/* Applied Micro X-Gene SoC DMA engine Driver
 *
 * Copyright (c) 2015, Applied Micro Circuits Corporation
 * Authors: Rameshwar Prasad Sahu <rsahu@apm.com>
 *	    Loc Ho <lho@apm.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __XGENE_DMA_H__
#define __XGENE_DMA_H__

/* DMA flyby opcode */
#define FLYBY_CRC16		0x1
#define FLYBY_ISCSI_CRC32C	0x2
#define FLYBY_CRC32		0x3
#define FLYBY_CHECKSUM		0x4

#define DMA_MAX_FLYBY_BYTE_CNT	0x7FFF	/* (32 KB - 1) */
#define DEFAULT_CRC32C_SEED	0xFFFFFFFF

/* Function to prepare flyby operations like CRC32c */
int xgene_dma_prep_flyby(void (*cb_func)(void *ctx, u32 result),
			 void *ctx, struct scatterlist *src_sg,
			 u32 nbytes, u32 seed, u8 opcode);

/* Allocate resources to flyby DMA channel */
int xgene_dma_flyby_alloc_resources(void);

/* Free all resources allocated to flyby DMA channel */
void xgene_dma_flyby_free_resources(void);

/* Dump DMA channel's Tx and Rx queue */
void xgene_dma_chan_hw_queue_desc_dump(int chan_id);

/* Dump all DMA channel's Tx and Rx queue */
void xgene_dma_hw_queue_desc_dump_all(void);

/* Dump DMA channel's sw ld pending, running, completed queues */
void xgene_dma_chan_sw_ld_queue_dump(int chan_id);

/* Dump all DMA channel's sw ld pending, running, completed queues */
void xgene_dma_sw_ld_queue_dump_all(void);

#endif /* __XGENE_DMA_H__ */
