/**
 * apm_pcp_csr.h - AppliedMicro X-Gene PCP Bridge Header
 *
 * Copyright (c) 2013, Applied Micro Circuits Corporation
 * Author: Loc Ho <lho@apm.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */
#ifndef APM_PCP_CSR_H
#define APM_PCP_CSR_H

/*
 Potenza / Storm PCP RB register addresses.
 Generated from potenza release 18256.
 Using //depot/projects/potenza/MAIN/uarch/lib/yaml/registers/registers.yaml.epy#212.
 Generated on 2012-10-11.
*/

/*       Global Base Address       */

#define PCP_RB_BASE_ADDR            0x000000007C000000U

/*    Register Bus page addresses. */

#define PCP_RB_CPU0_ROM_PAGE        0x00000000U
#define PCP_RB_CPU0_DBG_PAGE        0x00010000U
#define PCP_RB_CPU0_CTI_PAGE        0x00020000U
#define PCP_RB_CPU0_PMU_PAGE        0x00030000U
#define PCP_RB_CPU0_TRC_PAGE        0x00040000U
#define PCP_RB_CPU0_CPU_PAGE        0x000C0000U
#define PCP_RB_CPU0_L2C_PAGE        0x000D0000U
#define PCP_RB_CPU0_MEMERR_L2C_PAGE 0x000E0000U
#define PCP_RB_CPU0_MEMERR_CPU_PAGE 0x000F0000U

#define PCP_RB_CPU1_ROM_PAGE        0x00100000U
#define PCP_RB_CPU1_DBG_PAGE        0x00110000U
#define PCP_RB_CPU1_CTI_PAGE        0x00120000U
#define PCP_RB_CPU1_PMU_PAGE        0x00130000U
#define PCP_RB_CPU1_TRC_PAGE        0x00140000U
#define PCP_RB_CPU1_CPU_PAGE        0x001C0000U
#define PCP_RB_CPU1_L2C_PAGE        0x001D0000U
#define PCP_RB_CPU1_MEMERR_L2C_PAGE 0x001E0000U
#define PCP_RB_CPU1_MEMERR_CPU_PAGE 0x001F0000U

#define PCP_RB_CPU2_ROM_PAGE        0x00200000U
#define PCP_RB_CPU2_DBG_PAGE        0x00210000U
#define PCP_RB_CPU2_CTI_PAGE        0x00220000U
#define PCP_RB_CPU2_PMU_PAGE        0x00230000U
#define PCP_RB_CPU2_TRC_PAGE        0x00240000U
#define PCP_RB_CPU2_CPU_PAGE        0x002C0000U
#define PCP_RB_CPU2_L2C_PAGE        0x002D0000U
#define PCP_RB_CPU2_MEMERR_L2C_PAGE 0x002E0000U
#define PCP_RB_CPU2_MEMERR_CPU_PAGE 0x002F0000U

#define PCP_RB_CPU3_ROM_PAGE        0x00300000U
#define PCP_RB_CPU3_DBG_PAGE        0x00310000U
#define PCP_RB_CPU3_CTI_PAGE        0x00320000U
#define PCP_RB_CPU3_PMU_PAGE        0x00330000U
#define PCP_RB_CPU3_TRC_PAGE        0x00340000U
#define PCP_RB_CPU3_CPU_PAGE        0x003C0000U
#define PCP_RB_CPU3_L2C_PAGE        0x003D0000U
#define PCP_RB_CPU3_MEMERR_L2C_PAGE 0x003E0000U
#define PCP_RB_CPU3_MEMERR_CPU_PAGE 0x003F0000U

#define PCP_RB_CPU4_ROM_PAGE        0x00400000U
#define PCP_RB_CPU4_DBG_PAGE        0x00410000U
#define PCP_RB_CPU4_CTI_PAGE        0x00420000U
#define PCP_RB_CPU4_PMU_PAGE        0x00430000U
#define PCP_RB_CPU4_TRC_PAGE        0x00440000U
#define PCP_RB_CPU4_CPU_PAGE        0x004C0000U
#define PCP_RB_CPU4_L2C_PAGE        0x004D0000U
#define PCP_RB_CPU4_MEMERR_L2C_PAGE 0x004E0000U
#define PCP_RB_CPU4_MEMERR_CPU_PAGE 0x004F0000U

#define PCP_RB_CPU5_ROM_PAGE        0x00500000U
#define PCP_RB_CPU5_DBG_PAGE        0x00510000U
#define PCP_RB_CPU5_CTI_PAGE        0x00520000U
#define PCP_RB_CPU5_PMU_PAGE        0x00530000U
#define PCP_RB_CPU5_TRC_PAGE        0x00540000U
#define PCP_RB_CPU5_CPU_PAGE        0x005C0000U
#define PCP_RB_CPU5_L2C_PAGE        0x005D0000U
#define PCP_RB_CPU5_MEMERR_L2C_PAGE 0x005E0000U
#define PCP_RB_CPU5_MEMERR_CPU_PAGE 0x005F0000U

#define PCP_RB_CPU6_ROM_PAGE        0x00600000U
#define PCP_RB_CPU6_DBG_PAGE        0x00610000U
#define PCP_RB_CPU6_CTI_PAGE        0x00620000U
#define PCP_RB_CPU6_PMU_PAGE        0x00630000U
#define PCP_RB_CPU6_TRC_PAGE        0x00640000U
#define PCP_RB_CPU6_CPU_PAGE        0x006C0000U
#define PCP_RB_CPU6_L2C_PAGE        0x006D0000U
#define PCP_RB_CPU6_MEMERR_L2C_PAGE 0x006E0000U
#define PCP_RB_CPU6_MEMERR_CPU_PAGE 0x006F0000U

#define PCP_RB_CPU7_ROM_PAGE        0x00700000U
#define PCP_RB_CPU7_DBG_PAGE        0x00710000U
#define PCP_RB_CPU7_CTI_PAGE        0x00720000U
#define PCP_RB_CPU7_PMU_PAGE        0x00730000U
#define PCP_RB_CPU7_TRC_PAGE        0x00740000U
#define PCP_RB_CPU7_CPU_PAGE        0x007C0000U
#define PCP_RB_CPU7_L2C_PAGE        0x007D0000U
#define PCP_RB_CPU7_MEMERR_L2C_PAGE 0x007E0000U
#define PCP_RB_CPU7_MEMERR_CPU_PAGE 0x007F0000U


#define PCP_RB_RBM_PAGE             0x02000000U
#define PCP_RB_CSW_PAGE             0x02200000U
#define PCP_RB_CSW_DFT_PAGE         0x02210000U
#define PCP_RB_CSW_TRACE_PAGE       0x02300000U
#define PCP_RB_CSW_CTI_PAGE         0x02310000U

#define PCP_RB_DFT_PMD0_PAGE        0x02400000U
#define PCP_RB_DFT_PMD1_PAGE        0x02410000U
#define PCP_RB_DFT_PMD2_PAGE        0x02420000U
#define PCP_RB_DFT_PMD3_PAGE        0x02430000U

#define PCP_RB_L3C_PAGE             0x02600000U
#define PCP_RB_L3C_PMU_PAGE         0x02610000U
#define PCP_RB_MCB0_PAGE            0x02700000U
#define PCP_RB_MCB0_PMU_PAGE        0x02710000U
#define PCP_RB_MCB1_PAGE            0x02720000U
#define PCP_RB_MCB1_PMU_PAGE        0x02730000U

#define PCP_RB_MCU0A_PAGE           0x02800000U
#define PCP_RB_MCU0A_PMU_PAGE       0x02810000U
#define PCP_RB_MCU0A_PHY_PAGE       0x02820000U
#define PCP_RB_MCU0B_PAGE           0x02840000U
#define PCP_RB_MCU0B_PMU_PAGE       0x02850000U
#define PCP_RB_MCU0B_PHY_PAGE       0x02860000U
#define PCP_RB_MCU1A_PAGE           0x02880000U
#define PCP_RB_MCU1A_PMU_PAGE       0x02890000U
#define PCP_RB_MCU1A_PHY_PAGE       0x028A0000U
#define PCP_RB_MCU1B_PAGE           0x028C0000U
#define PCP_RB_MCU1B_PMU_PAGE       0x028D0000U
#define PCP_RB_MCU1B_PHY_PAGE       0x028E0000U

#define PCP_RB_IOB_CSW_PAGE         0x02900000U
#define PCP_RB_IOB_CFG_PAGE         0x02920000U
#define PCP_RB_IOB_ERR_PAGE         0x02930000U
#define PCP_RB_IOB_PMU_PAGE         0x02940000U
#define PCP_RB_IOB_DBG_PAGE         0x02950000U

/*     Register page offsets.    */

#define PCP_RB_CPUX_CTI_ASICCTL_PAGE_OFFSET                     0x00000144U
#define PCP_RB_CPUX_CPU_CCCR_PAGE_OFFSET                        0x00000000U
#define PCP_RB_CPUX_CPU_CPE0_PAGE_OFFSET                        0x00000010U
#define PCP_RB_CPUX_CPU_CPE1_PAGE_OFFSET                        0x00000014U
#define PCP_RB_CPUX_CPU_CPE2_PAGE_OFFSET                        0x00000018U
#define PCP_RB_CPUX_CPU_CPE3_PAGE_OFFSET                        0x0000001cU
#define PCP_RB_CPUX_CPU_CRCR_PAGE_OFFSET                        0x00000004U
#define PCP_RB_CSW_CSWASR_PAGE_OFFSET                           0x00000004U
#define PCP_RB_IOB_CSW_CSWCCR_PAGE_OFFSET                       0x00000008U
#define PCP_RB_CSW_CSWCR_PAGE_OFFSET                            0x00000000U
#define PCP_RB_IOB_CSW_CSWRCR_PAGE_OFFSET                       0x00000000U
#define PCP_RB_CSW_CSWSER_PAGE_OFFSET                           0x00000008U
#define PCP_RB_CPUX_CTI_CTIAPPCLEAR_PAGE_OFFSET                 0x00000018U
#define PCP_RB_CPUX_CTI_CTIAPPSET_PAGE_OFFSET                   0x00000014U
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_PAGE_OFFSET               0x00000fb8U
#define PCP_RB_CPUX_CTI_CTICHANPULSE_PAGE_OFFSET                0x0000001cU
#define PCP_RB_CPUX_CTI_CTICHINSTATUS_PAGE_OFFSET               0x00000138U
#define PCP_RB_CPUX_CTI_CTICHOUTSTATUS_PAGE_OFFSET              0x0000013cU
#define PCP_RB_CPUX_CTI_CTICIDR0_PAGE_OFFSET                    0x00000ff0U
#define PCP_RB_CPUX_CTI_CTICIDR1_PAGE_OFFSET                    0x00000ff4U
#define PCP_RB_CPUX_CTI_CTICIDR2_PAGE_OFFSET                    0x00000ff8U
#define PCP_RB_CPUX_CTI_CTICIDR3_PAGE_OFFSET                    0x00000ffcU
#define PCP_RB_CPUX_CTI_CTICONTROL_PAGE_OFFSET                  0x00000000U
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_PAGE_OFFSET                  0x00000fa8U
#define PCP_RB_CPUX_CTI_CTIDEVAFF1_PAGE_OFFSET                  0x00000facU
#define PCP_RB_CPUX_CTI_CTIDEVARCH_PAGE_OFFSET                  0x00000fbcU
#define PCP_RB_CPUX_CTI_CTIDEVID0_PAGE_OFFSET                   0x00000fc8U
#define PCP_RB_CPUX_CTI_CTIDEVID1_PAGE_OFFSET                   0x00000fc4U
#define PCP_RB_CPUX_CTI_CTIDEVID2_PAGE_OFFSET                   0x00000fc0U
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_PAGE_OFFSET                  0x00000fccU
#define PCP_RB_CPUX_CTI_CTIGATE_PAGE_OFFSET                     0x00000140U
#define PCP_RB_CPUX_CTI_CTIINEN0_PAGE_OFFSET                    0x00000020U
#define PCP_RB_CPUX_CTI_CTIINEN1_PAGE_OFFSET                    0x00000024U
#define PCP_RB_CPUX_CTI_CTIINEN2_PAGE_OFFSET                    0x00000028U
#define PCP_RB_CPUX_CTI_CTIINEN3_PAGE_OFFSET                    0x0000002cU
#define PCP_RB_CPUX_CTI_CTIINEN4_PAGE_OFFSET                    0x00000030U
#define PCP_RB_CPUX_CTI_CTIINEN5_PAGE_OFFSET                    0x00000034U
#define PCP_RB_CPUX_CTI_CTIINEN6_PAGE_OFFSET                    0x00000038U
#define PCP_RB_CPUX_CTI_CTIINEN7_PAGE_OFFSET                    0x0000003cU
#define PCP_RB_CPUX_CTI_CTIINTACK_PAGE_OFFSET                   0x00000010U
#define PCP_RB_CPUX_CTI_CTILAR_PAGE_OFFSET                      0x00000fb0U
#define PCP_RB_CPUX_CTI_CTILSR_PAGE_OFFSET                      0x00000fb4U
#define PCP_RB_CPUX_CTI_CTIOUTEN0_PAGE_OFFSET                   0x000000a0U
#define PCP_RB_CPUX_CTI_CTIOUTEN1_PAGE_OFFSET                   0x000000a4U
#define PCP_RB_CPUX_CTI_CTIOUTEN2_PAGE_OFFSET                   0x000000a8U
#define PCP_RB_CPUX_CTI_CTIOUTEN3_PAGE_OFFSET                   0x000000acU
#define PCP_RB_CPUX_CTI_CTIOUTEN4_PAGE_OFFSET                   0x000000b0U
#define PCP_RB_CPUX_CTI_CTIOUTEN5_PAGE_OFFSET                   0x000000b4U
#define PCP_RB_CPUX_CTI_CTIOUTEN6_PAGE_OFFSET                   0x000000b8U
#define PCP_RB_CPUX_CTI_CTIOUTEN7_PAGE_OFFSET                   0x000000bcU
#define PCP_RB_CPUX_CTI_CTIPIDR0_PAGE_OFFSET                    0x00000fe0U
#define PCP_RB_CPUX_CTI_CTIPIDR1_PAGE_OFFSET                    0x00000fe4U
#define PCP_RB_CPUX_CTI_CTIPIDR2_PAGE_OFFSET                    0x00000fe8U
#define PCP_RB_CPUX_CTI_CTIPIDR3_PAGE_OFFSET                    0x00000fecU
#define PCP_RB_CPUX_CTI_CTIPIDR4_PAGE_OFFSET                    0x00000fd0U
#define PCP_RB_CPUX_CTI_CTITRIGINSTATUS_PAGE_OFFSET             0x00000130U
#define PCP_RB_CPUX_CTI_CTITRIGOUTSTATUS_PAGE_OFFSET            0x00000134U
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_PAGE_OFFSET           0x00000fb8U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_PAGE_OFFSET                 0x00000408U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_PAGE_OFFSET                 0x00000418U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_PAGE_OFFSET                 0x00000428U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_PAGE_OFFSET                 0x00000438U
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_PAGE_OFFSET                 0x00000400U
#define PCP_RB_CPUX_DBG_DBGBXVR0_PAGE_OFFSET                    0x00000404U
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_PAGE_OFFSET                 0x00000410U
#define PCP_RB_CPUX_DBG_DBGBXVR1_PAGE_OFFSET                    0x00000414U
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_PAGE_OFFSET                 0x00000420U
#define PCP_RB_CPUX_DBG_DBGBXVR2_PAGE_OFFSET                    0x00000424U
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_PAGE_OFFSET                 0x00000430U
#define PCP_RB_CPUX_DBG_DBGBXVR3_PAGE_OFFSET                    0x00000434U
#define PCP_RB_CPUX_DBG_DBGCLAIMCLR_EL1_PAGE_OFFSET             0x00000fa4U
#define PCP_RB_CPUX_DBG_DBGCLAIMSET_EL1_PAGE_OFFSET             0x00000fa0U
#define PCP_RB_CPUX_DBG_DBGDTRRX_EL0_PAGE_OFFSET                0x00000080U
#define PCP_RB_CPUX_DBG_DBGDTRTX_EL0_PAGE_OFFSET                0x0000008cU
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_PAGE_OFFSET                 0x00000800U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_PAGE_OFFSET                 0x00000808U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_PAGE_OFFSET                 0x00000818U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_PAGE_OFFSET                 0x00000828U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_PAGE_OFFSET                 0x00000838U
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_PAGE_OFFSET                 0x00000800U
#define PCP_RB_CPUX_DBG_DBGWXVR0_PAGE_OFFSET                    0x00000804U
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_PAGE_OFFSET                 0x00000810U
#define PCP_RB_CPUX_DBG_DBGWXVR1_PAGE_OFFSET                    0x00000814U
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_PAGE_OFFSET                 0x00000820U
#define PCP_RB_CPUX_DBG_DBGWXVR2_PAGE_OFFSET                    0x00000824U
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_PAGE_OFFSET                 0x00000830U
#define PCP_RB_CPUX_DBG_DBGWXVR3_PAGE_OFFSET                    0x00000834U
#define PCP_RB_CPUX_DBG_EDACR_PAGE_OFFSET                       0x00000094U
#define PCP_RB_CPUX_DBG_EDCIDR0_PAGE_OFFSET                     0x00000ff0U
#define PCP_RB_CPUX_DBG_EDCIDR1_PAGE_OFFSET                     0x00000ff4U
#define PCP_RB_CPUX_DBG_EDCIDR2_PAGE_OFFSET                     0x00000ff8U
#define PCP_RB_CPUX_DBG_EDCIDR3_PAGE_OFFSET                     0x00000ffcU
#define PCP_RB_CPUX_DBG_EDDEVAFF0_PAGE_OFFSET                   0x00000fa8U
#define PCP_RB_CPUX_DBG_EDDEVAFF1_PAGE_OFFSET                   0x00000facU
#define PCP_RB_CPUX_DBG_EDDEVARCH_PAGE_OFFSET                   0x00000fbcU
#define PCP_RB_CPUX_DBG_EDDEVID_PAGE_OFFSET                     0x00000fc8U
#define PCP_RB_CPUX_DBG_EDDEVID1_PAGE_OFFSET                    0x00000fc4U
#define PCP_RB_CPUX_DBG_EDDEVID2_PAGE_OFFSET                    0x00000fc0U
#define PCP_RB_CPUX_DBG_EDDEVTYPE_PAGE_OFFSET                   0x00000fccU
#define PCP_RB_CPUX_DBG_EDECCR_PAGE_OFFSET                      0x00000098U
#define PCP_RB_CPUX_DBG_EDECR_PAGE_OFFSET                       0x00000024U
#define PCP_RB_CPUX_DBG_EDESR_PAGE_OFFSET                       0x00000020U
#define PCP_RB_CPUX_DBG_EDITCTRL_PAGE_OFFSET                    0x00000f00U
#define PCP_RB_CPUX_DBG_EDITR_PAGE_OFFSET                       0x00000084U
#define PCP_RB_CPUX_DBG_EDLAR_PAGE_OFFSET                       0x00000fb0U
#define PCP_RB_CPUX_DBG_EDLSR_PAGE_OFFSET                       0x00000fb4U
#define PCP_RB_CPUX_DBG_EDPIDR0_PAGE_OFFSET                     0x00000fe0U
#define PCP_RB_CPUX_DBG_EDPIDR1_PAGE_OFFSET                     0x00000fe4U
#define PCP_RB_CPUX_DBG_EDPIDR2_PAGE_OFFSET                     0x00000fe8U
#define PCP_RB_CPUX_DBG_EDPIDR3_PAGE_OFFSET                     0x00000fecU
#define PCP_RB_CPUX_DBG_EDPIDR4_PAGE_OFFSET                     0x00000fd0U
#define PCP_RB_CPUX_DBG_EDPRCR_PAGE_OFFSET                      0x00000310U
#define PCP_RB_CPUX_CPU_EDPRCR_ERROR_PAGE_OFFSET                0x00000ff8U
#define PCP_RB_CPUX_DBG_EDPRSR_PAGE_OFFSET                      0x00000314U
#define PCP_RB_CPUX_DBG_EDRCR_PAGE_OFFSET                       0x00000090U
#define PCP_RB_CPUX_DBG_EDSCR_PAGE_OFFSET                       0x00000088U
#define PCP_RB_CPUX_DBG_EDWARHI_PAGE_OFFSET                     0x00000034U
#define PCP_RB_CPUX_DBG_EDWARLO_PAGE_OFFSET                     0x00000030U
#define PCP_RB_CSW_TRACE_EXTEVENT_PAGE_OFFSET                   0x000000f0U
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_PAGE_OFFSET               0x00000000U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_PAGE_OFFSET               0x00000004U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_PAGE_OFFSET              0x00000804U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_PAGE_OFFSET             0x00000d28U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1HI_PAGE_OFFSET           0x00000d2cU
#define PCP_RB_CPUX_DBG_ID_AA64DFR1_EL1_PAGE_OFFSET             0x00000d48U
#define PCP_RB_CPUX_DBG_ID_AA64DFR1_EL1HI_PAGE_OFFSET           0x00000d4cU
#define PCP_RB_CPUX_DBG_ID_AA64ISAR0_EL1_PAGE_OFFSET            0x00000d30U
#define PCP_RB_CPUX_DBG_ID_AA64ISAR0_EL1HI_PAGE_OFFSET          0x00000d34U
#define PCP_RB_CPUX_DBG_ID_AA64ISAR1_EL1_PAGE_OFFSET            0x00000d50U
#define PCP_RB_CPUX_DBG_ID_AA64ISAR1_EL1HI_PAGE_OFFSET          0x00000d54U
#define PCP_RB_CPUX_DBG_ID_AA64MMFR0_EL1_PAGE_OFFSET            0x00000d38U
#define PCP_RB_CPUX_DBG_ID_AA64MMFR0_EL1HI_PAGE_OFFSET          0x00000d3cU
#define PCP_RB_CPUX_DBG_ID_AA64MMFR1_EL1_PAGE_OFFSET            0x00000d58U
#define PCP_RB_CPUX_DBG_ID_AA64MMFR1_EL1HI_PAGE_OFFSET          0x00000d5cU
#define PCP_RB_CPUX_DBG_ID_AA64PFR0_EL1_PAGE_OFFSET             0x00000d20U
#define PCP_RB_CPUX_DBG_ID_AA64PFR0_EL1HI_PAGE_OFFSET           0x00000d24U
#define PCP_RB_CPUX_DBG_ID_AA64PFR1_EL1_PAGE_OFFSET             0x00000d40U
#define PCP_RB_CPUX_DBG_ID_AA64PFR1_EL1HI_PAGE_OFFSET           0x00000d44U
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_PAGE_OFFSET       0x00000004U
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_PAGE_OFFSET       0x00000000U
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRREQINFOH_PAGE_OFFSET     0x0000000cU
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRREQINFOL_PAGE_OFFSET     0x00000008U
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_PAGE_OFFSET       0x00000014U
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_PAGE_OFFSET       0x00000010U
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRREQINFOH_PAGE_OFFSET     0x0000001cU
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRREQINFOL_PAGE_OFFSET     0x00000018U
#define PCP_RB_IOB_CFG_IOBBACFG_PAGE_OFFSET                     0x00000010U
#define PCP_RB_IOB_ERR_IOBBATRANSERRCSWREQID_PAGE_OFFSET        0x00000040U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_PAGE_OFFSET          0x00000034U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_PAGE_OFFSET          0x00000030U
#define PCP_RB_IOB_ERR_IOBBATRANSERRREQINFOH_PAGE_OFFSET        0x0000003cU
#define PCP_RB_IOB_ERR_IOBBATRANSERRREQINFOL_PAGE_OFFSET        0x00000038U
#define PCP_RB_IOB_ERR_IOBERRRESP_PAGE_OFFSET                   0x000007fcU
#define PCP_RB_IOB_CFG_IOBMEMCFG_PAGE_OFFSET                    0x00000100U
#define PCP_RB_IOB_CFG_IOBPACFG_PAGE_OFFSET                     0x00000000U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_PAGE_OFFSET          0x00000024U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_PAGE_OFFSET          0x00000020U
#define PCP_RB_IOB_CFG_IOBREMAPBAR_PAGE_OFFSET                  0x00000040U
#define PCP_RB_IOB_CFG_IOBREMAPRNG_PAGE_OFFSET                  0x00000020U
#define PCP_RB_IOB_CFG_IOBREMAPTAR5_PAGE_OFFSET                 0x00000074U
#define PCP_RB_IOB_CFG_IOBTRACECFG_PAGE_OFFSET                  0x00000080U
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_PAGE_OFFSET                0x00000004U
#define PCP_RB_CPUX_L2C_L2CR_PAGE_OFFSET                        0x00000000U
#define PCP_RB_CPUX_MEMERR_L2C_L2EAHR_PAGE_OFFSET               0x0000000cU
#define PCP_RB_CPUX_MEMERR_L2C_L2EALR_PAGE_OFFSET               0x00000008U
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_PAGE_OFFSET                0x00000000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_PAGE_OFFSET                0x00000004U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_PAGE_OFFSET               0x00000804U
#define PCP_RB_CPUX_L2C_L2PE0_PAGE_OFFSET                       0x00000020U
#define PCP_RB_CPUX_L2C_L2PE1_PAGE_OFFSET                       0x00000024U
#define PCP_RB_CPUX_L2C_L2PE2_PAGE_OFFSET                       0x00000028U
#define PCP_RB_CPUX_L2C_L2PE3_PAGE_OFFSET                       0x0000002cU
#define PCP_RB_CPUX_L2C_L2PE4_PAGE_OFFSET                       0x00000030U
#define PCP_RB_CPUX_L2C_L2RTOAHR_PAGE_OFFSET                    0x0000001cU
#define PCP_RB_CPUX_L2C_L2RTOALR_PAGE_OFFSET                    0x00000018U
#define PCP_RB_CPUX_L2C_L2RTOCR_PAGE_OFFSET                     0x00000010U
#define PCP_RB_CPUX_L2C_L2RTOSR_PAGE_OFFSET                     0x00000014U
#define PCP_RB_CPUX_L2C_L2WD0_PAGE_OFFSET                       0x00000008U
#define PCP_RB_CPUX_L2C_L2WD1_PAGE_OFFSET                       0x0000000cU
#define PCP_RB_CSW_L3ASR_PAGE_OFFSET                            0x0000030cU
#define PCP_RB_CSW_L3CCR_PAGE_OFFSET                            0x00000308U
#define PCP_RB_CSW_L3RCR_PAGE_OFFSET                            0x00000300U
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_PAGE_OFFSET               0x00000008U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_PAGE_OFFSET               0x0000000cU
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_PAGE_OFFSET              0x0000080cU
#define PCP_RB_CSW_MCB0ASR_PAGE_OFFSET                          0x0000040cU
#define PCP_RB_CSW_MCB0CCR_PAGE_OFFSET                          0x00000408U
#define PCP_RB_CSW_MCB0RCR_PAGE_OFFSET                          0x00000400U
#define PCP_RB_CSW_MCB1ASR_PAGE_OFFSET                          0x0000050cU
#define PCP_RB_CSW_MCB1CCR_PAGE_OFFSET                          0x00000508U
#define PCP_RB_CSW_MCB1RCR_PAGE_OFFSET                          0x00000500U
#define PCP_RB_CPUX_DBG_MIDR_EL1_PAGE_OFFSET                    0x00000d00U
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_PAGE_OFFSET               0x00000010U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_PAGE_OFFSET               0x00000014U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_PAGE_OFFSET              0x00000814U
#define PCP_RB_CPUX_DBG_OSLAR_EL1_PAGE_OFFSET                   0x00000300U
#define PCP_RB_CPUX_CPU_OSLK_DLK_ERROR_PAGE_OFFSET              0x00000ffcU
#define PCP_RB_CSW_P0ASR_PAGE_OFFSET                            0x00000208U
#define PCP_RB_CSW_P0CCR0_PAGE_OFFSET                           0x00000200U
#define PCP_RB_CSW_P0CCR1_PAGE_OFFSET                           0x00000204U
#define PCP_RB_CSW_P0RCR_PAGE_OFFSET                            0x00000100U
#define PCP_RB_CSW_P0RSR_PAGE_OFFSET                            0x00000104U
#define PCP_RB_CSW_P1ASR_PAGE_OFFSET                            0x00000218U
#define PCP_RB_CSW_P1CCR0_PAGE_OFFSET                           0x00000210U
#define PCP_RB_CSW_P1CCR1_PAGE_OFFSET                           0x00000214U
#define PCP_RB_CSW_P1RCR_PAGE_OFFSET                            0x00000110U
#define PCP_RB_CSW_P1RSR_PAGE_OFFSET                            0x00000114U
#define PCP_RB_CSW_P2ASR_PAGE_OFFSET                            0x00000228U
#define PCP_RB_CSW_P2CCR0_PAGE_OFFSET                           0x00000220U
#define PCP_RB_CSW_P2CCR1_PAGE_OFFSET                           0x00000224U
#define PCP_RB_CSW_P2RCR_PAGE_OFFSET                            0x00000120U
#define PCP_RB_CSW_P2RSR_PAGE_OFFSET                            0x00000124U
#define PCP_RB_CSW_P3ASR_PAGE_OFFSET                            0x00000238U
#define PCP_RB_CSW_P3CCR0_PAGE_OFFSET                           0x00000230U
#define PCP_RB_CSW_P3CCR1_PAGE_OFFSET                           0x00000234U
#define PCP_RB_CSW_P3RCR_PAGE_OFFSET                            0x00000130U
#define PCP_RB_CSW_P3RSR_PAGE_OFFSET                            0x00000134U
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_PAGE_OFFSET                 0x00000200U
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_PAGE_OFFSET                 0x00000204U
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_PAGE_OFFSET                0x00000fb8U
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_PAGE_OFFSET               0x0000047cU
#define PCP_RB_CPUX_PMU_PMCCNTRLO_EL0_PAGE_OFFSET               0x000000f8U
#define PCP_RB_CPUX_PMU_PMCCNTRHI_EL0_PAGE_OFFSET               0x000000fcU
#define PCP_RB_CPUX_PMU_PMCEID0_EL0_PAGE_OFFSET                 0x00000e20U
#define PCP_RB_CPUX_PMU_PMCEID1_EL0_PAGE_OFFSET                 0x00000e24U
#define PCP_RB_CPUX_PMU_PMCFGR_PAGE_OFFSET                      0x00000e00U
#define PCP_RB_CPUX_PMU_PMCIDR0_PAGE_OFFSET                     0x00000ff0U
#define PCP_RB_CPUX_PMU_PMCIDR1_PAGE_OFFSET                     0x00000ff4U
#define PCP_RB_CPUX_PMU_PMCIDR2_PAGE_OFFSET                     0x00000ff8U
#define PCP_RB_CPUX_PMU_PMCIDR3_PAGE_OFFSET                     0x00000ffcU
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_PAGE_OFFSET              0x00000c20U
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_PAGE_OFFSET              0x00000c00U
#define PCP_RB_CPUX_PMU_PMCR_EL0_PAGE_OFFSET                    0x00000e04U
#define PCP_RB_CPUX_PMU_PMDEVAFF0_PAGE_OFFSET                   0x00000fa8U
#define PCP_RB_CPUX_PMU_PMDEVAFF1_PAGE_OFFSET                   0x00000facU
#define PCP_RB_CPUX_PMU_PMDEVARCH_PAGE_OFFSET                   0x00000fbcU
#define PCP_RB_CPUX_PMU_PMDEVTYPE_PAGE_OFFSET                   0x00000fccU
#define PCP_RB_CPUX_PMU_PMEVCNTR0_EL0_PAGE_OFFSET               0x00000000U
#define PCP_RB_CPUX_PMU_PMEVCNTR1_EL0_PAGE_OFFSET               0x00000008U
#define PCP_RB_CPUX_PMU_PMEVCNTR2_EL0_PAGE_OFFSET               0x00000010U
#define PCP_RB_CPUX_PMU_PMEVCNTR3_EL0_PAGE_OFFSET               0x00000018U
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_PAGE_OFFSET              0x00000400U
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_PAGE_OFFSET              0x00000404U
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_PAGE_OFFSET              0x00000408U
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_PAGE_OFFSET              0x0000040cU
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_PAGE_OFFSET              0x00000c60U
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_PAGE_OFFSET              0x00000c40U
#define PCP_RB_CPUX_PMU_PMITCTRL_PAGE_OFFSET                    0x00000f00U
#define PCP_RB_CPUX_PMU_PMLAR_PAGE_OFFSET                       0x00000fb0U
#define PCP_RB_CPUX_PMU_PMLSR_PAGE_OFFSET                       0x00000fb4U
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_PAGE_OFFSET                0x00000c80U
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_PAGE_OFFSET                0x00000cc0U
#define PCP_RB_CPUX_PMU_PMPIDR0_PAGE_OFFSET                     0x00000fe0U
#define PCP_RB_CPUX_PMU_PMPIDR1_PAGE_OFFSET                     0x00000fe4U
#define PCP_RB_CPUX_PMU_PMPIDR2_PAGE_OFFSET                     0x00000fe8U
#define PCP_RB_CPUX_PMU_PMPIDR3_PAGE_OFFSET                     0x00000fecU
#define PCP_RB_CPUX_PMU_PMPIDR4_PAGE_OFFSET                     0x00000fd0U
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_PAGE_OFFSET                 0x00000ca0U
#define PCP_RB_RBM_RBASR_PAGE_OFFSET                            0x00000004U
#define PCP_RB_RBM_RBCSR_PAGE_OFFSET                            0x00000000U
#define PCP_RB_RBM_RBEIR_PAGE_OFFSET                            0x00000008U
#define PCP_RB_CPUX_CPU_RBER_PAGE_OFFSET                        0x0000000cU
#define PCP_RB_RBM_RBNR_PAGE_OFFSET                             0x0000000cU
#define PCP_RB_CPUX_TRC_TRCACATR0_PAGE_OFFSET                   0x00000480U
#define PCP_RB_CPUX_TRC_TRCACATR1_PAGE_OFFSET                   0x00000488U
#define PCP_RB_CPUX_TRC_TRCACATR10_PAGE_OFFSET                  0x000004d0U
#define PCP_RB_CPUX_TRC_TRCACATR11_PAGE_OFFSET                  0x000004d8U
#define PCP_RB_CPUX_TRC_TRCACATR12_PAGE_OFFSET                  0x000004e0U
#define PCP_RB_CPUX_TRC_TRCACATR13_PAGE_OFFSET                  0x000004e8U
#define PCP_RB_CPUX_TRC_TRCACATR14_PAGE_OFFSET                  0x000004f0U
#define PCP_RB_CPUX_TRC_TRCACATR15_PAGE_OFFSET                  0x000004f8U
#define PCP_RB_CPUX_TRC_TRCACATR2_PAGE_OFFSET                   0x00000490U
#define PCP_RB_CPUX_TRC_TRCACATR3_PAGE_OFFSET                   0x00000498U
#define PCP_RB_CPUX_TRC_TRCACATR4_PAGE_OFFSET                   0x000004a0U
#define PCP_RB_CPUX_TRC_TRCACATR5_PAGE_OFFSET                   0x000004a8U
#define PCP_RB_CPUX_TRC_TRCACATR6_PAGE_OFFSET                   0x000004b0U
#define PCP_RB_CPUX_TRC_TRCACATR7_PAGE_OFFSET                   0x000004b8U
#define PCP_RB_CPUX_TRC_TRCACATR8_PAGE_OFFSET                   0x000004c0U
#define PCP_RB_CPUX_TRC_TRCACATR9_PAGE_OFFSET                   0x000004c8U
#define PCP_RB_CSW_TRACE_TRCACKWP_PAGE_OFFSET                   0x00000014U
#define PCP_RB_CPUX_TRC_TRCACVR0LO_PAGE_OFFSET                  0x00000400U
#define PCP_RB_CPUX_TRC_TRCACVR0HI_PAGE_OFFSET                  0x00000404U
#define PCP_RB_CPUX_TRC_TRCACVR1LO_PAGE_OFFSET                  0x00000408U
#define PCP_RB_CPUX_TRC_TRCACVR1HI_PAGE_OFFSET                  0x0000040cU
#define PCP_RB_CPUX_TRC_TRCACVR10LO_PAGE_OFFSET                 0x00000450U
#define PCP_RB_CPUX_TRC_TRCACVR10HI_PAGE_OFFSET                 0x00000454U
#define PCP_RB_CPUX_TRC_TRCACVR11LO_PAGE_OFFSET                 0x00000458U
#define PCP_RB_CPUX_TRC_TRCACVR11HI_PAGE_OFFSET                 0x0000045cU
#define PCP_RB_CPUX_TRC_TRCACVR12LO_PAGE_OFFSET                 0x00000460U
#define PCP_RB_CPUX_TRC_TRCACVR12HI_PAGE_OFFSET                 0x00000464U
#define PCP_RB_CPUX_TRC_TRCACVR13LO_PAGE_OFFSET                 0x00000468U
#define PCP_RB_CPUX_TRC_TRCACVR13HI_PAGE_OFFSET                 0x0000046cU
#define PCP_RB_CPUX_TRC_TRCACVR14LO_PAGE_OFFSET                 0x00000470U
#define PCP_RB_CPUX_TRC_TRCACVR14HI_PAGE_OFFSET                 0x00000474U
#define PCP_RB_CPUX_TRC_TRCACVR15LO_PAGE_OFFSET                 0x00000478U
#define PCP_RB_CPUX_TRC_TRCACVR15HI_PAGE_OFFSET                 0x0000047cU
#define PCP_RB_CPUX_TRC_TRCACVR2LO_PAGE_OFFSET                  0x00000410U
#define PCP_RB_CPUX_TRC_TRCACVR2HI_PAGE_OFFSET                  0x00000414U
#define PCP_RB_CPUX_TRC_TRCACVR3LO_PAGE_OFFSET                  0x00000418U
#define PCP_RB_CPUX_TRC_TRCACVR3HI_PAGE_OFFSET                  0x0000041cU
#define PCP_RB_CPUX_TRC_TRCACVR4LO_PAGE_OFFSET                  0x00000420U
#define PCP_RB_CPUX_TRC_TRCACVR4HI_PAGE_OFFSET                  0x00000424U
#define PCP_RB_CPUX_TRC_TRCACVR5LO_PAGE_OFFSET                  0x00000428U
#define PCP_RB_CPUX_TRC_TRCACVR5HI_PAGE_OFFSET                  0x0000042cU
#define PCP_RB_CPUX_TRC_TRCACVR6LO_PAGE_OFFSET                  0x00000430U
#define PCP_RB_CPUX_TRC_TRCACVR6HI_PAGE_OFFSET                  0x00000434U
#define PCP_RB_CPUX_TRC_TRCACVR7LO_PAGE_OFFSET                  0x00000438U
#define PCP_RB_CPUX_TRC_TRCACVR7HI_PAGE_OFFSET                  0x0000043cU
#define PCP_RB_CPUX_TRC_TRCACVR8LO_PAGE_OFFSET                  0x00000440U
#define PCP_RB_CPUX_TRC_TRCACVR8HI_PAGE_OFFSET                  0x00000444U
#define PCP_RB_CPUX_TRC_TRCACVR9LO_PAGE_OFFSET                  0x00000448U
#define PCP_RB_CPUX_TRC_TRCACVR9HI_PAGE_OFFSET                  0x0000044cU
#define PCP_RB_CSW_TRACE_TRCADDR_PAGE_OFFSET                    0x00000020U
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_PAGE_OFFSET               0x00000fb8U
#define PCP_RB_CPUX_TRC_TRCAUXCTLR_PAGE_OFFSET                  0x00000018U
#define PCP_RB_CPUX_TRC_TRCBBCTLR_PAGE_OFFSET                   0x0000003cU
#define PCP_RB_CPUX_TRC_TRCCCCTLR_PAGE_OFFSET                   0x00000038U
#define PCP_RB_CPUX_TRC_TRCCIDCCTLR0_PAGE_OFFSET                0x00000680U
#define PCP_RB_CPUX_TRC_TRCCIDCCTLR1_PAGE_OFFSET                0x00000684U
#define PCP_RB_CPUX_TRC_TRCCIDCVR0_PAGE_OFFSET                  0x00000600U
#define PCP_RB_CPUX_TRC_TRCCIDCVR1_PAGE_OFFSET                  0x00000608U
#define PCP_RB_CPUX_TRC_TRCCIDCVR2_PAGE_OFFSET                  0x00000610U
#define PCP_RB_CPUX_TRC_TRCCIDCVR3_PAGE_OFFSET                  0x00000618U
#define PCP_RB_CPUX_TRC_TRCCIDCVR4_PAGE_OFFSET                  0x00000620U
#define PCP_RB_CPUX_TRC_TRCCIDCVR5_PAGE_OFFSET                  0x00000628U
#define PCP_RB_CPUX_TRC_TRCCIDCVR6_PAGE_OFFSET                  0x00000630U
#define PCP_RB_CPUX_TRC_TRCCIDCVR7_PAGE_OFFSET                  0x00000638U
#define PCP_RB_CPUX_TRC_TRCCIDR0_PAGE_OFFSET                    0x00000ff0U
#define PCP_RB_CPUX_TRC_TRCCIDR1_PAGE_OFFSET                    0x00000ff4U
#define PCP_RB_CPUX_TRC_TRCCIDR2_PAGE_OFFSET                    0x00000ff8U
#define PCP_RB_CPUX_TRC_TRCCIDR3_PAGE_OFFSET                    0x00000ffcU
#define PCP_RB_CPUX_TRC_TRCCLAIMCLR_PAGE_OFFSET                 0x00000fa4U
#define PCP_RB_CPUX_TRC_TRCCLAIMSET_PAGE_OFFSET                 0x00000fa0U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_PAGE_OFFSET                 0x00000150U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_PAGE_OFFSET                 0x00000154U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR2_PAGE_OFFSET                 0x00000158U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR3_PAGE_OFFSET                 0x0000015cU
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR0_PAGE_OFFSET                0x00000140U
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR1_PAGE_OFFSET                0x00000144U
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR2_PAGE_OFFSET                0x00000148U
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR3_PAGE_OFFSET                0x0000014cU
#define PCP_RB_CPUX_TRC_TRCCNTVR0_PAGE_OFFSET                   0x00000160U
#define PCP_RB_CPUX_TRC_TRCCNTVR1_PAGE_OFFSET                   0x00000164U
#define PCP_RB_CPUX_TRC_TRCCNTVR2_PAGE_OFFSET                   0x00000168U
#define PCP_RB_CPUX_TRC_TRCCNTVR3_PAGE_OFFSET                   0x0000016cU
#define PCP_RB_CPUX_TRC_TRCCONFIGR_PAGE_OFFSET                  0x00000010U
#define PCP_RB_CSW_TRACE_TRCCTL_PAGE_OFFSET                     0x00000000U
#define PCP_RB_CSW_TRACE_TRCDATA_PAGE_OFFSET                    0x00000024U
#define PCP_RB_CSW_TRACE_TRCDATAWP_PAGE_OFFSET                  0x00000018U
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_PAGE_OFFSET                  0x00000fa8U
#define PCP_RB_CPUX_TRC_TRCDEVAFF1_PAGE_OFFSET                  0x00000facU
#define PCP_RB_CPUX_TRC_TRCDEVARCH_PAGE_OFFSET                  0x00000fbcU
#define PCP_RB_CPUX_TRC_TRCDEVID_PAGE_OFFSET                    0x00000fc8U
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_PAGE_OFFSET                  0x00000fccU
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_PAGE_OFFSET               0x00000020U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_PAGE_OFFSET               0x00000024U
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_PAGE_OFFSET                0x00000120U
#define PCP_RB_CPUX_TRC_TRCIDR0_PAGE_OFFSET                     0x000001e0U
#define PCP_RB_CPUX_TRC_TRCIDR1_PAGE_OFFSET                     0x000001e4U
#define PCP_RB_CPUX_TRC_TRCIDR10_PAGE_OFFSET                    0x00000188U
#define PCP_RB_CPUX_TRC_TRCIDR11_PAGE_OFFSET                    0x0000018cU
#define PCP_RB_CPUX_TRC_TRCIDR12_PAGE_OFFSET                    0x00000190U
#define PCP_RB_CPUX_TRC_TRCIDR13_PAGE_OFFSET                    0x00000194U
#define PCP_RB_CPUX_TRC_TRCIDR2_PAGE_OFFSET                     0x000001e8U
#define PCP_RB_CPUX_TRC_TRCIDR3_PAGE_OFFSET                     0x000001ecU
#define PCP_RB_CPUX_TRC_TRCIDR4_PAGE_OFFSET                     0x000001f0U
#define PCP_RB_CPUX_TRC_TRCIDR5_PAGE_OFFSET                     0x000001f4U
#define PCP_RB_CPUX_TRC_TRCIDR6_PAGE_OFFSET                     0x000001f8U
#define PCP_RB_CPUX_TRC_TRCIDR7_PAGE_OFFSET                     0x000001fcU
#define PCP_RB_CPUX_TRC_TRCIDR8_PAGE_OFFSET                     0x00000180U
#define PCP_RB_CPUX_TRC_TRCIDR9_PAGE_OFFSET                     0x00000184U
#define PCP_RB_CPUX_TRC_TRCIMPSPEC1_PAGE_OFFSET                 0x000001c4U
#define PCP_RB_CPUX_TRC_TRCIMPSPEC2_PAGE_OFFSET                 0x000001c8U
#define PCP_RB_CPUX_TRC_TRCIMPSPEC3_PAGE_OFFSET                 0x000001ccU
#define PCP_RB_CPUX_TRC_TRCIMPSPEC4_PAGE_OFFSET                 0x000001d0U
#define PCP_RB_CPUX_TRC_TRCIMPSPEC5_PAGE_OFFSET                 0x000001d4U
#define PCP_RB_CPUX_TRC_TRCIMPSPEC6_PAGE_OFFSET                 0x000001d8U
#define PCP_RB_CPUX_TRC_TRCIMPSPEC7_PAGE_OFFSET                 0x000001dcU
#define PCP_RB_CPUX_TRC_TRCIMSPEC0_PAGE_OFFSET                  0x000001c0U
#define PCP_RB_CSW_TRACE_TRCINT_PAGE_OFFSET                     0x0000000cU
#define PCP_RB_CPUX_TRC_TRCITCTRL_PAGE_OFFSET                   0x00000f00U
#define PCP_RB_CPUX_TRC_TRCLAR_PAGE_OFFSET                      0x00000fb0U
#define PCP_RB_CPUX_TRC_TRCLSR_PAGE_OFFSET                      0x00000fb4U
#define PCP_RB_CPUX_TRC_TRCOSLAR_PAGE_OFFSET                    0x00000300U
#define PCP_RB_CPUX_TRC_TRCOSLSR_PAGE_OFFSET                    0x00000304U
#define PCP_RB_CPUX_TRC_TRCPDCR_PAGE_OFFSET                     0x00000310U
#define PCP_RB_CPUX_TRC_TRCPDSR_PAGE_OFFSET                     0x00000314U
#define PCP_RB_CPUX_TRC_TRCPIDR0_PAGE_OFFSET                    0x00000fe0U
#define PCP_RB_CPUX_TRC_TRCPIDR1_PAGE_OFFSET                    0x00000fe4U
#define PCP_RB_CPUX_TRC_TRCPIDR2_PAGE_OFFSET                    0x00000fe8U
#define PCP_RB_CPUX_TRC_TRCPIDR3_PAGE_OFFSET                    0x00000fecU
#define PCP_RB_CPUX_TRC_TRCPIDR4_PAGE_OFFSET                    0x00000fd0U
#define PCP_RB_CPUX_TRC_TRCPIDR5_PAGE_OFFSET                    0x00000fd4U
#define PCP_RB_CPUX_TRC_TRCPIDR6_PAGE_OFFSET                    0x00000fd8U
#define PCP_RB_CPUX_TRC_TRCPIDR7_PAGE_OFFSET                    0x00000fdcU
#define PCP_RB_CPUX_TRC_TRCPRGCTLR_PAGE_OFFSET                  0x00000004U
#define PCP_RB_CPUX_TRC_TRCPROCSELR_PAGE_OFFSET                 0x00000008U
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_PAGE_OFFSET                 0x00000228U
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_PAGE_OFFSET                 0x0000022cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_PAGE_OFFSET                 0x00000230U
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_PAGE_OFFSET                 0x00000234U
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_PAGE_OFFSET                 0x00000238U
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_PAGE_OFFSET                 0x0000023cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR16_PAGE_OFFSET                 0x00000240U
#define PCP_RB_CPUX_TRC_TRCRSCTLR17_PAGE_OFFSET                 0x00000244U
#define PCP_RB_CPUX_TRC_TRCRSCTLR18_PAGE_OFFSET                 0x00000248U
#define PCP_RB_CPUX_TRC_TRCRSCTLR19_PAGE_OFFSET                 0x0000024cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_PAGE_OFFSET                  0x00000208U
#define PCP_RB_CPUX_TRC_TRCRSCTLR20_PAGE_OFFSET                 0x00000250U
#define PCP_RB_CPUX_TRC_TRCRSCTLR21_PAGE_OFFSET                 0x00000254U
#define PCP_RB_CPUX_TRC_TRCRSCTLR22_PAGE_OFFSET                 0x00000258U
#define PCP_RB_CPUX_TRC_TRCRSCTLR23_PAGE_OFFSET                 0x0000025cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR24_PAGE_OFFSET                 0x00000260U
#define PCP_RB_CPUX_TRC_TRCRSCTLR25_PAGE_OFFSET                 0x00000264U
#define PCP_RB_CPUX_TRC_TRCRSCTLR26_PAGE_OFFSET                 0x00000268U
#define PCP_RB_CPUX_TRC_TRCRSCTLR27_PAGE_OFFSET                 0x0000026cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR28_PAGE_OFFSET                 0x00000270U
#define PCP_RB_CPUX_TRC_TRCRSCTLR29_PAGE_OFFSET                 0x00000274U
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_PAGE_OFFSET                  0x0000020cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR30_PAGE_OFFSET                 0x00000278U
#define PCP_RB_CPUX_TRC_TRCRSCTLR31_PAGE_OFFSET                 0x0000027cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_PAGE_OFFSET                  0x00000210U
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_PAGE_OFFSET                  0x00000214U
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_PAGE_OFFSET                  0x00000218U
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_PAGE_OFFSET                  0x0000021cU
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_PAGE_OFFSET                  0x00000220U
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_PAGE_OFFSET                  0x00000224U
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_PAGE_OFFSET                  0x00000100U
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_PAGE_OFFSET                  0x00000104U
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_PAGE_OFFSET                  0x00000108U
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_PAGE_OFFSET                0x00000118U
#define PCP_RB_CPUX_TRC_TRCSEQSTR_PAGE_OFFSET                   0x0000011cU
#define PCP_RB_CSW_TRACE_TRCSNPWP_PAGE_OFFSET                   0x00000010U
#define PCP_RB_CPUX_TRC_TRCSSCCR0_PAGE_OFFSET                   0x00000280U
#define PCP_RB_CPUX_TRC_TRCSSCCR1_PAGE_OFFSET                   0x00000284U
#define PCP_RB_CPUX_TRC_TRCSSCCR2_PAGE_OFFSET                   0x00000288U
#define PCP_RB_CPUX_TRC_TRCSSCCR3_PAGE_OFFSET                   0x0000028cU
#define PCP_RB_CPUX_TRC_TRCSSCCR4_PAGE_OFFSET                   0x00000290U
#define PCP_RB_CPUX_TRC_TRCSSCCR5_PAGE_OFFSET                   0x00000294U
#define PCP_RB_CPUX_TRC_TRCSSCCR6_PAGE_OFFSET                   0x00000298U
#define PCP_RB_CPUX_TRC_TRCSSCCR7_PAGE_OFFSET                   0x0000029cU
#define PCP_RB_CPUX_TRC_TRCSSCSR0_PAGE_OFFSET                   0x000002a0U
#define PCP_RB_CPUX_TRC_TRCSSCSR1_PAGE_OFFSET                   0x000002a4U
#define PCP_RB_CPUX_TRC_TRCSSCSR2_PAGE_OFFSET                   0x000002a8U
#define PCP_RB_CPUX_TRC_TRCSSCSR3_PAGE_OFFSET                   0x000002acU
#define PCP_RB_CPUX_TRC_TRCSSCSR4_PAGE_OFFSET                   0x000002b0U
#define PCP_RB_CPUX_TRC_TRCSSCSR5_PAGE_OFFSET                   0x000002b4U
#define PCP_RB_CPUX_TRC_TRCSSCSR6_PAGE_OFFSET                   0x000002b8U
#define PCP_RB_CPUX_TRC_TRCSSCSR7_PAGE_OFFSET                   0x000002bcU
#define PCP_RB_CSW_TRACE_TRCST_PAGE_OFFSET                      0x00000004U
#define PCP_RB_CPUX_TRC_TRCSTALLCTLR_PAGE_OFFSET                0x0000002cU
#define PCP_RB_CPUX_TRC_TRCSTATR_PAGE_OFFSET                    0x0000000cU
#define PCP_RB_CPUX_TRC_TRCSYNCPR_PAGE_OFFSET                   0x00000034U
#define PCP_RB_CPUX_TRC_TRCTRACEIDR_PAGE_OFFSET                 0x00000040U
#define PCP_RB_CPUX_TRC_TRCTSCTLR_PAGE_OFFSET                   0x00000030U
#define PCP_RB_CPUX_TRC_TRCVDARCCTLR_PAGE_OFFSET                0x000000a8U
#define PCP_RB_CPUX_TRC_TRCVDCTLR_PAGE_OFFSET                   0x000000a0U
#define PCP_RB_CPUX_TRC_TRCVDSACCTLR_PAGE_OFFSET                0x000000a4U
#define PCP_RB_CPUX_TRC_TRCVICTLR_PAGE_OFFSET                   0x00000080U
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_PAGE_OFFSET                 0x00000084U
#define PCP_RB_CPUX_TRC_TRCVIPCSSCTLR_PAGE_OFFSET               0x0000008cU
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_PAGE_OFFSET                 0x00000088U
#define PCP_RB_CPUX_TRC_TRCVMIDCCTLR0_PAGE_OFFSET               0x00000688U
#define PCP_RB_CPUX_TRC_TRCVMIDCCTLR1_PAGE_OFFSET               0x0000068cU
#define PCP_RB_CPUX_TRC_TRCVMIDCVR0_PAGE_OFFSET                 0x00000640U
#define PCP_RB_CPUX_TRC_TRCVMIDCVR1_PAGE_OFFSET                 0x00000648U
#define PCP_RB_CPUX_TRC_TRCVMIDCVR2_PAGE_OFFSET                 0x00000650U
#define PCP_RB_CPUX_TRC_TRCVMIDCVR3_PAGE_OFFSET                 0x00000658U
#define PCP_RB_CPUX_TRC_TRCVMIDCVR4_PAGE_OFFSET                 0x00000660U
#define PCP_RB_CPUX_TRC_TRCVMIDCVR5_PAGE_OFFSET                 0x00000668U
#define PCP_RB_CPUX_TRC_TRCVMIDCVR6_PAGE_OFFSET                 0x00000670U
#define PCP_RB_CPUX_TRC_TRCVMIDCVR7_PAGE_OFFSET                 0x00000678U
#define PCP_RB_CSW_TRACE_TRCWM_PAGE_OFFSET                      0x00000008U
#define PCP_RB_CSW_TRACE_VREG0_PAGE_OFFSET                      0x00000030U
#define PCP_RB_CSW_TRACE_VREG1_PAGE_OFFSET                      0x00000034U
#define PCP_RB_CSW_TRACE_VREG10_PAGE_OFFSET                     0x00000058U
#define PCP_RB_CSW_TRACE_VREG11_PAGE_OFFSET                     0x0000005cU
#define PCP_RB_CSW_TRACE_VREG12_PAGE_OFFSET                     0x00000060U
#define PCP_RB_CSW_TRACE_VREG13_PAGE_OFFSET                     0x00000064U
#define PCP_RB_CSW_TRACE_VREG2_PAGE_OFFSET                      0x00000038U
#define PCP_RB_CSW_TRACE_VREG4_PAGE_OFFSET                      0x00000040U
#define PCP_RB_CSW_TRACE_VREG6_PAGE_OFFSET                      0x00000048U
#define PCP_RB_CSW_TRACE_VREG7_PAGE_OFFSET                      0x0000004cU
#define PCP_RB_CSW_TRACE_VREG8_PAGE_OFFSET                      0x00000050U
#define PCP_RB_CSW_TRACE_VREG9_PAGE_OFFSET                      0x00000054U
#define PCP_RB_CSW_TRACE_WP0ADDRH_PAGE_OFFSET                   0x00000074U
#define PCP_RB_CSW_TRACE_WP0ADDRL_PAGE_OFFSET                   0x00000070U
#define PCP_RB_CSW_TRACE_WP0AGENT_PAGE_OFFSET                   0x000000b0U
#define PCP_RB_CSW_TRACE_WP0COUNT_PAGE_OFFSET                   0x000000d0U
#define PCP_RB_CSW_TRACE_WP0EVENT_PAGE_OFFSET                   0x000000d4U
#define PCP_RB_CSW_TRACE_WP0MASKH_PAGE_OFFSET                   0x00000094U
#define PCP_RB_CSW_TRACE_WP0MASKL_PAGE_OFFSET                   0x00000090U
#define PCP_RB_CSW_TRACE_WP0TYPE_PAGE_OFFSET                    0x000000b4U
#define PCP_RB_CSW_TRACE_WP1ADDRH_PAGE_OFFSET                   0x0000007cU
#define PCP_RB_CSW_TRACE_WP1ADDRL_PAGE_OFFSET                   0x00000078U
#define PCP_RB_CSW_TRACE_WP1AGENT_PAGE_OFFSET                   0x000000b8U
#define PCP_RB_CSW_TRACE_WP1COUNT_PAGE_OFFSET                   0x000000d8U
#define PCP_RB_CSW_TRACE_WP1EVENT_PAGE_OFFSET                   0x000000dcU
#define PCP_RB_CSW_TRACE_WP1MASKH_PAGE_OFFSET                   0x0000009cU
#define PCP_RB_CSW_TRACE_WP1MASKL_PAGE_OFFSET                   0x00000098U
#define PCP_RB_CSW_TRACE_WP1TYPE_PAGE_OFFSET                    0x000000bcU
#define PCP_RB_CSW_TRACE_WP2ADDRH_PAGE_OFFSET                   0x00000084U
#define PCP_RB_CSW_TRACE_WP2ADDRL_PAGE_OFFSET                   0x00000080U
#define PCP_RB_CSW_TRACE_WP2AGENT_PAGE_OFFSET                   0x000000c0U
#define PCP_RB_CSW_TRACE_WP2COUNT_PAGE_OFFSET                   0x000000e0U
#define PCP_RB_CSW_TRACE_WP2EVENT_PAGE_OFFSET                   0x000000e4U
#define PCP_RB_CSW_TRACE_WP2MASKH_PAGE_OFFSET                   0x000000a4U
#define PCP_RB_CSW_TRACE_WP2MASKL_PAGE_OFFSET                   0x000000a0U
#define PCP_RB_CSW_TRACE_WP2TYPE_PAGE_OFFSET                    0x000000c4U
#define PCP_RB_CSW_TRACE_WP3ADDRH_PAGE_OFFSET                   0x0000008cU
#define PCP_RB_CSW_TRACE_WP3ADDRL_PAGE_OFFSET                   0x00000088U
#define PCP_RB_CSW_TRACE_WP3AGENT_PAGE_OFFSET                   0x000000c8U
#define PCP_RB_CSW_TRACE_WP3COUNT_PAGE_OFFSET                   0x000000e8U
#define PCP_RB_CSW_TRACE_WP3EVENT_PAGE_OFFSET                   0x000000ecU
#define PCP_RB_CSW_TRACE_WP3MASKH_PAGE_OFFSET                   0x000000acU
#define PCP_RB_CSW_TRACE_WP3MASKL_PAGE_OFFSET                   0x000000a8U
#define PCP_RB_CSW_TRACE_WP3TYPE_PAGE_OFFSET                    0x000000ccU
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_PAGE_OFFSET           0x00000054U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_PAGE_OFFSET           0x00000050U
#define PCP_RB_IOB_ERR_XGICTRANSERRREQINFO_PAGE_OFFSET          0x00000058U

#define PCP_RB_CPUX_CPU_CCCR_IBPFRC_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_IBPFRC_SHIFT                  25
#define PCP_RB_CPUX_CPU_CCCR_IBPFRC_MASK                   0x02000000U
#define PCP_RB_CPUX_CPU_CCCR_IBPFRC_RD(src)                (((src) & 0x02000000U)>>25)
#define PCP_RB_CPUX_CPU_CCCR_IBPFRC_SET(dst,src)           (((dst) & ~0x02000000U) | (((u32)(src)<<25) & 0x02000000U))
#define PCP_RB_CPUX_CPU_CCCR_LGHISTPATHFRC_WIDTH           1
#define PCP_RB_CPUX_CPU_CCCR_LGHISTPATHFRC_SHIFT           24
#define PCP_RB_CPUX_CPU_CCCR_LGHISTPATHFRC_MASK            0x01000000U
#define PCP_RB_CPUX_CPU_CCCR_LGHISTPATHFRC_RD(src)         (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_CPU_CCCR_LGHISTPATHFRC_SET(dst,src)    (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_CPU_CCCR_FULLTBINVFRC_WIDTH            1
#define PCP_RB_CPUX_CPU_CCCR_FULLTBINVFRC_SHIFT            23
#define PCP_RB_CPUX_CPU_CCCR_FULLTBINVFRC_MASK             0x00800000U
#define PCP_RB_CPUX_CPU_CCCR_FULLTBINVFRC_RD(src)          (((src) & 0x00800000U)>>23)
#define PCP_RB_CPUX_CPU_CCCR_FULLTBINVFRC_SET(dst,src)     (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CPUX_CPU_CCCR_PWCDIS_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_PWCDIS_SHIFT                  22
#define PCP_RB_CPUX_CPU_CCCR_PWCDIS_MASK                   0x00400000U
#define PCP_RB_CPUX_CPU_CCCR_PWCDIS_RD(src)                (((src) & 0x00400000U)>>22)
#define PCP_RB_CPUX_CPU_CCCR_PWCDIS_SET(dst,src)           (((dst) & ~0x00400000U) | (((u32)(src)<<22) & 0x00400000U))
#define PCP_RB_CPUX_CPU_CCCR_UTBS2DIS_WIDTH                1
#define PCP_RB_CPUX_CPU_CCCR_UTBS2DIS_SHIFT                21
#define PCP_RB_CPUX_CPU_CCCR_UTBS2DIS_MASK                 0x00200000U
#define PCP_RB_CPUX_CPU_CCCR_UTBS2DIS_RD(src)              (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_CPU_CCCR_UTBS2DIS_SET(dst,src)         (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_CPU_CCCR_UTBDIS_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_UTBDIS_SHIFT                  20
#define PCP_RB_CPUX_CPU_CCCR_UTBDIS_MASK                   0x00100000U
#define PCP_RB_CPUX_CPU_CCCR_UTBDIS_RD(src)                (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_CPU_CCCR_UTBDIS_SET(dst,src)           (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_CPU_CCCR_SPRTODIS_WIDTH                1
#define PCP_RB_CPUX_CPU_CCCR_SPRTODIS_SHIFT                19
#define PCP_RB_CPUX_CPU_CCCR_SPRTODIS_MASK                 0x00080000U
#define PCP_RB_CPUX_CPU_CCCR_SPRTODIS_RD(src)              (((src) & 0x00080000U)>>19)
#define PCP_RB_CPUX_CPU_CCCR_SPRTODIS_SET(dst,src)         (((dst) & ~0x00080000U) | (((u32)(src)<<19) & 0x00080000U))
#define PCP_RB_CPUX_CPU_CCCR_SPRCSFRC_WIDTH                1
#define PCP_RB_CPUX_CPU_CCCR_SPRCSFRC_SHIFT                18
#define PCP_RB_CPUX_CPU_CCCR_SPRCSFRC_MASK                 0x00040000U
#define PCP_RB_CPUX_CPU_CCCR_SPRCSFRC_RD(src)              (((src) & 0x00040000U)>>18)
#define PCP_RB_CPUX_CPU_CCCR_SPRCSFRC_SET(dst,src)         (((dst) & ~0x00040000U) | (((u32)(src)<<18) & 0x00040000U))
#define PCP_RB_CPUX_CPU_CCCR_DECMODE_WIDTH                 2
#define PCP_RB_CPUX_CPU_CCCR_DECMODE_SHIFT                 16
#define PCP_RB_CPUX_CPU_CCCR_DECMODE_MASK                  0x00030000U
#define PCP_RB_CPUX_CPU_CCCR_DECMODE_RD(src)               (((src) & 0x00030000U)>>16)
#define PCP_RB_CPUX_CPU_CCCR_DECMODE_SET(dst,src)          (((dst) & ~0x00030000U) | (((u32)(src)<<16) & 0x00030000U))
#define PCP_RB_CPUX_CPU_CCCR_FXBYPDIS_WIDTH                2
#define PCP_RB_CPUX_CPU_CCCR_FXBYPDIS_SHIFT                14
#define PCP_RB_CPUX_CPU_CCCR_FXBYPDIS_MASK                 0x0000C000U
#define PCP_RB_CPUX_CPU_CCCR_FXBYPDIS_RD(src)              (((src) & 0x0000C000U)>>14)
#define PCP_RB_CPUX_CPU_CCCR_FXBYPDIS_SET(dst,src)         (((dst) & ~0x0000C000U) | (((u32)(src)<<14) & 0x0000C000U))
#define PCP_RB_CPUX_CPU_CCCR_FXPDIS_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_FXPDIS_SHIFT                  13
#define PCP_RB_CPUX_CPU_CCCR_FXPDIS_MASK                   0x00002000U
#define PCP_RB_CPUX_CPU_CCCR_FXPDIS_RD(src)                (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_CPU_CCCR_FXPDIS_SET(dst,src)           (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_CPU_CCCR_WRCMBDIS_WIDTH                1
#define PCP_RB_CPUX_CPU_CCCR_WRCMBDIS_SHIFT                12
#define PCP_RB_CPUX_CPU_CCCR_WRCMBDIS_MASK                 0x00001000U
#define PCP_RB_CPUX_CPU_CCCR_WRCMBDIS_RD(src)              (((src) & 0x00001000U)>>12)
#define PCP_RB_CPUX_CPU_CCCR_WRCMBDIS_SET(dst,src)         (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_CPUX_CPU_CCCR_TLBSYNCDIS_WIDTH              1
#define PCP_RB_CPUX_CPU_CCCR_TLBSYNCDIS_SHIFT              11
#define PCP_RB_CPUX_CPU_CCCR_TLBSYNCDIS_MASK               0x00000800U
#define PCP_RB_CPUX_CPU_CCCR_TLBSYNCDIS_RD(src)            (((src) & 0x00000800U)>>11)
#define PCP_RB_CPUX_CPU_CCCR_TLBSYNCDIS_SET(dst,src)       (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CPUX_CPU_CCCR_OOLIDIS_WIDTH                 1
#define PCP_RB_CPUX_CPU_CCCR_OOLIDIS_SHIFT                 10
#define PCP_RB_CPUX_CPU_CCCR_OOLIDIS_MASK                  0x00000400U
#define PCP_RB_CPUX_CPU_CCCR_OOLIDIS_RD(src)               (((src) & 0x00000400U)>>10)
#define PCP_RB_CPUX_CPU_CCCR_OOLIDIS_SET(dst,src)          (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CPUX_CPU_CCCR_DL1HDPDIS_WIDTH               1
#define PCP_RB_CPUX_CPU_CCCR_DL1HDPDIS_SHIFT               9
#define PCP_RB_CPUX_CPU_CCCR_DL1HDPDIS_MASK                0x00000200U
#define PCP_RB_CPUX_CPU_CCCR_DL1HDPDIS_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CPUX_CPU_CCCR_DL1HDPDIS_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CPUX_CPU_CCCR_DL1FILLINH_WIDTH              1
#define PCP_RB_CPUX_CPU_CCCR_DL1FILLINH_SHIFT              8
#define PCP_RB_CPUX_CPU_CCCR_DL1FILLINH_MASK               0x00000100U
#define PCP_RB_CPUX_CPU_CCCR_DL1FILLINH_RD(src)            (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_CPU_CCCR_DL1FILLINH_SET(dst,src)       (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_CPU_CCCR_IXADIS_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_IXADIS_SHIFT                  7
#define PCP_RB_CPUX_CPU_CCCR_IXADIS_MASK                   0x00000080U
#define PCP_RB_CPUX_CPU_CCCR_IXADIS_RD(src)                (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_CPU_CCCR_IXADIS_SET(dst,src)           (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_CPU_CCCR_IXBAISSOPTDIS_WIDTH           1
#define PCP_RB_CPUX_CPU_CCCR_IXBAISSOPTDIS_SHIFT           6
#define PCP_RB_CPUX_CPU_CCCR_IXBAISSOPTDIS_MASK            0x00000040U
#define PCP_RB_CPUX_CPU_CCCR_IXBAISSOPTDIS_RD(src)         (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_CPU_CCCR_IXBAISSOPTDIS_SET(dst,src)    (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_CPU_CCCR_PFAB_WIDTH                    1
#define PCP_RB_CPUX_CPU_CCCR_PFAB_SHIFT                    5
#define PCP_RB_CPUX_CPU_CCCR_PFAB_MASK                     0x00000020U
#define PCP_RB_CPUX_CPU_CCCR_PFAB_RD(src)                  (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_CPU_CCCR_PFAB_SET(dst,src)             (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_CPU_CCCR_OBCFRC_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_OBCFRC_SHIFT                  4
#define PCP_RB_CPUX_CPU_CCCR_OBCFRC_MASK                   0x00000010U
#define PCP_RB_CPUX_CPU_CCCR_OBCFRC_RD(src)                (((src) & 0x00000010U)>>4)
#define PCP_RB_CPUX_CPU_CCCR_OBCFRC_SET(dst,src)           (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CPUX_CPU_CCCR_CBPDIS_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_CBPDIS_SHIFT                  3
#define PCP_RB_CPUX_CPU_CCCR_CBPDIS_MASK                   0x00000008U
#define PCP_RB_CPUX_CPU_CCCR_CBPDIS_RD(src)                (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_CPU_CCCR_CBPDIS_SET(dst,src)           (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_CPU_CCCR_BMPDIS_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_BMPDIS_SHIFT                  2
#define PCP_RB_CPUX_CPU_CCCR_BMPDIS_MASK                   0x00000004U
#define PCP_RB_CPUX_CPU_CCCR_BMPDIS_RD(src)                (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_CPU_CCCR_BMPDIS_SET(dst,src)           (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_CPU_CCCR_BTBDIS_WIDTH                  1
#define PCP_RB_CPUX_CPU_CCCR_BTBDIS_SHIFT                  1
#define PCP_RB_CPUX_CPU_CCCR_BTBDIS_MASK                   0x00000002U
#define PCP_RB_CPUX_CPU_CCCR_BTBDIS_RD(src)                (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_CPU_CCCR_BTBDIS_SET(dst,src)           (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_CPU_CCCR_IL1FILLINH_WIDTH              1
#define PCP_RB_CPUX_CPU_CCCR_IL1FILLINH_SHIFT              0
#define PCP_RB_CPUX_CPU_CCCR_IL1FILLINH_MASK               0x00000001U
#define PCP_RB_CPUX_CPU_CCCR_IL1FILLINH_RD(src)            (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_CPU_CCCR_IL1FILLINH_SET(dst,src)       (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_CPU_CPE0_CNT_WIDTH                     28
#define PCP_RB_CPUX_CPU_CPE0_CNT_SHIFT                     0
#define PCP_RB_CPUX_CPU_CPE0_CNT_MASK                      0x0FFFFFFFU
#define PCP_RB_CPUX_CPU_CPE0_CNT_RD(src)                   (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_CPU_CPE0_CNT_SET(dst,src)              (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_CPU_CPE1_CNT_WIDTH                     28
#define PCP_RB_CPUX_CPU_CPE1_CNT_SHIFT                     0
#define PCP_RB_CPUX_CPU_CPE1_CNT_MASK                      0x0FFFFFFFU
#define PCP_RB_CPUX_CPU_CPE1_CNT_RD(src)                   (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_CPU_CPE1_CNT_SET(dst,src)              (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_CPU_CPE2_CNT_WIDTH                     28
#define PCP_RB_CPUX_CPU_CPE2_CNT_SHIFT                     0
#define PCP_RB_CPUX_CPU_CPE2_CNT_MASK                      0x0FFFFFFFU
#define PCP_RB_CPUX_CPU_CPE2_CNT_RD(src)                   (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_CPU_CPE2_CNT_SET(dst,src)              (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_CPU_CPE3_CNT_WIDTH                     28
#define PCP_RB_CPUX_CPU_CPE3_CNT_SHIFT                     0
#define PCP_RB_CPUX_CPU_CPE3_CNT_MASK                      0x0FFFFFFFU
#define PCP_RB_CPUX_CPU_CPE3_CNT_RD(src)                   (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_CPU_CPE3_CNT_SET(dst,src)              (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_CPU_CRCR_CWRR_WIDTH                    1
#define PCP_RB_CPUX_CPU_CRCR_CWRR_SHIFT                    1
#define PCP_RB_CPUX_CPU_CRCR_CWRR_MASK                     0x00000002U
#define PCP_RB_CPUX_CPU_CRCR_CWRR_RD(src)                  (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_CPU_CRCR_CWRR_SET(dst,src)             (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_CSWASR_IOBSNPACTV_WIDTH                 1
#define PCP_RB_CSW_CSWASR_IOBSNPACTV_SHIFT                 23
#define PCP_RB_CSW_CSWASR_IOBSNPACTV_MASK                  0x00800000U
#define PCP_RB_CSW_CSWASR_IOBSNPACTV_RD(src)               (((src) & 0x00800000U)>>23)
#define PCP_RB_CSW_CSWASR_IOBSNPACTV_SET(dst,src)          (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CSW_CSWASR_L3CALLOCACTV_WIDTH               1
#define PCP_RB_CSW_CSWASR_L3CALLOCACTV_SHIFT               18
#define PCP_RB_CSW_CSWASR_L3CALLOCACTV_MASK                0x00040000U
#define PCP_RB_CSW_CSWASR_L3CALLOCACTV_RD(src)             (((src) & 0x00040000U)>>18)
#define PCP_RB_CSW_CSWASR_L3CALLOCACTV_SET(dst,src)        (((dst) & ~0x00040000U) | (((u32)(src)<<18) & 0x00040000U))
#define PCP_RB_CSW_CSWASR_L3CSNPACTV_WIDTH                 1
#define PCP_RB_CSW_CSWASR_L3CSNPACTV_SHIFT                 16
#define PCP_RB_CSW_CSWASR_L3CSNPACTV_MASK                  0x00010000U
#define PCP_RB_CSW_CSWASR_L3CSNPACTV_RD(src)               (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_CSWASR_L3CSNPACTV_SET(dst,src)          (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_CSWASR_PMD3SNPACTV_WIDTH                1
#define PCP_RB_CSW_CSWASR_PMD3SNPACTV_SHIFT                3
#define PCP_RB_CSW_CSWASR_PMD3SNPACTV_MASK                 0x00000008U
#define PCP_RB_CSW_CSWASR_PMD3SNPACTV_RD(src)              (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_CSWASR_PMD3SNPACTV_SET(dst,src)         (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_CSWASR_PMD2SNPACTV_WIDTH                1
#define PCP_RB_CSW_CSWASR_PMD2SNPACTV_SHIFT                2
#define PCP_RB_CSW_CSWASR_PMD2SNPACTV_MASK                 0x00000004U
#define PCP_RB_CSW_CSWASR_PMD2SNPACTV_RD(src)              (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_CSWASR_PMD2SNPACTV_SET(dst,src)         (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_CSWASR_PMD1SNPACTV_WIDTH                1
#define PCP_RB_CSW_CSWASR_PMD1SNPACTV_SHIFT                1
#define PCP_RB_CSW_CSWASR_PMD1SNPACTV_MASK                 0x00000002U
#define PCP_RB_CSW_CSWASR_PMD1SNPACTV_RD(src)              (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_CSWASR_PMD1SNPACTV_SET(dst,src)         (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_CSWASR_PMD0SNPACTV_WIDTH                1
#define PCP_RB_CSW_CSWASR_PMD0SNPACTV_SHIFT                0
#define PCP_RB_CSW_CSWASR_PMD0SNPACTV_MASK                 0x00000001U
#define PCP_RB_CSW_CSWASR_PMD0SNPACTV_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_CSWASR_PMD0SNPACTV_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CSW_CSWCCR_DCCSIGN_WIDTH                1
#define PCP_RB_IOB_CSW_CSWCCR_DCCSIGN_SHIFT                31
#define PCP_RB_IOB_CSW_CSWCCR_DCCSIGN_MASK                 0x80000000U
#define PCP_RB_IOB_CSW_CSWCCR_DCCSIGN_RD(src)              (((src) & 0x80000000U)>>31)
#define PCP_RB_IOB_CSW_CSWCCR_DCCSIGN_SET(dst,src)         (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_IOB_CSW_CSWCCR_DCCVAL_WIDTH                 3
#define PCP_RB_IOB_CSW_CSWCCR_DCCVAL_SHIFT                 28
#define PCP_RB_IOB_CSW_CSWCCR_DCCVAL_MASK                  0x70000000U
#define PCP_RB_IOB_CSW_CSWCCR_DCCVAL_RD(src)               (((src) & 0x70000000U)>>28)
#define PCP_RB_IOB_CSW_CSWCCR_DCCVAL_SET(dst,src)          (((dst) & ~0x70000000U) | (((u32)(src)<<28) & 0x70000000U))
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACRST_WIDTH           1
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACRST_SHIFT           15
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACRST_MASK            0x00008000U
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACRST_RD(src)         (((src) & 0x00008000U)>>15)
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACRST_SET(dst,src)    (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACPD_WIDTH            1
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACPD_SHIFT            14
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACPD_MASK             0x00004000U
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACPD_RD(src)          (((src) & 0x00004000U)>>14)
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKMACPD_SET(dst,src)     (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKEN_WIDTH               1
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKEN_SHIFT               11
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKEN_MASK                0x00000800U
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKEN_RD(src)             (((src) & 0x00000800U)>>11)
#define PCP_RB_IOB_CSW_CSWCCR_CSWCLKEN_SET(dst,src)        (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERBYPEN_WIDTH           1
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERBYPEN_SHIFT           9
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERBYPEN_MASK            0x00000200U
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERBYPEN_RD(src)         (((src) & 0x00000200U)>>9)
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERBYPEN_SET(dst,src)    (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERCLKEN_WIDTH           1
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERCLKEN_SHIFT           8
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERCLKEN_MASK            0x00000100U
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERCLKEN_RD(src)         (((src) & 0x00000100U)>>8)
#define PCP_RB_IOB_CSW_CSWCCR_CLUSTERCLKEN_SET(dst,src)    (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_IOB_CSW_CSWCCR_CMLNSCORREN_WIDTH            1
#define PCP_RB_IOB_CSW_CSWCCR_CMLNSCORREN_SHIFT            5
#define PCP_RB_IOB_CSW_CSWCCR_CMLNSCORREN_MASK             0x00000020U
#define PCP_RB_IOB_CSW_CSWCCR_CMLNSCORREN_RD(src)          (((src) & 0x00000020U)>>5)
#define PCP_RB_IOB_CSW_CSWCCR_CMLNSCORREN_SET(dst,src)     (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_IOB_CSW_CSWCCR_SKEWTUNE_WIDTH               5
#define PCP_RB_IOB_CSW_CSWCCR_SKEWTUNE_SHIFT               0
#define PCP_RB_IOB_CSW_CSWCCR_SKEWTUNE_MASK                0x0000001FU
#define PCP_RB_IOB_CSW_CSWCCR_SKEWTUNE_RD(src)             (((src) & 0x0000001FU)>>0)
#define PCP_RB_IOB_CSW_CSWCCR_SKEWTUNE_SET(dst,src)        (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_CSWCR_MCB1SYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_CSWCR_MCB1SYNCCLKDIVEN_SHIFT            31
#define PCP_RB_CSW_CSWCR_MCB1SYNCCLKDIVEN_MASK             0x80000000U
#define PCP_RB_CSW_CSWCR_MCB1SYNCCLKDIVEN_RD(src)          (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_CSWCR_MCB1SYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_CSWCR_MCB0SYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_CSWCR_MCB0SYNCCLKDIVEN_SHIFT            30
#define PCP_RB_CSW_CSWCR_MCB0SYNCCLKDIVEN_MASK             0x40000000U
#define PCP_RB_CSW_CSWCR_MCB0SYNCCLKDIVEN_RD(src)          (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_CSWCR_MCB0SYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_CSWCR_IOBSYNCCLKDIVEN_WIDTH             1
#define PCP_RB_CSW_CSWCR_IOBSYNCCLKDIVEN_SHIFT             29
#define PCP_RB_CSW_CSWCR_IOBSYNCCLKDIVEN_MASK              0x20000000U
#define PCP_RB_CSW_CSWCR_IOBSYNCCLKDIVEN_RD(src)           (((src) & 0x20000000U)>>29)
#define PCP_RB_CSW_CSWCR_IOBSYNCCLKDIVEN_SET(dst,src)      (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CSW_CSWCR_L3CSYNCCLKDIVEN_WIDTH             1
#define PCP_RB_CSW_CSWCR_L3CSYNCCLKDIVEN_SHIFT             28
#define PCP_RB_CSW_CSWCR_L3CSYNCCLKDIVEN_MASK              0x10000000U
#define PCP_RB_CSW_CSWCR_L3CSYNCCLKDIVEN_RD(src)           (((src) & 0x10000000U)>>28)
#define PCP_RB_CSW_CSWCR_L3CSYNCCLKDIVEN_SET(dst,src)      (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CSW_CSWCR_PMD3SYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_CSWCR_PMD3SYNCCLKDIVEN_SHIFT            27
#define PCP_RB_CSW_CSWCR_PMD3SYNCCLKDIVEN_MASK             0x08000000U
#define PCP_RB_CSW_CSWCR_PMD3SYNCCLKDIVEN_RD(src)          (((src) & 0x08000000U)>>27)
#define PCP_RB_CSW_CSWCR_PMD3SYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CSW_CSWCR_PMD2SYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_CSWCR_PMD2SYNCCLKDIVEN_SHIFT            26
#define PCP_RB_CSW_CSWCR_PMD2SYNCCLKDIVEN_MASK             0x04000000U
#define PCP_RB_CSW_CSWCR_PMD2SYNCCLKDIVEN_RD(src)          (((src) & 0x04000000U)>>26)
#define PCP_RB_CSW_CSWCR_PMD2SYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_CSW_CSWCR_PMD1SYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_CSWCR_PMD1SYNCCLKDIVEN_SHIFT            25
#define PCP_RB_CSW_CSWCR_PMD1SYNCCLKDIVEN_MASK             0x02000000U
#define PCP_RB_CSW_CSWCR_PMD1SYNCCLKDIVEN_RD(src)          (((src) & 0x02000000U)>>25)
#define PCP_RB_CSW_CSWCR_PMD1SYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x02000000U) | (((u32)(src)<<25) & 0x02000000U))
#define PCP_RB_CSW_CSWCR_PMD0SYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_CSWCR_PMD0SYNCCLKDIVEN_SHIFT            24
#define PCP_RB_CSW_CSWCR_PMD0SYNCCLKDIVEN_MASK             0x01000000U
#define PCP_RB_CSW_CSWCR_PMD0SYNCCLKDIVEN_RD(src)          (((src) & 0x01000000U)>>24)
#define PCP_RB_CSW_CSWCR_PMD0SYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CSW_CSWCR_REQTHROT_WIDTH                    10
#define PCP_RB_CSW_CSWCR_REQTHROT_SHIFT                    12
#define PCP_RB_CSW_CSWCR_REQTHROT_MASK                     0x003FF000U
#define PCP_RB_CSW_CSWCR_REQTHROT_RD(src)                  (((src) & 0x003FF000U)>>12)
#define PCP_RB_CSW_CSWCR_REQTHROT_SET(dst,src)             (((dst) & ~0x003FF000U) | (((u32)(src)<<12) & 0x003FF000U))
#define PCP_RB_CSW_CSWCR_L3OCFRC_WIDTH                     1
#define PCP_RB_CSW_CSWCR_L3OCFRC_SHIFT                     8
#define PCP_RB_CSW_CSWCR_L3OCFRC_MASK                      0x00000100U
#define PCP_RB_CSW_CSWCR_L3OCFRC_RD(src)                   (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_CSWCR_L3OCFRC_SET(dst,src)              (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_CSWCR_PMDREQPRI_WIDTH                   1
#define PCP_RB_CSW_CSWCR_PMDREQPRI_SHIFT                   3
#define PCP_RB_CSW_CSWCR_PMDREQPRI_MASK                    0x00000008U
#define PCP_RB_CSW_CSWCR_PMDREQPRI_RD(src)                 (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_CSWCR_PMDREQPRI_SET(dst,src)            (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_CSWCR_FULLADDR_WIDTH                    1
#define PCP_RB_CSW_CSWCR_FULLADDR_SHIFT                    2
#define PCP_RB_CSW_CSWCR_FULLADDR_MASK                     0x00000004U
#define PCP_RB_CSW_CSWCR_FULLADDR_RD(src)                  (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_CSWCR_FULLADDR_SET(dst,src)             (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_CSWCR_MCBINTLV_WIDTH                    1
#define PCP_RB_CSW_CSWCR_MCBINTLV_SHIFT                    1
#define PCP_RB_CSW_CSWCR_MCBINTLV_MASK                     0x00000002U
#define PCP_RB_CSW_CSWCR_MCBINTLV_RD(src)                  (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_CSWCR_MCBINTLV_SET(dst,src)             (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_CSWCR_DUALMCB_WIDTH                     1
#define PCP_RB_CSW_CSWCR_DUALMCB_SHIFT                     0
#define PCP_RB_CSW_CSWCR_DUALMCB_MASK                      0x00000001U
#define PCP_RB_CSW_CSWCR_DUALMCB_RD(src)                   (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_CSWCR_DUALMCB_SET(dst,src)              (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CSW_CSWRCR_RESETCSW_WIDTH               1
#define PCP_RB_IOB_CSW_CSWRCR_RESETCSW_SHIFT               0
#define PCP_RB_IOB_CSW_CSWRCR_RESETCSW_MASK                0x00000001U
#define PCP_RB_IOB_CSW_CSWRCR_RESETCSW_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_CSW_CSWRCR_RESETCSW_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_CSWSER_IOBSNPEN_WIDTH                   1
#define PCP_RB_CSW_CSWSER_IOBSNPEN_SHIFT                   23
#define PCP_RB_CSW_CSWSER_IOBSNPEN_MASK                    0x00800000U
#define PCP_RB_CSW_CSWSER_IOBSNPEN_RD(src)                 (((src) & 0x00800000U)>>23)
#define PCP_RB_CSW_CSWSER_IOBSNPEN_SET(dst,src)            (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CSW_CSWSER_L3CALLOCEN_WIDTH                 1
#define PCP_RB_CSW_CSWSER_L3CALLOCEN_SHIFT                 18
#define PCP_RB_CSW_CSWSER_L3CALLOCEN_MASK                  0x00040000U
#define PCP_RB_CSW_CSWSER_L3CALLOCEN_RD(src)               (((src) & 0x00040000U)>>18)
#define PCP_RB_CSW_CSWSER_L3CALLOCEN_SET(dst,src)          (((dst) & ~0x00040000U) | (((u32)(src)<<18) & 0x00040000U))
#define PCP_RB_CSW_CSWSER_L3CSNPEN_WIDTH                   1
#define PCP_RB_CSW_CSWSER_L3CSNPEN_SHIFT                   16
#define PCP_RB_CSW_CSWSER_L3CSNPEN_MASK                    0x00010000U
#define PCP_RB_CSW_CSWSER_L3CSNPEN_RD(src)                 (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_CSWSER_L3CSNPEN_SET(dst,src)            (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_CSWSER_PMD3SNPEN_WIDTH                  1
#define PCP_RB_CSW_CSWSER_PMD3SNPEN_SHIFT                  3
#define PCP_RB_CSW_CSWSER_PMD3SNPEN_MASK                   0x00000008U
#define PCP_RB_CSW_CSWSER_PMD3SNPEN_RD(src)                (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_CSWSER_PMD3SNPEN_SET(dst,src)           (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_CSWSER_PMD2SNPEN_WIDTH                  1
#define PCP_RB_CSW_CSWSER_PMD2SNPEN_SHIFT                  2
#define PCP_RB_CSW_CSWSER_PMD2SNPEN_MASK                   0x00000004U
#define PCP_RB_CSW_CSWSER_PMD2SNPEN_RD(src)                (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_CSWSER_PMD2SNPEN_SET(dst,src)           (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_CSWSER_PMD1SNPEN_WIDTH                  1
#define PCP_RB_CSW_CSWSER_PMD1SNPEN_SHIFT                  1
#define PCP_RB_CSW_CSWSER_PMD1SNPEN_MASK                   0x00000002U
#define PCP_RB_CSW_CSWSER_PMD1SNPEN_RD(src)                (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_CSWSER_PMD1SNPEN_SET(dst,src)           (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_CSWSER_PMD0SNPEN_WIDTH                  1
#define PCP_RB_CSW_CSWSER_PMD0SNPEN_SHIFT                  0
#define PCP_RB_CSW_CSWSER_PMD0SNPEN_MASK                   0x00000001U
#define PCP_RB_CSW_CSWSER_PMD0SNPEN_RD(src)                (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_CSWSER_PMD0SNPEN_SET(dst,src)           (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_CTI_CTIAPPCLEAR_C_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIAPPCLEAR_C_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIAPPCLEAR_C_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIAPPCLEAR_C_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIAPPCLEAR_C_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIAPPSET_S_WIDTH                  4
#define PCP_RB_CPUX_CTI_CTIAPPSET_S_SHIFT                  0
#define PCP_RB_CPUX_CTI_CTIAPPSET_S_MASK                   0x0000000FU
#define PCP_RB_CPUX_CTI_CTIAPPSET_S_RD(src)                (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIAPPSET_S_SET(dst,src)           (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SNID_WIDTH           2
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SNID_SHIFT           6
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SNID_MASK            0x000000C0U
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SNID_RD(src)         (((src) & 0x000000C0U)>>6)
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SNID_SET(dst,src)    (((dst) & ~0x000000C0U) | (((u32)(src)<<6) & 0x000000C0U))
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SID_WIDTH            2
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SID_SHIFT            4
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SID_MASK             0x00000030U
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SID_RD(src)          (((src) & 0x00000030U)>>4)
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_SID_SET(dst,src)     (((dst) & ~0x00000030U) | (((u32)(src)<<4) & 0x00000030U))
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSNID_WIDTH          2
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSNID_SHIFT          2
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSNID_MASK           0x0000000CU
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSNID_RD(src)        (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSNID_SET(dst,src)   (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSID_WIDTH           2
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSID_SHIFT           0
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSID_MASK            0x00000003U
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSID_RD(src)         (((src) & 0x00000003U)>>0)
#define PCP_RB_CPUX_CTI_CTIAUTHSTATUS_NSID_SET(dst,src)    (((dst) & ~0x00000003U) | (((u32)(src)<<0) & 0x00000003U))
#define PCP_RB_CPUX_CTI_CTICHANPULSE_P_WIDTH               4
#define PCP_RB_CPUX_CTI_CTICHANPULSE_P_SHIFT               0
#define PCP_RB_CPUX_CTI_CTICHANPULSE_P_MASK                0x0000000FU
#define PCP_RB_CPUX_CTI_CTICHANPULSE_P_RD(src)             (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTICHANPULSE_P_SET(dst,src)        (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTICIDR0_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTICIDR0_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTICIDR0_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTICIDR0_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTICIDR0_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTICIDR1_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTICIDR1_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTICIDR1_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTICIDR1_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTICIDR1_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTICIDR2_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTICIDR2_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTICIDR2_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTICIDR2_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTICIDR2_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTICIDR3_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTICIDR3_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTICIDR3_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTICIDR3_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTICIDR3_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTICONTROL_GLBEN_WIDTH             1
#define PCP_RB_CPUX_CTI_CTICONTROL_GLBEN_SHIFT             0
#define PCP_RB_CPUX_CTI_CTICONTROL_GLBEN_MASK              0x00000001U
#define PCP_RB_CPUX_CTI_CTICONTROL_GLBEN_RD(src)           (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_CTI_CTICONTROL_GLBEN_SET(dst,src)      (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_RES1_WIDTH              1
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_RES1_SHIFT              31
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_RES1_MASK               0x80000000U
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_RES1_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_RES1_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_U_WIDTH                 1
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_U_SHIFT                 30
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_U_MASK                  0x40000000U
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_U_RD(src)               (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_U_SET(dst,src)          (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_MT_WIDTH                1
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_MT_SHIFT                24
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_MT_MASK                 0x01000000U
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_MT_RD(src)              (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_MT_SET(dst,src)         (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF2_WIDTH              8
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF2_SHIFT              16
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF2_MASK               0x00FF0000U
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF2_RD(src)            (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF2_SET(dst,src)       (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF1_WIDTH              8
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF1_SHIFT              8
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF1_MASK               0x0000FF00U
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF1_RD(src)            (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF1_SET(dst,src)       (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF0_WIDTH              8
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF0_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF0_MASK               0x000000FFU
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_CTI_CTIDEVAFF0_AFF0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_CTI_CTIDEVAFF1_AFF3_WIDTH              8
#define PCP_RB_CPUX_CTI_CTIDEVAFF1_AFF3_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIDEVAFF1_AFF3_MASK               0x000000FFU
#define PCP_RB_CPUX_CTI_CTIDEVAFF1_AFF3_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_CTI_CTIDEVAFF1_AFF3_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_CTI_CTIDEVARCH_BIT31_0_WIDTH           32
#define PCP_RB_CPUX_CTI_CTIDEVARCH_BIT31_0_SHIFT           0
#define PCP_RB_CPUX_CTI_CTIDEVARCH_BIT31_0_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTIDEVARCH_BIT31_0_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTIDEVARCH_BIT31_0_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTIDEVID0_INOUT_WIDTH              2
#define PCP_RB_CPUX_CTI_CTIDEVID0_INOUT_SHIFT              24
#define PCP_RB_CPUX_CTI_CTIDEVID0_INOUT_MASK               0x03000000U
#define PCP_RB_CPUX_CTI_CTIDEVID0_INOUT_RD(src)            (((src) & 0x03000000U)>>24)
#define PCP_RB_CPUX_CTI_CTIDEVID0_INOUT_SET(dst,src)       (((dst) & ~0x03000000U) | (((u32)(src)<<24) & 0x03000000U))
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMCHAN_WIDTH            6
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMCHAN_SHIFT            16
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMCHAN_MASK             0x003F0000U
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMCHAN_RD(src)          (((src) & 0x003F0000U)>>16)
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMCHAN_SET(dst,src)     (((dst) & ~0x003F0000U) | (((u32)(src)<<16) & 0x003F0000U))
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMTRIG_WIDTH            6
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMTRIG_SHIFT            8
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMTRIG_MASK             0x00003F00U
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMTRIG_RD(src)          (((src) & 0x00003F00U)>>8)
#define PCP_RB_CPUX_CTI_CTIDEVID0_NUMTRIG_SET(dst,src)     (((dst) & ~0x00003F00U) | (((u32)(src)<<8) & 0x00003F00U))
#define PCP_RB_CPUX_CTI_CTIDEVID0_EXTMAXNUM_WIDTH          5
#define PCP_RB_CPUX_CTI_CTIDEVID0_EXTMAXNUM_SHIFT          0
#define PCP_RB_CPUX_CTI_CTIDEVID0_EXTMAXNUM_MASK           0x0000001FU
#define PCP_RB_CPUX_CTI_CTIDEVID0_EXTMAXNUM_RD(src)        (((src) & 0x0000001FU)>>0)
#define PCP_RB_CPUX_CTI_CTIDEVID0_EXTMAXNUM_SET(dst,src)   (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_SUB_WIDTH               4
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_SUB_SHIFT               4
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_SUB_MASK                0x000000F0U
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_SUB_RD(src)             (((src) & 0x000000F0U)>>4)
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_SUB_SET(dst,src)        (((dst) & ~0x000000F0U) | (((u32)(src)<<4) & 0x000000F0U))
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_MAJOR_WIDTH             4
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_MAJOR_SHIFT             0
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_MAJOR_MASK              0x0000000FU
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_MAJOR_RD(src)           (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIDEVTYPE_MAJOR_SET(dst,src)      (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIGATE_G_WIDTH                    4
#define PCP_RB_CPUX_CTI_CTIGATE_G_SHIFT                    0
#define PCP_RB_CPUX_CTI_CTIGATE_G_MASK                     0x0000000FU
#define PCP_RB_CPUX_CTI_CTIGATE_G_RD(src)                  (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIGATE_G_SET(dst,src)             (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN0_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN0_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN0_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN0_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN0_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN1_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN1_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN1_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN1_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN1_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN2_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN2_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN2_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN2_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN2_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN3_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN3_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN3_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN3_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN3_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN4_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN4_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN4_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN4_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN4_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN5_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN5_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN5_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN5_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN5_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN6_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN6_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN6_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN6_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN6_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINEN7_INEN_WIDTH                4
#define PCP_RB_CPUX_CTI_CTIINEN7_INEN_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINEN7_INEN_MASK                 0x0000000FU
#define PCP_RB_CPUX_CTI_CTIINEN7_INEN_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIINEN7_INEN_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIINTACK_ACK_WIDTH                1
#define PCP_RB_CPUX_CTI_CTIINTACK_ACK_SHIFT                0
#define PCP_RB_CPUX_CTI_CTIINTACK_ACK_MASK                 0x00000001U
#define PCP_RB_CPUX_CTI_CTIINTACK_ACK_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_CTI_CTIINTACK_ACK_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_CTI_CTILAR_CTL_WIDTH                   32
#define PCP_RB_CPUX_CTI_CTILAR_CTL_SHIFT                   0
#define PCP_RB_CPUX_CTI_CTILAR_CTL_MASK                    0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTILAR_CTL_RD(src)                 (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTILAR_CTL_SET(dst,src)            (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTILSR_NTT_WIDTH                   1
#define PCP_RB_CPUX_CTI_CTILSR_NTT_SHIFT                   2
#define PCP_RB_CPUX_CTI_CTILSR_NTT_MASK                    0x00000004U
#define PCP_RB_CPUX_CTI_CTILSR_NTT_RD(src)                 (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_CTI_CTILSR_NTT_SET(dst,src)            (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_CTI_CTILSR_SLK_WIDTH                   1
#define PCP_RB_CPUX_CTI_CTILSR_SLK_SHIFT                   1
#define PCP_RB_CPUX_CTI_CTILSR_SLK_MASK                    0x00000002U
#define PCP_RB_CPUX_CTI_CTILSR_SLK_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_CTI_CTILSR_SLK_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_CTI_CTILSR_SLI_WIDTH                   1
#define PCP_RB_CPUX_CTI_CTILSR_SLI_SHIFT                   0
#define PCP_RB_CPUX_CTI_CTILSR_SLI_MASK                    0x00000001U
#define PCP_RB_CPUX_CTI_CTILSR_SLI_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_CTI_CTILSR_SLI_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_CTI_CTIOUTEN0_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN0_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN0_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN0_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN0_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIOUTEN1_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN1_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN1_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN1_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN1_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIOUTEN2_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN2_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN2_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN2_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN2_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIOUTEN3_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN3_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN3_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN3_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN3_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIOUTEN4_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN4_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN4_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN4_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN4_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIOUTEN5_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN5_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN5_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN5_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN5_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIOUTEN6_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN6_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN6_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN6_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN6_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIOUTEN7_OUTEN_WIDTH              4
#define PCP_RB_CPUX_CTI_CTIOUTEN7_OUTEN_SHIFT              0
#define PCP_RB_CPUX_CTI_CTIOUTEN7_OUTEN_MASK               0x0000000FU
#define PCP_RB_CPUX_CTI_CTIOUTEN7_OUTEN_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_CTI_CTIOUTEN7_OUTEN_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_CTI_CTIPIDR0_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTIPIDR0_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTIPIDR0_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTIPIDR0_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTIPIDR0_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTIPIDR1_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTIPIDR1_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTIPIDR1_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTIPIDR1_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTIPIDR1_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTIPIDR2_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTIPIDR2_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTIPIDR2_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTIPIDR2_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTIPIDR2_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTIPIDR3_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTIPIDR3_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTIPIDR3_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTIPIDR3_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTIPIDR3_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTIPIDR4_BIT31_0_WIDTH             32
#define PCP_RB_CPUX_CTI_CTIPIDR4_BIT31_0_SHIFT             0
#define PCP_RB_CPUX_CTI_CTIPIDR4_BIT31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_CTI_CTIPIDR4_BIT31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_CTI_CTIPIDR4_BIT31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_CTI_CTITRIGOUTSTATUS_TROUT_WIDTH       1
#define PCP_RB_CPUX_CTI_CTITRIGOUTSTATUS_TROUT_SHIFT       0
#define PCP_RB_CPUX_CTI_CTITRIGOUTSTATUS_TROUT_MASK        0x00000001U
#define PCP_RB_CPUX_CTI_CTITRIGOUTSTATUS_TROUT_RD(src)     (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_CTI_CTITRIGOUTSTATUS_TROUT_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SNID_WIDTH       2
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SNID_SHIFT       6
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SNID_MASK        0x000000C0U
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SNID_RD(src)     (((src) & 0x000000C0U)>>6)
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SNID_SET(dst,src) (((dst) & ~0x000000C0U) | (((u32)(src)<<6) & 0x000000C0U))
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SID_WIDTH        2
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SID_SHIFT        4
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SID_MASK         0x00000030U
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SID_RD(src)      (((src) & 0x00000030U)>>4)
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_SID_SET(dst,src) (((dst) & ~0x00000030U) | (((u32)(src)<<4) & 0x00000030U))
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSNID_WIDTH      2
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSNID_SHIFT      2
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSNID_MASK       0x0000000CU
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSNID_RD(src)    (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSNID_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSID_WIDTH       2
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSID_SHIFT       0
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSID_MASK        0x00000003U
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSID_RD(src)     (((src) & 0x00000003U)>>0)
#define PCP_RB_CPUX_DBG_DBGAUTHSTATUS_EL1_NSID_SET(dst,src) (((dst) & ~0x00000003U) | (((u32)(src)<<0) & 0x00000003U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT3_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT3_SHIFT              23
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT3_MASK               0x00800000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT3_RD(src)            (((src) & 0x00800000U)>>23)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT3_SET(dst,src)       (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT2_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT2_SHIFT              22
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT2_MASK               0x00400000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT2_RD(src)            (((src) & 0x00400000U)>>22)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT2_SET(dst,src)       (((dst) & ~0x00400000U) | (((u32)(src)<<22) & 0x00400000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT1_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT1_SHIFT              21
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT1_MASK               0x00200000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT1_RD(src)            (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT1_SET(dst,src)       (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT0_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT0_SHIFT              20
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT0_MASK               0x00100000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT0_RD(src)            (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BT0_SET(dst,src)       (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS3_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS3_SHIFT             8
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS3_MASK              0x00000100U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS3_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS3_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS2_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS2_SHIFT             7
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS2_MASK              0x00000080U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS2_RD(src)           (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS2_SET(dst,src)      (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS1_SHIFT             6
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS1_MASK              0x00000040U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS1_RD(src)           (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS1_SET(dst,src)      (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS0_SHIFT             5
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS0_MASK              0x00000020U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS0_RD(src)           (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_BAS0_SET(dst,src)      (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_PMC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_PMC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_PMC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_PMC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_PMC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGBCR0_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT3_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT3_SHIFT              23
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT3_MASK               0x00800000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT3_RD(src)            (((src) & 0x00800000U)>>23)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT3_SET(dst,src)       (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT2_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT2_SHIFT              22
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT2_MASK               0x00400000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT2_RD(src)            (((src) & 0x00400000U)>>22)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT2_SET(dst,src)       (((dst) & ~0x00400000U) | (((u32)(src)<<22) & 0x00400000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT1_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT1_SHIFT              21
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT1_MASK               0x00200000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT1_RD(src)            (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT1_SET(dst,src)       (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT0_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT0_SHIFT              20
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT0_MASK               0x00100000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT0_RD(src)            (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BT0_SET(dst,src)       (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS3_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS3_SHIFT             8
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS3_MASK              0x00000100U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS3_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS3_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS2_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS2_SHIFT             7
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS2_MASK              0x00000080U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS2_RD(src)           (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS2_SET(dst,src)      (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS1_SHIFT             6
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS1_MASK              0x00000040U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS1_RD(src)           (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS1_SET(dst,src)      (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS0_SHIFT             5
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS0_MASK              0x00000020U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS0_RD(src)           (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_BAS0_SET(dst,src)      (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_PMC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_PMC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_PMC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_PMC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_PMC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGBCR1_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT3_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT3_SHIFT              23
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT3_MASK               0x00800000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT3_RD(src)            (((src) & 0x00800000U)>>23)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT3_SET(dst,src)       (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT2_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT2_SHIFT              22
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT2_MASK               0x00400000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT2_RD(src)            (((src) & 0x00400000U)>>22)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT2_SET(dst,src)       (((dst) & ~0x00400000U) | (((u32)(src)<<22) & 0x00400000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT1_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT1_SHIFT              21
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT1_MASK               0x00200000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT1_RD(src)            (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT1_SET(dst,src)       (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT0_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT0_SHIFT              20
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT0_MASK               0x00100000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT0_RD(src)            (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BT0_SET(dst,src)       (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS3_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS3_SHIFT             8
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS3_MASK              0x00000100U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS3_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS3_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS2_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS2_SHIFT             7
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS2_MASK              0x00000080U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS2_RD(src)           (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS2_SET(dst,src)      (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS1_SHIFT             6
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS1_MASK              0x00000040U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS1_RD(src)           (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS1_SET(dst,src)      (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS0_SHIFT             5
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS0_MASK              0x00000020U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS0_RD(src)           (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_BAS0_SET(dst,src)      (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_PMC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_PMC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_PMC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_PMC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_PMC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGBCR2_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT3_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT3_SHIFT              23
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT3_MASK               0x00800000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT3_RD(src)            (((src) & 0x00800000U)>>23)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT3_SET(dst,src)       (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT2_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT2_SHIFT              22
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT2_MASK               0x00400000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT2_RD(src)            (((src) & 0x00400000U)>>22)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT2_SET(dst,src)       (((dst) & ~0x00400000U) | (((u32)(src)<<22) & 0x00400000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT1_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT1_SHIFT              21
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT1_MASK               0x00200000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT1_RD(src)            (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT1_SET(dst,src)       (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT0_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT0_SHIFT              20
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT0_MASK               0x00100000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT0_RD(src)            (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BT0_SET(dst,src)       (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS3_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS3_SHIFT             8
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS3_MASK              0x00000100U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS3_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS3_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS2_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS2_SHIFT             7
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS2_MASK              0x00000080U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS2_RD(src)           (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS2_SET(dst,src)      (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS1_SHIFT             6
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS1_MASK              0x00000040U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS1_RD(src)           (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS1_SET(dst,src)      (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS0_SHIFT             5
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS0_MASK              0x00000020U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS0_RD(src)           (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_BAS0_SET(dst,src)      (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_PMC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_PMC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_PMC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_PMC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_PMC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGBCR3_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA64_WIDTH           49
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA64_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA64_MASK            0x1FFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA64_RD(src)         (((src) & 0x1FFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA64_SET(dst,src)    (((dst) & ~0x1FFFFFFFFFFFFU) | (((u32)(src)<<0) & 0x1FFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA32_WIDTH           32
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA32_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA32_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA32_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR0_EL1_DATA32_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA64_WIDTH           49
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA64_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA64_MASK            0x1FFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA64_RD(src)         (((src) & 0x1FFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA64_SET(dst,src)    (((dst) & ~0x1FFFFFFFFFFFFU) | (((u32)(src)<<0) & 0x1FFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA32_WIDTH           32
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA32_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA32_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA32_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR1_EL1_DATA32_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA64_WIDTH           49
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA64_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA64_MASK            0x1FFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA64_RD(src)         (((src) & 0x1FFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA64_SET(dst,src)    (((dst) & ~0x1FFFFFFFFFFFFU) | (((u32)(src)<<0) & 0x1FFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA32_WIDTH           32
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA32_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA32_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA32_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR2_EL1_DATA32_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA64_WIDTH           49
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA64_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA64_MASK            0x1FFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA64_RD(src)         (((src) & 0x1FFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA64_SET(dst,src)    (((dst) & ~0x1FFFFFFFFFFFFU) | (((u32)(src)<<0) & 0x1FFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA32_WIDTH           32
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA32_SHIFT           0
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA32_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA32_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGBVR3_EL1_DATA32_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_DBG_DBGCLAIMCLR_EL1_BITS7_0_WIDTH      8
#define PCP_RB_CPUX_DBG_DBGCLAIMCLR_EL1_BITS7_0_SHIFT      0
#define PCP_RB_CPUX_DBG_DBGCLAIMCLR_EL1_BITS7_0_MASK       0x000000FFU
#define PCP_RB_CPUX_DBG_DBGCLAIMCLR_EL1_BITS7_0_RD(src)    (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_DBGCLAIMCLR_EL1_BITS7_0_SET(dst,src) (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_DBGCLAIMSET_EL1_BITS7_0_WIDTH      8
#define PCP_RB_CPUX_DBG_DBGCLAIMSET_EL1_BITS7_0_SHIFT      0
#define PCP_RB_CPUX_DBG_DBGCLAIMSET_EL1_BITS7_0_MASK       0x000000FFU
#define PCP_RB_CPUX_DBG_DBGCLAIMSET_EL1_BITS7_0_RD(src)    (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_DBGCLAIMSET_EL1_BITS7_0_SET(dst,src) (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_DBGDTR_EL0_BITS63_0_WIDTH          64
#define PCP_RB_CPUX_DBG_DBGDTR_EL0_BITS63_0_SHIFT          0
#define PCP_RB_CPUX_DBG_DBGDTR_EL0_BITS63_0_MASK           0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_DBGDTR_EL0_BITS63_0_RD(src)        (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_DBGDTR_EL0_BITS63_0_SET(dst,src)   (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_ENABLE_WIDTH           1
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_ENABLE_SHIFT           15
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_ENABLE_MASK            0x00008000U
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_ENABLE_RD(src)         (((src) & 0x00008000U)>>15)
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_ENABLE_SET(dst,src)    (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_DELAY_WIDTH            12
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_DELAY_SHIFT            0
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_DELAY_MASK             0x00000FFFU
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_DELAY_RD(src)          (((src) & 0x00000FFFU)>>0)
#define PCP_RB_IOB_DBG_DBGPCPPLLCCR_DELAY_SET(dst,src)     (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_MASK_WIDTH             5
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_MASK_SHIFT             24
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_MASK_MASK              0x1F000000U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_MASK_RD(src)           (((src) & 0x1F000000U)>>24)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_MASK_SET(dst,src)      (((dst) & ~0x1F000000U) | (((u32)(src)<<24) & 0x1F000000U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_WT_WIDTH               1
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_WT_SHIFT               20
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_WT_MASK                0x00100000U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_WT_RD(src)             (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_WT_SET(dst,src)        (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_BAS_WIDTH              8
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_BAS_SHIFT              5
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_BAS_MASK               0x00001FE0U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_BAS_RD(src)            (((src) & 0x00001FE0U)>>5)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_BAS_SET(dst,src)       (((dst) & ~0x00001FE0U) | (((u32)(src)<<5) & 0x00001FE0U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LSC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LSC_SHIFT              3
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LSC_MASK               0x00000018U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LSC_RD(src)            (((src) & 0x00000018U)>>3)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_LSC_SET(dst,src)       (((dst) & ~0x00000018U) | (((u32)(src)<<3) & 0x00000018U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_PAC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_PAC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_PAC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_PAC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_PAC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGWCR0_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_MASK_WIDTH             5
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_MASK_SHIFT             24
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_MASK_MASK              0x1F000000U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_MASK_RD(src)           (((src) & 0x1F000000U)>>24)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_MASK_SET(dst,src)      (((dst) & ~0x1F000000U) | (((u32)(src)<<24) & 0x1F000000U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_WT_WIDTH               1
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_WT_SHIFT               20
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_WT_MASK                0x00100000U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_WT_RD(src)             (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_WT_SET(dst,src)        (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_BAS_WIDTH              8
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_BAS_SHIFT              5
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_BAS_MASK               0x00001FE0U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_BAS_RD(src)            (((src) & 0x00001FE0U)>>5)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_BAS_SET(dst,src)       (((dst) & ~0x00001FE0U) | (((u32)(src)<<5) & 0x00001FE0U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LSC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LSC_SHIFT              3
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LSC_MASK               0x00000018U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LSC_RD(src)            (((src) & 0x00000018U)>>3)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_LSC_SET(dst,src)       (((dst) & ~0x00000018U) | (((u32)(src)<<3) & 0x00000018U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_PAC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_PAC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_PAC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_PAC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_PAC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGWCR1_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_MASK_WIDTH             5
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_MASK_SHIFT             24
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_MASK_MASK              0x1F000000U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_MASK_RD(src)           (((src) & 0x1F000000U)>>24)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_MASK_SET(dst,src)      (((dst) & ~0x1F000000U) | (((u32)(src)<<24) & 0x1F000000U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_WT_WIDTH               1
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_WT_SHIFT               20
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_WT_MASK                0x00100000U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_WT_RD(src)             (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_WT_SET(dst,src)        (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_BAS_WIDTH              8
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_BAS_SHIFT              5
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_BAS_MASK               0x00001FE0U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_BAS_RD(src)            (((src) & 0x00001FE0U)>>5)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_BAS_SET(dst,src)       (((dst) & ~0x00001FE0U) | (((u32)(src)<<5) & 0x00001FE0U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LSC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LSC_SHIFT              3
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LSC_MASK               0x00000018U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LSC_RD(src)            (((src) & 0x00000018U)>>3)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_LSC_SET(dst,src)       (((dst) & ~0x00000018U) | (((u32)(src)<<3) & 0x00000018U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_PAC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_PAC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_PAC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_PAC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_PAC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGWCR2_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_MASK_WIDTH             5
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_MASK_SHIFT             24
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_MASK_MASK              0x1F000000U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_MASK_RD(src)           (((src) & 0x1F000000U)>>24)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_MASK_SET(dst,src)      (((dst) & ~0x1F000000U) | (((u32)(src)<<24) & 0x1F000000U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_WT_WIDTH               1
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_WT_SHIFT               20
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_WT_MASK                0x00100000U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_WT_RD(src)             (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_WT_SET(dst,src)        (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LBN_WIDTH              4
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LBN_SHIFT              16
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LBN_MASK               0x000F0000U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LBN_RD(src)            (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LBN_SET(dst,src)       (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC1_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC1_SHIFT             15
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC1_MASK              0x00008000U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC1_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC1_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC0_WIDTH             1
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC0_SHIFT             14
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC0_MASK              0x00004000U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC0_RD(src)           (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_SSC0_SET(dst,src)      (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_HMC_WIDTH              1
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_HMC_SHIFT              13
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_HMC_MASK               0x00002000U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_HMC_RD(src)            (((src) & 0x00002000U)>>13)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_HMC_SET(dst,src)       (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_BAS_WIDTH              8
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_BAS_SHIFT              5
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_BAS_MASK               0x00001FE0U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_BAS_RD(src)            (((src) & 0x00001FE0U)>>5)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_BAS_SET(dst,src)       (((dst) & ~0x00001FE0U) | (((u32)(src)<<5) & 0x00001FE0U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LSC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LSC_SHIFT              3
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LSC_MASK               0x00000018U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LSC_RD(src)            (((src) & 0x00000018U)>>3)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_LSC_SET(dst,src)       (((dst) & ~0x00000018U) | (((u32)(src)<<3) & 0x00000018U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_PAC_WIDTH              2
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_PAC_SHIFT              1
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_PAC_MASK               0x00000006U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_PAC_RD(src)            (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_PAC_SET(dst,src)       (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_E_WIDTH                1
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_E_SHIFT                0
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_E_MASK                 0x00000001U
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_E_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_DBGWCR3_EL1_E_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA49_WIDTH             47
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA49_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA49_MASK              0x1FFFFFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA49_RD(src)           (((src) & 0x1FFFFFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA49_SET(dst,src)      (((dst) & ~0x1FFFFFFFFFFFCU) | (((u32)(src)<<2) & 0x1FFFFFFFFFFFCU))
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA32_WIDTH             30
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA32_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA32_MASK              0xFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA32_RD(src)           (((src) & 0xFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR0_EL1_VA32_SET(dst,src)      (((dst) & ~0xFFFFFFFCU) | (((u32)(src)<<2) & 0xFFFFFFFCU))
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA49_WIDTH             47
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA49_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA49_MASK              0x1FFFFFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA49_RD(src)           (((src) & 0x1FFFFFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA49_SET(dst,src)      (((dst) & ~0x1FFFFFFFFFFFCU) | (((u32)(src)<<2) & 0x1FFFFFFFFFFFCU))
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA32_WIDTH             30
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA32_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA32_MASK              0xFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA32_RD(src)           (((src) & 0xFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR1_EL1_VA32_SET(dst,src)      (((dst) & ~0xFFFFFFFCU) | (((u32)(src)<<2) & 0xFFFFFFFCU))
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA49_WIDTH             47
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA49_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA49_MASK              0x1FFFFFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA49_RD(src)           (((src) & 0x1FFFFFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA49_SET(dst,src)      (((dst) & ~0x1FFFFFFFFFFFCU) | (((u32)(src)<<2) & 0x1FFFFFFFFFFFCU))
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA32_WIDTH             30
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA32_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA32_MASK              0xFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA32_RD(src)           (((src) & 0xFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR2_EL1_VA32_SET(dst,src)      (((dst) & ~0xFFFFFFFCU) | (((u32)(src)<<2) & 0xFFFFFFFCU))
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_RESS_WIDTH             15
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_RESS_SHIFT             49
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_RESS_MASK              0xFFFE000000000000U
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_RESS_RD(src)           (((src) & 0xFFFE000000000000U)>>49)
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_RESS_SET(dst,src)      (((dst) & ~0xFFFE000000000000U) | (((u32)(src)<<49) & 0xFFFE000000000000U))
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA49_WIDTH             47
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA49_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA49_MASK              0x1FFFFFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA49_RD(src)           (((src) & 0x1FFFFFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA49_SET(dst,src)      (((dst) & ~0x1FFFFFFFFFFFCU) | (((u32)(src)<<2) & 0x1FFFFFFFFFFFCU))
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA32_WIDTH             30
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA32_SHIFT             2
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA32_MASK              0xFFFFFFFCU
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA32_RD(src)           (((src) & 0xFFFFFFFCU)>>2)
#define PCP_RB_CPUX_DBG_DBGWVR3_EL1_VA32_SET(dst,src)      (((dst) & ~0xFFFFFFFCU) | (((u32)(src)<<2) & 0xFFFFFFFCU))
#define PCP_RB_CPUX_DBG_EDCIDR0_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDCIDR0_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDCIDR0_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDCIDR0_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDCIDR0_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDCIDR1_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDCIDR1_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDCIDR1_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDCIDR1_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDCIDR1_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDCIDR2_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDCIDR2_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDCIDR2_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDCIDR2_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDCIDR2_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDCIDR3_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDCIDR3_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDCIDR3_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDCIDR3_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDCIDR3_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDDEVAFF0_RES1_WIDTH               1
#define PCP_RB_CPUX_DBG_EDDEVAFF0_RES1_SHIFT               31
#define PCP_RB_CPUX_DBG_EDDEVAFF0_RES1_MASK                0x80000000U
#define PCP_RB_CPUX_DBG_EDDEVAFF0_RES1_RD(src)             (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_DBG_EDDEVAFF0_RES1_SET(dst,src)        (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_DBG_EDDEVAFF0_U_WIDTH                  1
#define PCP_RB_CPUX_DBG_EDDEVAFF0_U_SHIFT                  30
#define PCP_RB_CPUX_DBG_EDDEVAFF0_U_MASK                   0x40000000U
#define PCP_RB_CPUX_DBG_EDDEVAFF0_U_RD(src)                (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_DBG_EDDEVAFF0_U_SET(dst,src)           (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_DBG_EDDEVAFF0_MT_WIDTH                 1
#define PCP_RB_CPUX_DBG_EDDEVAFF0_MT_SHIFT                 24
#define PCP_RB_CPUX_DBG_EDDEVAFF0_MT_MASK                  0x01000000U
#define PCP_RB_CPUX_DBG_EDDEVAFF0_MT_RD(src)               (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_DBG_EDDEVAFF0_MT_SET(dst,src)          (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF2_WIDTH               8
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF2_SHIFT               16
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF2_MASK                0x00FF0000U
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF2_RD(src)             (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF2_SET(dst,src)        (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF1_WIDTH               8
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF1_SHIFT               8
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF1_MASK                0x0000FF00U
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF1_RD(src)             (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF1_SET(dst,src)        (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDDEVAFF0_AFF0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDDEVAFF1_AFF3_WIDTH               8
#define PCP_RB_CPUX_DBG_EDDEVAFF1_AFF3_SHIFT               0
#define PCP_RB_CPUX_DBG_EDDEVAFF1_AFF3_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDDEVAFF1_AFF3_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDDEVAFF1_AFF3_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHITECT_WIDTH          11
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHITECT_SHIFT          21
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHITECT_MASK           0xFFE00000U
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHITECT_RD(src)        (((src) & 0xFFE00000U)>>21)
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHITECT_SET(dst,src)   (((dst) & ~0xFFE00000U) | (((u32)(src)<<21) & 0xFFE00000U))
#define PCP_RB_CPUX_DBG_EDDEVARCH_PRESENT_WIDTH            1
#define PCP_RB_CPUX_DBG_EDDEVARCH_PRESENT_SHIFT            20
#define PCP_RB_CPUX_DBG_EDDEVARCH_PRESENT_MASK             0x00100000U
#define PCP_RB_CPUX_DBG_EDDEVARCH_PRESENT_RD(src)          (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_EDDEVARCH_PRESENT_SET(dst,src)     (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_EDDEVARCH_REVISION_WIDTH           4
#define PCP_RB_CPUX_DBG_EDDEVARCH_REVISION_SHIFT           16
#define PCP_RB_CPUX_DBG_EDDEVARCH_REVISION_MASK            0x000F0000U
#define PCP_RB_CPUX_DBG_EDDEVARCH_REVISION_RD(src)         (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_EDDEVARCH_REVISION_SET(dst,src)    (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHVER_WIDTH            4
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHVER_SHIFT            12
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHVER_MASK             0x0000F000U
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHVER_RD(src)          (((src) & 0x0000F000U)>>12)
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHVER_SET(dst,src)     (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHPART_WIDTH           12
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHPART_SHIFT           0
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHPART_MASK            0x00000FFFU
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHPART_RD(src)         (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CPUX_DBG_EDDEVARCH_ARCHPART_SET(dst,src)    (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_DBG_EDDEVID_PCSAMPLE_WIDTH             4
#define PCP_RB_CPUX_DBG_EDDEVID_PCSAMPLE_SHIFT             0
#define PCP_RB_CPUX_DBG_EDDEVID_PCSAMPLE_MASK              0x0000000FU
#define PCP_RB_CPUX_DBG_EDDEVID_PCSAMPLE_RD(src)           (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_DBG_EDDEVID_PCSAMPLE_SET(dst,src)      (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_DBG_EDDEVID_AUXREGS_WIDTH              4
#define PCP_RB_CPUX_DBG_EDDEVID_AUXREGS_SHIFT              24
#define PCP_RB_CPUX_DBG_EDDEVID_AUXREGS_MASK               0x0F000000U
#define PCP_RB_CPUX_DBG_EDDEVID_AUXREGS_RD(src)            (((src) & 0x0F000000U)>>24)
#define PCP_RB_CPUX_DBG_EDDEVID_AUXREGS_SET(dst,src)       (((dst) & ~0x0F000000U) | (((u32)(src)<<24) & 0x0F000000U))
#define PCP_RB_CPUX_DBG_EDDEVID1_PCSROFFSET_WIDTH          4
#define PCP_RB_CPUX_DBG_EDDEVID1_PCSROFFSET_SHIFT          0
#define PCP_RB_CPUX_DBG_EDDEVID1_PCSROFFSET_MASK           0x0000000FU
#define PCP_RB_CPUX_DBG_EDDEVID1_PCSROFFSET_RD(src)        (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_DBG_EDDEVID1_PCSROFFSET_SET(dst,src)   (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_DBG_EDDEVTYPE_SUB_WIDTH                4
#define PCP_RB_CPUX_DBG_EDDEVTYPE_SUB_SHIFT                4
#define PCP_RB_CPUX_DBG_EDDEVTYPE_SUB_MASK                 0x000000F0U
#define PCP_RB_CPUX_DBG_EDDEVTYPE_SUB_RD(src)              (((src) & 0x000000F0U)>>4)
#define PCP_RB_CPUX_DBG_EDDEVTYPE_SUB_SET(dst,src)         (((dst) & ~0x000000F0U) | (((u32)(src)<<4) & 0x000000F0U))
#define PCP_RB_CPUX_DBG_EDDEVTYPE_MAJOR_WIDTH              4
#define PCP_RB_CPUX_DBG_EDDEVTYPE_MAJOR_SHIFT              0
#define PCP_RB_CPUX_DBG_EDDEVTYPE_MAJOR_MASK               0x0000000FU
#define PCP_RB_CPUX_DBG_EDDEVTYPE_MAJOR_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_DBG_EDDEVTYPE_MAJOR_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_DBG_EDECCR_NSE_WIDTH                   4
#define PCP_RB_CPUX_DBG_EDECCR_NSE_SHIFT                   4
#define PCP_RB_CPUX_DBG_EDECCR_NSE_MASK                    0x000000F0U
#define PCP_RB_CPUX_DBG_EDECCR_NSE_RD(src)                 (((src) & 0x000000F0U)>>4)
#define PCP_RB_CPUX_DBG_EDECCR_NSE_SET(dst,src)            (((dst) & ~0x000000F0U) | (((u32)(src)<<4) & 0x000000F0U))
#define PCP_RB_CPUX_DBG_EDECCR_SE_WIDTH                    4
#define PCP_RB_CPUX_DBG_EDECCR_SE_SHIFT                    0
#define PCP_RB_CPUX_DBG_EDECCR_SE_MASK                     0x0000000FU
#define PCP_RB_CPUX_DBG_EDECCR_SE_RD(src)                  (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_DBG_EDECCR_SE_SET(dst,src)             (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_DBG_EDECR_SS_WIDTH                     1
#define PCP_RB_CPUX_DBG_EDECR_SS_SHIFT                     2
#define PCP_RB_CPUX_DBG_EDECR_SS_MASK                      0x00000004U
#define PCP_RB_CPUX_DBG_EDECR_SS_RD(src)                   (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_DBG_EDECR_SS_SET(dst,src)              (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_DBG_EDECR_RCE_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDECR_RCE_SHIFT                    1
#define PCP_RB_CPUX_DBG_EDECR_RCE_MASK                     0x00000002U
#define PCP_RB_CPUX_DBG_EDECR_RCE_RD(src)                  (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_DBG_EDECR_RCE_SET(dst,src)             (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_DBG_EDECR_OSUCE_WIDTH                  1
#define PCP_RB_CPUX_DBG_EDECR_OSUCE_SHIFT                  0
#define PCP_RB_CPUX_DBG_EDECR_OSUCE_MASK                   0x00000001U
#define PCP_RB_CPUX_DBG_EDECR_OSUCE_RD(src)                (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_EDECR_OSUCE_SET(dst,src)           (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_EDESR_SS_WIDTH                     1
#define PCP_RB_CPUX_DBG_EDESR_SS_SHIFT                     2
#define PCP_RB_CPUX_DBG_EDESR_SS_MASK                      0x00000004U
#define PCP_RB_CPUX_DBG_EDESR_SS_RD(src)                   (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_DBG_EDESR_SS_SET(dst,src)              (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_DBG_EDESR_RC_WIDTH                     1
#define PCP_RB_CPUX_DBG_EDESR_RC_SHIFT                     1
#define PCP_RB_CPUX_DBG_EDESR_RC_MASK                      0x00000002U
#define PCP_RB_CPUX_DBG_EDESR_RC_RD(src)                   (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_DBG_EDESR_RC_SET(dst,src)              (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_DBG_EDESR_OSUC_WIDTH                   1
#define PCP_RB_CPUX_DBG_EDESR_OSUC_SHIFT                   0
#define PCP_RB_CPUX_DBG_EDESR_OSUC_MASK                    0x00000001U
#define PCP_RB_CPUX_DBG_EDESR_OSUC_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_EDESR_OSUC_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_EDLSR_NTT_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDLSR_NTT_SHIFT                    2
#define PCP_RB_CPUX_DBG_EDLSR_NTT_MASK                     0x00000004U
#define PCP_RB_CPUX_DBG_EDLSR_NTT_RD(src)                  (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_DBG_EDLSR_NTT_SET(dst,src)             (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_DBG_EDLSR_SLK_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDLSR_SLK_SHIFT                    1
#define PCP_RB_CPUX_DBG_EDLSR_SLK_MASK                     0x00000002U
#define PCP_RB_CPUX_DBG_EDLSR_SLK_RD(src)                  (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_DBG_EDLSR_SLK_SET(dst,src)             (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_DBG_EDLSR_SLI_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDLSR_SLI_SHIFT                    0
#define PCP_RB_CPUX_DBG_EDLSR_SLI_MASK                     0x00000001U
#define PCP_RB_CPUX_DBG_EDLSR_SLI_RD(src)                  (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_EDLSR_SLI_SET(dst,src)             (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_EDPIDR0_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDPIDR0_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDPIDR0_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDPIDR0_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDPIDR0_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDPIDR1_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDPIDR1_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDPIDR1_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDPIDR1_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDPIDR1_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDPIDR2_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDPIDR2_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDPIDR2_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDPIDR2_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDPIDR2_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDPIDR3_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDPIDR3_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDPIDR3_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDPIDR3_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDPIDR3_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDPIDR4_BIT7_0_WIDTH               8
#define PCP_RB_CPUX_DBG_EDPIDR4_BIT7_0_SHIFT               0
#define PCP_RB_CPUX_DBG_EDPIDR4_BIT7_0_MASK                0x000000FFU
#define PCP_RB_CPUX_DBG_EDPIDR4_BIT7_0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_DBG_EDPIDR4_BIT7_0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_DBG_EDPRCR_COREPURQ_WIDTH              1
#define PCP_RB_CPUX_DBG_EDPRCR_COREPURQ_SHIFT              3
#define PCP_RB_CPUX_DBG_EDPRCR_COREPURQ_MASK               0x00000008U
#define PCP_RB_CPUX_DBG_EDPRCR_COREPURQ_RD(src)            (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_DBG_EDPRCR_COREPURQ_SET(dst,src)       (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_DBG_EDPRCR_CWRR_WIDTH                  1
#define PCP_RB_CPUX_DBG_EDPRCR_CWRR_SHIFT                  1
#define PCP_RB_CPUX_DBG_EDPRCR_CWRR_MASK                   0x00000002U
#define PCP_RB_CPUX_DBG_EDPRCR_CWRR_RD(src)                (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_DBG_EDPRCR_CWRR_SET(dst,src)           (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_DBG_EDPRCR_CORENPDRQ_WIDTH             1
#define PCP_RB_CPUX_DBG_EDPRCR_CORENPDRQ_SHIFT             0
#define PCP_RB_CPUX_DBG_EDPRCR_CORENPDRQ_MASK              0x00000001U
#define PCP_RB_CPUX_DBG_EDPRCR_CORENPDRQ_RD(src)           (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_EDPRCR_CORENPDRQ_SET(dst,src)      (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_CPU_EDPRCR_ERROR_COREPURQ_WIDTH        1
#define PCP_RB_CPUX_CPU_EDPRCR_ERROR_COREPURQ_SHIFT        3
#define PCP_RB_CPUX_CPU_EDPRCR_ERROR_COREPURQ_MASK         0x00000008U
#define PCP_RB_CPUX_CPU_EDPRCR_ERROR_COREPURQ_RD(src)      (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_CPU_EDPRCR_ERROR_COREPURQ_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_DBG_EDPRSR_SDR_WIDTH                   1
#define PCP_RB_CPUX_DBG_EDPRSR_SDR_SHIFT                   11
#define PCP_RB_CPUX_DBG_EDPRSR_SDR_MASK                    0x00000800U
#define PCP_RB_CPUX_DBG_EDPRSR_SDR_RD(src)                 (((src) & 0x00000800U)>>11)
#define PCP_RB_CPUX_DBG_EDPRSR_SDR_SET(dst,src)            (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CPUX_DBG_EDPRSR_SPMAD_WIDTH                 1
#define PCP_RB_CPUX_DBG_EDPRSR_SPMAD_SHIFT                 10
#define PCP_RB_CPUX_DBG_EDPRSR_SPMAD_MASK                  0x00000400U
#define PCP_RB_CPUX_DBG_EDPRSR_SPMAD_RD(src)               (((src) & 0x00000400U)>>10)
#define PCP_RB_CPUX_DBG_EDPRSR_SPMAD_SET(dst,src)          (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CPUX_DBG_EDPRSR_EPMAD_WIDTH                 1
#define PCP_RB_CPUX_DBG_EDPRSR_EPMAD_SHIFT                 9
#define PCP_RB_CPUX_DBG_EDPRSR_EPMAD_MASK                  0x00000200U
#define PCP_RB_CPUX_DBG_EDPRSR_EPMAD_RD(src)               (((src) & 0x00000200U)>>9)
#define PCP_RB_CPUX_DBG_EDPRSR_EPMAD_SET(dst,src)          (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CPUX_DBG_EDPRSR_SDAD_WIDTH                  1
#define PCP_RB_CPUX_DBG_EDPRSR_SDAD_SHIFT                  8
#define PCP_RB_CPUX_DBG_EDPRSR_SDAD_MASK                   0x00000100U
#define PCP_RB_CPUX_DBG_EDPRSR_SDAD_RD(src)                (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_DBG_EDPRSR_SDAD_SET(dst,src)           (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_DBG_EDPRSR_EDAD_WIDTH                  1
#define PCP_RB_CPUX_DBG_EDPRSR_EDAD_SHIFT                  7
#define PCP_RB_CPUX_DBG_EDPRSR_EDAD_MASK                   0x00000080U
#define PCP_RB_CPUX_DBG_EDPRSR_EDAD_RD(src)                (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_DBG_EDPRSR_EDAD_SET(dst,src)           (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_DBG_EDPRSR_DLK_WIDTH                   1
#define PCP_RB_CPUX_DBG_EDPRSR_DLK_SHIFT                   6
#define PCP_RB_CPUX_DBG_EDPRSR_DLK_MASK                    0x00000040U
#define PCP_RB_CPUX_DBG_EDPRSR_DLK_RD(src)                 (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_DBG_EDPRSR_DLK_SET(dst,src)            (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_DBG_EDPRSR_OSLK_WIDTH                  1
#define PCP_RB_CPUX_DBG_EDPRSR_OSLK_SHIFT                  5
#define PCP_RB_CPUX_DBG_EDPRSR_OSLK_MASK                   0x00000020U
#define PCP_RB_CPUX_DBG_EDPRSR_OSLK_RD(src)                (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_DBG_EDPRSR_OSLK_SET(dst,src)           (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_DBG_EDPRSR_HALTED_WIDTH                1
#define PCP_RB_CPUX_DBG_EDPRSR_HALTED_SHIFT                4
#define PCP_RB_CPUX_DBG_EDPRSR_HALTED_MASK                 0x00000010U
#define PCP_RB_CPUX_DBG_EDPRSR_HALTED_RD(src)              (((src) & 0x00000010U)>>4)
#define PCP_RB_CPUX_DBG_EDPRSR_HALTED_SET(dst,src)         (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CPUX_DBG_EDPRSR_SR_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDPRSR_SR_SHIFT                    3
#define PCP_RB_CPUX_DBG_EDPRSR_SR_MASK                     0x00000008U
#define PCP_RB_CPUX_DBG_EDPRSR_SR_RD(src)                  (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_DBG_EDPRSR_SR_SET(dst,src)             (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_DBG_EDPRSR_R_WIDTH                     1
#define PCP_RB_CPUX_DBG_EDPRSR_R_SHIFT                     2
#define PCP_RB_CPUX_DBG_EDPRSR_R_MASK                      0x00000004U
#define PCP_RB_CPUX_DBG_EDPRSR_R_RD(src)                   (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_DBG_EDPRSR_R_SET(dst,src)              (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_DBG_EDPRSR_SPD_WIDTH                   1
#define PCP_RB_CPUX_DBG_EDPRSR_SPD_SHIFT                   1
#define PCP_RB_CPUX_DBG_EDPRSR_SPD_MASK                    0x00000002U
#define PCP_RB_CPUX_DBG_EDPRSR_SPD_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_DBG_EDPRSR_SPD_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_DBG_EDPRSR_PU_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDPRSR_PU_SHIFT                    0
#define PCP_RB_CPUX_DBG_EDPRSR_PU_MASK                     0x00000001U
#define PCP_RB_CPUX_DBG_EDPRSR_PU_RD(src)                  (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_DBG_EDPRSR_PU_SET(dst,src)             (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_EDRCR_CSPA_WIDTH                   1
#define PCP_RB_CPUX_DBG_EDRCR_CSPA_SHIFT                   3
#define PCP_RB_CPUX_DBG_EDRCR_CSPA_MASK                    0x00000008U
#define PCP_RB_CPUX_DBG_EDRCR_CSPA_RD(src)                 (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_DBG_EDRCR_CSPA_SET(dst,src)            (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_DBG_EDRCR_CSE_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDRCR_CSE_SHIFT                    2
#define PCP_RB_CPUX_DBG_EDRCR_CSE_MASK                     0x00000004U
#define PCP_RB_CPUX_DBG_EDRCR_CSE_RD(src)                  (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_DBG_EDRCR_CSE_SET(dst,src)             (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_DBG_EDSCR_RXFULL_WIDTH                 1
#define PCP_RB_CPUX_DBG_EDSCR_RXFULL_SHIFT                 30
#define PCP_RB_CPUX_DBG_EDSCR_RXFULL_MASK                  0x40000000U
#define PCP_RB_CPUX_DBG_EDSCR_RXFULL_RD(src)               (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_DBG_EDSCR_RXFULL_SET(dst,src)          (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_DBG_EDSCR_TXFULL_WIDTH                 1
#define PCP_RB_CPUX_DBG_EDSCR_TXFULL_SHIFT                 29
#define PCP_RB_CPUX_DBG_EDSCR_TXFULL_MASK                  0x20000000U
#define PCP_RB_CPUX_DBG_EDSCR_TXFULL_RD(src)               (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_DBG_EDSCR_TXFULL_SET(dst,src)          (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_DBG_EDSCR_ITO_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_ITO_SHIFT                    28
#define PCP_RB_CPUX_DBG_EDSCR_ITO_MASK                     0x10000000U
#define PCP_RB_CPUX_DBG_EDSCR_ITO_RD(src)                  (((src) & 0x10000000U)>>28)
#define PCP_RB_CPUX_DBG_EDSCR_ITO_SET(dst,src)             (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CPUX_DBG_EDSCR_RXO_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_RXO_SHIFT                    27
#define PCP_RB_CPUX_DBG_EDSCR_RXO_MASK                     0x08000000U
#define PCP_RB_CPUX_DBG_EDSCR_RXO_RD(src)                  (((src) & 0x08000000U)>>27)
#define PCP_RB_CPUX_DBG_EDSCR_RXO_SET(dst,src)             (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CPUX_DBG_EDSCR_TXU_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_TXU_SHIFT                    26
#define PCP_RB_CPUX_DBG_EDSCR_TXU_MASK                     0x04000000U
#define PCP_RB_CPUX_DBG_EDSCR_TXU_RD(src)                  (((src) & 0x04000000U)>>26)
#define PCP_RB_CPUX_DBG_EDSCR_TXU_SET(dst,src)             (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_CPUX_DBG_EDSCR_PIPEADV_WIDTH                1
#define PCP_RB_CPUX_DBG_EDSCR_PIPEADV_SHIFT                25
#define PCP_RB_CPUX_DBG_EDSCR_PIPEADV_MASK                 0x02000000U
#define PCP_RB_CPUX_DBG_EDSCR_PIPEADV_RD(src)              (((src) & 0x02000000U)>>25)
#define PCP_RB_CPUX_DBG_EDSCR_PIPEADV_SET(dst,src)         (((dst) & ~0x02000000U) | (((u32)(src)<<25) & 0x02000000U))
#define PCP_RB_CPUX_DBG_EDSCR_ITE_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_ITE_SHIFT                    24
#define PCP_RB_CPUX_DBG_EDSCR_ITE_MASK                     0x01000000U
#define PCP_RB_CPUX_DBG_EDSCR_ITE_RD(src)                  (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_DBG_EDSCR_ITE_SET(dst,src)             (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_DBG_EDSCR_INTDIS_WIDTH                 2
#define PCP_RB_CPUX_DBG_EDSCR_INTDIS_SHIFT                 22
#define PCP_RB_CPUX_DBG_EDSCR_INTDIS_MASK                  0x00C00000U
#define PCP_RB_CPUX_DBG_EDSCR_INTDIS_RD(src)               (((src) & 0x00C00000U)>>22)
#define PCP_RB_CPUX_DBG_EDSCR_INTDIS_SET(dst,src)          (((dst) & ~0x00C00000U) | (((u32)(src)<<22) & 0x00C00000U))
#define PCP_RB_CPUX_DBG_EDSCR_TDA_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_TDA_SHIFT                    21
#define PCP_RB_CPUX_DBG_EDSCR_TDA_MASK                     0x00200000U
#define PCP_RB_CPUX_DBG_EDSCR_TDA_RD(src)                  (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_DBG_EDSCR_TDA_SET(dst,src)             (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_DBG_EDSCR_MA_WIDTH                     1
#define PCP_RB_CPUX_DBG_EDSCR_MA_SHIFT                     20
#define PCP_RB_CPUX_DBG_EDSCR_MA_MASK                      0x00100000U
#define PCP_RB_CPUX_DBG_EDSCR_MA_RD(src)                   (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_DBG_EDSCR_MA_SET(dst,src)              (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_DBG_EDSCR_NS_WIDTH                     1
#define PCP_RB_CPUX_DBG_EDSCR_NS_SHIFT                     18
#define PCP_RB_CPUX_DBG_EDSCR_NS_MASK                      0x00040000U
#define PCP_RB_CPUX_DBG_EDSCR_NS_RD(src)                   (((src) & 0x00040000U)>>18)
#define PCP_RB_CPUX_DBG_EDSCR_NS_SET(dst,src)              (((dst) & ~0x00040000U) | (((u32)(src)<<18) & 0x00040000U))
#define PCP_RB_CPUX_DBG_EDSCR_SDD_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_SDD_SHIFT                    16
#define PCP_RB_CPUX_DBG_EDSCR_SDD_MASK                     0x00010000U
#define PCP_RB_CPUX_DBG_EDSCR_SDD_RD(src)                  (((src) & 0x00010000U)>>16)
#define PCP_RB_CPUX_DBG_EDSCR_SDD_SET(dst,src)             (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CPUX_DBG_EDSCR_HDE_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_HDE_SHIFT                    14
#define PCP_RB_CPUX_DBG_EDSCR_HDE_MASK                     0x00004000U
#define PCP_RB_CPUX_DBG_EDSCR_HDE_RD(src)                  (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_DBG_EDSCR_HDE_SET(dst,src)             (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_DBG_EDSCR_RW_WIDTH                     4
#define PCP_RB_CPUX_DBG_EDSCR_RW_SHIFT                     10
#define PCP_RB_CPUX_DBG_EDSCR_RW_MASK                      0x00003C00U
#define PCP_RB_CPUX_DBG_EDSCR_RW_RD(src)                   (((src) & 0x00003C00U)>>10)
#define PCP_RB_CPUX_DBG_EDSCR_RW_SET(dst,src)              (((dst) & ~0x00003C00U) | (((u32)(src)<<10) & 0x00003C00U))
#define PCP_RB_CPUX_DBG_EDSCR_EL_WIDTH                     2
#define PCP_RB_CPUX_DBG_EDSCR_EL_SHIFT                     8
#define PCP_RB_CPUX_DBG_EDSCR_EL_MASK                      0x00000300U
#define PCP_RB_CPUX_DBG_EDSCR_EL_RD(src)                   (((src) & 0x00000300U)>>8)
#define PCP_RB_CPUX_DBG_EDSCR_EL_SET(dst,src)              (((dst) & ~0x00000300U) | (((u32)(src)<<8) & 0x00000300U))
#define PCP_RB_CPUX_DBG_EDSCR_A_WIDTH                      1
#define PCP_RB_CPUX_DBG_EDSCR_A_SHIFT                      7
#define PCP_RB_CPUX_DBG_EDSCR_A_MASK                       0x00000080U
#define PCP_RB_CPUX_DBG_EDSCR_A_RD(src)                    (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_DBG_EDSCR_A_SET(dst,src)               (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_DBG_EDSCR_ERR_WIDTH                    1
#define PCP_RB_CPUX_DBG_EDSCR_ERR_SHIFT                    6
#define PCP_RB_CPUX_DBG_EDSCR_ERR_MASK                     0x00000040U
#define PCP_RB_CPUX_DBG_EDSCR_ERR_RD(src)                  (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_DBG_EDSCR_ERR_SET(dst,src)             (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_DBG_EDSCR_STATUS_WIDTH                 6
#define PCP_RB_CPUX_DBG_EDSCR_STATUS_SHIFT                 0
#define PCP_RB_CPUX_DBG_EDSCR_STATUS_MASK                  0x0000003FU
#define PCP_RB_CPUX_DBG_EDSCR_STATUS_RD(src)               (((src) & 0x0000003FU)>>0)
#define PCP_RB_CPUX_DBG_EDSCR_STATUS_SET(dst,src)          (((dst) & ~0x0000003FU) | (((u32)(src)<<0) & 0x0000003FU))
#define PCP_RB_CPUX_DBG_EDWARHI_BITS31_0_WIDTH             32
#define PCP_RB_CPUX_DBG_EDWARHI_BITS31_0_SHIFT             0
#define PCP_RB_CPUX_DBG_EDWARHI_BITS31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_DBG_EDWARHI_BITS31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_EDWARHI_BITS31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_DBG_EDWARLO_BITS31_0_WIDTH             32
#define PCP_RB_CPUX_DBG_EDWARLO_BITS31_0_SHIFT             0
#define PCP_RB_CPUX_DBG_EDWARLO_BITS31_0_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_DBG_EDWARLO_BITS31_0_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_EDWARLO_BITS31_0_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_EXTEVENT_STOPCOUNT_WIDTH          12
#define PCP_RB_CSW_TRACE_EXTEVENT_STOPCOUNT_SHIFT          0
#define PCP_RB_CSW_TRACE_EXTEVENT_STOPCOUNT_MASK           0x00000FFFU
#define PCP_RB_CSW_TRACE_EXTEVENT_STOPCOUNT_RD(src)        (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_EXTEVENT_STOPCOUNT_SET(dst,src)   (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_CACHERREN_WIDTH      1
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_CACHERREN_SHIFT      9
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_CACHERREN_MASK       0x00000200U
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_CACHERREN_RD(src)    (((src) & 0x00000200U)>>9)
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_CACHERREN_SET(dst,src) (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_ITBERREN_WIDTH       1
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_ITBERREN_SHIFT       8
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_ITBERREN_MASK        0x00000100U
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_ITBERREN_RD(src)     (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_ITBERREN_SET(dst,src) (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_INTRENC_WIDTH        1
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_INTRENC_SHIFT        0
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_INTRENC_MASK         0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_INTRENC_RD(src)      (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_ICFECR_INTRENC_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRWAY_WIDTH         8
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRWAY_SHIFT         24
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRWAY_MASK          0xFF000000U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRWAY_RD(src)       (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRWAY_SET(dst,src)  (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINDEX_WIDTH       6
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINDEX_SHIFT       16
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINDEX_MASK        0x003F0000U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINDEX_RD(src)     (((src) & 0x003F0000U)>>16)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINDEX_SET(dst,src) (((dst) & ~0x003F0000U) | (((u32)(src)<<16) & 0x003F0000U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINFO_WIDTH        8
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINFO_SHIFT        8
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINFO_MASK         0x0000FF00U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINFO_RD(src)      (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRINFO_SET(dst,src) (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRTYPE_WIDTH        3
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRTYPE_SHIFT        4
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRTYPE_MASK         0x00000070U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRTYPE_RD(src)      (((src) & 0x00000070U)>>4)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_ERRTYPE_SET(dst,src) (((dst) & ~0x00000070U) | (((u32)(src)<<4) & 0x00000070U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_MULTCERR_WIDTH       1
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_MULTCERR_SHIFT       2
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_MULTCERR_MASK        0x00000004U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_MULTCERR_RD(src)     (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_MULTCERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_CERR_WIDTH           1
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_CERR_SHIFT           0
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_CERR_MASK            0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_CERR_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESR_CERR_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRWAY_WIDTH        8
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRWAY_SHIFT        24
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRWAY_MASK         0xFF000000U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRWAY_RD(src)      (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRWAY_SET(dst,src) (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINDEX_WIDTH      6
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINDEX_SHIFT      16
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINDEX_MASK       0x003F0000U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINDEX_RD(src)    (((src) & 0x003F0000U)>>16)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINDEX_SET(dst,src) (((dst) & ~0x003F0000U) | (((u32)(src)<<16) & 0x003F0000U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINFO_WIDTH       8
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINFO_SHIFT       8
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINFO_MASK        0x0000FF00U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINFO_RD(src)     (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRINFO_SET(dst,src) (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRTYPE_WIDTH       3
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRTYPE_SHIFT       4
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRTYPE_MASK        0x00000070U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRTYPE_RD(src)     (((src) & 0x00000070U)>>4)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_ERRTYPE_SET(dst,src) (((dst) & ~0x00000070U) | (((u32)(src)<<4) & 0x00000070U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_MULTCERR_WIDTH      1
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_MULTCERR_SHIFT      2
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_MULTCERR_MASK       0x00000004U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_MULTCERR_RD(src)    (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_MULTCERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_CERR_WIDTH          1
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_CERR_SHIFT          0
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_CERR_MASK           0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_CERR_RD(src)        (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_ICFESRA_CERR_SET(dst,src)   (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT63_32_WIDTH     32
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT63_32_SHIFT     32
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT63_32_MASK      0xFFFFFFFF00000000U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT63_32_RD(src)   (((src) & 0xFFFFFFFF00000000U)>>32)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT63_32_SET(dst,src) (((dst) & ~0xFFFFFFFF00000000U) | (((u64)(src)<<32) & 0xFFFFFFFF00000000U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_CTX_CMPS_WIDTH     4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_CTX_CMPS_SHIFT     28
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_CTX_CMPS_MASK      0x00000000F0000000U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_CTX_CMPS_RD(src)   (((src) & 0x00000000F0000000U)>>28)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_CTX_CMPS_SET(dst,src) (((dst) & ~0x00000000F0000000U) | (((u64)(src)<<28) & 0x00000000F0000000U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT27_24_WIDTH     4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT27_24_SHIFT     24
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT27_24_MASK      0x000000000F000000U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT27_24_RD(src)   (((src) & 0x000000000F000000U)>>24)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT27_24_SET(dst,src) (((dst) & ~0x000000000F000000U) | (((u64)(src)<<24) & 0x000000000F000000U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_WRPS_WIDTH         4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_WRPS_SHIFT         20
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_WRPS_MASK          0x0000000000F00000U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_WRPS_RD(src)       (((src) & 0x0000000000F00000U)>>20)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_WRPS_SET(dst,src)  (((dst) & ~0x0000000000F00000U) | (((u64)(src)<<20) & 0x0000000000F00000U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT19_16_WIDTH     4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT19_16_SHIFT     16
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT19_16_MASK      0x00000000000F0000U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT19_16_RD(src)   (((src) & 0x00000000000F0000U)>>16)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BIT19_16_SET(dst,src) (((dst) & ~0x00000000000F0000U) | (((u64)(src)<<16) & 0x00000000000F0000U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BRPS_WIDTH         4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BRPS_SHIFT         12
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BRPS_MASK          0x000000000000F000U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BRPS_RD(src)       (((src) & 0x000000000000F000U)>>12)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_BRPS_SET(dst,src)  (((dst) & ~0x000000000000F000U) | (((u64)(src)<<12) & 0x000000000000F000U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_PME_WIDTH          4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_PME_SHIFT          8
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_PME_MASK           0x0000000000000F00U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_PME_RD(src)        (((src) & 0x0000000000000F00U)>>8)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_PME_SET(dst,src)   (((dst) & ~0x0000000000000F00U) | (((u64)(src)<<8) & 0x0000000000000F00U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_TA_WIDTH           4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_TA_SHIFT           4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_TA_MASK            0x00000000000000F0U
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_TA_RD(src)         (((src) & 0x00000000000000F0U)>>4)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_TA_SET(dst,src)    (((dst) & ~0x00000000000000F0U) | (((u64)(src)<<4) & 0x00000000000000F0U))
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_DA_WIDTH           4
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_DA_SHIFT           0
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_DA_MASK            0x000000000000000FU
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_DA_RD(src)         (((src) & 0x000000000000000FU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64DFR0_EL1_DA_SET(dst,src)    (((dst) & ~0x000000000000000FU) | (((u64)(src)<<0) & 0x000000000000000FU))
#define PCP_RB_CPUX_DBG_ID_AA64DFR1_EL1_BIT63_0_WIDTH      64
#define PCP_RB_CPUX_DBG_ID_AA64DFR1_EL1_BIT63_0_SHIFT      0
#define PCP_RB_CPUX_DBG_ID_AA64DFR1_EL1_BIT63_0_MASK       0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_ID_AA64DFR1_EL1_BIT63_0_RD(src)    (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64DFR1_EL1_BIT63_0_SET(dst,src) (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_ID_AA64ISAR0_EL1_BIT63_0_WIDTH     64
#define PCP_RB_CPUX_DBG_ID_AA64ISAR0_EL1_BIT63_0_SHIFT     0
#define PCP_RB_CPUX_DBG_ID_AA64ISAR0_EL1_BIT63_0_MASK      0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_ID_AA64ISAR0_EL1_BIT63_0_RD(src)   (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64ISAR0_EL1_BIT63_0_SET(dst,src) (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_ID_AA64ISAR1_EL1_BIT63_0_WIDTH     64
#define PCP_RB_CPUX_DBG_ID_AA64ISAR1_EL1_BIT63_0_SHIFT     0
#define PCP_RB_CPUX_DBG_ID_AA64ISAR1_EL1_BIT63_0_MASK      0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_ID_AA64ISAR1_EL1_BIT63_0_RD(src)   (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64ISAR1_EL1_BIT63_0_SET(dst,src) (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_ID_AA64MMFR0_EL1_BIT63_0_WIDTH     64
#define PCP_RB_CPUX_DBG_ID_AA64MMFR0_EL1_BIT63_0_SHIFT     0
#define PCP_RB_CPUX_DBG_ID_AA64MMFR0_EL1_BIT63_0_MASK      0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_ID_AA64MMFR0_EL1_BIT63_0_RD(src)   (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64MMFR0_EL1_BIT63_0_SET(dst,src) (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_ID_AA64MMFR1_EL1_BIT63_0_WIDTH     64
#define PCP_RB_CPUX_DBG_ID_AA64MMFR1_EL1_BIT63_0_SHIFT     0
#define PCP_RB_CPUX_DBG_ID_AA64MMFR1_EL1_BIT63_0_MASK      0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_ID_AA64MMFR1_EL1_BIT63_0_RD(src)   (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64MMFR1_EL1_BIT63_0_SET(dst,src) (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_ID_AA64PFR0_EL1_BIT63_0_WIDTH      64
#define PCP_RB_CPUX_DBG_ID_AA64PFR0_EL1_BIT63_0_SHIFT      0
#define PCP_RB_CPUX_DBG_ID_AA64PFR0_EL1_BIT63_0_MASK       0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_ID_AA64PFR0_EL1_BIT63_0_RD(src)    (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64PFR0_EL1_BIT63_0_SET(dst,src) (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_DBG_ID_AA64PFR1_EL1_BIT63_0_WIDTH      64
#define PCP_RB_CPUX_DBG_ID_AA64PFR1_EL1_BIT63_0_SHIFT      0
#define PCP_RB_CPUX_DBG_ID_AA64PFR1_EL1_BIT63_0_MASK       0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_DBG_ID_AA64PFR1_EL1_BIT63_0_RD(src)    (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_DBG_ID_AA64PFR1_EL1_BIT63_0_SET(dst,src) (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u64)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_MILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_MILLEGALACCESS_SHIFT 1
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_MILLEGALACCESS_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_MILLEGALACCESS_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_MILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_ILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_ILLEGALACCESS_SHIFT 0
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_ILLEGALACCESS_MASK 0x00000001U
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_ILLEGALACCESS_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTMSK_ILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_MILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_MILLEGALACCESS_SHIFT 1
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_MILLEGALACCESS_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_MILLEGALACCESS_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_MILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_ILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_ILLEGALACCESS_SHIFT 0
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_ILLEGALACCESS_MASK 0x00000001U
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_ILLEGALACCESS_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBAXIS0TRANSERRINTSTS_ILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_MILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_MILLEGALACCESS_SHIFT 1
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_MILLEGALACCESS_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_MILLEGALACCESS_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_MILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_ILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_ILLEGALACCESS_SHIFT 0
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_ILLEGALACCESS_MASK 0x00000001U
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_ILLEGALACCESS_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTMSK_ILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_MILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_MILLEGALACCESS_SHIFT 1
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_MILLEGALACCESS_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_MILLEGALACCESS_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_MILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_ILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_ILLEGALACCESS_SHIFT 0
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_ILLEGALACCESS_MASK 0x00000001U
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_ILLEGALACCESS_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBAXIS1TRANSERRINTSTS_ILLEGALACCESS_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_IOBBACFG_MAXREQ_WIDTH               2
#define PCP_RB_IOB_CFG_IOBBACFG_MAXREQ_SHIFT               0
#define PCP_RB_IOB_CFG_IOBBACFG_MAXREQ_MASK                0x00000003U
#define PCP_RB_IOB_CFG_IOBBACFG_MAXREQ_RD(src)             (((src) & 0x00000003U)>>0)
#define PCP_RB_IOB_CFG_IOBBACFG_MAXREQ_SET(dst,src)        (((dst) & ~0x00000003U) | (((u32)(src)<<0) & 0x00000003U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MILLEGALACCESS_SHIFT 15
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MILLEGALACCESS_MASK 0x00008000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MILLEGALACCESS_RD(src) (((src) & 0x00008000U)>>15)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MILLEGALACCESS_SET(dst,src) (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_ILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_ILLEGALACCESS_SHIFT 14
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_ILLEGALACCESS_MASK 0x00004000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_ILLEGALACCESS_RD(src) (((src) & 0x00004000U)>>14)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_ILLEGALACCESS_SET(dst,src) (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWIDRAMCORRUPT_SHIFT 13
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWIDRAMCORRUPT_MASK 0x00002000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWIDRAMCORRUPT_RD(src) (((src) & 0x00002000U)>>13)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WIDRAMCORRUPT_SHIFT 12
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WIDRAMCORRUPT_MASK 0x00001000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WIDRAMCORRUPT_RD(src) (((src) & 0x00001000U)>>12)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRIDRAMCORRUPT_SHIFT 11
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRIDRAMCORRUPT_MASK 0x00000800U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRIDRAMCORRUPT_RD(src) (((src) & 0x00000800U)>>11)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RIDRAMCORRUPT_SHIFT 10
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RIDRAMCORRUPT_MASK 0x00000400U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RIDRAMCORRUPT_RD(src) (((src) & 0x00000400U)>>10)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MTRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MTRANSCORRUPT_SHIFT 9
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MTRANSCORRUPT_MASK 0x00000200U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MTRANSCORRUPT_RD(src) (((src) & 0x00000200U)>>9)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MTRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_TRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_TRANSCORRUPT_SHIFT 8
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_TRANSCORRUPT_MASK 0x00000100U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_TRANSCORRUPT_RD(src) (((src) & 0x00000100U)>>8)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_TRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWDATACORRUPT_SHIFT 7
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWDATACORRUPT_MASK 0x00000080U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWDATACORRUPT_RD(src) (((src) & 0x00000080U)>>7)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWDATACORRUPT_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WDATACORRUPT_SHIFT 6
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WDATACORRUPT_MASK 0x00000040U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WDATACORRUPT_RD(src) (((src) & 0x00000040U)>>6)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WDATACORRUPT_SET(dst,src) (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRBMPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRBMPOISONEDREQ_SHIFT 5
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRBMPOISONEDREQ_MASK 0x00000020U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRBMPOISONEDREQ_RD(src) (((src) & 0x00000020U)>>5)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MRBMPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RBMPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RBMPOISONEDREQ_SHIFT 4
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RBMPOISONEDREQ_MASK 0x00000010U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RBMPOISONEDREQ_RD(src) (((src) & 0x00000010U)>>4)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_RBMPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MXGICPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MXGICPOISONEDREQ_SHIFT 3
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MXGICPOISONEDREQ_MASK 0x00000008U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MXGICPOISONEDREQ_RD(src) (((src) & 0x00000008U)>>3)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MXGICPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_XGICPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_XGICPOISONEDREQ_SHIFT 2
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_XGICPOISONEDREQ_MASK 0x00000004U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_XGICPOISONEDREQ_RD(src) (((src) & 0x00000004U)>>2)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_XGICPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWRERRRESP_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWRERRRESP_SHIFT 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWRERRRESP_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWRERRRESP_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_MWRERRRESP_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WRERRRESP_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WRERRRESP_SHIFT 0
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WRERRRESP_MASK  0x00000001U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WRERRRESP_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTMSK_WRERRRESP_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MILLEGALACCESS_SHIFT 15
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MILLEGALACCESS_MASK 0x00008000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MILLEGALACCESS_RD(src) (((src) & 0x00008000U)>>15)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MILLEGALACCESS_SET(dst,src) (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_ILLEGALACCESS_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_ILLEGALACCESS_SHIFT 14
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_ILLEGALACCESS_MASK 0x00004000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_ILLEGALACCESS_RD(src) (((src) & 0x00004000U)>>14)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_ILLEGALACCESS_SET(dst,src) (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWIDRAMCORRUPT_SHIFT 13
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWIDRAMCORRUPT_MASK 0x00002000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWIDRAMCORRUPT_RD(src) (((src) & 0x00002000U)>>13)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WIDRAMCORRUPT_SHIFT 12
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WIDRAMCORRUPT_MASK 0x00001000U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WIDRAMCORRUPT_RD(src) (((src) & 0x00001000U)>>12)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRIDRAMCORRUPT_SHIFT 11
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRIDRAMCORRUPT_MASK 0x00000800U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRIDRAMCORRUPT_RD(src) (((src) & 0x00000800U)>>11)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RIDRAMCORRUPT_SHIFT 10
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RIDRAMCORRUPT_MASK 0x00000400U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RIDRAMCORRUPT_RD(src) (((src) & 0x00000400U)>>10)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MTRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MTRANSCORRUPT_SHIFT 9
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MTRANSCORRUPT_MASK 0x00000200U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MTRANSCORRUPT_RD(src) (((src) & 0x00000200U)>>9)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MTRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_TRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_TRANSCORRUPT_SHIFT 8
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_TRANSCORRUPT_MASK 0x00000100U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_TRANSCORRUPT_RD(src) (((src) & 0x00000100U)>>8)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_TRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWDATACORRUPT_SHIFT 7
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWDATACORRUPT_MASK 0x00000080U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWDATACORRUPT_RD(src) (((src) & 0x00000080U)>>7)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWDATACORRUPT_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WDATACORRUPT_SHIFT 6
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WDATACORRUPT_MASK 0x00000040U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WDATACORRUPT_RD(src) (((src) & 0x00000040U)>>6)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WDATACORRUPT_SET(dst,src) (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRBMPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRBMPOISONEDREQ_SHIFT 5
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRBMPOISONEDREQ_MASK 0x00000020U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRBMPOISONEDREQ_RD(src) (((src) & 0x00000020U)>>5)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MRBMPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RBMPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RBMPOISONEDREQ_SHIFT 4
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RBMPOISONEDREQ_MASK 0x00000010U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RBMPOISONEDREQ_RD(src) (((src) & 0x00000010U)>>4)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_RBMPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MXGICPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MXGICPOISONEDREQ_SHIFT 3
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MXGICPOISONEDREQ_MASK 0x00000008U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MXGICPOISONEDREQ_RD(src) (((src) & 0x00000008U)>>3)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MXGICPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_XGICPOISONEDREQ_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_XGICPOISONEDREQ_SHIFT 2
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_XGICPOISONEDREQ_MASK 0x00000004U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_XGICPOISONEDREQ_RD(src) (((src) & 0x00000004U)>>2)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_XGICPOISONEDREQ_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWRERRRESP_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWRERRRESP_SHIFT 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWRERRRESP_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWRERRRESP_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_MWRERRRESP_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WRERRRESP_WIDTH 1
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WRERRRESP_SHIFT 0
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WRERRRESP_MASK  0x00000001U
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WRERRRESP_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBBATRANSERRINTSTS_WRERRRESP_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMB_WIDTH                 2
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMB_SHIFT                 6
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMB_MASK                  0x000000C0U
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMB_RD(src)               (((src) & 0x000000C0U)>>6)
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMB_SET(dst,src)          (((dst) & ~0x000000C0U) | (((u32)(src)<<6) & 0x000000C0U))
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEB_WIDTH                1
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEB_SHIFT                4
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEB_MASK                 0x00000010U
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEB_RD(src)              (((src) & 0x00000010U)>>4)
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEB_SET(dst,src)         (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMA_WIDTH                 2
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMA_SHIFT                 2
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMA_MASK                  0x0000000CU
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMA_RD(src)               (((src) & 0x0000000CU)>>2)
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMA_SET(dst,src)          (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEA_WIDTH                1
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEA_SHIFT                0
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEA_MASK                 0x00000001U
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEA_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_CFG_IOBMEMCFG_RMEA_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_IOBPACFG_COS2CREDITCFG_WIDTH        5
#define PCP_RB_IOB_CFG_IOBPACFG_COS2CREDITCFG_SHIFT        24
#define PCP_RB_IOB_CFG_IOBPACFG_COS2CREDITCFG_MASK         0x1F000000U
#define PCP_RB_IOB_CFG_IOBPACFG_COS2CREDITCFG_RD(src)      (((src) & 0x1F000000U)>>24)
#define PCP_RB_IOB_CFG_IOBPACFG_COS2CREDITCFG_SET(dst,src) (((dst) & ~0x1F000000U) | (((u32)(src)<<24) & 0x1F000000U))
#define PCP_RB_IOB_CFG_IOBPACFG_COS1CREDITCFG_WIDTH        5
#define PCP_RB_IOB_CFG_IOBPACFG_COS1CREDITCFG_SHIFT        16
#define PCP_RB_IOB_CFG_IOBPACFG_COS1CREDITCFG_MASK         0x001F0000U
#define PCP_RB_IOB_CFG_IOBPACFG_COS1CREDITCFG_RD(src)      (((src) & 0x001F0000U)>>16)
#define PCP_RB_IOB_CFG_IOBPACFG_COS1CREDITCFG_SET(dst,src) (((dst) & ~0x001F0000U) | (((u32)(src)<<16) & 0x001F0000U))
#define PCP_RB_IOB_CFG_IOBPACFG_COS0CREDITCFG_WIDTH        5
#define PCP_RB_IOB_CFG_IOBPACFG_COS0CREDITCFG_SHIFT        8
#define PCP_RB_IOB_CFG_IOBPACFG_COS0CREDITCFG_MASK         0x00001F00U
#define PCP_RB_IOB_CFG_IOBPACFG_COS0CREDITCFG_RD(src)      (((src) & 0x00001F00U)>>8)
#define PCP_RB_IOB_CFG_IOBPACFG_COS0CREDITCFG_SET(dst,src) (((dst) & ~0x00001F00U) | (((u32)(src)<<8) & 0x00001F00U))
#define PCP_RB_IOB_CFG_IOBPACFG_MAXREQ_WIDTH               3
#define PCP_RB_IOB_CFG_IOBPACFG_MAXREQ_SHIFT               0
#define PCP_RB_IOB_CFG_IOBPACFG_MAXREQ_MASK                0x00000007U
#define PCP_RB_IOB_CFG_IOBPACFG_MAXREQ_RD(src)             (((src) & 0x00000007U)>>0)
#define PCP_RB_IOB_CFG_IOBPACFG_MAXREQ_SET(dst,src)        (((dst) & ~0x00000007U) | (((u32)(src)<<0) & 0x00000007U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MREQIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MREQIDRAMCORRUPT_SHIFT 7
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MREQIDRAMCORRUPT_MASK 0x00000080U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MREQIDRAMCORRUPT_RD(src) (((src) & 0x00000080U)>>7)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MREQIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_REQIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_REQIDRAMCORRUPT_SHIFT 6
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_REQIDRAMCORRUPT_MASK 0x00000040U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_REQIDRAMCORRUPT_RD(src) (((src) & 0x00000040U)>>6)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_REQIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MTRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MTRANSCORRUPT_SHIFT 5
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MTRANSCORRUPT_MASK 0x00000020U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MTRANSCORRUPT_RD(src) (((src) & 0x00000020U)>>5)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MTRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_TRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_TRANSCORRUPT_SHIFT 4
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_TRANSCORRUPT_MASK 0x00000010U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_TRANSCORRUPT_RD(src) (((src) & 0x00000010U)>>4)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_TRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MWDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MWDATACORRUPT_SHIFT 3
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MWDATACORRUPT_MASK 0x00000008U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MWDATACORRUPT_RD(src) (((src) & 0x00000008U)>>3)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MWDATACORRUPT_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_WDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_WDATACORRUPT_SHIFT 2
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_WDATACORRUPT_MASK 0x00000004U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_WDATACORRUPT_RD(src) (((src) & 0x00000004U)>>2)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_WDATACORRUPT_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MRDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MRDATACORRUPT_SHIFT 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MRDATACORRUPT_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MRDATACORRUPT_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_MRDATACORRUPT_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_RDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_RDATACORRUPT_SHIFT 0
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_RDATACORRUPT_MASK 0x00000001U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_RDATACORRUPT_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTMSK_RDATACORRUPT_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MREQIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MREQIDRAMCORRUPT_SHIFT 7
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MREQIDRAMCORRUPT_MASK 0x00000080U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MREQIDRAMCORRUPT_RD(src) (((src) & 0x00000080U)>>7)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MREQIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_REQIDRAMCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_REQIDRAMCORRUPT_SHIFT 6
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_REQIDRAMCORRUPT_MASK 0x00000040U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_REQIDRAMCORRUPT_RD(src) (((src) & 0x00000040U)>>6)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_REQIDRAMCORRUPT_SET(dst,src) (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MTRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MTRANSCORRUPT_SHIFT 5
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MTRANSCORRUPT_MASK 0x00000020U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MTRANSCORRUPT_RD(src) (((src) & 0x00000020U)>>5)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MTRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_TRANSCORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_TRANSCORRUPT_SHIFT 4
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_TRANSCORRUPT_MASK 0x00000010U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_TRANSCORRUPT_RD(src) (((src) & 0x00000010U)>>4)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_TRANSCORRUPT_SET(dst,src) (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MWDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MWDATACORRUPT_SHIFT 3
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MWDATACORRUPT_MASK 0x00000008U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MWDATACORRUPT_RD(src) (((src) & 0x00000008U)>>3)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MWDATACORRUPT_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_WDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_WDATACORRUPT_SHIFT 2
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_WDATACORRUPT_MASK 0x00000004U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_WDATACORRUPT_RD(src) (((src) & 0x00000004U)>>2)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_WDATACORRUPT_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MRDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MRDATACORRUPT_SHIFT 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MRDATACORRUPT_MASK 0x00000002U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MRDATACORRUPT_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_MRDATACORRUPT_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_RDATACORRUPT_WIDTH 1
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_RDATACORRUPT_SHIFT 0
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_RDATACORRUPT_MASK 0x00000001U
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_RDATACORRUPT_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_IOBPATRANSERRINTSTS_RDATACORRUPT_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_IOBREMAPBAR_BASEADDRESS_WIDTH       22
#define PCP_RB_IOB_CFG_IOBREMAPBAR_BASEADDRESS_SHIFT       0
#define PCP_RB_IOB_CFG_IOBREMAPBAR_BASEADDRESS_MASK        0x003FFFFFU
#define PCP_RB_IOB_CFG_IOBREMAPBAR_BASEADDRESS_RD(src)     (((src) & 0x003FFFFFU)>>0)
#define PCP_RB_IOB_CFG_IOBREMAPBAR_BASEADDRESS_SET(dst,src) (((dst) & ~0x003FFFFFU) | (((u32)(src)<<0) & 0x003FFFFFU))
#define PCP_RB_IOB_CFG_IOBREMAPRNG_BARRANGE_WIDTH          10
#define PCP_RB_IOB_CFG_IOBREMAPRNG_BARRANGE_SHIFT          1
#define PCP_RB_IOB_CFG_IOBREMAPRNG_BARRANGE_MASK           0x000007FEU
#define PCP_RB_IOB_CFG_IOBREMAPRNG_BARRANGE_RD(src)        (((src) & 0x000007FEU)>>1)
#define PCP_RB_IOB_CFG_IOBREMAPRNG_BARRANGE_SET(dst,src)   (((dst) & ~0x000007FEU) | (((u32)(src)<<1) & 0x000007FEU))
#define PCP_RB_IOB_CFG_IOBREMAPRNG_REMAPEN_WIDTH           1
#define PCP_RB_IOB_CFG_IOBREMAPRNG_REMAPEN_SHIFT           0
#define PCP_RB_IOB_CFG_IOBREMAPRNG_REMAPEN_MASK            0x00000001U
#define PCP_RB_IOB_CFG_IOBREMAPRNG_REMAPEN_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_CFG_IOBREMAPRNG_REMAPEN_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_IOBREMAPTAR5_TRANSADDRESS_WIDTH     22
#define PCP_RB_IOB_CFG_IOBREMAPTAR5_TRANSADDRESS_SHIFT     0
#define PCP_RB_IOB_CFG_IOBREMAPTAR5_TRANSADDRESS_MASK      0x003FFFFFU
#define PCP_RB_IOB_CFG_IOBREMAPTAR5_TRANSADDRESS_RD(src)   (((src) & 0x003FFFFFU)>>0)
#define PCP_RB_IOB_CFG_IOBREMAPTAR5_TRANSADDRESS_SET(dst,src) (((dst) & ~0x003FFFFFU) | (((u32)(src)<<0) & 0x003FFFFFU))
#define PCP_RB_IOB_CFG_IOBTRACECFG_PADDRDBG31_WIDTH        1
#define PCP_RB_IOB_CFG_IOBTRACECFG_PADDRDBG31_SHIFT        1
#define PCP_RB_IOB_CFG_IOBTRACECFG_PADDRDBG31_MASK         0x00000002U
#define PCP_RB_IOB_CFG_IOBTRACECFG_PADDRDBG31_RD(src)      (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_CFG_IOBTRACECFG_PADDRDBG31_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_CFG_IOBTRACECFG_DISABLEATBBURST_WIDTH   1
#define PCP_RB_IOB_CFG_IOBTRACECFG_DISABLEATBBURST_SHIFT   0
#define PCP_RB_IOB_CFG_IOBTRACECFG_DISABLEATBBURST_MASK    0x00000001U
#define PCP_RB_IOB_CFG_IOBTRACECFG_DISABLEATBBURST_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_CFG_IOBTRACECFG_DISABLEATBBURST_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VCTOKENINIT_WIDTH     1
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VCTOKENINIT_SHIFT     31
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VCTOKENINIT_MASK      0x80000000U
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VCTOKENINIT_RD(src)   (((src) & 0x80000000U)>>31)
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VCTOKENINIT_SET(dst,src) (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC2TOKEN_WIDTH        9
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC2TOKEN_SHIFT        20
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC2TOKEN_MASK         0x1FF00000U
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC2TOKEN_RD(src)      (((src) & 0x1FF00000U)>>20)
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC2TOKEN_SET(dst,src) (((dst) & ~0x1FF00000U) | (((u32)(src)<<20) & 0x1FF00000U))
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC1TOKEN_WIDTH        9
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC1TOKEN_SHIFT        10
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC1TOKEN_MASK         0x0007FC00U
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC1TOKEN_RD(src)      (((src) & 0x0007FC00U)>>10)
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC1TOKEN_SET(dst,src) (((dst) & ~0x0007FC00U) | (((u32)(src)<<10) & 0x0007FC00U))
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC0TOKEN_WIDTH        9
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC0TOKEN_SHIFT        0
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC0TOKEN_MASK         0x000001FFU
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC0TOKEN_RD(src)      (((src) & 0x000001FFU)>>0)
#define PCP_RB_IOB_CFG_IOBVCTOKENCFG_VC0TOKEN_SET(dst,src) (((dst) & ~0x000001FFU) | (((u32)(src)<<0) & 0x000001FFU))
#define PCP_RB_CPUX_L2C_L2CR_OOCFFRC_WIDTH                 1
#define PCP_RB_CPUX_L2C_L2CR_OOCFFRC_SHIFT                 4
#define PCP_RB_CPUX_L2C_L2CR_OOCFFRC_MASK                  0x00000010U
#define PCP_RB_CPUX_L2C_L2CR_OOCFFRC_RD(src)               (((src) & 0x00000010U)>>4)
#define PCP_RB_CPUX_L2C_L2CR_OOCFFRC_SET(dst,src)          (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CPUX_L2C_L2CR_HDPDIS_WIDTH                  1
#define PCP_RB_CPUX_L2C_L2CR_HDPDIS_SHIFT                  3
#define PCP_RB_CPUX_L2C_L2CR_HDPDIS_MASK                   0x00000008U
#define PCP_RB_CPUX_L2C_L2CR_HDPDIS_RD(src)                (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_L2C_L2CR_HDPDIS_SET(dst,src)           (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_L2C_L2CR_PIPEDIS_WIDTH                 1
#define PCP_RB_CPUX_L2C_L2CR_PIPEDIS_SHIFT                 2
#define PCP_RB_CPUX_L2C_L2CR_PIPEDIS_MASK                  0x00000004U
#define PCP_RB_CPUX_L2C_L2CR_PIPEDIS_RD(src)               (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_L2C_L2CR_PIPEDIS_SET(dst,src)          (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_L2C_L2CR_NCWRCMBDIS_WIDTH              1
#define PCP_RB_CPUX_L2C_L2CR_NCWRCMBDIS_SHIFT              1
#define PCP_RB_CPUX_L2C_L2CR_NCWRCMBDIS_MASK               0x00000002U
#define PCP_RB_CPUX_L2C_L2CR_NCWRCMBDIS_RD(src)            (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_L2C_L2CR_NCWRCMBDIS_SET(dst,src)       (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_L2C_L2CR_CWRDMBDIS_WIDTH               1
#define PCP_RB_CPUX_L2C_L2CR_CWRDMBDIS_SHIFT               0
#define PCP_RB_CPUX_L2C_L2CR_CWRDMBDIS_MASK                0x00000001U
#define PCP_RB_CPUX_L2C_L2CR_CWRDMBDIS_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_L2C_L2CR_CWRDMBDIS_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_L2C_L2EAHR_PHYSADDRHI_WIDTH     10
#define PCP_RB_CPUX_MEMERR_L2C_L2EAHR_PHYSADDRHI_SHIFT     0
#define PCP_RB_CPUX_MEMERR_L2C_L2EAHR_PHYSADDRHI_MASK      0x000003FFU
#define PCP_RB_CPUX_MEMERR_L2C_L2EAHR_PHYSADDRHI_RD(src)   (((src) & 0x000003FFU)>>0)
#define PCP_RB_CPUX_MEMERR_L2C_L2EAHR_PHYSADDRHI_SET(dst,src) (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CPUX_MEMERR_L2C_L2EALR_PHYSADDRLO_WIDTH     26
#define PCP_RB_CPUX_MEMERR_L2C_L2EALR_PHYSADDRLO_SHIFT     6
#define PCP_RB_CPUX_MEMERR_L2C_L2EALR_PHYSADDRLO_MASK      0xFFFFFFC0U
#define PCP_RB_CPUX_MEMERR_L2C_L2EALR_PHYSADDRLO_RD(src)   (((src) & 0xFFFFFFC0U)>>6)
#define PCP_RB_CPUX_MEMERR_L2C_L2EALR_PHYSADDRLO_SET(dst,src) (((dst) & ~0xFFFFFFC0U) | (((u32)(src)<<6) & 0xFFFFFFC0U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_TAGERREN_WIDTH        1
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_TAGERREN_SHIFT        10
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_TAGERREN_MASK         0x00000400U
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_TAGERREN_RD(src)      (((src) & 0x00000400U)>>10)
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_TAGERREN_SET(dst,src) (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_DATAERREN_WIDTH       1
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_DATAERREN_SHIFT       9
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_DATAERREN_MASK        0x00000200U
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_DATAERREN_RD(src)     (((src) & 0x00000200U)>>9)
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_DATAERREN_SET(dst,src) (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_SDBERREN_WIDTH        1
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_SDBERREN_SHIFT        8
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_SDBERREN_MASK         0x00000100U
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_SDBERREN_RD(src)      (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_SDBERREN_SET(dst,src) (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_UCINTREN_WIDTH        1
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_UCINTREN_SHIFT        1
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_UCINTREN_MASK         0x00000002U
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_UCINTREN_RD(src)      (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_UCINTREN_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_CINTREN_WIDTH         1
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_CINTREN_SHIFT         0
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_CINTREN_MASK          0x00000001U
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_CINTREN_RD(src)       (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_L2C_L2ECR_CINTREN_SET(dst,src)  (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRSYN_WIDTH          8
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRSYN_SHIFT          24
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRSYN_MASK           0xFF000000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRSYN_RD(src)        (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRSYN_SET(dst,src)   (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRWAY_WIDTH          6
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRWAY_SHIFT          18
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRWAY_MASK           0x00FC0000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRWAY_RD(src)        (((src) & 0x00FC0000U)>>18)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRWAY_SET(dst,src)   (((dst) & ~0x00FC0000U) | (((u32)(src)<<18) & 0x00FC0000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRCPU_WIDTH          1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRCPU_SHIFT          17
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRCPU_MASK           0x00020000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRCPU_RD(src)        (((src) & 0x00020000U)>>17)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRCPU_SET(dst,src)   (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTIGRP_WIDTH        1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTIGRP_SHIFT        16
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTIGRP_MASK         0x00010000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTIGRP_RD(src)      (((src) & 0x00010000U)>>16)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTIGRP_SET(dst,src) (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRGROUP_WIDTH        3
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRGROUP_SHIFT        13
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRGROUP_MASK         0x0000E000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRGROUP_RD(src)      (((src) & 0x0000E000U)>>13)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRGROUP_SET(dst,src) (((dst) & ~0x0000E000U) | (((u32)(src)<<13) & 0x0000E000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRACTION_WIDTH       3
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRACTION_SHIFT       10
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRACTION_MASK        0x00001C00U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRACTION_RD(src)     (((src) & 0x00001C00U)>>10)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRACTION_SET(dst,src) (((dst) & ~0x00001C00U) | (((u32)(src)<<10) & 0x00001C00U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRTYPE_WIDTH         2
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRTYPE_SHIFT         8
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRTYPE_MASK          0x00000300U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRTYPE_RD(src)       (((src) & 0x00000300U)>>8)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERRTYPE_SET(dst,src)  (((dst) & ~0x00000300U) | (((u32)(src)<<8) & 0x00000300U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTUCERR_WIDTH       1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTUCERR_SHIFT       3
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTUCERR_MASK        0x00000008U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTUCERR_RD(src)     (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTUCERR_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTICERR_WIDTH       1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTICERR_SHIFT       2
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTICERR_MASK        0x00000004U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTICERR_RD(src)     (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_MULTICERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_UCERR_WIDTH           1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_UCERR_SHIFT           1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_UCERR_MASK            0x00000002U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_UCERR_RD(src)         (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_UCERR_SET(dst,src)    (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERR_WIDTH             1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERR_SHIFT             0
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERR_MASK              0x00000001U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERR_RD(src)           (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESR_ERR_SET(dst,src)      (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRSYN_WIDTH         8
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRSYN_SHIFT         24
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRSYN_MASK          0xFF000000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRSYN_RD(src)       (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRSYN_SET(dst,src)  (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRWAY_WIDTH         6
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRWAY_SHIFT         18
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRWAY_MASK          0x00FC0000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRWAY_RD(src)       (((src) & 0x00FC0000U)>>18)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRWAY_SET(dst,src)  (((dst) & ~0x00FC0000U) | (((u32)(src)<<18) & 0x00FC0000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRCPU_WIDTH         1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRCPU_SHIFT         17
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRCPU_MASK          0x00020000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRCPU_RD(src)       (((src) & 0x00020000U)>>17)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRCPU_SET(dst,src)  (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTIGRP_WIDTH       1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTIGRP_SHIFT       16
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTIGRP_MASK        0x00010000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTIGRP_RD(src)     (((src) & 0x00010000U)>>16)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTIGRP_SET(dst,src) (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRGROUP_WIDTH       3
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRGROUP_SHIFT       13
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRGROUP_MASK        0x0000E000U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRGROUP_RD(src)     (((src) & 0x0000E000U)>>13)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRGROUP_SET(dst,src) (((dst) & ~0x0000E000U) | (((u32)(src)<<13) & 0x0000E000U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRACTION_WIDTH      3
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRACTION_SHIFT      10
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRACTION_MASK       0x00001C00U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRACTION_RD(src)    (((src) & 0x00001C00U)>>10)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRACTION_SET(dst,src) (((dst) & ~0x00001C00U) | (((u32)(src)<<10) & 0x00001C00U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRTYPE_WIDTH        2
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRTYPE_SHIFT        8
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRTYPE_MASK         0x00000300U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRTYPE_RD(src)      (((src) & 0x00000300U)>>8)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERRTYPE_SET(dst,src) (((dst) & ~0x00000300U) | (((u32)(src)<<8) & 0x00000300U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTUCERR_WIDTH      1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTUCERR_SHIFT      3
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTUCERR_MASK       0x00000008U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTUCERR_RD(src)    (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTUCERR_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTICERR_WIDTH      1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTICERR_SHIFT      2
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTICERR_MASK       0x00000004U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTICERR_RD(src)    (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_MULTICERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_UCERR_WIDTH          1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_UCERR_SHIFT          1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_UCERR_MASK           0x00000002U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_UCERR_RD(src)        (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_UCERR_SET(dst,src)   (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERR_WIDTH            1
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERR_SHIFT            0
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERR_MASK             0x00000001U
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERR_RD(src)          (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_L2C_L2ESRA_ERR_SET(dst,src)     (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_L2C_L2PE0_CNT_WIDTH                    28
#define PCP_RB_CPUX_L2C_L2PE0_CNT_SHIFT                    0
#define PCP_RB_CPUX_L2C_L2PE0_CNT_MASK                     0x0FFFFFFFU
#define PCP_RB_CPUX_L2C_L2PE0_CNT_RD(src)                  (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_L2C_L2PE0_CNT_SET(dst,src)             (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_L2C_L2PE1_CNT_WIDTH                    28
#define PCP_RB_CPUX_L2C_L2PE1_CNT_SHIFT                    0
#define PCP_RB_CPUX_L2C_L2PE1_CNT_MASK                     0x0FFFFFFFU
#define PCP_RB_CPUX_L2C_L2PE1_CNT_RD(src)                  (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_L2C_L2PE1_CNT_SET(dst,src)             (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_L2C_L2PE2_CNT_WIDTH                    28
#define PCP_RB_CPUX_L2C_L2PE2_CNT_SHIFT                    0
#define PCP_RB_CPUX_L2C_L2PE2_CNT_MASK                     0x0FFFFFFFU
#define PCP_RB_CPUX_L2C_L2PE2_CNT_RD(src)                  (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_L2C_L2PE2_CNT_SET(dst,src)             (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_L2C_L2PE3_CNT_WIDTH                    28
#define PCP_RB_CPUX_L2C_L2PE3_CNT_SHIFT                    0
#define PCP_RB_CPUX_L2C_L2PE3_CNT_MASK                     0x0FFFFFFFU
#define PCP_RB_CPUX_L2C_L2PE3_CNT_RD(src)                  (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_L2C_L2PE3_CNT_SET(dst,src)             (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_L2C_L2PE4_CNT_WIDTH                    28
#define PCP_RB_CPUX_L2C_L2PE4_CNT_SHIFT                    0
#define PCP_RB_CPUX_L2C_L2PE4_CNT_MASK                     0x0FFFFFFFU
#define PCP_RB_CPUX_L2C_L2PE4_CNT_RD(src)                  (((src) & 0x0FFFFFFFU)>>0)
#define PCP_RB_CPUX_L2C_L2PE4_CNT_SET(dst,src)             (((dst) & ~0x0FFFFFFFU) | (((u32)(src)<<0) & 0x0FFFFFFFU))
#define PCP_RB_CPUX_L2C_L2RTOAHR_PA_WIDTH                  10
#define PCP_RB_CPUX_L2C_L2RTOAHR_PA_SHIFT                  0
#define PCP_RB_CPUX_L2C_L2RTOAHR_PA_MASK                   0x000003FFU
#define PCP_RB_CPUX_L2C_L2RTOAHR_PA_RD(src)                (((src) & 0x000003FFU)>>0)
#define PCP_RB_CPUX_L2C_L2RTOAHR_PA_SET(dst,src)           (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CPUX_L2C_L2RTOALR_PA_WIDTH                  26
#define PCP_RB_CPUX_L2C_L2RTOALR_PA_SHIFT                  6
#define PCP_RB_CPUX_L2C_L2RTOALR_PA_MASK                   0xFFFFFFC0U
#define PCP_RB_CPUX_L2C_L2RTOALR_PA_RD(src)                (((src) & 0xFFFFFFC0U)>>6)
#define PCP_RB_CPUX_L2C_L2RTOALR_PA_SET(dst,src)           (((dst) & ~0xFFFFFFC0U) | (((u32)(src)<<6) & 0xFFFFFFC0U))
#define PCP_RB_CPUX_L2C_L2RTOCR_RTODIS_WIDTH               1
#define PCP_RB_CPUX_L2C_L2RTOCR_RTODIS_SHIFT               9
#define PCP_RB_CPUX_L2C_L2RTOCR_RTODIS_MASK                0x00000200U
#define PCP_RB_CPUX_L2C_L2RTOCR_RTODIS_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CPUX_L2C_L2RTOCR_RTODIS_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CPUX_L2C_L2RTOCR_RTOIRQEN_WIDTH             1
#define PCP_RB_CPUX_L2C_L2RTOCR_RTOIRQEN_SHIFT             8
#define PCP_RB_CPUX_L2C_L2RTOCR_RTOIRQEN_MASK              0x00000100U
#define PCP_RB_CPUX_L2C_L2RTOCR_RTOIRQEN_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_L2C_L2RTOCR_RTOIRQEN_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_L2C_L2RTOCR_COUNT_WIDTH                6
#define PCP_RB_CPUX_L2C_L2RTOCR_COUNT_SHIFT                0
#define PCP_RB_CPUX_L2C_L2RTOCR_COUNT_MASK                 0x0000003FU
#define PCP_RB_CPUX_L2C_L2RTOCR_COUNT_RD(src)              (((src) & 0x0000003FU)>>0)
#define PCP_RB_CPUX_L2C_L2RTOCR_COUNT_SET(dst,src)         (((dst) & ~0x0000003FU) | (((u32)(src)<<0) & 0x0000003FU))
#define PCP_RB_CPUX_L2C_L2RTOSR_DATAVLD_WIDTH              1
#define PCP_RB_CPUX_L2C_L2RTOSR_DATAVLD_SHIFT              31
#define PCP_RB_CPUX_L2C_L2RTOSR_DATAVLD_MASK               0x80000000U
#define PCP_RB_CPUX_L2C_L2RTOSR_DATAVLD_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_L2C_L2RTOSR_DATAVLD_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKLCL_WIDTH               1
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKLCL_SHIFT               30
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKLCL_MASK                0x40000000U
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKLCL_RD(src)             (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKLCL_SET(dst,src)        (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKVLD_WIDTH               1
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKVLD_SHIFT               29
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKVLD_MASK                0x20000000U
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKVLD_RD(src)             (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_L2C_L2RTOSR_ACKVLD_SET(dst,src)        (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_REQORD_WIDTH               1
#define PCP_RB_CPUX_L2C_L2RTOSR_REQORD_SHIFT               28
#define PCP_RB_CPUX_L2C_L2RTOSR_REQORD_MASK                0x10000000U
#define PCP_RB_CPUX_L2C_L2RTOSR_REQORD_RD(src)             (((src) & 0x10000000U)>>28)
#define PCP_RB_CPUX_L2C_L2RTOSR_REQORD_SET(dst,src)        (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLSTATE_WIDTH            4
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLSTATE_SHIFT            24
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLSTATE_MASK             0x0F000000U
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLSTATE_RD(src)          (((src) & 0x0F000000U)>>24)
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLSTATE_SET(dst,src)     (((dst) & ~0x0F000000U) | (((u32)(src)<<24) & 0x0F000000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_REQTYPE_WIDTH              5
#define PCP_RB_CPUX_L2C_L2RTOSR_REQTYPE_SHIFT              19
#define PCP_RB_CPUX_L2C_L2RTOSR_REQTYPE_MASK               0x00F80000U
#define PCP_RB_CPUX_L2C_L2RTOSR_REQTYPE_RD(src)            (((src) & 0x00F80000U)>>19)
#define PCP_RB_CPUX_L2C_L2RTOSR_REQTYPE_SET(dst,src)       (((dst) & ~0x00F80000U) | (((u32)(src)<<19) & 0x00F80000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_DEFFWD_WIDTH               1
#define PCP_RB_CPUX_L2C_L2RTOSR_DEFFWD_SHIFT               18
#define PCP_RB_CPUX_L2C_L2RTOSR_DEFFWD_MASK                0x00040000U
#define PCP_RB_CPUX_L2C_L2RTOSR_DEFFWD_RD(src)             (((src) & 0x00040000U)>>18)
#define PCP_RB_CPUX_L2C_L2RTOSR_DEFFWD_SET(dst,src)        (((dst) & ~0x00040000U) | (((u32)(src)<<18) & 0x00040000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLDEST_WIDTH             2
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLDEST_SHIFT             16
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLDEST_MASK              0x00030000U
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLDEST_RD(src)           (((src) & 0x00030000U)>>16)
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLDEST_SET(dst,src)      (((dst) & ~0x00030000U) | (((u32)(src)<<16) & 0x00030000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_CPU_WIDTH                  1
#define PCP_RB_CPUX_L2C_L2RTOSR_CPU_SHIFT                  15
#define PCP_RB_CPUX_L2C_L2RTOSR_CPU_MASK                   0x00008000U
#define PCP_RB_CPUX_L2C_L2RTOSR_CPU_RD(src)                (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_L2C_L2RTOSR_CPU_SET(dst,src)           (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLID_WIDTH               3
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLID_SHIFT               12
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLID_MASK                0x00007000U
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLID_RD(src)             (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_L2C_L2RTOSR_FILLID_SET(dst,src)        (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_L2C_L2RTOSR_REQID_WIDTH                6
#define PCP_RB_CPUX_L2C_L2RTOSR_REQID_SHIFT                4
#define PCP_RB_CPUX_L2C_L2RTOSR_REQID_MASK                 0x000003F0U
#define PCP_RB_CPUX_L2C_L2RTOSR_REQID_RD(src)              (((src) & 0x000003F0U)>>4)
#define PCP_RB_CPUX_L2C_L2RTOSR_REQID_SET(dst,src)         (((dst) & ~0x000003F0U) | (((u32)(src)<<4) & 0x000003F0U))
#define PCP_RB_CPUX_L2C_L2RTOSR_MULTERR_WIDTH              1
#define PCP_RB_CPUX_L2C_L2RTOSR_MULTERR_SHIFT              1
#define PCP_RB_CPUX_L2C_L2RTOSR_MULTERR_MASK               0x00000002U
#define PCP_RB_CPUX_L2C_L2RTOSR_MULTERR_RD(src)            (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_L2C_L2RTOSR_MULTERR_SET(dst,src)       (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_L2C_L2RTOSR_ERR_WIDTH                  1
#define PCP_RB_CPUX_L2C_L2RTOSR_ERR_SHIFT                  0
#define PCP_RB_CPUX_L2C_L2RTOSR_ERR_MASK                   0x00000001U
#define PCP_RB_CPUX_L2C_L2RTOSR_ERR_RD(src)                (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_L2C_L2RTOSR_ERR_SET(dst,src)           (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_L2C_L2WD0_WAYDIS_WIDTH                 32
#define PCP_RB_CPUX_L2C_L2WD0_WAYDIS_SHIFT                 0
#define PCP_RB_CPUX_L2C_L2WD0_WAYDIS_MASK                  0xFFFFFFFFU
#define PCP_RB_CPUX_L2C_L2WD0_WAYDIS_RD(src)               (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_L2C_L2WD0_WAYDIS_SET(dst,src)          (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_L2C_L2WD1_WAYDIS_WIDTH                 32
#define PCP_RB_CPUX_L2C_L2WD1_WAYDIS_SHIFT                 0
#define PCP_RB_CPUX_L2C_L2WD1_WAYDIS_MASK                  0xFFFFFFFFU
#define PCP_RB_CPUX_L2C_L2WD1_WAYDIS_RD(src)               (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_L2C_L2WD1_WAYDIS_SET(dst,src)          (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_L3ASR_DATAQUIESCEDL3C_WIDTH             1
#define PCP_RB_CSW_L3ASR_DATAQUIESCEDL3C_SHIFT             1
#define PCP_RB_CSW_L3ASR_DATAQUIESCEDL3C_MASK              0x00000002U
#define PCP_RB_CSW_L3ASR_DATAQUIESCEDL3C_RD(src)           (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_L3ASR_DATAQUIESCEDL3C_SET(dst,src)      (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_L3ASR_SELFQUIESCEDL3C_WIDTH             1
#define PCP_RB_CSW_L3ASR_SELFQUIESCEDL3C_SHIFT             0
#define PCP_RB_CSW_L3ASR_SELFQUIESCEDL3C_MASK              0x00000001U
#define PCP_RB_CSW_L3ASR_SELFQUIESCEDL3C_RD(src)           (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_L3ASR_SELFQUIESCEDL3C_SET(dst,src)      (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_L3CCR_L3CCLKMACRST_WIDTH                1
#define PCP_RB_CSW_L3CCR_L3CCLKMACRST_SHIFT                15
#define PCP_RB_CSW_L3CCR_L3CCLKMACRST_MASK                 0x00008000U
#define PCP_RB_CSW_L3CCR_L3CCLKMACRST_RD(src)              (((src) & 0x00008000U)>>15)
#define PCP_RB_CSW_L3CCR_L3CCLKMACRST_SET(dst,src)         (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CSW_L3CCR_L3CCLKMACPD_WIDTH                 1
#define PCP_RB_CSW_L3CCR_L3CCLKMACPD_SHIFT                 14
#define PCP_RB_CSW_L3CCR_L3CCLKMACPD_MASK                  0x00004000U
#define PCP_RB_CSW_L3CCR_L3CCLKMACPD_RD(src)               (((src) & 0x00004000U)>>14)
#define PCP_RB_CSW_L3CCR_L3CCLKMACPD_SET(dst,src)          (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CSW_L3CCR_L3CCLKEN_WIDTH                    1
#define PCP_RB_CSW_L3CCR_L3CCLKEN_SHIFT                    11
#define PCP_RB_CSW_L3CCR_L3CCLKEN_MASK                     0x00000800U
#define PCP_RB_CSW_L3CCR_L3CCLKEN_RD(src)                  (((src) & 0x00000800U)>>11)
#define PCP_RB_CSW_L3CCR_L3CCLKEN_SET(dst,src)             (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CSW_L3CCR_CMLNSCORREN_WIDTH                 1
#define PCP_RB_CSW_L3CCR_CMLNSCORREN_SHIFT                 5
#define PCP_RB_CSW_L3CCR_CMLNSCORREN_MASK                  0x00000020U
#define PCP_RB_CSW_L3CCR_CMLNSCORREN_RD(src)               (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_L3CCR_CMLNSCORREN_SET(dst,src)          (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_L3RCR_RESETL3C_WIDTH                    1
#define PCP_RB_CSW_L3RCR_RESETL3C_SHIFT                    0
#define PCP_RB_CSW_L3RCR_RESETL3C_MASK                     0x00000001U
#define PCP_RB_CSW_L3RCR_RESETL3C_RD(src)                  (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_L3RCR_RESETL3C_SET(dst,src)             (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_CACHERREN_WIDTH      1
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_CACHERREN_SHIFT      9
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_CACHERREN_MASK       0x00000200U
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_CACHERREN_RD(src)    (((src) & 0x00000200U)>>9)
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_CACHERREN_SET(dst,src) (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_DTBERREN_WIDTH       1
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_DTBERREN_SHIFT       8
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_DTBERREN_MASK        0x00000100U
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_DTBERREN_RD(src)     (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_DTBERREN_SET(dst,src) (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_INTRENC_WIDTH        1
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_INTRENC_SHIFT        0
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_INTRENC_MASK         0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_INTRENC_RD(src)      (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_LSUECR_INTRENC_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRWAY_WIDTH         8
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRWAY_SHIFT         24
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRWAY_MASK          0xFF000000U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRWAY_RD(src)       (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRWAY_SET(dst,src)  (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINDEX_WIDTH       6
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINDEX_SHIFT       16
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINDEX_MASK        0x003F0000U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINDEX_RD(src)     (((src) & 0x003F0000U)>>16)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINDEX_SET(dst,src) (((dst) & ~0x003F0000U) | (((u32)(src)<<16) & 0x003F0000U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINFO_WIDTH        8
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINFO_SHIFT        8
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINFO_MASK         0x0000FF00U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINFO_RD(src)      (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRINFO_SET(dst,src) (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRTYPE_WIDTH        3
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRTYPE_SHIFT        4
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRTYPE_MASK         0x00000070U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRTYPE_RD(src)      (((src) & 0x00000070U)>>4)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_ERRTYPE_SET(dst,src) (((dst) & ~0x00000070U) | (((u32)(src)<<4) & 0x00000070U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_MULTCERR_WIDTH       1
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_MULTCERR_SHIFT       2
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_MULTCERR_MASK        0x00000004U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_MULTCERR_RD(src)     (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_MULTCERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_CERR_WIDTH           1
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_CERR_SHIFT           0
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_CERR_MASK            0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_CERR_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESR_CERR_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRWAY_WIDTH        8
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRWAY_SHIFT        24
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRWAY_MASK         0xFF000000U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRWAY_RD(src)      (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRWAY_SET(dst,src) (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINDEX_WIDTH      6
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINDEX_SHIFT      16
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINDEX_MASK       0x003F0000U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINDEX_RD(src)    (((src) & 0x003F0000U)>>16)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINDEX_SET(dst,src) (((dst) & ~0x003F0000U) | (((u32)(src)<<16) & 0x003F0000U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINFO_WIDTH       8
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINFO_SHIFT       8
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINFO_MASK        0x0000FF00U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINFO_RD(src)     (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRINFO_SET(dst,src) (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRTYPE_WIDTH       3
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRTYPE_SHIFT       4
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRTYPE_MASK        0x00000070U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRTYPE_RD(src)     (((src) & 0x00000070U)>>4)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_ERRTYPE_SET(dst,src) (((dst) & ~0x00000070U) | (((u32)(src)<<4) & 0x00000070U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_MULTCERR_WIDTH      1
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_MULTCERR_SHIFT      2
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_MULTCERR_MASK       0x00000004U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_MULTCERR_RD(src)    (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_MULTCERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_CERR_WIDTH          1
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_CERR_SHIFT          0
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_CERR_MASK           0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_CERR_RD(src)        (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_LSUESRA_CERR_SET(dst,src)   (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACRST_WIDTH              1
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACRST_SHIFT              15
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACRST_MASK               0x00008000U
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACRST_RD(src)            (((src) & 0x00008000U)>>15)
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACRST_SET(dst,src)       (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACPD_WIDTH               1
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACPD_SHIFT               14
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACPD_MASK                0x00004000U
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACPD_RD(src)             (((src) & 0x00004000U)>>14)
#define PCP_RB_CSW_MCB0CCR_MCBCLKMACPD_SET(dst,src)        (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CSW_MCB0CCR_MCBCLKEN_WIDTH                  1
#define PCP_RB_CSW_MCB0CCR_MCBCLKEN_SHIFT                  11
#define PCP_RB_CSW_MCB0CCR_MCBCLKEN_MASK                   0x00000800U
#define PCP_RB_CSW_MCB0CCR_MCBCLKEN_RD(src)                (((src) & 0x00000800U)>>11)
#define PCP_RB_CSW_MCB0CCR_MCBCLKEN_SET(dst,src)           (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CSW_MCB0CCR_CMLNSCORREN_WIDTH               1
#define PCP_RB_CSW_MCB0CCR_CMLNSCORREN_SHIFT               5
#define PCP_RB_CSW_MCB0CCR_CMLNSCORREN_MASK                0x00000020U
#define PCP_RB_CSW_MCB0CCR_CMLNSCORREN_RD(src)             (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_MCB0CCR_CMLNSCORREN_SET(dst,src)        (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_MCB0RCR_RESETMCB_WIDTH                  1
#define PCP_RB_CSW_MCB0RCR_RESETMCB_SHIFT                  0
#define PCP_RB_CSW_MCB0RCR_RESETMCB_MASK                   0x00000001U
#define PCP_RB_CSW_MCB0RCR_RESETMCB_RD(src)                (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_MCB0RCR_RESETMCB_SET(dst,src)           (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACRST_WIDTH              1
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACRST_SHIFT              15
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACRST_MASK               0x00008000U
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACRST_RD(src)            (((src) & 0x00008000U)>>15)
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACRST_SET(dst,src)       (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACPD_WIDTH               1
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACPD_SHIFT               14
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACPD_MASK                0x00004000U
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACPD_RD(src)             (((src) & 0x00004000U)>>14)
#define PCP_RB_CSW_MCB1CCR_MCBCLKMACPD_SET(dst,src)        (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CSW_MCB1CCR_MCBCLKEN_WIDTH                  1
#define PCP_RB_CSW_MCB1CCR_MCBCLKEN_SHIFT                  11
#define PCP_RB_CSW_MCB1CCR_MCBCLKEN_MASK                   0x00000800U
#define PCP_RB_CSW_MCB1CCR_MCBCLKEN_RD(src)                (((src) & 0x00000800U)>>11)
#define PCP_RB_CSW_MCB1CCR_MCBCLKEN_SET(dst,src)           (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CSW_MCB1CCR_CMLNSCORREN_WIDTH               1
#define PCP_RB_CSW_MCB1CCR_CMLNSCORREN_SHIFT               5
#define PCP_RB_CSW_MCB1CCR_CMLNSCORREN_MASK                0x00000020U
#define PCP_RB_CSW_MCB1CCR_CMLNSCORREN_RD(src)             (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_MCB1CCR_CMLNSCORREN_SET(dst,src)        (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_MCB1RCR_RESETMCB_WIDTH                  1
#define PCP_RB_CSW_MCB1RCR_RESETMCB_SHIFT                  0
#define PCP_RB_CSW_MCB1RCR_RESETMCB_MASK                   0x00000001U
#define PCP_RB_CSW_MCB1RCR_RESETMCB_RD(src)                (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_MCB1RCR_RESETMCB_SET(dst,src)           (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_DBG_MIDR_EL1_IMPLEMENTER_WIDTH         8
#define PCP_RB_CPUX_DBG_MIDR_EL1_IMPLEMENTER_SHIFT         24
#define PCP_RB_CPUX_DBG_MIDR_EL1_IMPLEMENTER_MASK          0xFF000000U
#define PCP_RB_CPUX_DBG_MIDR_EL1_IMPLEMENTER_RD(src)       (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_DBG_MIDR_EL1_IMPLEMENTER_SET(dst,src)  (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_DBG_MIDR_EL1_VARIANT_WIDTH             4
#define PCP_RB_CPUX_DBG_MIDR_EL1_VARIANT_SHIFT             20
#define PCP_RB_CPUX_DBG_MIDR_EL1_VARIANT_MASK              0x00F00000U
#define PCP_RB_CPUX_DBG_MIDR_EL1_VARIANT_RD(src)           (((src) & 0x00F00000U)>>20)
#define PCP_RB_CPUX_DBG_MIDR_EL1_VARIANT_SET(dst,src)      (((dst) & ~0x00F00000U) | (((u32)(src)<<20) & 0x00F00000U))
#define PCP_RB_CPUX_DBG_MIDR_EL1_ARCHITECTURE_WIDTH        4
#define PCP_RB_CPUX_DBG_MIDR_EL1_ARCHITECTURE_SHIFT        16
#define PCP_RB_CPUX_DBG_MIDR_EL1_ARCHITECTURE_MASK         0x000F0000U
#define PCP_RB_CPUX_DBG_MIDR_EL1_ARCHITECTURE_RD(src)      (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_DBG_MIDR_EL1_ARCHITECTURE_SET(dst,src) (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_DBG_MIDR_EL1_PRIMARY_PART_NUMBER_WIDTH 12
#define PCP_RB_CPUX_DBG_MIDR_EL1_PRIMARY_PART_NUMBER_SHIFT 4
#define PCP_RB_CPUX_DBG_MIDR_EL1_PRIMARY_PART_NUMBER_MASK  0x0000FFF0U
#define PCP_RB_CPUX_DBG_MIDR_EL1_PRIMARY_PART_NUMBER_RD(src) (((src) & 0x0000FFF0U)>>4)
#define PCP_RB_CPUX_DBG_MIDR_EL1_PRIMARY_PART_NUMBER_SET(dst,src) (((dst) & ~0x0000FFF0U) | (((u32)(src)<<4) & 0x0000FFF0U))
#define PCP_RB_CPUX_DBG_MIDR_EL1_REVISION_WIDTH            4
#define PCP_RB_CPUX_DBG_MIDR_EL1_REVISION_SHIFT            0
#define PCP_RB_CPUX_DBG_MIDR_EL1_REVISION_MASK             0x0000000FU
#define PCP_RB_CPUX_DBG_MIDR_EL1_REVISION_RD(src)          (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_DBG_MIDR_EL1_REVISION_SET(dst,src)     (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_UTBERREN_WIDTH       1
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_UTBERREN_SHIFT       8
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_UTBERREN_MASK        0x00000100U
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_UTBERREN_RD(src)     (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_UTBERREN_SET(dst,src) (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_INTRENC_WIDTH        1
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_INTRENC_SHIFT        0
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_INTRENC_MASK         0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_INTRENC_RD(src)      (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_MMUECR_INTRENC_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRWAY_WIDTH         8
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRWAY_SHIFT         24
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRWAY_MASK          0xFF000000U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRWAY_RD(src)       (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRWAY_SET(dst,src)  (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINDEX_WIDTH       7
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINDEX_SHIFT       16
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINDEX_MASK        0x007F0000U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINDEX_RD(src)     (((src) & 0x007F0000U)>>16)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINDEX_SET(dst,src) (((dst) & ~0x007F0000U) | (((u32)(src)<<16) & 0x007F0000U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINFO_WIDTH        8
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINFO_SHIFT        8
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINFO_MASK         0x0000FF00U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINFO_RD(src)      (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRINFO_SET(dst,src) (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRREQSTR_WIDTH      1
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRREQSTR_SHIFT      7
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRREQSTR_MASK       0x00000080U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRREQSTR_RD(src)    (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRREQSTR_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRTYPE_WIDTH        3
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRTYPE_SHIFT        4
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRTYPE_MASK         0x00000070U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRTYPE_RD(src)      (((src) & 0x00000070U)>>4)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_ERRTYPE_SET(dst,src) (((dst) & ~0x00000070U) | (((u32)(src)<<4) & 0x00000070U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_MULTCERR_WIDTH       1
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_MULTCERR_SHIFT       2
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_MULTCERR_MASK        0x00000004U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_MULTCERR_RD(src)     (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_MULTCERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_CERR_WIDTH           1
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_CERR_SHIFT           0
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_CERR_MASK            0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_CERR_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESR_CERR_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRWAY_WIDTH        8
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRWAY_SHIFT        24
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRWAY_MASK         0xFF000000U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRWAY_RD(src)      (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRWAY_SET(dst,src) (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINDEX_WIDTH      7
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINDEX_SHIFT      16
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINDEX_MASK       0x007F0000U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINDEX_RD(src)    (((src) & 0x007F0000U)>>16)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINDEX_SET(dst,src) (((dst) & ~0x007F0000U) | (((u32)(src)<<16) & 0x007F0000U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINFO_WIDTH       8
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINFO_SHIFT       8
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINFO_MASK        0x0000FF00U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINFO_RD(src)     (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRINFO_SET(dst,src) (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRREQSTR_WIDTH     1
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRREQSTR_SHIFT     7
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRREQSTR_MASK      0x00000080U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRREQSTR_RD(src)   (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRREQSTR_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRTYPE_WIDTH       3
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRTYPE_SHIFT       4
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRTYPE_MASK        0x00000070U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRTYPE_RD(src)     (((src) & 0x00000070U)>>4)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_ERRTYPE_SET(dst,src) (((dst) & ~0x00000070U) | (((u32)(src)<<4) & 0x00000070U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_MULTCERR_WIDTH      1
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_MULTCERR_SHIFT      2
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_MULTCERR_MASK       0x00000004U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_MULTCERR_RD(src)    (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_MULTCERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_CERR_WIDTH          1
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_CERR_SHIFT          0
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_CERR_MASK           0x00000001U
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_CERR_RD(src)        (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_MEMERR_CPU_MMUESRA_CERR_SET(dst,src)   (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDPMD_WIDTH             1
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDPMD_SHIFT             10
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDPMD_MASK              0x00000400U
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDPMD_RD(src)           (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDPMD_SET(dst,src)      (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P0ASR_DATAQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P0ASR_DATAQUIESCEDL2C_SHIFT             9
#define PCP_RB_CSW_P0ASR_DATAQUIESCEDL2C_MASK              0x00000200U
#define PCP_RB_CSW_P0ASR_DATAQUIESCEDL2C_RD(src)           (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P0ASR_DATAQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDL2C_SHIFT             8
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDL2C_MASK              0x00000100U
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDL2C_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC1_WIDTH              1
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC1_SHIFT              6
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC1_MASK               0x00000040U
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC1_RD(src)            (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC1_SET(dst,src)       (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC1_WIDTH            1
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC1_SHIFT            5
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC1_MASK             0x00000020U
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC1_RD(src)          (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC1_SET(dst,src)     (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC1_WIDTH           1
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC1_SHIFT           4
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC1_MASK            0x00000010U
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC1_RD(src)         (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC1_SET(dst,src)    (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC0_WIDTH              1
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC0_SHIFT              2
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC0_MASK               0x00000004U
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC0_RD(src)            (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P0ASR_SELFQUIESCEDC0_SET(dst,src)       (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC0_WIDTH            1
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC0_SHIFT            1
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC0_MASK             0x00000002U
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC0_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P0ASR_WAITINGFORINTRC0_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC0_WIDTH           1
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC0_SHIFT           0
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC0_MASK            0x00000001U
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC0_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P0ASR_WAITINGFOREVENTC0_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P0CCR0_PMDSYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_P0CCR0_PMDSYNCCLKDIVEN_SHIFT            16
#define PCP_RB_CSW_P0CCR0_PMDSYNCCLKDIVEN_MASK             0x00010000U
#define PCP_RB_CSW_P0CCR0_PMDSYNCCLKDIVEN_RD(src)          (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P0CCR0_PMDSYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P0CCR0_PMDCLKMACRST_WIDTH               1
#define PCP_RB_CSW_P0CCR0_PMDCLKMACRST_SHIFT               15
#define PCP_RB_CSW_P0CCR0_PMDCLKMACRST_MASK                0x00008000U
#define PCP_RB_CSW_P0CCR0_PMDCLKMACRST_RD(src)             (((src) & 0x00008000U)>>15)
#define PCP_RB_CSW_P0CCR0_PMDCLKMACRST_SET(dst,src)        (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CSW_P0CCR0_PMDCLKMACPD_WIDTH                1
#define PCP_RB_CSW_P0CCR0_PMDCLKMACPD_SHIFT                14
#define PCP_RB_CSW_P0CCR0_PMDCLKMACPD_MASK                 0x00004000U
#define PCP_RB_CSW_P0CCR0_PMDCLKMACPD_RD(src)              (((src) & 0x00004000U)>>14)
#define PCP_RB_CSW_P0CCR0_PMDCLKMACPD_SET(dst,src)         (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CSW_P0CCR0_PMDDIVRATIO_WIDTH                2
#define PCP_RB_CSW_P0CCR0_PMDDIVRATIO_SHIFT                12
#define PCP_RB_CSW_P0CCR0_PMDDIVRATIO_MASK                 0x00003000U
#define PCP_RB_CSW_P0CCR0_PMDDIVRATIO_RD(src)              (((src) & 0x00003000U)>>12)
#define PCP_RB_CSW_P0CCR0_PMDDIVRATIO_SET(dst,src)         (((dst) & ~0x00003000U) | (((u32)(src)<<12) & 0x00003000U))
#define PCP_RB_CSW_P0CCR0_PMDCLKEN_WIDTH                   1
#define PCP_RB_CSW_P0CCR0_PMDCLKEN_SHIFT                   11
#define PCP_RB_CSW_P0CCR0_PMDCLKEN_MASK                    0x00000800U
#define PCP_RB_CSW_P0CCR0_PMDCLKEN_RD(src)                 (((src) & 0x00000800U)>>11)
#define PCP_RB_CSW_P0CCR0_PMDCLKEN_SET(dst,src)            (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CSW_P0CCR0_DCCHOLD_WIDTH                    1
#define PCP_RB_CSW_P0CCR0_DCCHOLD_SHIFT                    10
#define PCP_RB_CSW_P0CCR0_DCCHOLD_MASK                     0x00000400U
#define PCP_RB_CSW_P0CCR0_DCCHOLD_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P0CCR0_DCCHOLD_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P0CCR0_CLUSTERBYPEN_WIDTH               1
#define PCP_RB_CSW_P0CCR0_CLUSTERBYPEN_SHIFT               9
#define PCP_RB_CSW_P0CCR0_CLUSTERBYPEN_MASK                0x00000200U
#define PCP_RB_CSW_P0CCR0_CLUSTERBYPEN_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P0CCR0_CLUSTERBYPEN_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P0CCR0_CLUSTERCLKEN_WIDTH               1
#define PCP_RB_CSW_P0CCR0_CLUSTERCLKEN_SHIFT               8
#define PCP_RB_CSW_P0CCR0_CLUSTERCLKEN_MASK                0x00000100U
#define PCP_RB_CSW_P0CCR0_CLUSTERCLKEN_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P0CCR0_CLUSTERCLKEN_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P0CCR0_CGCC1_WIDTH                      1
#define PCP_RB_CSW_P0CCR0_CGCC1_SHIFT                      5
#define PCP_RB_CSW_P0CCR0_CGCC1_MASK                       0x00000020U
#define PCP_RB_CSW_P0CCR0_CGCC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P0CCR0_CGCC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P0CCR0_CGCF1_WIDTH                      1
#define PCP_RB_CSW_P0CCR0_CGCF1_SHIFT                      4
#define PCP_RB_CSW_P0CCR0_CGCF1_MASK                       0x00000010U
#define PCP_RB_CSW_P0CCR0_CGCF1_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P0CCR0_CGCF1_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P0CCR0_CGCC0_WIDTH                      1
#define PCP_RB_CSW_P0CCR0_CGCC0_SHIFT                      1
#define PCP_RB_CSW_P0CCR0_CGCC0_MASK                       0x00000002U
#define PCP_RB_CSW_P0CCR0_CGCC0_RD(src)                    (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P0CCR0_CGCC0_SET(dst,src)               (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P0CCR0_CGCF0_WIDTH                      1
#define PCP_RB_CSW_P0CCR0_CGCF0_SHIFT                      0
#define PCP_RB_CSW_P0CCR0_CGCF0_MASK                       0x00000001U
#define PCP_RB_CSW_P0CCR0_CGCF0_RD(src)                    (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P0CCR0_CGCF0_SET(dst,src)               (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P0CCR1_DCCSIGN_WIDTH                    1
#define PCP_RB_CSW_P0CCR1_DCCSIGN_SHIFT                    31
#define PCP_RB_CSW_P0CCR1_DCCSIGN_MASK                     0x80000000U
#define PCP_RB_CSW_P0CCR1_DCCSIGN_RD(src)                  (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_P0CCR1_DCCSIGN_SET(dst,src)             (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_P0CCR1_DCCVAL_WIDTH                     3
#define PCP_RB_CSW_P0CCR1_DCCVAL_SHIFT                     28
#define PCP_RB_CSW_P0CCR1_DCCVAL_MASK                      0x70000000U
#define PCP_RB_CSW_P0CCR1_DCCVAL_RD(src)                   (((src) & 0x70000000U)>>28)
#define PCP_RB_CSW_P0CCR1_DCCVAL_SET(dst,src)              (((dst) & ~0x70000000U) | (((u32)(src)<<28) & 0x70000000U))
#define PCP_RB_CSW_P0CCR1_CMLNSCORREN_WIDTH                1
#define PCP_RB_CSW_P0CCR1_CMLNSCORREN_SHIFT                27
#define PCP_RB_CSW_P0CCR1_CMLNSCORREN_MASK                 0x08000000U
#define PCP_RB_CSW_P0CCR1_CMLNSCORREN_RD(src)              (((src) & 0x08000000U)>>27)
#define PCP_RB_CSW_P0CCR1_CMLNSCORREN_SET(dst,src)         (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CSW_P0CCR1_SKEWTUNEL2C_WIDTH                5
#define PCP_RB_CSW_P0CCR1_SKEWTUNEL2C_SHIFT                20
#define PCP_RB_CSW_P0CCR1_SKEWTUNEL2C_MASK                 0x01F00000U
#define PCP_RB_CSW_P0CCR1_SKEWTUNEL2C_RD(src)              (((src) & 0x01F00000U)>>20)
#define PCP_RB_CSW_P0CCR1_SKEWTUNEL2C_SET(dst,src)         (((dst) & ~0x01F00000U) | (((u32)(src)<<20) & 0x01F00000U))
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC1_WIDTH                 5
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC1_SHIFT                 15
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC1_MASK                  0x000F8000U
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC1_RD(src)               (((src) & 0x000F8000U)>>15)
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC1_SET(dst,src)          (((dst) & ~0x000F8000U) | (((u32)(src)<<15) & 0x000F8000U))
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC0_WIDTH                 5
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC0_SHIFT                 10
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC0_MASK                  0x00007C00U
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC0_RD(src)               (((src) & 0x00007C00U)>>10)
#define PCP_RB_CSW_P0CCR1_SKEWTUNEC0_SET(dst,src)          (((dst) & ~0x00007C00U) | (((u32)(src)<<10) & 0x00007C00U))
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF1_WIDTH                 5
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF1_SHIFT                 5
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF1_MASK                  0x000003E0U
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF1_RD(src)               (((src) & 0x000003E0U)>>5)
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF1_SET(dst,src)          (((dst) & ~0x000003E0U) | (((u32)(src)<<5) & 0x000003E0U))
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF0_WIDTH                 5
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF0_SHIFT                 0
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF0_MASK                  0x0000001FU
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF0_RD(src)               (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_P0CCR1_SKEWTUNEF0_SET(dst,src)          (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_P0RCR_DBGENC1_WIDTH                     1
#define PCP_RB_CSW_P0RCR_DBGENC1_SHIFT                     17
#define PCP_RB_CSW_P0RCR_DBGENC1_MASK                      0x00020000U
#define PCP_RB_CSW_P0RCR_DBGENC1_RD(src)                   (((src) & 0x00020000U)>>17)
#define PCP_RB_CSW_P0RCR_DBGENC1_SET(dst,src)              (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CSW_P0RCR_DBGENC0_WIDTH                     1
#define PCP_RB_CSW_P0RCR_DBGENC0_SHIFT                     16
#define PCP_RB_CSW_P0RCR_DBGENC0_MASK                      0x00010000U
#define PCP_RB_CSW_P0RCR_DBGENC0_RD(src)                   (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P0RCR_DBGENC0_SET(dst,src)              (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P0RCR_RESETDBGC1_WIDTH                  1
#define PCP_RB_CSW_P0RCR_RESETDBGC1_SHIFT                  13
#define PCP_RB_CSW_P0RCR_RESETDBGC1_MASK                   0x00002000U
#define PCP_RB_CSW_P0RCR_RESETDBGC1_RD(src)                (((src) & 0x00002000U)>>13)
#define PCP_RB_CSW_P0RCR_RESETDBGC1_SET(dst,src)           (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CSW_P0RCR_RESETDBGC0_WIDTH                  1
#define PCP_RB_CSW_P0RCR_RESETDBGC0_SHIFT                  12
#define PCP_RB_CSW_P0RCR_RESETDBGC0_MASK                   0x00001000U
#define PCP_RB_CSW_P0RCR_RESETDBGC0_RD(src)                (((src) & 0x00001000U)>>12)
#define PCP_RB_CSW_P0RCR_RESETDBGC0_SET(dst,src)           (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_CSW_P0RCR_RESETL2C_WIDTH                    1
#define PCP_RB_CSW_P0RCR_RESETL2C_SHIFT                    10
#define PCP_RB_CSW_P0RCR_RESETL2C_MASK                     0x00000400U
#define PCP_RB_CSW_P0RCR_RESETL2C_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P0RCR_RESETL2C_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P0RCR_RESETC1_WIDTH                     1
#define PCP_RB_CSW_P0RCR_RESETC1_SHIFT                     9
#define PCP_RB_CSW_P0RCR_RESETC1_MASK                      0x00000200U
#define PCP_RB_CSW_P0RCR_RESETC1_RD(src)                   (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P0RCR_RESETC1_SET(dst,src)              (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P0RCR_RESETC0_WIDTH                     1
#define PCP_RB_CSW_P0RCR_RESETC0_SHIFT                     8
#define PCP_RB_CSW_P0RCR_RESETC0_MASK                      0x00000100U
#define PCP_RB_CSW_P0RCR_RESETC0_RD(src)                   (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P0RCR_RESETC0_SET(dst,src)              (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P0RCR_INITL2C_WIDTH                     1
#define PCP_RB_CSW_P0RCR_INITL2C_SHIFT                     6
#define PCP_RB_CSW_P0RCR_INITL2C_MASK                      0x00000040U
#define PCP_RB_CSW_P0RCR_INITL2C_RD(src)                   (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P0RCR_INITL2C_SET(dst,src)              (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P0RCR_INITC1_WIDTH                      1
#define PCP_RB_CSW_P0RCR_INITC1_SHIFT                      5
#define PCP_RB_CSW_P0RCR_INITC1_MASK                       0x00000020U
#define PCP_RB_CSW_P0RCR_INITC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P0RCR_INITC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P0RCR_INITC0_WIDTH                      1
#define PCP_RB_CSW_P0RCR_INITC0_SHIFT                      4
#define PCP_RB_CSW_P0RCR_INITC0_MASK                       0x00000010U
#define PCP_RB_CSW_P0RCR_INITC0_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P0RCR_INITC0_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P0RCR_STARTC1_WIDTH                     1
#define PCP_RB_CSW_P0RCR_STARTC1_SHIFT                     1
#define PCP_RB_CSW_P0RCR_STARTC1_MASK                      0x00000002U
#define PCP_RB_CSW_P0RCR_STARTC1_RD(src)                   (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P0RCR_STARTC1_SET(dst,src)              (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P0RCR_STARTC0_WIDTH                     1
#define PCP_RB_CSW_P0RCR_STARTC0_SHIFT                     0
#define PCP_RB_CSW_P0RCR_STARTC0_MASK                      0x00000001U
#define PCP_RB_CSW_P0RCR_STARTC0_RD(src)                   (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P0RCR_STARTC0_SET(dst,src)              (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC1_WIDTH                1
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC1_SHIFT                3
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC1_MASK                 0x00000008U
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC1_RD(src)              (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC1_SET(dst,src)         (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC0_WIDTH                1
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC0_SHIFT                2
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC0_MASK                 0x00000004U
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC0_RD(src)              (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P0RSR_CWRRACTIVEC0_SET(dst,src)         (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P0RSR_INRESETC1_WIDTH                   1
#define PCP_RB_CSW_P0RSR_INRESETC1_SHIFT                   1
#define PCP_RB_CSW_P0RSR_INRESETC1_MASK                    0x00000002U
#define PCP_RB_CSW_P0RSR_INRESETC1_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P0RSR_INRESETC1_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P0RSR_INRESETC0_WIDTH                   1
#define PCP_RB_CSW_P0RSR_INRESETC0_SHIFT                   0
#define PCP_RB_CSW_P0RSR_INRESETC0_MASK                    0x00000001U
#define PCP_RB_CSW_P0RSR_INRESETC0_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P0RSR_INRESETC0_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDPMD_WIDTH             1
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDPMD_SHIFT             10
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDPMD_MASK              0x00000400U
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDPMD_RD(src)           (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDPMD_SET(dst,src)      (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P1ASR_DATAQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P1ASR_DATAQUIESCEDL2C_SHIFT             9
#define PCP_RB_CSW_P1ASR_DATAQUIESCEDL2C_MASK              0x00000200U
#define PCP_RB_CSW_P1ASR_DATAQUIESCEDL2C_RD(src)           (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P1ASR_DATAQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDL2C_SHIFT             8
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDL2C_MASK              0x00000100U
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDL2C_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC1_WIDTH              1
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC1_SHIFT              6
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC1_MASK               0x00000040U
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC1_RD(src)            (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC1_SET(dst,src)       (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC1_WIDTH            1
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC1_SHIFT            5
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC1_MASK             0x00000020U
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC1_RD(src)          (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC1_SET(dst,src)     (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC1_WIDTH           1
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC1_SHIFT           4
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC1_MASK            0x00000010U
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC1_RD(src)         (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC1_SET(dst,src)    (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC0_WIDTH              1
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC0_SHIFT              2
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC0_MASK               0x00000004U
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC0_RD(src)            (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P1ASR_SELFQUIESCEDC0_SET(dst,src)       (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC0_WIDTH            1
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC0_SHIFT            1
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC0_MASK             0x00000002U
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC0_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P1ASR_WAITINGFORINTRC0_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC0_WIDTH           1
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC0_SHIFT           0
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC0_MASK            0x00000001U
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC0_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P1ASR_WAITINGFOREVENTC0_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P1CCR0_PMDSYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_P1CCR0_PMDSYNCCLKDIVEN_SHIFT            16
#define PCP_RB_CSW_P1CCR0_PMDSYNCCLKDIVEN_MASK             0x00010000U
#define PCP_RB_CSW_P1CCR0_PMDSYNCCLKDIVEN_RD(src)          (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P1CCR0_PMDSYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P1CCR0_PMDCLKMACRST_WIDTH               1
#define PCP_RB_CSW_P1CCR0_PMDCLKMACRST_SHIFT               15
#define PCP_RB_CSW_P1CCR0_PMDCLKMACRST_MASK                0x00008000U
#define PCP_RB_CSW_P1CCR0_PMDCLKMACRST_RD(src)             (((src) & 0x00008000U)>>15)
#define PCP_RB_CSW_P1CCR0_PMDCLKMACRST_SET(dst,src)        (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CSW_P1CCR0_PMDCLKMACPD_WIDTH                1
#define PCP_RB_CSW_P1CCR0_PMDCLKMACPD_SHIFT                14
#define PCP_RB_CSW_P1CCR0_PMDCLKMACPD_MASK                 0x00004000U
#define PCP_RB_CSW_P1CCR0_PMDCLKMACPD_RD(src)              (((src) & 0x00004000U)>>14)
#define PCP_RB_CSW_P1CCR0_PMDCLKMACPD_SET(dst,src)         (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CSW_P1CCR0_PMDDIVRATIO_WIDTH                2
#define PCP_RB_CSW_P1CCR0_PMDDIVRATIO_SHIFT                12
#define PCP_RB_CSW_P1CCR0_PMDDIVRATIO_MASK                 0x00003000U
#define PCP_RB_CSW_P1CCR0_PMDDIVRATIO_RD(src)              (((src) & 0x00003000U)>>12)
#define PCP_RB_CSW_P1CCR0_PMDDIVRATIO_SET(dst,src)         (((dst) & ~0x00003000U) | (((u32)(src)<<12) & 0x00003000U))
#define PCP_RB_CSW_P1CCR0_PMDCLKEN_WIDTH                   1
#define PCP_RB_CSW_P1CCR0_PMDCLKEN_SHIFT                   11
#define PCP_RB_CSW_P1CCR0_PMDCLKEN_MASK                    0x00000800U
#define PCP_RB_CSW_P1CCR0_PMDCLKEN_RD(src)                 (((src) & 0x00000800U)>>11)
#define PCP_RB_CSW_P1CCR0_PMDCLKEN_SET(dst,src)            (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CSW_P1CCR0_DCCHOLD_WIDTH                    1
#define PCP_RB_CSW_P1CCR0_DCCHOLD_SHIFT                    10
#define PCP_RB_CSW_P1CCR0_DCCHOLD_MASK                     0x00000400U
#define PCP_RB_CSW_P1CCR0_DCCHOLD_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P1CCR0_DCCHOLD_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P1CCR0_CLUSTERBYPEN_WIDTH               1
#define PCP_RB_CSW_P1CCR0_CLUSTERBYPEN_SHIFT               9
#define PCP_RB_CSW_P1CCR0_CLUSTERBYPEN_MASK                0x00000200U
#define PCP_RB_CSW_P1CCR0_CLUSTERBYPEN_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P1CCR0_CLUSTERBYPEN_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P1CCR0_CLUSTERCLKEN_WIDTH               1
#define PCP_RB_CSW_P1CCR0_CLUSTERCLKEN_SHIFT               8
#define PCP_RB_CSW_P1CCR0_CLUSTERCLKEN_MASK                0x00000100U
#define PCP_RB_CSW_P1CCR0_CLUSTERCLKEN_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P1CCR0_CLUSTERCLKEN_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P1CCR0_CGCC1_WIDTH                      1
#define PCP_RB_CSW_P1CCR0_CGCC1_SHIFT                      5
#define PCP_RB_CSW_P1CCR0_CGCC1_MASK                       0x00000020U
#define PCP_RB_CSW_P1CCR0_CGCC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P1CCR0_CGCC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P1CCR0_CGCF1_WIDTH                      1
#define PCP_RB_CSW_P1CCR0_CGCF1_SHIFT                      4
#define PCP_RB_CSW_P1CCR0_CGCF1_MASK                       0x00000010U
#define PCP_RB_CSW_P1CCR0_CGCF1_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P1CCR0_CGCF1_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P1CCR0_CGCC0_WIDTH                      1
#define PCP_RB_CSW_P1CCR0_CGCC0_SHIFT                      1
#define PCP_RB_CSW_P1CCR0_CGCC0_MASK                       0x00000002U
#define PCP_RB_CSW_P1CCR0_CGCC0_RD(src)                    (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P1CCR0_CGCC0_SET(dst,src)               (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P1CCR0_CGCF0_WIDTH                      1
#define PCP_RB_CSW_P1CCR0_CGCF0_SHIFT                      0
#define PCP_RB_CSW_P1CCR0_CGCF0_MASK                       0x00000001U
#define PCP_RB_CSW_P1CCR0_CGCF0_RD(src)                    (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P1CCR0_CGCF0_SET(dst,src)               (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P1CCR1_DCCSIGN_WIDTH                    1
#define PCP_RB_CSW_P1CCR1_DCCSIGN_SHIFT                    31
#define PCP_RB_CSW_P1CCR1_DCCSIGN_MASK                     0x80000000U
#define PCP_RB_CSW_P1CCR1_DCCSIGN_RD(src)                  (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_P1CCR1_DCCSIGN_SET(dst,src)             (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_P1CCR1_DCCVAL_WIDTH                     3
#define PCP_RB_CSW_P1CCR1_DCCVAL_SHIFT                     28
#define PCP_RB_CSW_P1CCR1_DCCVAL_MASK                      0x70000000U
#define PCP_RB_CSW_P1CCR1_DCCVAL_RD(src)                   (((src) & 0x70000000U)>>28)
#define PCP_RB_CSW_P1CCR1_DCCVAL_SET(dst,src)              (((dst) & ~0x70000000U) | (((u32)(src)<<28) & 0x70000000U))
#define PCP_RB_CSW_P1CCR1_CMLNSCORREN_WIDTH                1
#define PCP_RB_CSW_P1CCR1_CMLNSCORREN_SHIFT                27
#define PCP_RB_CSW_P1CCR1_CMLNSCORREN_MASK                 0x08000000U
#define PCP_RB_CSW_P1CCR1_CMLNSCORREN_RD(src)              (((src) & 0x08000000U)>>27)
#define PCP_RB_CSW_P1CCR1_CMLNSCORREN_SET(dst,src)         (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CSW_P1CCR1_SKEWTUNEL2C_WIDTH                5
#define PCP_RB_CSW_P1CCR1_SKEWTUNEL2C_SHIFT                20
#define PCP_RB_CSW_P1CCR1_SKEWTUNEL2C_MASK                 0x01F00000U
#define PCP_RB_CSW_P1CCR1_SKEWTUNEL2C_RD(src)              (((src) & 0x01F00000U)>>20)
#define PCP_RB_CSW_P1CCR1_SKEWTUNEL2C_SET(dst,src)         (((dst) & ~0x01F00000U) | (((u32)(src)<<20) & 0x01F00000U))
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC1_WIDTH                 5
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC1_SHIFT                 15
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC1_MASK                  0x000F8000U
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC1_RD(src)               (((src) & 0x000F8000U)>>15)
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC1_SET(dst,src)          (((dst) & ~0x000F8000U) | (((u32)(src)<<15) & 0x000F8000U))
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC0_WIDTH                 5
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC0_SHIFT                 10
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC0_MASK                  0x00007C00U
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC0_RD(src)               (((src) & 0x00007C00U)>>10)
#define PCP_RB_CSW_P1CCR1_SKEWTUNEC0_SET(dst,src)          (((dst) & ~0x00007C00U) | (((u32)(src)<<10) & 0x00007C00U))
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF1_WIDTH                 5
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF1_SHIFT                 5
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF1_MASK                  0x000003E0U
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF1_RD(src)               (((src) & 0x000003E0U)>>5)
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF1_SET(dst,src)          (((dst) & ~0x000003E0U) | (((u32)(src)<<5) & 0x000003E0U))
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF0_WIDTH                 5
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF0_SHIFT                 0
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF0_MASK                  0x0000001FU
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF0_RD(src)               (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_P1CCR1_SKEWTUNEF0_SET(dst,src)          (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_P1RCR_DBGENC1_WIDTH                     1
#define PCP_RB_CSW_P1RCR_DBGENC1_SHIFT                     17
#define PCP_RB_CSW_P1RCR_DBGENC1_MASK                      0x00020000U
#define PCP_RB_CSW_P1RCR_DBGENC1_RD(src)                   (((src) & 0x00020000U)>>17)
#define PCP_RB_CSW_P1RCR_DBGENC1_SET(dst,src)              (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CSW_P1RCR_DBGENC0_WIDTH                     1
#define PCP_RB_CSW_P1RCR_DBGENC0_SHIFT                     16
#define PCP_RB_CSW_P1RCR_DBGENC0_MASK                      0x00010000U
#define PCP_RB_CSW_P1RCR_DBGENC0_RD(src)                   (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P1RCR_DBGENC0_SET(dst,src)              (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P1RCR_RESETDBGC1_WIDTH                  1
#define PCP_RB_CSW_P1RCR_RESETDBGC1_SHIFT                  13
#define PCP_RB_CSW_P1RCR_RESETDBGC1_MASK                   0x00002000U
#define PCP_RB_CSW_P1RCR_RESETDBGC1_RD(src)                (((src) & 0x00002000U)>>13)
#define PCP_RB_CSW_P1RCR_RESETDBGC1_SET(dst,src)           (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CSW_P1RCR_RESETDBGC0_WIDTH                  1
#define PCP_RB_CSW_P1RCR_RESETDBGC0_SHIFT                  12
#define PCP_RB_CSW_P1RCR_RESETDBGC0_MASK                   0x00001000U
#define PCP_RB_CSW_P1RCR_RESETDBGC0_RD(src)                (((src) & 0x00001000U)>>12)
#define PCP_RB_CSW_P1RCR_RESETDBGC0_SET(dst,src)           (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_CSW_P1RCR_RESETL2C_WIDTH                    1
#define PCP_RB_CSW_P1RCR_RESETL2C_SHIFT                    10
#define PCP_RB_CSW_P1RCR_RESETL2C_MASK                     0x00000400U
#define PCP_RB_CSW_P1RCR_RESETL2C_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P1RCR_RESETL2C_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P1RCR_RESETC1_WIDTH                     1
#define PCP_RB_CSW_P1RCR_RESETC1_SHIFT                     9
#define PCP_RB_CSW_P1RCR_RESETC1_MASK                      0x00000200U
#define PCP_RB_CSW_P1RCR_RESETC1_RD(src)                   (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P1RCR_RESETC1_SET(dst,src)              (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P1RCR_RESETC0_WIDTH                     1
#define PCP_RB_CSW_P1RCR_RESETC0_SHIFT                     8
#define PCP_RB_CSW_P1RCR_RESETC0_MASK                      0x00000100U
#define PCP_RB_CSW_P1RCR_RESETC0_RD(src)                   (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P1RCR_RESETC0_SET(dst,src)              (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P1RCR_INITL2C_WIDTH                     1
#define PCP_RB_CSW_P1RCR_INITL2C_SHIFT                     6
#define PCP_RB_CSW_P1RCR_INITL2C_MASK                      0x00000040U
#define PCP_RB_CSW_P1RCR_INITL2C_RD(src)                   (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P1RCR_INITL2C_SET(dst,src)              (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P1RCR_INITC1_WIDTH                      1
#define PCP_RB_CSW_P1RCR_INITC1_SHIFT                      5
#define PCP_RB_CSW_P1RCR_INITC1_MASK                       0x00000020U
#define PCP_RB_CSW_P1RCR_INITC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P1RCR_INITC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P1RCR_INITC0_WIDTH                      1
#define PCP_RB_CSW_P1RCR_INITC0_SHIFT                      4
#define PCP_RB_CSW_P1RCR_INITC0_MASK                       0x00000010U
#define PCP_RB_CSW_P1RCR_INITC0_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P1RCR_INITC0_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P1RCR_STARTC1_WIDTH                     1
#define PCP_RB_CSW_P1RCR_STARTC1_SHIFT                     1
#define PCP_RB_CSW_P1RCR_STARTC1_MASK                      0x00000002U
#define PCP_RB_CSW_P1RCR_STARTC1_RD(src)                   (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P1RCR_STARTC1_SET(dst,src)              (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P1RCR_STARTC0_WIDTH                     1
#define PCP_RB_CSW_P1RCR_STARTC0_SHIFT                     0
#define PCP_RB_CSW_P1RCR_STARTC0_MASK                      0x00000001U
#define PCP_RB_CSW_P1RCR_STARTC0_RD(src)                   (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P1RCR_STARTC0_SET(dst,src)              (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC1_WIDTH                1
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC1_SHIFT                3
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC1_MASK                 0x00000008U
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC1_RD(src)              (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC1_SET(dst,src)         (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC0_WIDTH                1
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC0_SHIFT                2
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC0_MASK                 0x00000004U
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC0_RD(src)              (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P1RSR_CWRRACTIVEC0_SET(dst,src)         (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P1RSR_INRESETC1_WIDTH                   1
#define PCP_RB_CSW_P1RSR_INRESETC1_SHIFT                   1
#define PCP_RB_CSW_P1RSR_INRESETC1_MASK                    0x00000002U
#define PCP_RB_CSW_P1RSR_INRESETC1_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P1RSR_INRESETC1_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P1RSR_INRESETC0_WIDTH                   1
#define PCP_RB_CSW_P1RSR_INRESETC0_SHIFT                   0
#define PCP_RB_CSW_P1RSR_INRESETC0_MASK                    0x00000001U
#define PCP_RB_CSW_P1RSR_INRESETC0_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P1RSR_INRESETC0_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDPMD_WIDTH             1
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDPMD_SHIFT             10
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDPMD_MASK              0x00000400U
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDPMD_RD(src)           (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDPMD_SET(dst,src)      (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P2ASR_DATAQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P2ASR_DATAQUIESCEDL2C_SHIFT             9
#define PCP_RB_CSW_P2ASR_DATAQUIESCEDL2C_MASK              0x00000200U
#define PCP_RB_CSW_P2ASR_DATAQUIESCEDL2C_RD(src)           (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P2ASR_DATAQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDL2C_SHIFT             8
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDL2C_MASK              0x00000100U
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDL2C_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC1_WIDTH              1
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC1_SHIFT              6
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC1_MASK               0x00000040U
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC1_RD(src)            (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC1_SET(dst,src)       (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC1_WIDTH            1
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC1_SHIFT            5
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC1_MASK             0x00000020U
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC1_RD(src)          (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC1_SET(dst,src)     (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC1_WIDTH           1
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC1_SHIFT           4
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC1_MASK            0x00000010U
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC1_RD(src)         (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC1_SET(dst,src)    (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC0_WIDTH              1
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC0_SHIFT              2
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC0_MASK               0x00000004U
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC0_RD(src)            (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P2ASR_SELFQUIESCEDC0_SET(dst,src)       (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC0_WIDTH            1
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC0_SHIFT            1
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC0_MASK             0x00000002U
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC0_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P2ASR_WAITINGFORINTRC0_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC0_WIDTH           1
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC0_SHIFT           0
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC0_MASK            0x00000001U
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC0_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P2ASR_WAITINGFOREVENTC0_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P2CCR0_PMDSYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_P2CCR0_PMDSYNCCLKDIVEN_SHIFT            16
#define PCP_RB_CSW_P2CCR0_PMDSYNCCLKDIVEN_MASK             0x00010000U
#define PCP_RB_CSW_P2CCR0_PMDSYNCCLKDIVEN_RD(src)          (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P2CCR0_PMDSYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P2CCR0_PMDCLKMACRST_WIDTH               1
#define PCP_RB_CSW_P2CCR0_PMDCLKMACRST_SHIFT               15
#define PCP_RB_CSW_P2CCR0_PMDCLKMACRST_MASK                0x00008000U
#define PCP_RB_CSW_P2CCR0_PMDCLKMACRST_RD(src)             (((src) & 0x00008000U)>>15)
#define PCP_RB_CSW_P2CCR0_PMDCLKMACRST_SET(dst,src)        (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CSW_P2CCR0_PMDCLKMACPD_WIDTH                1
#define PCP_RB_CSW_P2CCR0_PMDCLKMACPD_SHIFT                14
#define PCP_RB_CSW_P2CCR0_PMDCLKMACPD_MASK                 0x00004000U
#define PCP_RB_CSW_P2CCR0_PMDCLKMACPD_RD(src)              (((src) & 0x00004000U)>>14)
#define PCP_RB_CSW_P2CCR0_PMDCLKMACPD_SET(dst,src)         (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CSW_P2CCR0_PMDDIVRATIO_WIDTH                2
#define PCP_RB_CSW_P2CCR0_PMDDIVRATIO_SHIFT                12
#define PCP_RB_CSW_P2CCR0_PMDDIVRATIO_MASK                 0x00003000U
#define PCP_RB_CSW_P2CCR0_PMDDIVRATIO_RD(src)              (((src) & 0x00003000U)>>12)
#define PCP_RB_CSW_P2CCR0_PMDDIVRATIO_SET(dst,src)         (((dst) & ~0x00003000U) | (((u32)(src)<<12) & 0x00003000U))
#define PCP_RB_CSW_P2CCR0_PMDCLKEN_WIDTH                   1
#define PCP_RB_CSW_P2CCR0_PMDCLKEN_SHIFT                   11
#define PCP_RB_CSW_P2CCR0_PMDCLKEN_MASK                    0x00000800U
#define PCP_RB_CSW_P2CCR0_PMDCLKEN_RD(src)                 (((src) & 0x00000800U)>>11)
#define PCP_RB_CSW_P2CCR0_PMDCLKEN_SET(dst,src)            (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CSW_P2CCR0_DCCHOLD_WIDTH                    1
#define PCP_RB_CSW_P2CCR0_DCCHOLD_SHIFT                    10
#define PCP_RB_CSW_P2CCR0_DCCHOLD_MASK                     0x00000400U
#define PCP_RB_CSW_P2CCR0_DCCHOLD_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P2CCR0_DCCHOLD_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P2CCR0_CLUSTERBYPEN_WIDTH               1
#define PCP_RB_CSW_P2CCR0_CLUSTERBYPEN_SHIFT               9
#define PCP_RB_CSW_P2CCR0_CLUSTERBYPEN_MASK                0x00000200U
#define PCP_RB_CSW_P2CCR0_CLUSTERBYPEN_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P2CCR0_CLUSTERBYPEN_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P2CCR0_CLUSTERCLKEN_WIDTH               1
#define PCP_RB_CSW_P2CCR0_CLUSTERCLKEN_SHIFT               8
#define PCP_RB_CSW_P2CCR0_CLUSTERCLKEN_MASK                0x00000100U
#define PCP_RB_CSW_P2CCR0_CLUSTERCLKEN_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P2CCR0_CLUSTERCLKEN_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P2CCR0_CGCC1_WIDTH                      1
#define PCP_RB_CSW_P2CCR0_CGCC1_SHIFT                      5
#define PCP_RB_CSW_P2CCR0_CGCC1_MASK                       0x00000020U
#define PCP_RB_CSW_P2CCR0_CGCC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P2CCR0_CGCC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P2CCR0_CGCF1_WIDTH                      1
#define PCP_RB_CSW_P2CCR0_CGCF1_SHIFT                      4
#define PCP_RB_CSW_P2CCR0_CGCF1_MASK                       0x00000010U
#define PCP_RB_CSW_P2CCR0_CGCF1_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P2CCR0_CGCF1_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P2CCR0_CGCC0_WIDTH                      1
#define PCP_RB_CSW_P2CCR0_CGCC0_SHIFT                      1
#define PCP_RB_CSW_P2CCR0_CGCC0_MASK                       0x00000002U
#define PCP_RB_CSW_P2CCR0_CGCC0_RD(src)                    (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P2CCR0_CGCC0_SET(dst,src)               (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P2CCR0_CGCF0_WIDTH                      1
#define PCP_RB_CSW_P2CCR0_CGCF0_SHIFT                      0
#define PCP_RB_CSW_P2CCR0_CGCF0_MASK                       0x00000001U
#define PCP_RB_CSW_P2CCR0_CGCF0_RD(src)                    (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P2CCR0_CGCF0_SET(dst,src)               (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P2CCR1_DCCSIGN_WIDTH                    1
#define PCP_RB_CSW_P2CCR1_DCCSIGN_SHIFT                    31
#define PCP_RB_CSW_P2CCR1_DCCSIGN_MASK                     0x80000000U
#define PCP_RB_CSW_P2CCR1_DCCSIGN_RD(src)                  (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_P2CCR1_DCCSIGN_SET(dst,src)             (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_P2CCR1_DCCVAL_WIDTH                     3
#define PCP_RB_CSW_P2CCR1_DCCVAL_SHIFT                     28
#define PCP_RB_CSW_P2CCR1_DCCVAL_MASK                      0x70000000U
#define PCP_RB_CSW_P2CCR1_DCCVAL_RD(src)                   (((src) & 0x70000000U)>>28)
#define PCP_RB_CSW_P2CCR1_DCCVAL_SET(dst,src)              (((dst) & ~0x70000000U) | (((u32)(src)<<28) & 0x70000000U))
#define PCP_RB_CSW_P2CCR1_CMLNSCORREN_WIDTH                1
#define PCP_RB_CSW_P2CCR1_CMLNSCORREN_SHIFT                27
#define PCP_RB_CSW_P2CCR1_CMLNSCORREN_MASK                 0x08000000U
#define PCP_RB_CSW_P2CCR1_CMLNSCORREN_RD(src)              (((src) & 0x08000000U)>>27)
#define PCP_RB_CSW_P2CCR1_CMLNSCORREN_SET(dst,src)         (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CSW_P2CCR1_SKEWTUNEL2C_WIDTH                5
#define PCP_RB_CSW_P2CCR1_SKEWTUNEL2C_SHIFT                20
#define PCP_RB_CSW_P2CCR1_SKEWTUNEL2C_MASK                 0x01F00000U
#define PCP_RB_CSW_P2CCR1_SKEWTUNEL2C_RD(src)              (((src) & 0x01F00000U)>>20)
#define PCP_RB_CSW_P2CCR1_SKEWTUNEL2C_SET(dst,src)         (((dst) & ~0x01F00000U) | (((u32)(src)<<20) & 0x01F00000U))
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC1_WIDTH                 5
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC1_SHIFT                 15
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC1_MASK                  0x000F8000U
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC1_RD(src)               (((src) & 0x000F8000U)>>15)
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC1_SET(dst,src)          (((dst) & ~0x000F8000U) | (((u32)(src)<<15) & 0x000F8000U))
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC0_WIDTH                 5
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC0_SHIFT                 10
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC0_MASK                  0x00007C00U
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC0_RD(src)               (((src) & 0x00007C00U)>>10)
#define PCP_RB_CSW_P2CCR1_SKEWTUNEC0_SET(dst,src)          (((dst) & ~0x00007C00U) | (((u32)(src)<<10) & 0x00007C00U))
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF1_WIDTH                 5
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF1_SHIFT                 5
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF1_MASK                  0x000003E0U
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF1_RD(src)               (((src) & 0x000003E0U)>>5)
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF1_SET(dst,src)          (((dst) & ~0x000003E0U) | (((u32)(src)<<5) & 0x000003E0U))
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF0_WIDTH                 5
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF0_SHIFT                 0
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF0_MASK                  0x0000001FU
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF0_RD(src)               (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_P2CCR1_SKEWTUNEF0_SET(dst,src)          (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_P2RCR_DBGENC1_WIDTH                     1
#define PCP_RB_CSW_P2RCR_DBGENC1_SHIFT                     17
#define PCP_RB_CSW_P2RCR_DBGENC1_MASK                      0x00020000U
#define PCP_RB_CSW_P2RCR_DBGENC1_RD(src)                   (((src) & 0x00020000U)>>17)
#define PCP_RB_CSW_P2RCR_DBGENC1_SET(dst,src)              (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CSW_P2RCR_DBGENC0_WIDTH                     1
#define PCP_RB_CSW_P2RCR_DBGENC0_SHIFT                     16
#define PCP_RB_CSW_P2RCR_DBGENC0_MASK                      0x00010000U
#define PCP_RB_CSW_P2RCR_DBGENC0_RD(src)                   (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P2RCR_DBGENC0_SET(dst,src)              (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P2RCR_RESETDBGC1_WIDTH                  1
#define PCP_RB_CSW_P2RCR_RESETDBGC1_SHIFT                  13
#define PCP_RB_CSW_P2RCR_RESETDBGC1_MASK                   0x00002000U
#define PCP_RB_CSW_P2RCR_RESETDBGC1_RD(src)                (((src) & 0x00002000U)>>13)
#define PCP_RB_CSW_P2RCR_RESETDBGC1_SET(dst,src)           (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CSW_P2RCR_RESETDBGC0_WIDTH                  1
#define PCP_RB_CSW_P2RCR_RESETDBGC0_SHIFT                  12
#define PCP_RB_CSW_P2RCR_RESETDBGC0_MASK                   0x00001000U
#define PCP_RB_CSW_P2RCR_RESETDBGC0_RD(src)                (((src) & 0x00001000U)>>12)
#define PCP_RB_CSW_P2RCR_RESETDBGC0_SET(dst,src)           (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_CSW_P2RCR_RESETL2C_WIDTH                    1
#define PCP_RB_CSW_P2RCR_RESETL2C_SHIFT                    10
#define PCP_RB_CSW_P2RCR_RESETL2C_MASK                     0x00000400U
#define PCP_RB_CSW_P2RCR_RESETL2C_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P2RCR_RESETL2C_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P2RCR_RESETC1_WIDTH                     1
#define PCP_RB_CSW_P2RCR_RESETC1_SHIFT                     9
#define PCP_RB_CSW_P2RCR_RESETC1_MASK                      0x00000200U
#define PCP_RB_CSW_P2RCR_RESETC1_RD(src)                   (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P2RCR_RESETC1_SET(dst,src)              (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P2RCR_RESETC0_WIDTH                     1
#define PCP_RB_CSW_P2RCR_RESETC0_SHIFT                     8
#define PCP_RB_CSW_P2RCR_RESETC0_MASK                      0x00000100U
#define PCP_RB_CSW_P2RCR_RESETC0_RD(src)                   (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P2RCR_RESETC0_SET(dst,src)              (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P2RCR_INITL2C_WIDTH                     1
#define PCP_RB_CSW_P2RCR_INITL2C_SHIFT                     6
#define PCP_RB_CSW_P2RCR_INITL2C_MASK                      0x00000040U
#define PCP_RB_CSW_P2RCR_INITL2C_RD(src)                   (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P2RCR_INITL2C_SET(dst,src)              (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P2RCR_INITC1_WIDTH                      1
#define PCP_RB_CSW_P2RCR_INITC1_SHIFT                      5
#define PCP_RB_CSW_P2RCR_INITC1_MASK                       0x00000020U
#define PCP_RB_CSW_P2RCR_INITC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P2RCR_INITC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P2RCR_INITC0_WIDTH                      1
#define PCP_RB_CSW_P2RCR_INITC0_SHIFT                      4
#define PCP_RB_CSW_P2RCR_INITC0_MASK                       0x00000010U
#define PCP_RB_CSW_P2RCR_INITC0_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P2RCR_INITC0_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P2RCR_STARTC1_WIDTH                     1
#define PCP_RB_CSW_P2RCR_STARTC1_SHIFT                     1
#define PCP_RB_CSW_P2RCR_STARTC1_MASK                      0x00000002U
#define PCP_RB_CSW_P2RCR_STARTC1_RD(src)                   (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P2RCR_STARTC1_SET(dst,src)              (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P2RCR_STARTC0_WIDTH                     1
#define PCP_RB_CSW_P2RCR_STARTC0_SHIFT                     0
#define PCP_RB_CSW_P2RCR_STARTC0_MASK                      0x00000001U
#define PCP_RB_CSW_P2RCR_STARTC0_RD(src)                   (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P2RCR_STARTC0_SET(dst,src)              (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC1_WIDTH                1
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC1_SHIFT                3
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC1_MASK                 0x00000008U
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC1_RD(src)              (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC1_SET(dst,src)         (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC0_WIDTH                1
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC0_SHIFT                2
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC0_MASK                 0x00000004U
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC0_RD(src)              (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P2RSR_CWRRACTIVEC0_SET(dst,src)         (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P2RSR_INRESETC1_WIDTH                   1
#define PCP_RB_CSW_P2RSR_INRESETC1_SHIFT                   1
#define PCP_RB_CSW_P2RSR_INRESETC1_MASK                    0x00000002U
#define PCP_RB_CSW_P2RSR_INRESETC1_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P2RSR_INRESETC1_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P2RSR_INRESETC0_WIDTH                   1
#define PCP_RB_CSW_P2RSR_INRESETC0_SHIFT                   0
#define PCP_RB_CSW_P2RSR_INRESETC0_MASK                    0x00000001U
#define PCP_RB_CSW_P2RSR_INRESETC0_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P2RSR_INRESETC0_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDPMD_WIDTH             1
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDPMD_SHIFT             10
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDPMD_MASK              0x00000400U
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDPMD_RD(src)           (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDPMD_SET(dst,src)      (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P3ASR_DATAQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P3ASR_DATAQUIESCEDL2C_SHIFT             9
#define PCP_RB_CSW_P3ASR_DATAQUIESCEDL2C_MASK              0x00000200U
#define PCP_RB_CSW_P3ASR_DATAQUIESCEDL2C_RD(src)           (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P3ASR_DATAQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDL2C_WIDTH             1
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDL2C_SHIFT             8
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDL2C_MASK              0x00000100U
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDL2C_RD(src)           (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDL2C_SET(dst,src)      (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC1_WIDTH              1
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC1_SHIFT              6
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC1_MASK               0x00000040U
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC1_RD(src)            (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC1_SET(dst,src)       (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC1_WIDTH            1
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC1_SHIFT            5
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC1_MASK             0x00000020U
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC1_RD(src)          (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC1_SET(dst,src)     (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC1_WIDTH           1
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC1_SHIFT           4
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC1_MASK            0x00000010U
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC1_RD(src)         (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC1_SET(dst,src)    (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC0_WIDTH              1
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC0_SHIFT              2
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC0_MASK               0x00000004U
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC0_RD(src)            (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P3ASR_SELFQUIESCEDC0_SET(dst,src)       (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC0_WIDTH            1
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC0_SHIFT            1
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC0_MASK             0x00000002U
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC0_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P3ASR_WAITINGFORINTRC0_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC0_WIDTH           1
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC0_SHIFT           0
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC0_MASK            0x00000001U
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC0_RD(src)         (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P3ASR_WAITINGFOREVENTC0_SET(dst,src)    (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P3CCR0_PMDSYNCCLKDIVEN_WIDTH            1
#define PCP_RB_CSW_P3CCR0_PMDSYNCCLKDIVEN_SHIFT            16
#define PCP_RB_CSW_P3CCR0_PMDSYNCCLKDIVEN_MASK             0x00010000U
#define PCP_RB_CSW_P3CCR0_PMDSYNCCLKDIVEN_RD(src)          (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P3CCR0_PMDSYNCCLKDIVEN_SET(dst,src)     (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P3CCR0_PMDCLKMACRST_WIDTH               1
#define PCP_RB_CSW_P3CCR0_PMDCLKMACRST_SHIFT               15
#define PCP_RB_CSW_P3CCR0_PMDCLKMACRST_MASK                0x00008000U
#define PCP_RB_CSW_P3CCR0_PMDCLKMACRST_RD(src)             (((src) & 0x00008000U)>>15)
#define PCP_RB_CSW_P3CCR0_PMDCLKMACRST_SET(dst,src)        (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CSW_P3CCR0_PMDCLKMACPD_WIDTH                1
#define PCP_RB_CSW_P3CCR0_PMDCLKMACPD_SHIFT                14
#define PCP_RB_CSW_P3CCR0_PMDCLKMACPD_MASK                 0x00004000U
#define PCP_RB_CSW_P3CCR0_PMDCLKMACPD_RD(src)              (((src) & 0x00004000U)>>14)
#define PCP_RB_CSW_P3CCR0_PMDCLKMACPD_SET(dst,src)         (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CSW_P3CCR0_PMDDIVRATIO_WIDTH                2
#define PCP_RB_CSW_P3CCR0_PMDDIVRATIO_SHIFT                12
#define PCP_RB_CSW_P3CCR0_PMDDIVRATIO_MASK                 0x00003000U
#define PCP_RB_CSW_P3CCR0_PMDDIVRATIO_RD(src)              (((src) & 0x00003000U)>>12)
#define PCP_RB_CSW_P3CCR0_PMDDIVRATIO_SET(dst,src)         (((dst) & ~0x00003000U) | (((u32)(src)<<12) & 0x00003000U))
#define PCP_RB_CSW_P3CCR0_PMDCLKEN_WIDTH                   1
#define PCP_RB_CSW_P3CCR0_PMDCLKEN_SHIFT                   11
#define PCP_RB_CSW_P3CCR0_PMDCLKEN_MASK                    0x00000800U
#define PCP_RB_CSW_P3CCR0_PMDCLKEN_RD(src)                 (((src) & 0x00000800U)>>11)
#define PCP_RB_CSW_P3CCR0_PMDCLKEN_SET(dst,src)            (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CSW_P3CCR0_DCCHOLD_WIDTH                    1
#define PCP_RB_CSW_P3CCR0_DCCHOLD_SHIFT                    10
#define PCP_RB_CSW_P3CCR0_DCCHOLD_MASK                     0x00000400U
#define PCP_RB_CSW_P3CCR0_DCCHOLD_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P3CCR0_DCCHOLD_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P3CCR0_CLUSTERBYPEN_WIDTH               1
#define PCP_RB_CSW_P3CCR0_CLUSTERBYPEN_SHIFT               9
#define PCP_RB_CSW_P3CCR0_CLUSTERBYPEN_MASK                0x00000200U
#define PCP_RB_CSW_P3CCR0_CLUSTERBYPEN_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P3CCR0_CLUSTERBYPEN_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P3CCR0_CLUSTERCLKEN_WIDTH               1
#define PCP_RB_CSW_P3CCR0_CLUSTERCLKEN_SHIFT               8
#define PCP_RB_CSW_P3CCR0_CLUSTERCLKEN_MASK                0x00000100U
#define PCP_RB_CSW_P3CCR0_CLUSTERCLKEN_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P3CCR0_CLUSTERCLKEN_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P3CCR0_CGCC1_WIDTH                      1
#define PCP_RB_CSW_P3CCR0_CGCC1_SHIFT                      5
#define PCP_RB_CSW_P3CCR0_CGCC1_MASK                       0x00000020U
#define PCP_RB_CSW_P3CCR0_CGCC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P3CCR0_CGCC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P3CCR0_CGCF1_WIDTH                      1
#define PCP_RB_CSW_P3CCR0_CGCF1_SHIFT                      4
#define PCP_RB_CSW_P3CCR0_CGCF1_MASK                       0x00000010U
#define PCP_RB_CSW_P3CCR0_CGCF1_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P3CCR0_CGCF1_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P3CCR0_CGCC0_WIDTH                      1
#define PCP_RB_CSW_P3CCR0_CGCC0_SHIFT                      1
#define PCP_RB_CSW_P3CCR0_CGCC0_MASK                       0x00000002U
#define PCP_RB_CSW_P3CCR0_CGCC0_RD(src)                    (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P3CCR0_CGCC0_SET(dst,src)               (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P3CCR0_CGCF0_WIDTH                      1
#define PCP_RB_CSW_P3CCR0_CGCF0_SHIFT                      0
#define PCP_RB_CSW_P3CCR0_CGCF0_MASK                       0x00000001U
#define PCP_RB_CSW_P3CCR0_CGCF0_RD(src)                    (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P3CCR0_CGCF0_SET(dst,src)               (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P3CCR1_DCCSIGN_WIDTH                    1
#define PCP_RB_CSW_P3CCR1_DCCSIGN_SHIFT                    31
#define PCP_RB_CSW_P3CCR1_DCCSIGN_MASK                     0x80000000U
#define PCP_RB_CSW_P3CCR1_DCCSIGN_RD(src)                  (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_P3CCR1_DCCSIGN_SET(dst,src)             (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_P3CCR1_DCCVAL_WIDTH                     3
#define PCP_RB_CSW_P3CCR1_DCCVAL_SHIFT                     28
#define PCP_RB_CSW_P3CCR1_DCCVAL_MASK                      0x70000000U
#define PCP_RB_CSW_P3CCR1_DCCVAL_RD(src)                   (((src) & 0x70000000U)>>28)
#define PCP_RB_CSW_P3CCR1_DCCVAL_SET(dst,src)              (((dst) & ~0x70000000U) | (((u32)(src)<<28) & 0x70000000U))
#define PCP_RB_CSW_P3CCR1_CMLNSCORREN_WIDTH                1
#define PCP_RB_CSW_P3CCR1_CMLNSCORREN_SHIFT                27
#define PCP_RB_CSW_P3CCR1_CMLNSCORREN_MASK                 0x08000000U
#define PCP_RB_CSW_P3CCR1_CMLNSCORREN_RD(src)              (((src) & 0x08000000U)>>27)
#define PCP_RB_CSW_P3CCR1_CMLNSCORREN_SET(dst,src)         (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CSW_P3CCR1_SKEWTUNEL2C_WIDTH                5
#define PCP_RB_CSW_P3CCR1_SKEWTUNEL2C_SHIFT                20
#define PCP_RB_CSW_P3CCR1_SKEWTUNEL2C_MASK                 0x01F00000U
#define PCP_RB_CSW_P3CCR1_SKEWTUNEL2C_RD(src)              (((src) & 0x01F00000U)>>20)
#define PCP_RB_CSW_P3CCR1_SKEWTUNEL2C_SET(dst,src)         (((dst) & ~0x01F00000U) | (((u32)(src)<<20) & 0x01F00000U))
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC1_WIDTH                 5
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC1_SHIFT                 15
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC1_MASK                  0x000F8000U
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC1_RD(src)               (((src) & 0x000F8000U)>>15)
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC1_SET(dst,src)          (((dst) & ~0x000F8000U) | (((u32)(src)<<15) & 0x000F8000U))
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC0_WIDTH                 5
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC0_SHIFT                 10
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC0_MASK                  0x00007C00U
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC0_RD(src)               (((src) & 0x00007C00U)>>10)
#define PCP_RB_CSW_P3CCR1_SKEWTUNEC0_SET(dst,src)          (((dst) & ~0x00007C00U) | (((u32)(src)<<10) & 0x00007C00U))
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF1_WIDTH                 5
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF1_SHIFT                 5
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF1_MASK                  0x000003E0U
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF1_RD(src)               (((src) & 0x000003E0U)>>5)
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF1_SET(dst,src)          (((dst) & ~0x000003E0U) | (((u32)(src)<<5) & 0x000003E0U))
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF0_WIDTH                 5
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF0_SHIFT                 0
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF0_MASK                  0x0000001FU
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF0_RD(src)               (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_P3CCR1_SKEWTUNEF0_SET(dst,src)          (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_P3RCR_DBGENC1_WIDTH                     1
#define PCP_RB_CSW_P3RCR_DBGENC1_SHIFT                     17
#define PCP_RB_CSW_P3RCR_DBGENC1_MASK                      0x00020000U
#define PCP_RB_CSW_P3RCR_DBGENC1_RD(src)                   (((src) & 0x00020000U)>>17)
#define PCP_RB_CSW_P3RCR_DBGENC1_SET(dst,src)              (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CSW_P3RCR_DBGENC0_WIDTH                     1
#define PCP_RB_CSW_P3RCR_DBGENC0_SHIFT                     16
#define PCP_RB_CSW_P3RCR_DBGENC0_MASK                      0x00010000U
#define PCP_RB_CSW_P3RCR_DBGENC0_RD(src)                   (((src) & 0x00010000U)>>16)
#define PCP_RB_CSW_P3RCR_DBGENC0_SET(dst,src)              (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CSW_P3RCR_RESETDBGC1_WIDTH                  1
#define PCP_RB_CSW_P3RCR_RESETDBGC1_SHIFT                  13
#define PCP_RB_CSW_P3RCR_RESETDBGC1_MASK                   0x00002000U
#define PCP_RB_CSW_P3RCR_RESETDBGC1_RD(src)                (((src) & 0x00002000U)>>13)
#define PCP_RB_CSW_P3RCR_RESETDBGC1_SET(dst,src)           (((dst) & ~0x00002000U) | (((u32)(src)<<13) & 0x00002000U))
#define PCP_RB_CSW_P3RCR_RESETDBGC0_WIDTH                  1
#define PCP_RB_CSW_P3RCR_RESETDBGC0_SHIFT                  12
#define PCP_RB_CSW_P3RCR_RESETDBGC0_MASK                   0x00001000U
#define PCP_RB_CSW_P3RCR_RESETDBGC0_RD(src)                (((src) & 0x00001000U)>>12)
#define PCP_RB_CSW_P3RCR_RESETDBGC0_SET(dst,src)           (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_CSW_P3RCR_RESETL2C_WIDTH                    1
#define PCP_RB_CSW_P3RCR_RESETL2C_SHIFT                    10
#define PCP_RB_CSW_P3RCR_RESETL2C_MASK                     0x00000400U
#define PCP_RB_CSW_P3RCR_RESETL2C_RD(src)                  (((src) & 0x00000400U)>>10)
#define PCP_RB_CSW_P3RCR_RESETL2C_SET(dst,src)             (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CSW_P3RCR_RESETC1_WIDTH                     1
#define PCP_RB_CSW_P3RCR_RESETC1_SHIFT                     9
#define PCP_RB_CSW_P3RCR_RESETC1_MASK                      0x00000200U
#define PCP_RB_CSW_P3RCR_RESETC1_RD(src)                   (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_P3RCR_RESETC1_SET(dst,src)              (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_P3RCR_RESETC0_WIDTH                     1
#define PCP_RB_CSW_P3RCR_RESETC0_SHIFT                     8
#define PCP_RB_CSW_P3RCR_RESETC0_MASK                      0x00000100U
#define PCP_RB_CSW_P3RCR_RESETC0_RD(src)                   (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_P3RCR_RESETC0_SET(dst,src)              (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_P3RCR_INITL2C_WIDTH                     1
#define PCP_RB_CSW_P3RCR_INITL2C_SHIFT                     6
#define PCP_RB_CSW_P3RCR_INITL2C_MASK                      0x00000040U
#define PCP_RB_CSW_P3RCR_INITL2C_RD(src)                   (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_P3RCR_INITL2C_SET(dst,src)              (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_P3RCR_INITC1_WIDTH                      1
#define PCP_RB_CSW_P3RCR_INITC1_SHIFT                      5
#define PCP_RB_CSW_P3RCR_INITC1_MASK                       0x00000020U
#define PCP_RB_CSW_P3RCR_INITC1_RD(src)                    (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_P3RCR_INITC1_SET(dst,src)               (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_P3RCR_INITC0_WIDTH                      1
#define PCP_RB_CSW_P3RCR_INITC0_SHIFT                      4
#define PCP_RB_CSW_P3RCR_INITC0_MASK                       0x00000010U
#define PCP_RB_CSW_P3RCR_INITC0_RD(src)                    (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_P3RCR_INITC0_SET(dst,src)               (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_P3RCR_STARTC1_WIDTH                     1
#define PCP_RB_CSW_P3RCR_STARTC1_SHIFT                     1
#define PCP_RB_CSW_P3RCR_STARTC1_MASK                      0x00000002U
#define PCP_RB_CSW_P3RCR_STARTC1_RD(src)                   (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P3RCR_STARTC1_SET(dst,src)              (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P3RCR_STARTC0_WIDTH                     1
#define PCP_RB_CSW_P3RCR_STARTC0_SHIFT                     0
#define PCP_RB_CSW_P3RCR_STARTC0_MASK                      0x00000001U
#define PCP_RB_CSW_P3RCR_STARTC0_RD(src)                   (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P3RCR_STARTC0_SET(dst,src)              (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC1_WIDTH                1
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC1_SHIFT                3
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC1_MASK                 0x00000008U
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC1_RD(src)              (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC1_SET(dst,src)         (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC0_WIDTH                1
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC0_SHIFT                2
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC0_MASK                 0x00000004U
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC0_RD(src)              (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_P3RSR_CWRRACTIVEC0_SET(dst,src)         (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_P3RSR_INRESETC1_WIDTH                   1
#define PCP_RB_CSW_P3RSR_INRESETC1_SHIFT                   1
#define PCP_RB_CSW_P3RSR_INRESETC1_MASK                    0x00000002U
#define PCP_RB_CSW_P3RSR_INRESETC1_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_P3RSR_INRESETC1_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_P3RSR_INRESETC0_WIDTH                   1
#define PCP_RB_CSW_P3RSR_INRESETC0_SHIFT                   0
#define PCP_RB_CSW_P3RSR_INRESETC0_MASK                    0x00000001U
#define PCP_RB_CSW_P3RSR_INRESETC0_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_P3RSR_INRESETC0_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_MCUPOWERON_WIDTH       4
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_MCUPOWERON_SHIFT       8
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_MCUPOWERON_MASK        0x00000F00U
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_MCUPOWERON_RD(src)     (((src) & 0x00000F00U)>>8)
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_MCUPOWERON_SET(dst,src) (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CINCRPOWERON_WIDTH   4
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CINCRPOWERON_SHIFT   4
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CINCRPOWERON_MASK    0x000000F0U
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CINCRPOWERON_RD(src) (((src) & 0x000000F0U)>>4)
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CINCRPOWERON_SET(dst,src) (((dst) & ~0x000000F0U) | (((u32)(src)<<4) & 0x000000F0U))
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CPOWERON_WIDTH       1
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CPOWERON_SHIFT       3
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CPOWERON_MASK        0x00000008U
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CPOWERON_RD(src)     (((src) & 0x00000008U)>>3)
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_L3CPOWERON_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_PCPEFUSESTART_WIDTH    1
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_PCPEFUSESTART_SHIFT    0
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_PCPEFUSESTART_MASK     0x00000001U
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_PCPEFUSESTART_RD(src)  (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_CFG_PCPEFUSECTRL_PCPEFUSESTART_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTFAIL_WIDTH        1
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTFAIL_SHIFT        1
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTFAIL_MASK         0x00000002U
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTFAIL_RD(src)      (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTFAIL_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTRDY_WIDTH         1
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTRDY_SHIFT         0
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTRDY_MASK          0x00000001U
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTRDY_RD(src)       (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_CFG_PCPEFUSESTAT_MBISTRDY_SET(dst,src)  (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SNID_WIDTH            2
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SNID_SHIFT            6
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SNID_MASK             0x000000C0U
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SNID_RD(src)          (((src) & 0x000000C0U)>>6)
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SNID_SET(dst,src)     (((dst) & ~0x000000C0U) | (((u32)(src)<<6) & 0x000000C0U))
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SID_WIDTH             2
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SID_SHIFT             4
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SID_MASK              0x00000030U
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SID_RD(src)           (((src) & 0x00000030U)>>4)
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_SID_SET(dst,src)      (((dst) & ~0x00000030U) | (((u32)(src)<<4) & 0x00000030U))
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSNID_WIDTH           2
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSNID_SHIFT           2
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSNID_MASK            0x0000000CU
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSNID_RD(src)         (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSNID_SET(dst,src)    (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSID_WIDTH            2
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSID_SHIFT            0
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSID_MASK             0x00000003U
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSID_RD(src)          (((src) & 0x00000003U)>>0)
#define PCP_RB_CPUX_PMU_PMAUTHSTATUS_NSID_SET(dst,src)     (((dst) & ~0x00000003U) | (((u32)(src)<<0) & 0x00000003U))
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_P_WIDTH              1
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_P_SHIFT              31
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_P_MASK               0x80000000U
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_P_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_P_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_U_WIDTH              1
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_U_SHIFT              30
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_U_MASK               0x40000000U
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_U_RD(src)            (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_U_SET(dst,src)       (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSK_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSK_SHIFT            29
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSK_MASK             0x20000000U
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSK_RD(src)          (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSK_SET(dst,src)     (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSU_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSU_SHIFT            28
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSU_MASK             0x10000000U
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSU_RD(src)          (((src) & 0x10000000U)>>28)
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSU_SET(dst,src)     (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSH_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSH_SHIFT            27
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSH_MASK             0x08000000U
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSH_RD(src)          (((src) & 0x08000000U)>>27)
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_NSH_SET(dst,src)     (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_M_WIDTH              1
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_M_SHIFT              26
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_M_MASK               0x04000000U
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_M_RD(src)            (((src) & 0x04000000U)>>26)
#define PCP_RB_CPUX_PMU_PMCCFILTR_EL0_M_SET(dst,src)       (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_CPUX_PMU_PMCCNTR_EL0_CCNT_WIDTH             64
#define PCP_RB_CPUX_PMU_PMCCNTR_EL0_CCNT_SHIFT             0
#define PCP_RB_CPUX_PMU_PMCCNTR_EL0_CCNT_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMCCNTR_EL0_CCNT_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMCCNTR_EL0_CCNT_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMCEID0_EL0_BIT31_0_WIDTH          32
#define PCP_RB_CPUX_PMU_PMCEID0_EL0_BIT31_0_SHIFT          0
#define PCP_RB_CPUX_PMU_PMCEID0_EL0_BIT31_0_MASK           0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMCEID0_EL0_BIT31_0_RD(src)        (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMCEID0_EL0_BIT31_0_SET(dst,src)   (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMCFGR_EX_WIDTH                    1
#define PCP_RB_CPUX_PMU_PMCFGR_EX_SHIFT                    16
#define PCP_RB_CPUX_PMU_PMCFGR_EX_MASK                     0x00010000U
#define PCP_RB_CPUX_PMU_PMCFGR_EX_RD(src)                  (((src) & 0x00010000U)>>16)
#define PCP_RB_CPUX_PMU_PMCFGR_EX_SET(dst,src)             (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CPUX_PMU_PMCFGR_CCD_WIDTH                   1
#define PCP_RB_CPUX_PMU_PMCFGR_CCD_SHIFT                   15
#define PCP_RB_CPUX_PMU_PMCFGR_CCD_MASK                    0x00008000U
#define PCP_RB_CPUX_PMU_PMCFGR_CCD_RD(src)                 (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_PMU_PMCFGR_CCD_SET(dst,src)            (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_PMU_PMCFGR_CC_WIDTH                    1
#define PCP_RB_CPUX_PMU_PMCFGR_CC_SHIFT                    14
#define PCP_RB_CPUX_PMU_PMCFGR_CC_MASK                     0x00004000U
#define PCP_RB_CPUX_PMU_PMCFGR_CC_RD(src)                  (((src) & 0x00004000U)>>14)
#define PCP_RB_CPUX_PMU_PMCFGR_CC_SET(dst,src)             (((dst) & ~0x00004000U) | (((u32)(src)<<14) & 0x00004000U))
#define PCP_RB_CPUX_PMU_PMCFGR_SIZE_WIDTH                  6
#define PCP_RB_CPUX_PMU_PMCFGR_SIZE_SHIFT                  8
#define PCP_RB_CPUX_PMU_PMCFGR_SIZE_MASK                   0x00003F00U
#define PCP_RB_CPUX_PMU_PMCFGR_SIZE_RD(src)                (((src) & 0x00003F00U)>>8)
#define PCP_RB_CPUX_PMU_PMCFGR_SIZE_SET(dst,src)           (((dst) & ~0x00003F00U) | (((u32)(src)<<8) & 0x00003F00U))
#define PCP_RB_CPUX_PMU_PMCFGR_N_WIDTH                     8
#define PCP_RB_CPUX_PMU_PMCFGR_N_SHIFT                     0
#define PCP_RB_CPUX_PMU_PMCFGR_N_MASK                      0x000000FFU
#define PCP_RB_CPUX_PMU_PMCFGR_N_RD(src)                   (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_PMU_PMCFGR_N_SET(dst,src)              (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_PMU_PMCIDR0_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMCIDR0_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMCIDR0_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMCIDR0_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMCIDR0_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMCIDR1_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMCIDR1_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMCIDR1_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMCIDR1_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMCIDR1_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMCIDR2_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMCIDR2_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMCIDR2_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMCIDR2_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMCIDR2_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMCIDR3_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMCIDR3_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMCIDR3_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMCIDR3_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMCIDR3_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_C_WIDTH             1
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_C_SHIFT             31
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_C_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_C_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_C_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P3_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P3_SHIFT            3
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P3_MASK             0x00000008U
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P3_RD(src)          (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P3_SET(dst,src)     (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P2_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P2_SHIFT            2
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P2_MASK             0x00000004U
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P2_RD(src)          (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P2_SET(dst,src)     (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P1_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P1_SHIFT            1
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P1_MASK             0x00000002U
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P1_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P1_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P0_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P0_SHIFT            0
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P0_MASK             0x00000001U
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P0_RD(src)          (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMCNTENCLR_EL0_P0_SET(dst,src)     (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_C_WIDTH             1
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_C_SHIFT             31
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_C_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_C_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_C_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P3_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P3_SHIFT            3
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P3_MASK             0x00000008U
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P3_RD(src)          (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P3_SET(dst,src)     (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P2_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P2_SHIFT            2
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P2_MASK             0x00000004U
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P2_RD(src)          (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P2_SET(dst,src)     (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P1_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P1_SHIFT            1
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P1_MASK             0x00000002U
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P1_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P1_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P0_WIDTH            1
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P0_SHIFT            0
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P0_MASK             0x00000001U
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P0_RD(src)          (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMCNTENSET_EL0_P0_SET(dst,src)     (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_IMP_WIDTH                 8
#define PCP_RB_CPUX_PMU_PMCR_EL0_IMP_SHIFT                 24
#define PCP_RB_CPUX_PMU_PMCR_EL0_IMP_MASK                  0xFF000000U
#define PCP_RB_CPUX_PMU_PMCR_EL0_IMP_RD(src)               (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_PMU_PMCR_EL0_IMP_SET(dst,src)          (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_IDCODE_WIDTH              8
#define PCP_RB_CPUX_PMU_PMCR_EL0_IDCODE_SHIFT              16
#define PCP_RB_CPUX_PMU_PMCR_EL0_IDCODE_MASK               0x00FF0000U
#define PCP_RB_CPUX_PMU_PMCR_EL0_IDCODE_RD(src)            (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CPUX_PMU_PMCR_EL0_IDCODE_SET(dst,src)       (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_N_WIDTH                   5
#define PCP_RB_CPUX_PMU_PMCR_EL0_N_SHIFT                   11
#define PCP_RB_CPUX_PMU_PMCR_EL0_N_MASK                    0x0000F800U
#define PCP_RB_CPUX_PMU_PMCR_EL0_N_RD(src)                 (((src) & 0x0000F800U)>>11)
#define PCP_RB_CPUX_PMU_PMCR_EL0_N_SET(dst,src)            (((dst) & ~0x0000F800U) | (((u32)(src)<<11) & 0x0000F800U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_LC_WIDTH                  1
#define PCP_RB_CPUX_PMU_PMCR_EL0_LC_SHIFT                  6
#define PCP_RB_CPUX_PMU_PMCR_EL0_LC_MASK                   0x00000040U
#define PCP_RB_CPUX_PMU_PMCR_EL0_LC_RD(src)                (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_PMU_PMCR_EL0_LC_SET(dst,src)           (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_DP_WIDTH                  1
#define PCP_RB_CPUX_PMU_PMCR_EL0_DP_SHIFT                  5
#define PCP_RB_CPUX_PMU_PMCR_EL0_DP_MASK                   0x00000020U
#define PCP_RB_CPUX_PMU_PMCR_EL0_DP_RD(src)                (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_PMU_PMCR_EL0_DP_SET(dst,src)           (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_X_WIDTH                   1
#define PCP_RB_CPUX_PMU_PMCR_EL0_X_SHIFT                   4
#define PCP_RB_CPUX_PMU_PMCR_EL0_X_MASK                    0x00000010U
#define PCP_RB_CPUX_PMU_PMCR_EL0_X_RD(src)                 (((src) & 0x00000010U)>>4)
#define PCP_RB_CPUX_PMU_PMCR_EL0_X_SET(dst,src)            (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_D_WIDTH                   1
#define PCP_RB_CPUX_PMU_PMCR_EL0_D_SHIFT                   3
#define PCP_RB_CPUX_PMU_PMCR_EL0_D_MASK                    0x00000008U
#define PCP_RB_CPUX_PMU_PMCR_EL0_D_RD(src)                 (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMCR_EL0_D_SET(dst,src)            (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_C_WIDTH                   1
#define PCP_RB_CPUX_PMU_PMCR_EL0_C_SHIFT                   2
#define PCP_RB_CPUX_PMU_PMCR_EL0_C_MASK                    0x00000004U
#define PCP_RB_CPUX_PMU_PMCR_EL0_C_RD(src)                 (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMCR_EL0_C_SET(dst,src)            (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_P_WIDTH                   1
#define PCP_RB_CPUX_PMU_PMCR_EL0_P_SHIFT                   1
#define PCP_RB_CPUX_PMU_PMCR_EL0_P_MASK                    0x00000002U
#define PCP_RB_CPUX_PMU_PMCR_EL0_P_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMCR_EL0_P_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMCR_EL0_E_WIDTH                   1
#define PCP_RB_CPUX_PMU_PMCR_EL0_E_SHIFT                   0
#define PCP_RB_CPUX_PMU_PMCR_EL0_E_MASK                    0x00000001U
#define PCP_RB_CPUX_PMU_PMCR_EL0_E_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMCR_EL0_E_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMDEVAFF0_RES1_WIDTH               1
#define PCP_RB_CPUX_PMU_PMDEVAFF0_RES1_SHIFT               31
#define PCP_RB_CPUX_PMU_PMDEVAFF0_RES1_MASK                0x80000000U
#define PCP_RB_CPUX_PMU_PMDEVAFF0_RES1_RD(src)             (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMDEVAFF0_RES1_SET(dst,src)        (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMDEVAFF0_U_WIDTH                  1
#define PCP_RB_CPUX_PMU_PMDEVAFF0_U_SHIFT                  30
#define PCP_RB_CPUX_PMU_PMDEVAFF0_U_MASK                   0x40000000U
#define PCP_RB_CPUX_PMU_PMDEVAFF0_U_RD(src)                (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_PMU_PMDEVAFF0_U_SET(dst,src)           (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_PMU_PMDEVAFF0_MT_WIDTH                 1
#define PCP_RB_CPUX_PMU_PMDEVAFF0_MT_SHIFT                 24
#define PCP_RB_CPUX_PMU_PMDEVAFF0_MT_MASK                  0x01000000U
#define PCP_RB_CPUX_PMU_PMDEVAFF0_MT_RD(src)               (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_PMU_PMDEVAFF0_MT_SET(dst,src)          (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF2_WIDTH               8
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF2_SHIFT               16
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF2_MASK                0x00FF0000U
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF2_RD(src)             (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF2_SET(dst,src)        (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF1_WIDTH               8
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF1_SHIFT               8
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF1_MASK                0x0000FF00U
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF1_RD(src)             (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF1_SET(dst,src)        (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF0_WIDTH               8
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF0_SHIFT               0
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF0_MASK                0x000000FFU
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF0_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_PMU_PMDEVAFF0_AFF0_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_PMU_PMDEVAFF1_AFF3_WIDTH               8
#define PCP_RB_CPUX_PMU_PMDEVAFF1_AFF3_SHIFT               0
#define PCP_RB_CPUX_PMU_PMDEVAFF1_AFF3_MASK                0x000000FFU
#define PCP_RB_CPUX_PMU_PMDEVAFF1_AFF3_RD(src)             (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_PMU_PMDEVAFF1_AFF3_SET(dst,src)        (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHITECT_WIDTH          11
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHITECT_SHIFT          21
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHITECT_MASK           0xFFE00000U
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHITECT_RD(src)        (((src) & 0xFFE00000U)>>21)
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHITECT_SET(dst,src)   (((dst) & ~0xFFE00000U) | (((u32)(src)<<21) & 0xFFE00000U))
#define PCP_RB_CPUX_PMU_PMDEVARCH_PRESENT_WIDTH            1
#define PCP_RB_CPUX_PMU_PMDEVARCH_PRESENT_SHIFT            20
#define PCP_RB_CPUX_PMU_PMDEVARCH_PRESENT_MASK             0x00100000U
#define PCP_RB_CPUX_PMU_PMDEVARCH_PRESENT_RD(src)          (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_PMU_PMDEVARCH_PRESENT_SET(dst,src)     (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_PMU_PMDEVARCH_REVISION_WIDTH           4
#define PCP_RB_CPUX_PMU_PMDEVARCH_REVISION_SHIFT           16
#define PCP_RB_CPUX_PMU_PMDEVARCH_REVISION_MASK            0x000F0000U
#define PCP_RB_CPUX_PMU_PMDEVARCH_REVISION_RD(src)         (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_PMU_PMDEVARCH_REVISION_SET(dst,src)    (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHVER_WIDTH            4
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHVER_SHIFT            12
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHVER_MASK             0x0000F000U
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHVER_RD(src)          (((src) & 0x0000F000U)>>12)
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHVER_SET(dst,src)     (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHPART_WIDTH           12
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHPART_SHIFT           0
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHPART_MASK            0x00000FFFU
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHPART_RD(src)         (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CPUX_PMU_PMDEVARCH_ARCHPART_SET(dst,src)    (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_PMU_PMDEVTYPE_SUB_WIDTH                4
#define PCP_RB_CPUX_PMU_PMDEVTYPE_SUB_SHIFT                4
#define PCP_RB_CPUX_PMU_PMDEVTYPE_SUB_MASK                 0x000000F0U
#define PCP_RB_CPUX_PMU_PMDEVTYPE_SUB_RD(src)              (((src) & 0x000000F0U)>>4)
#define PCP_RB_CPUX_PMU_PMDEVTYPE_SUB_SET(dst,src)         (((dst) & ~0x000000F0U) | (((u32)(src)<<4) & 0x000000F0U))
#define PCP_RB_CPUX_PMU_PMDEVTYPE_MAJOR_WIDTH              4
#define PCP_RB_CPUX_PMU_PMDEVTYPE_MAJOR_SHIFT              0
#define PCP_RB_CPUX_PMU_PMDEVTYPE_MAJOR_MASK               0x0000000FU
#define PCP_RB_CPUX_PMU_PMDEVTYPE_MAJOR_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_PMU_PMDEVTYPE_MAJOR_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_PMU_PMEVCNTR0_EL0_PMNX_WIDTH           32
#define PCP_RB_CPUX_PMU_PMEVCNTR0_EL0_PMNX_SHIFT           0
#define PCP_RB_CPUX_PMU_PMEVCNTR0_EL0_PMNX_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMEVCNTR0_EL0_PMNX_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVCNTR0_EL0_PMNX_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMEVCNTR1_EL0_PMNX_WIDTH           32
#define PCP_RB_CPUX_PMU_PMEVCNTR1_EL0_PMNX_SHIFT           0
#define PCP_RB_CPUX_PMU_PMEVCNTR1_EL0_PMNX_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMEVCNTR1_EL0_PMNX_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVCNTR1_EL0_PMNX_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMEVCNTR2_EL0_PMNX_WIDTH           32
#define PCP_RB_CPUX_PMU_PMEVCNTR2_EL0_PMNX_SHIFT           0
#define PCP_RB_CPUX_PMU_PMEVCNTR2_EL0_PMNX_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMEVCNTR2_EL0_PMNX_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVCNTR2_EL0_PMNX_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMEVCNTR3_EL0_PMNX_WIDTH           32
#define PCP_RB_CPUX_PMU_PMEVCNTR3_EL0_PMNX_SHIFT           0
#define PCP_RB_CPUX_PMU_PMEVCNTR3_EL0_PMNX_MASK            0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMEVCNTR3_EL0_PMNX_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVCNTR3_EL0_PMNX_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_P_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_P_SHIFT             31
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_P_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_P_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_P_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_U_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_U_SHIFT             30
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_U_MASK              0x40000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_U_RD(src)           (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_U_SET(dst,src)      (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSK_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSK_SHIFT           29
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSK_MASK            0x20000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSK_RD(src)         (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSK_SET(dst,src)    (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSU_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSU_SHIFT           28
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSU_MASK            0x10000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSU_RD(src)         (((src) & 0x10000000U)>>28)
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSU_SET(dst,src)    (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSH_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSH_SHIFT           27
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSH_MASK            0x08000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSH_RD(src)         (((src) & 0x08000000U)>>27)
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_NSH_SET(dst,src)    (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_M_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_M_SHIFT             26
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_M_MASK              0x04000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_M_RD(src)           (((src) & 0x04000000U)>>26)
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_M_SET(dst,src)      (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_EVTCOUNT_WIDTH      10
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_EVTCOUNT_SHIFT      0
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_EVTCOUNT_MASK       0x000003FFU
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_EVTCOUNT_RD(src)    (((src) & 0x000003FFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVTYPER0_EL0_EVTCOUNT_SET(dst,src) (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_P_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_P_SHIFT             31
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_P_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_P_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_P_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_U_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_U_SHIFT             30
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_U_MASK              0x40000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_U_RD(src)           (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_U_SET(dst,src)      (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSK_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSK_SHIFT           29
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSK_MASK            0x20000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSK_RD(src)         (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSK_SET(dst,src)    (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSU_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSU_SHIFT           28
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSU_MASK            0x10000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSU_RD(src)         (((src) & 0x10000000U)>>28)
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSU_SET(dst,src)    (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSH_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSH_SHIFT           27
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSH_MASK            0x08000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSH_RD(src)         (((src) & 0x08000000U)>>27)
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_NSH_SET(dst,src)    (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_M_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_M_SHIFT             26
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_M_MASK              0x04000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_M_RD(src)           (((src) & 0x04000000U)>>26)
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_M_SET(dst,src)      (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_EVTCOUNT_WIDTH      10
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_EVTCOUNT_SHIFT      0
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_EVTCOUNT_MASK       0x000003FFU
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_EVTCOUNT_RD(src)    (((src) & 0x000003FFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVTYPER1_EL0_EVTCOUNT_SET(dst,src) (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_P_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_P_SHIFT             31
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_P_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_P_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_P_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_U_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_U_SHIFT             30
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_U_MASK              0x40000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_U_RD(src)           (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_U_SET(dst,src)      (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSK_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSK_SHIFT           29
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSK_MASK            0x20000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSK_RD(src)         (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSK_SET(dst,src)    (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSU_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSU_SHIFT           28
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSU_MASK            0x10000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSU_RD(src)         (((src) & 0x10000000U)>>28)
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSU_SET(dst,src)    (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSH_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSH_SHIFT           27
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSH_MASK            0x08000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSH_RD(src)         (((src) & 0x08000000U)>>27)
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_NSH_SET(dst,src)    (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_M_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_M_SHIFT             26
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_M_MASK              0x04000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_M_RD(src)           (((src) & 0x04000000U)>>26)
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_M_SET(dst,src)      (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_EVTCOUNT_WIDTH      10
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_EVTCOUNT_SHIFT      0
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_EVTCOUNT_MASK       0x000003FFU
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_EVTCOUNT_RD(src)    (((src) & 0x000003FFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVTYPER2_EL0_EVTCOUNT_SET(dst,src) (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_P_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_P_SHIFT             31
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_P_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_P_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_P_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_U_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_U_SHIFT             30
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_U_MASK              0x40000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_U_RD(src)           (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_U_SET(dst,src)      (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSK_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSK_SHIFT           29
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSK_MASK            0x20000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSK_RD(src)         (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSK_SET(dst,src)    (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSU_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSU_SHIFT           28
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSU_MASK            0x10000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSU_RD(src)         (((src) & 0x10000000U)>>28)
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSU_SET(dst,src)    (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSH_WIDTH           1
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSH_SHIFT           27
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSH_MASK            0x08000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSH_RD(src)         (((src) & 0x08000000U)>>27)
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_NSH_SET(dst,src)    (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_M_WIDTH             1
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_M_SHIFT             26
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_M_MASK              0x04000000U
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_M_RD(src)           (((src) & 0x04000000U)>>26)
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_M_SET(dst,src)      (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_EVTCOUNT_WIDTH      10
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_EVTCOUNT_SHIFT      0
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_EVTCOUNT_MASK       0x000003FFU
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_EVTCOUNT_RD(src)    (((src) & 0x000003FFU)>>0)
#define PCP_RB_CPUX_PMU_PMEVTYPER3_EL0_EVTCOUNT_SET(dst,src) (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_C_WIDTH             1
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_C_SHIFT             31
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_C_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_C_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_C_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P3_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P3_SHIFT            3
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P3_MASK             0x00000008U
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P3_RD(src)          (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P3_SET(dst,src)     (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P2_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P2_SHIFT            2
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P2_MASK             0x00000004U
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P2_RD(src)          (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P2_SET(dst,src)     (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P1_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P1_SHIFT            1
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P1_MASK             0x00000002U
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P1_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P1_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P0_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P0_SHIFT            0
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P0_MASK             0x00000001U
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P0_RD(src)          (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMINTENCLR_EL1_P0_SET(dst,src)     (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_C_WIDTH             1
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_C_SHIFT             31
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_C_MASK              0x80000000U
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_C_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_C_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P3_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P3_SHIFT            3
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P3_MASK             0x00000008U
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P3_RD(src)          (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P3_SET(dst,src)     (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P2_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P2_SHIFT            2
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P2_MASK             0x00000004U
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P2_RD(src)          (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P2_SET(dst,src)     (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P1_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P1_SHIFT            1
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P1_MASK             0x00000002U
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P1_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P1_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P0_WIDTH            1
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P0_SHIFT            0
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P0_MASK             0x00000001U
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P0_RD(src)          (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMINTENSET_EL1_P0_SET(dst,src)     (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMLSR_NTT_WIDTH                    1
#define PCP_RB_CPUX_PMU_PMLSR_NTT_SHIFT                    2
#define PCP_RB_CPUX_PMU_PMLSR_NTT_MASK                     0x00000004U
#define PCP_RB_CPUX_PMU_PMLSR_NTT_RD(src)                  (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMLSR_NTT_SET(dst,src)             (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMLSR_SLK_WIDTH                    1
#define PCP_RB_CPUX_PMU_PMLSR_SLK_SHIFT                    1
#define PCP_RB_CPUX_PMU_PMLSR_SLK_MASK                     0x00000002U
#define PCP_RB_CPUX_PMU_PMLSR_SLK_RD(src)                  (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMLSR_SLK_SET(dst,src)             (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMLSR_SLI_WIDTH                    1
#define PCP_RB_CPUX_PMU_PMLSR_SLI_SHIFT                    0
#define PCP_RB_CPUX_PMU_PMLSR_SLI_MASK                     0x00000001U
#define PCP_RB_CPUX_PMU_PMLSR_SLI_RD(src)                  (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMLSR_SLI_SET(dst,src)             (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_C_WIDTH               1
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_C_SHIFT               31
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_C_MASK                0x80000000U
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_C_RD(src)             (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_C_SET(dst,src)        (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P3_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P3_SHIFT              3
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P3_MASK               0x00000008U
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P3_RD(src)            (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P3_SET(dst,src)       (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P2_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P2_SHIFT              2
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P2_MASK               0x00000004U
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P2_RD(src)            (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P2_SET(dst,src)       (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P1_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P1_SHIFT              1
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P1_MASK               0x00000002U
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P1_RD(src)            (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P1_SET(dst,src)       (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P0_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P0_MASK               0x00000001U
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P0_RD(src)            (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMOVSCLR_EL0_P0_SET(dst,src)       (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_C_WIDTH               1
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_C_SHIFT               31
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_C_MASK                0x80000000U
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_C_RD(src)             (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_C_SET(dst,src)        (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P3_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P3_SHIFT              3
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P3_MASK               0x00000008U
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P3_RD(src)            (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P3_SET(dst,src)       (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P2_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P2_SHIFT              2
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P2_MASK               0x00000004U
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P2_RD(src)            (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P2_SET(dst,src)       (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P1_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P1_SHIFT              1
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P1_MASK               0x00000002U
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P1_RD(src)            (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P1_SET(dst,src)       (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P0_WIDTH              1
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P0_MASK               0x00000001U
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P0_RD(src)            (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMOVSSET_EL0_P0_SET(dst,src)       (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_PMU_PMPIDR0_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMPIDR0_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMPIDR0_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMPIDR0_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMPIDR0_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMPIDR1_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMPIDR1_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMPIDR1_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMPIDR1_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMPIDR1_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMPIDR2_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMPIDR2_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMPIDR2_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMPIDR2_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMPIDR2_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMPIDR3_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMPIDR3_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMPIDR3_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMPIDR3_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMPIDR3_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMPIDR4_BIT31_0_WIDTH              32
#define PCP_RB_CPUX_PMU_PMPIDR4_BIT31_0_SHIFT              0
#define PCP_RB_CPUX_PMU_PMPIDR4_BIT31_0_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_PMU_PMPIDR4_BIT31_0_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_PMU_PMPIDR4_BIT31_0_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P3_WIDTH               1
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P3_SHIFT               3
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P3_MASK                0x00000008U
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P3_RD(src)             (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P3_SET(dst,src)        (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P2_WIDTH               1
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P2_SHIFT               2
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P2_MASK                0x00000004U
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P2_RD(src)             (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P2_SET(dst,src)        (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P1_WIDTH               1
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P1_SHIFT               1
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P1_MASK                0x00000002U
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P1_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P1_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P0_WIDTH               1
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P0_SHIFT               0
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P0_MASK                0x00000001U
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P0_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_PMU_PMSWINC_EL0_P0_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_RBM_RBASR_L3CAVAIL_WIDTH                    1
#define PCP_RB_RBM_RBASR_L3CAVAIL_SHIFT                    16
#define PCP_RB_RBM_RBASR_L3CAVAIL_MASK                     0x00010000U
#define PCP_RB_RBM_RBASR_L3CAVAIL_RD(src)                  (((src) & 0x00010000U)>>16)
#define PCP_RB_RBM_RBASR_L3CAVAIL_SET(dst,src)             (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_RBM_RBASR_PMDAVAIL_WIDTH                    4
#define PCP_RB_RBM_RBASR_PMDAVAIL_SHIFT                    0
#define PCP_RB_RBM_RBASR_PMDAVAIL_MASK                     0x0000000FU
#define PCP_RB_RBM_RBASR_PMDAVAIL_RD(src)                  (((src) & 0x0000000FU)>>0)
#define PCP_RB_RBM_RBASR_PMDAVAIL_SET(dst,src)             (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_RBM_RBCSR_TIMEOUT_WIDTH                     1
#define PCP_RB_RBM_RBCSR_TIMEOUT_SHIFT                     1
#define PCP_RB_RBM_RBCSR_TIMEOUT_MASK                      0x00000002U
#define PCP_RB_RBM_RBCSR_TIMEOUT_RD(src)                   (((src) & 0x00000002U)>>1)
#define PCP_RB_RBM_RBCSR_TIMEOUT_SET(dst,src)              (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_RBM_RBCSR_STICKYERR_WIDTH                   1
#define PCP_RB_RBM_RBCSR_STICKYERR_SHIFT                   0
#define PCP_RB_RBM_RBCSR_STICKYERR_MASK                    0x00000001U
#define PCP_RB_RBM_RBCSR_STICKYERR_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_RBM_RBCSR_STICKYERR_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_RBM_RBEIR_CPUDBGACCESSERR_WIDTH             1
#define PCP_RB_RBM_RBEIR_CPUDBGACCESSERR_SHIFT             31
#define PCP_RB_RBM_RBEIR_CPUDBGACCESSERR_MASK              0x80000000U
#define PCP_RB_RBM_RBEIR_CPUDBGACCESSERR_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_RBM_RBEIR_CPUDBGACCESSERR_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_RBM_RBEIR_AGENTOFFLINEERR_WIDTH             1
#define PCP_RB_RBM_RBEIR_AGENTOFFLINEERR_SHIFT             30
#define PCP_RB_RBM_RBEIR_AGENTOFFLINEERR_MASK              0x40000000U
#define PCP_RB_RBM_RBEIR_AGENTOFFLINEERR_RD(src)           (((src) & 0x40000000U)>>30)
#define PCP_RB_RBM_RBEIR_AGENTOFFLINEERR_SET(dst,src)      (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_RBM_RBEIR_UNIMPLRBPAGEERR_WIDTH             1
#define PCP_RB_RBM_RBEIR_UNIMPLRBPAGEERR_SHIFT             29
#define PCP_RB_RBM_RBEIR_UNIMPLRBPAGEERR_MASK              0x20000000U
#define PCP_RB_RBM_RBEIR_UNIMPLRBPAGEERR_RD(src)           (((src) & 0x20000000U)>>29)
#define PCP_RB_RBM_RBEIR_UNIMPLRBPAGEERR_SET(dst,src)      (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_RBM_RBEIR_WORDALIGNEDERR_WIDTH              1
#define PCP_RB_RBM_RBEIR_WORDALIGNEDERR_SHIFT              28
#define PCP_RB_RBM_RBEIR_WORDALIGNEDERR_MASK               0x10000000U
#define PCP_RB_RBM_RBEIR_WORDALIGNEDERR_RD(src)            (((src) & 0x10000000U)>>28)
#define PCP_RB_RBM_RBEIR_WORDALIGNEDERR_SET(dst,src)       (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_RBM_RBEIR_PAGEACCESSERR_WIDTH               1
#define PCP_RB_RBM_RBEIR_PAGEACCESSERR_SHIFT               27
#define PCP_RB_RBM_RBEIR_PAGEACCESSERR_MASK                0x08000000U
#define PCP_RB_RBM_RBEIR_PAGEACCESSERR_RD(src)             (((src) & 0x08000000U)>>27)
#define PCP_RB_RBM_RBEIR_PAGEACCESSERR_SET(dst,src)        (((dst) & ~0x08000000U) | (((u32)(src)<<27) & 0x08000000U))
#define PCP_RB_RBM_RBEIR_WRITE_WIDTH                       1
#define PCP_RB_RBM_RBEIR_WRITE_SHIFT                       26
#define PCP_RB_RBM_RBEIR_WRITE_MASK                        0x04000000U
#define PCP_RB_RBM_RBEIR_WRITE_RD(src)                     (((src) & 0x04000000U)>>26)
#define PCP_RB_RBM_RBEIR_WRITE_SET(dst,src)                (((dst) & ~0x04000000U) | (((u32)(src)<<26) & 0x04000000U))
#define PCP_RB_RBM_RBEIR_ADDRESS_WIDTH                     26
#define PCP_RB_RBM_RBEIR_ADDRESS_SHIFT                     0
#define PCP_RB_RBM_RBEIR_ADDRESS_MASK                      0x03FFFFFFU
#define PCP_RB_RBM_RBEIR_ADDRESS_RD(src)                   (((src) & 0x03FFFFFFU)>>0)
#define PCP_RB_RBM_RBEIR_ADDRESS_SET(dst,src)              (((dst) & ~0x03FFFFFFU) | (((u32)(src)<<0) & 0x03FFFFFFU))
#define PCP_RB_CPUX_CPU_RBER_RBE_WIDTH                     1
#define PCP_RB_CPUX_CPU_RBER_RBE_SHIFT                     0
#define PCP_RB_CPUX_CPU_RBER_RBE_MASK                      0x00000001U
#define PCP_RB_CPUX_CPU_RBER_RBE_RD(src)                   (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_CPU_RBER_RBE_SET(dst,src)              (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCACATR0_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR0_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR0_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR0_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR0_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR0_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR0_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR0_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR0_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR0_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCACATR1_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR1_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR1_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR1_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR1_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR1_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR1_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR1_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR1_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR1_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCACATR2_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR2_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR2_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR2_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR2_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR2_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR2_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR2_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR2_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR2_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCACATR3_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR3_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR3_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR3_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR3_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR3_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR3_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR3_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR3_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR3_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCACATR4_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR4_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR4_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR4_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR4_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR4_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR4_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR4_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR4_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR4_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCACATR5_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR5_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR5_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR5_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR5_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR5_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR5_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR5_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR5_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR5_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCACATR6_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR6_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR6_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR6_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR6_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR6_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR6_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR6_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR6_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR6_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCACATR7_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCACATR7_EXLEVEL_NS_SHIFT         12
#define PCP_RB_CPUX_TRC_TRCACATR7_EXLEVEL_NS_MASK          0x00007000U
#define PCP_RB_CPUX_TRC_TRCACATR7_EXLEVEL_NS_RD(src)       (((src) & 0x00007000U)>>12)
#define PCP_RB_CPUX_TRC_TRCACATR7_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00007000U) | (((u32)(src)<<12) & 0x00007000U))
#define PCP_RB_CPUX_TRC_TRCACATR7_CONTEXTTYPE_WIDTH        2
#define PCP_RB_CPUX_TRC_TRCACATR7_CONTEXTTYPE_SHIFT        2
#define PCP_RB_CPUX_TRC_TRCACATR7_CONTEXTTYPE_MASK         0x0000000CU
#define PCP_RB_CPUX_TRC_TRCACATR7_CONTEXTTYPE_RD(src)      (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCACATR7_CONTEXTTYPE_SET(dst,src) (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CSW_TRACE_TRCACKWP_OVERFLOW_WIDTH           1
#define PCP_RB_CSW_TRACE_TRCACKWP_OVERFLOW_SHIFT           31
#define PCP_RB_CSW_TRACE_TRCACKWP_OVERFLOW_MASK            0x80000000U
#define PCP_RB_CSW_TRACE_TRCACKWP_OVERFLOW_RD(src)         (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_TRCACKWP_OVERFLOW_SET(dst,src)    (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_TRCACKWP_WRPTR_WIDTH              9
#define PCP_RB_CSW_TRACE_TRCACKWP_WRPTR_SHIFT              0
#define PCP_RB_CSW_TRACE_TRCACKWP_WRPTR_MASK               0x000001FFU
#define PCP_RB_CSW_TRACE_TRCACKWP_WRPTR_RD(src)            (((src) & 0x000001FFU)>>0)
#define PCP_RB_CSW_TRACE_TRCACKWP_WRPTR_SET(dst,src)       (((dst) & ~0x000001FFU) | (((u32)(src)<<0) & 0x000001FFU))
#define PCP_RB_CPUX_TRC_TRCACVR0_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR0_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR0_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR0_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR0_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCACVR1_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR1_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR1_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR1_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR1_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCACVR2_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR2_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR2_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR2_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR2_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCACVR3_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR3_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR3_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR3_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR3_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCACVR4_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR4_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR4_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR4_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR4_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCACVR5_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR5_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR5_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR5_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR5_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCACVR6_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR6_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR6_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR6_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR6_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCACVR7_ADDRESS_WIDTH             64
#define PCP_RB_CPUX_TRC_TRCACVR7_ADDRESS_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCACVR7_ADDRESS_MASK              0xFFFFFFFFFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCACVR7_ADDRESS_RD(src)           (((src) & 0xFFFFFFFFFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCACVR7_ADDRESS_SET(dst,src)      (((dst) & ~0xFFFFFFFFFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFFFFFFFFFU))
#define PCP_RB_CSW_TRACE_TRCADDR_TRACEACC_WIDTH            1
#define PCP_RB_CSW_TRACE_TRCADDR_TRACEACC_SHIFT            31
#define PCP_RB_CSW_TRACE_TRCADDR_TRACEACC_MASK             0x80000000U
#define PCP_RB_CSW_TRACE_TRCADDR_TRACEACC_RD(src)          (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_TRCADDR_TRACEACC_SET(dst,src)     (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_TRCADDR_DATAWRPTRUPD_WIDTH        1
#define PCP_RB_CSW_TRACE_TRCADDR_DATAWRPTRUPD_SHIFT        30
#define PCP_RB_CSW_TRACE_TRCADDR_DATAWRPTRUPD_MASK         0x40000000U
#define PCP_RB_CSW_TRACE_TRCADDR_DATAWRPTRUPD_RD(src)      (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_TRCADDR_DATAWRPTRUPD_SET(dst,src) (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_TRCADDR_ACKWRPTRUPD_WIDTH         1
#define PCP_RB_CSW_TRACE_TRCADDR_ACKWRPTRUPD_SHIFT         29
#define PCP_RB_CSW_TRACE_TRCADDR_ACKWRPTRUPD_MASK          0x20000000U
#define PCP_RB_CSW_TRACE_TRCADDR_ACKWRPTRUPD_RD(src)       (((src) & 0x20000000U)>>29)
#define PCP_RB_CSW_TRACE_TRCADDR_ACKWRPTRUPD_SET(dst,src)  (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CSW_TRACE_TRCADDR_SNPWRPTRUPD_WIDTH         1
#define PCP_RB_CSW_TRACE_TRCADDR_SNPWRPTRUPD_SHIFT         28
#define PCP_RB_CSW_TRACE_TRCADDR_SNPWRPTRUPD_MASK          0x10000000U
#define PCP_RB_CSW_TRACE_TRCADDR_SNPWRPTRUPD_RD(src)       (((src) & 0x10000000U)>>28)
#define PCP_RB_CSW_TRACE_TRCADDR_SNPWRPTRUPD_SET(dst,src)  (((dst) & ~0x10000000U) | (((u32)(src)<<28) & 0x10000000U))
#define PCP_RB_CSW_TRACE_TRCADDR_VIRTREG_WIDTH             4
#define PCP_RB_CSW_TRACE_TRCADDR_VIRTREG_SHIFT             9
#define PCP_RB_CSW_TRACE_TRCADDR_VIRTREG_MASK              0x00001E00U
#define PCP_RB_CSW_TRACE_TRCADDR_VIRTREG_RD(src)           (((src) & 0x00001E00U)>>9)
#define PCP_RB_CSW_TRACE_TRCADDR_VIRTREG_SET(dst,src)      (((dst) & ~0x00001E00U) | (((u32)(src)<<9) & 0x00001E00U))
#define PCP_RB_CSW_TRACE_TRCADDR_RDPTR_WIDTH               9
#define PCP_RB_CSW_TRACE_TRCADDR_RDPTR_SHIFT               0
#define PCP_RB_CSW_TRACE_TRCADDR_RDPTR_MASK                0x000001FFU
#define PCP_RB_CSW_TRACE_TRCADDR_RDPTR_RD(src)             (((src) & 0x000001FFU)>>0)
#define PCP_RB_CSW_TRACE_TRCADDR_RDPTR_SET(dst,src)        (((dst) & ~0x000001FFU) | (((u32)(src)<<0) & 0x000001FFU))
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SNID_WIDTH           2
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SNID_SHIFT           6
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SNID_MASK            0x000000C0U
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SNID_RD(src)         (((src) & 0x000000C0U)>>6)
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SNID_SET(dst,src)    (((dst) & ~0x000000C0U) | (((u32)(src)<<6) & 0x000000C0U))
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SID_WIDTH            2
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SID_SHIFT            4
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SID_MASK             0x00000030U
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SID_RD(src)          (((src) & 0x00000030U)>>4)
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_SID_SET(dst,src)     (((dst) & ~0x00000030U) | (((u32)(src)<<4) & 0x00000030U))
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSNID_WIDTH          2
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSNID_SHIFT          2
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSNID_MASK           0x0000000CU
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSNID_RD(src)        (((src) & 0x0000000CU)>>2)
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSNID_SET(dst,src)   (((dst) & ~0x0000000CU) | (((u32)(src)<<2) & 0x0000000CU))
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSID_WIDTH           2
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSID_SHIFT           0
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSID_MASK            0x00000003U
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSID_RD(src)         (((src) & 0x00000003U)>>0)
#define PCP_RB_CPUX_TRC_TRCAUTHSTATUS_NSID_SET(dst,src)    (((dst) & ~0x00000003U) | (((u32)(src)<<0) & 0x00000003U))
#define PCP_RB_CPUX_TRC_TRCBBCTLR_MODE_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCBBCTLR_MODE_SHIFT               8
#define PCP_RB_CPUX_TRC_TRCBBCTLR_MODE_MASK                0x00000100U
#define PCP_RB_CPUX_TRC_TRCBBCTLR_MODE_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CPUX_TRC_TRCBBCTLR_MODE_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CPUX_TRC_TRCBBCTLR_RANGE_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCBBCTLR_RANGE_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCBBCTLR_RANGE_MASK               0x0000000FU
#define PCP_RB_CPUX_TRC_TRCBBCTLR_RANGE_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCBBCTLR_RANGE_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCCCCTLR_THRESHOLD_WIDTH          12
#define PCP_RB_CPUX_TRC_TRCCCCTLR_THRESHOLD_SHIFT          0
#define PCP_RB_CPUX_TRC_TRCCCCTLR_THRESHOLD_MASK           0x00000FFFU
#define PCP_RB_CPUX_TRC_TRCCCCTLR_THRESHOLD_RD(src)        (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCCCTLR_THRESHOLD_SET(dst,src)   (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_TRC_TRCCIDCCTLR0_COMP0_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCCIDCCTLR0_COMP0_SHIFT           0
#define PCP_RB_CPUX_TRC_TRCCIDCCTLR0_COMP0_MASK            0x0000000FU
#define PCP_RB_CPUX_TRC_TRCCIDCCTLR0_COMP0_RD(src)         (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCCIDCCTLR0_COMP0_SET(dst,src)    (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCCIDCVR0_VALUE_WIDTH             32
#define PCP_RB_CPUX_TRC_TRCCIDCVR0_VALUE_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCCIDCVR0_VALUE_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCCIDCVR0_VALUE_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCIDCVR0_VALUE_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCCIDR0_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCCIDR0_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCIDR0_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCCIDR0_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCIDR0_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCCIDR1_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCCIDR1_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCIDR1_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCCIDR1_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCIDR1_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCCIDR2_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCCIDR2_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCIDR2_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCCIDR2_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCIDR2_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCCIDR3_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCCIDR3_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCIDR3_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCCIDR3_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCIDR3_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCCLAIMCLR_CLR_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCCLAIMCLR_CLR_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCLAIMCLR_CLR_MASK               0x0000000FU
#define PCP_RB_CPUX_TRC_TRCCLAIMCLR_CLR_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCCLAIMCLR_CLR_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCCLAIMSET_SET_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCCLAIMSET_SET_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCLAIMSET_SET_MASK               0x0000000FU
#define PCP_RB_CPUX_TRC_TRCCLAIMSET_SET_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCCLAIMSET_SET_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTCHAIN_WIDTH         1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTCHAIN_SHIFT         17
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTCHAIN_MASK          0x00020000U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTCHAIN_RD(src)       (((src) & 0x00020000U)>>17)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTCHAIN_SET(dst,src)  (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDSELF_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDSELF_SHIFT          16
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDSELF_MASK           0x00010000U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDSELF_RD(src)        (((src) & 0x00010000U)>>16)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDSELF_SET(dst,src)   (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTTYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTTYPE_SHIFT     15
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTTYPE_MASK      0x00008000U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTTYPE_RD(src)   (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTTYPE_SET(dst,src) (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTSEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTSEL_SHIFT      8
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTSEL_MASK       0x00000F00U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTSEL_RD(src)    (((src) & 0x00000F00U)>>8)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_RLDEVENTSEL_SET(dst,src) (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTTYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTTYPE_SHIFT     7
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTTYPE_MASK      0x00000080U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTTYPE_RD(src)   (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTTYPE_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTSEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTSEL_SHIFT      0
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTSEL_MASK       0x0000000FU
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTSEL_RD(src)    (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR0_CNTEVENTSEL_SET(dst,src) (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTCHAIN_WIDTH         1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTCHAIN_SHIFT         17
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTCHAIN_MASK          0x00020000U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTCHAIN_RD(src)       (((src) & 0x00020000U)>>17)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTCHAIN_SET(dst,src)  (((dst) & ~0x00020000U) | (((u32)(src)<<17) & 0x00020000U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDSELF_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDSELF_SHIFT          16
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDSELF_MASK           0x00010000U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDSELF_RD(src)        (((src) & 0x00010000U)>>16)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDSELF_SET(dst,src)   (((dst) & ~0x00010000U) | (((u32)(src)<<16) & 0x00010000U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTTYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTTYPE_SHIFT     15
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTTYPE_MASK      0x00008000U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTTYPE_RD(src)   (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTTYPE_SET(dst,src) (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTSEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTSEL_SHIFT      8
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTSEL_MASK       0x00000F00U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTSEL_RD(src)    (((src) & 0x00000F00U)>>8)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_RLDEVENTSEL_SET(dst,src) (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTTYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTTYPE_SHIFT     7
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTTYPE_MASK      0x00000080U
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTTYPE_RD(src)   (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTTYPE_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTSEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTSEL_SHIFT      0
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTSEL_MASK       0x0000000FU
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTSEL_RD(src)    (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCCNTCTLR1_CNTEVENTSEL_SET(dst,src) (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR0_VALUE_WIDTH           16
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR0_VALUE_SHIFT           0
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR0_VALUE_MASK            0x0000FFFFU
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR0_VALUE_RD(src)         (((src) & 0x0000FFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR0_VALUE_SET(dst,src)    (((dst) & ~0x0000FFFFU) | (((u32)(src)<<0) & 0x0000FFFFU))
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR1_VALUE_WIDTH           16
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR1_VALUE_SHIFT           0
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR1_VALUE_MASK            0x0000FFFFU
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR1_VALUE_RD(src)         (((src) & 0x0000FFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCNTRLDVR1_VALUE_SET(dst,src)    (((dst) & ~0x0000FFFFU) | (((u32)(src)<<0) & 0x0000FFFFU))
#define PCP_RB_CPUX_TRC_TRCCNTVR0_VALUE_WIDTH              16
#define PCP_RB_CPUX_TRC_TRCCNTVR0_VALUE_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCNTVR0_VALUE_MASK               0x0000FFFFU
#define PCP_RB_CPUX_TRC_TRCCNTVR0_VALUE_RD(src)            (((src) & 0x0000FFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCNTVR0_VALUE_SET(dst,src)       (((dst) & ~0x0000FFFFU) | (((u32)(src)<<0) & 0x0000FFFFU))
#define PCP_RB_CPUX_TRC_TRCCNTVR1_VALUE_WIDTH              16
#define PCP_RB_CPUX_TRC_TRCCNTVR1_VALUE_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCNTVR1_VALUE_MASK               0x0000FFFFU
#define PCP_RB_CPUX_TRC_TRCCNTVR1_VALUE_RD(src)            (((src) & 0x0000FFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCCNTVR1_VALUE_SET(dst,src)       (((dst) & ~0x0000FFFFU) | (((u32)(src)<<0) & 0x0000FFFFU))
#define PCP_RB_CPUX_TRC_TRCCONFIGR_TS_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCCONFIGR_TS_SHIFT                11
#define PCP_RB_CPUX_TRC_TRCCONFIGR_TS_MASK                 0x00000800U
#define PCP_RB_CPUX_TRC_TRCCONFIGR_TS_RD(src)              (((src) & 0x00000800U)>>11)
#define PCP_RB_CPUX_TRC_TRCCONFIGR_TS_SET(dst,src)         (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CPUX_TRC_TRCCONFIGR_VMID_WIDTH              1
#define PCP_RB_CPUX_TRC_TRCCONFIGR_VMID_SHIFT              7
#define PCP_RB_CPUX_TRC_TRCCONFIGR_VMID_MASK               0x00000080U
#define PCP_RB_CPUX_TRC_TRCCONFIGR_VMID_RD(src)            (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCCONFIGR_VMID_SET(dst,src)       (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CID_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CID_SHIFT               6
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CID_MASK                0x00000040U
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CID_RD(src)             (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CID_SET(dst,src)        (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CCI_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CCI_SHIFT               4
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CCI_MASK                0x00000010U
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CCI_RD(src)             (((src) & 0x00000010U)>>4)
#define PCP_RB_CPUX_TRC_TRCCONFIGR_CCI_SET(dst,src)        (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BB_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BB_SHIFT                3
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BB_MASK                 0x00000008U
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BB_RD(src)              (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BB_SET(dst,src)         (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BIT0_WIDTH              1
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BIT0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BIT0_MASK               0x00000001U
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BIT0_RD(src)            (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCCONFIGR_BIT0_SET(dst,src)       (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_TRCCTL_TRACEEN_WIDTH              1
#define PCP_RB_CSW_TRACE_TRCCTL_TRACEEN_SHIFT              31
#define PCP_RB_CSW_TRACE_TRCCTL_TRACEEN_MASK               0x80000000U
#define PCP_RB_CSW_TRACE_TRCCTL_TRACEEN_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_TRCCTL_TRACEEN_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_TRCCTL_TWEINTEN_WIDTH             1
#define PCP_RB_CSW_TRACE_TRCCTL_TWEINTEN_SHIFT             2
#define PCP_RB_CSW_TRACE_TRCCTL_TWEINTEN_MASK              0x00000004U
#define PCP_RB_CSW_TRACE_TRCCTL_TWEINTEN_RD(src)           (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_TRACE_TRCCTL_TWEINTEN_SET(dst,src)      (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_TRACE_TRCCTL_TAEINTEN_WIDTH             1
#define PCP_RB_CSW_TRACE_TRCCTL_TAEINTEN_SHIFT             1
#define PCP_RB_CSW_TRACE_TRCCTL_TAEINTEN_MASK              0x00000002U
#define PCP_RB_CSW_TRACE_TRCCTL_TAEINTEN_RD(src)           (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_TRCCTL_TAEINTEN_SET(dst,src)      (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_TRCCTL_TIEINTEN_WIDTH             1
#define PCP_RB_CSW_TRACE_TRCCTL_TIEINTEN_SHIFT             0
#define PCP_RB_CSW_TRACE_TRCCTL_TIEINTEN_MASK              0x00000001U
#define PCP_RB_CSW_TRACE_TRCCTL_TIEINTEN_RD(src)           (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_TRCCTL_TIEINTEN_SET(dst,src)      (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_TRCDATA_TRACEDATA_WIDTH           32
#define PCP_RB_CSW_TRACE_TRCDATA_TRACEDATA_SHIFT           0
#define PCP_RB_CSW_TRACE_TRCDATA_TRACEDATA_MASK            0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_TRCDATA_TRACEDATA_RD(src)         (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_TRCDATA_TRACEDATA_SET(dst,src)    (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_TRCDATAWP_OVERFLOW_WIDTH          1
#define PCP_RB_CSW_TRACE_TRCDATAWP_OVERFLOW_SHIFT          31
#define PCP_RB_CSW_TRACE_TRCDATAWP_OVERFLOW_MASK           0x80000000U
#define PCP_RB_CSW_TRACE_TRCDATAWP_OVERFLOW_RD(src)        (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_TRCDATAWP_OVERFLOW_SET(dst,src)   (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_TRCDATAWP_WRPTR_WIDTH             9
#define PCP_RB_CSW_TRACE_TRCDATAWP_WRPTR_SHIFT             0
#define PCP_RB_CSW_TRACE_TRCDATAWP_WRPTR_MASK              0x000001FFU
#define PCP_RB_CSW_TRACE_TRCDATAWP_WRPTR_RD(src)           (((src) & 0x000001FFU)>>0)
#define PCP_RB_CSW_TRACE_TRCDATAWP_WRPTR_SET(dst,src)      (((dst) & ~0x000001FFU) | (((u32)(src)<<0) & 0x000001FFU))
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_RES1_WIDTH              1
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_RES1_SHIFT              31
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_RES1_MASK               0x80000000U
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_RES1_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_RES1_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_U_WIDTH                 1
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_U_SHIFT                 30
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_U_MASK                  0x40000000U
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_U_RD(src)               (((src) & 0x40000000U)>>30)
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_U_SET(dst,src)          (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_MT_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_MT_SHIFT                24
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_MT_MASK                 0x01000000U
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_MT_RD(src)              (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_MT_SET(dst,src)         (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF2_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF2_SHIFT              16
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF2_MASK               0x00FF0000U
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF2_RD(src)            (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF2_SET(dst,src)       (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF1_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF1_SHIFT              8
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF1_MASK               0x0000FF00U
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF1_RD(src)            (((src) & 0x0000FF00U)>>8)
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF1_SET(dst,src)       (((dst) & ~0x0000FF00U) | (((u32)(src)<<8) & 0x0000FF00U))
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCDEVAFF0_AFF0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCDEVAFF1_AFF3_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCDEVAFF1_AFF3_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCDEVAFF1_AFF3_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCDEVAFF1_AFF3_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCDEVAFF1_AFF3_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHITECT_WIDTH         11
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHITECT_SHIFT         21
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHITECT_MASK          0xFFE00000U
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHITECT_RD(src)       (((src) & 0xFFE00000U)>>21)
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHITECT_SET(dst,src)  (((dst) & ~0xFFE00000U) | (((u32)(src)<<21) & 0xFFE00000U))
#define PCP_RB_CPUX_TRC_TRCDEVARCH_PRESENT_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCDEVARCH_PRESENT_SHIFT           20
#define PCP_RB_CPUX_TRC_TRCDEVARCH_PRESENT_MASK            0x00100000U
#define PCP_RB_CPUX_TRC_TRCDEVARCH_PRESENT_RD(src)         (((src) & 0x00100000U)>>20)
#define PCP_RB_CPUX_TRC_TRCDEVARCH_PRESENT_SET(dst,src)    (((dst) & ~0x00100000U) | (((u32)(src)<<20) & 0x00100000U))
#define PCP_RB_CPUX_TRC_TRCDEVARCH_REVISION_WIDTH          4
#define PCP_RB_CPUX_TRC_TRCDEVARCH_REVISION_SHIFT          16
#define PCP_RB_CPUX_TRC_TRCDEVARCH_REVISION_MASK           0x000F0000U
#define PCP_RB_CPUX_TRC_TRCDEVARCH_REVISION_RD(src)        (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCDEVARCH_REVISION_SET(dst,src)   (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHVER_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHVER_SHIFT           12
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHVER_MASK            0x0000F000U
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHVER_RD(src)         (((src) & 0x0000F000U)>>12)
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHVER_SET(dst,src)    (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHPART_WIDTH          12
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHPART_SHIFT          0
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHPART_MASK           0x00000FFFU
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHPART_RD(src)        (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCDEVARCH_ARCHPART_SET(dst,src)   (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_SUB_WIDTH               4
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_SUB_SHIFT               4
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_SUB_MASK                0x000000F0U
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_SUB_RD(src)             (((src) & 0x000000F0U)>>4)
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_SUB_SET(dst,src)        (((dst) & ~0x000000F0U) | (((u32)(src)<<4) & 0x000000F0U))
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_MAJOR_WIDTH             4
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_MAJOR_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_MAJOR_MASK              0x0000000FU
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_MAJOR_RD(src)           (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCDEVTYPE_MAJOR_SET(dst,src)      (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3TYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3TYPE_SHIFT     31
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3TYPE_MASK      0x80000000U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3TYPE_RD(src)   (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3TYPE_SET(dst,src) (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3SEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3SEL_SHIFT      24
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3SEL_MASK       0x0F000000U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3SEL_RD(src)    (((src) & 0x0F000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT3SEL_SET(dst,src) (((dst) & ~0x0F000000U) | (((u32)(src)<<24) & 0x0F000000U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2TYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2TYPE_SHIFT     23
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2TYPE_MASK      0x00800000U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2TYPE_RD(src)   (((src) & 0x00800000U)>>23)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2TYPE_SET(dst,src) (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2SEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2SEL_SHIFT      16
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2SEL_MASK       0x000F0000U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2SEL_RD(src)    (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT2SEL_SET(dst,src) (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1TYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1TYPE_SHIFT     15
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1TYPE_MASK      0x00008000U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1TYPE_RD(src)   (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1TYPE_SET(dst,src) (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1SEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1SEL_SHIFT      8
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1SEL_MASK       0x00000F00U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1SEL_RD(src)    (((src) & 0x00000F00U)>>8)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT1SEL_SET(dst,src) (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0TYPE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0TYPE_SHIFT     7
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0TYPE_MASK      0x00000080U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0TYPE_RD(src)   (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0TYPE_SET(dst,src) (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0SEL_WIDTH      4
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0SEL_SHIFT      0
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0SEL_MASK       0x0000000FU
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0SEL_RD(src)    (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL0R_EVENT0SEL_SET(dst,src) (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_LPOVERRIDE_WIDTH     1
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_LPOVERRIDE_SHIFT     12
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_LPOVERRIDE_MASK      0x00001000U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_LPOVERRIDE_RD(src)   (((src) & 0x00001000U)>>12)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_LPOVERRIDE_SET(dst,src) (((dst) & ~0x00001000U) | (((u32)(src)<<12) & 0x00001000U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_ATB_WIDTH            1
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_ATB_SHIFT            11
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_ATB_MASK             0x00000800U
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_ATB_RD(src)          (((src) & 0x00000800U)>>11)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_ATB_SET(dst,src)     (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_INSTEN_WIDTH         4
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_INSTEN_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_INSTEN_MASK          0x0000000FU
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_INSTEN_RD(src)       (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCEVENTCTL1R_INSTEN_SET(dst,src)  (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL3_WIDTH            3
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL3_SHIFT            24
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL3_MASK             0x07000000U
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL3_RD(src)          (((src) & 0x07000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL3_SET(dst,src)     (((dst) & ~0x07000000U) | (((u32)(src)<<24) & 0x07000000U))
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL2_WIDTH            3
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL2_SHIFT            16
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL2_MASK             0x00070000U
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL2_RD(src)          (((src) & 0x00070000U)>>16)
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL2_SET(dst,src)     (((dst) & ~0x00070000U) | (((u32)(src)<<16) & 0x00070000U))
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL1_WIDTH            3
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL1_SHIFT            8
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL1_MASK             0x00000700U
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL1_RD(src)          (((src) & 0x00000700U)>>8)
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL1_SET(dst,src)     (((dst) & ~0x00000700U) | (((u32)(src)<<8) & 0x00000700U))
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL0_WIDTH            3
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL0_SHIFT            0
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL0_MASK             0x00000007U
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL0_RD(src)          (((src) & 0x00000007U)>>0)
#define PCP_RB_CPUX_TRC_TRCEXTINSELR_SEL0_SET(dst,src)     (((dst) & ~0x00000007U) | (((u32)(src)<<0) & 0x00000007U))
#define PCP_RB_CPUX_TRC_TRCIDR0_COMMOPT_WIDTH              1
#define PCP_RB_CPUX_TRC_TRCIDR0_COMMOPT_SHIFT              29
#define PCP_RB_CPUX_TRC_TRCIDR0_COMMOPT_MASK               0x20000000U
#define PCP_RB_CPUX_TRC_TRCIDR0_COMMOPT_RD(src)            (((src) & 0x20000000U)>>29)
#define PCP_RB_CPUX_TRC_TRCIDR0_COMMOPT_SET(dst,src)       (((dst) & ~0x20000000U) | (((u32)(src)<<29) & 0x20000000U))
#define PCP_RB_CPUX_TRC_TRCIDR0_TSSIZE_WIDTH               5
#define PCP_RB_CPUX_TRC_TRCIDR0_TSSIZE_SHIFT               24
#define PCP_RB_CPUX_TRC_TRCIDR0_TSSIZE_MASK                0x1F000000U
#define PCP_RB_CPUX_TRC_TRCIDR0_TSSIZE_RD(src)             (((src) & 0x1F000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCIDR0_TSSIZE_SET(dst,src)        (((dst) & ~0x1F000000U) | (((u32)(src)<<24) & 0x1F000000U))
#define PCP_RB_CPUX_TRC_TRCIDR0_NUMEVENT_WIDTH             2
#define PCP_RB_CPUX_TRC_TRCIDR0_NUMEVENT_SHIFT             10
#define PCP_RB_CPUX_TRC_TRCIDR0_NUMEVENT_MASK              0x00000C00U
#define PCP_RB_CPUX_TRC_TRCIDR0_NUMEVENT_RD(src)           (((src) & 0x00000C00U)>>10)
#define PCP_RB_CPUX_TRC_TRCIDR0_NUMEVENT_SET(dst,src)      (((dst) & ~0x00000C00U) | (((u32)(src)<<10) & 0x00000C00U))
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCCI_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCCI_SHIFT               7
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCCI_MASK                0x00000080U
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCCI_RD(src)             (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCCI_SET(dst,src)        (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCOND_WIDTH              1
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCOND_SHIFT              6
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCOND_MASK               0x00000040U
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCOND_RD(src)            (((src) & 0x00000040U)>>6)
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCCOND_SET(dst,src)       (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCBB_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCBB_SHIFT                5
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCBB_MASK                 0x00000020U
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCBB_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCBB_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCDATA_WIDTH              2
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCDATA_SHIFT              3
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCDATA_MASK               0x00000018U
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCDATA_RD(src)            (((src) & 0x00000018U)>>3)
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCDATA_SET(dst,src)       (((dst) & ~0x00000018U) | (((u32)(src)<<3) & 0x00000018U))
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCINSTP0_WIDTH            2
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCINSTP0_SHIFT            1
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCINSTP0_MASK             0x00000006U
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCINSTP0_RD(src)          (((src) & 0x00000006U)>>1)
#define PCP_RB_CPUX_TRC_TRCIDR0_TRCINSTP0_SET(dst,src)     (((dst) & ~0x00000006U) | (((u32)(src)<<1) & 0x00000006U))
#define PCP_RB_CPUX_TRC_TRCIDR0_BIT0_WIDTH                 1
#define PCP_RB_CPUX_TRC_TRCIDR0_BIT0_SHIFT                 0
#define PCP_RB_CPUX_TRC_TRCIDR0_BIT0_MASK                  0x00000001U
#define PCP_RB_CPUX_TRC_TRCIDR0_BIT0_RD(src)               (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR0_BIT0_SET(dst,src)          (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCIDR1_DESIGNER_WIDTH             8
#define PCP_RB_CPUX_TRC_TRCIDR1_DESIGNER_SHIFT             24
#define PCP_RB_CPUX_TRC_TRCIDR1_DESIGNER_MASK              0xFF000000U
#define PCP_RB_CPUX_TRC_TRCIDR1_DESIGNER_RD(src)           (((src) & 0xFF000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCIDR1_DESIGNER_SET(dst,src)      (((dst) & ~0xFF000000U) | (((u32)(src)<<24) & 0xFF000000U))
#define PCP_RB_CPUX_TRC_TRCIDR1_BITS_15_12_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCIDR1_BITS_15_12_SHIFT           12
#define PCP_RB_CPUX_TRC_TRCIDR1_BITS_15_12_MASK            0x0000F000U
#define PCP_RB_CPUX_TRC_TRCIDR1_BITS_15_12_RD(src)         (((src) & 0x0000F000U)>>12)
#define PCP_RB_CPUX_TRC_TRCIDR1_BITS_15_12_SET(dst,src)    (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMAJ_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMAJ_SHIFT           8
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMAJ_MASK            0x00000F00U
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMAJ_RD(src)         (((src) & 0x00000F00U)>>8)
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMAJ_SET(dst,src)    (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMIN_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMIN_SHIFT           4
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMIN_MASK            0x000000F0U
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMIN_RD(src)         (((src) & 0x000000F0U)>>4)
#define PCP_RB_CPUX_TRC_TRCIDR1_TRCARCHMIN_SET(dst,src)    (((dst) & ~0x000000F0U) | (((u32)(src)<<4) & 0x000000F0U))
#define PCP_RB_CPUX_TRC_TRCIDR1_REVISION_WIDTH             4
#define PCP_RB_CPUX_TRC_TRCIDR1_REVISION_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCIDR1_REVISION_MASK              0x0000000FU
#define PCP_RB_CPUX_TRC_TRCIDR1_REVISION_RD(src)           (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR1_REVISION_SET(dst,src)      (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCIDR10_NUMP1KEY_WIDTH            32
#define PCP_RB_CPUX_TRC_TRCIDR10_NUMP1KEY_SHIFT            0
#define PCP_RB_CPUX_TRC_TRCIDR10_NUMP1KEY_MASK             0xFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCIDR10_NUMP1KEY_RD(src)          (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR10_NUMP1KEY_SET(dst,src)     (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCIDR11_NUMP1SPC_WIDTH            32
#define PCP_RB_CPUX_TRC_TRCIDR11_NUMP1SPC_SHIFT            0
#define PCP_RB_CPUX_TRC_TRCIDR11_NUMP1SPC_MASK             0xFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCIDR11_NUMP1SPC_RD(src)          (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR11_NUMP1SPC_SET(dst,src)     (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCIDR12_NUMCONDKEY_WIDTH          32
#define PCP_RB_CPUX_TRC_TRCIDR12_NUMCONDKEY_SHIFT          0
#define PCP_RB_CPUX_TRC_TRCIDR12_NUMCONDKEY_MASK           0xFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCIDR12_NUMCONDKEY_RD(src)        (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR12_NUMCONDKEY_SET(dst,src)   (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCIDR13_NUMCONDSPC_WIDTH          32
#define PCP_RB_CPUX_TRC_TRCIDR13_NUMCONDSPC_SHIFT          0
#define PCP_RB_CPUX_TRC_TRCIDR13_NUMCONDSPC_MASK           0xFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCIDR13_NUMCONDSPC_RD(src)        (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR13_NUMCONDSPC_SET(dst,src)   (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCIDR2_VMIDSIZE_WIDTH             5
#define PCP_RB_CPUX_TRC_TRCIDR2_VMIDSIZE_SHIFT             10
#define PCP_RB_CPUX_TRC_TRCIDR2_VMIDSIZE_MASK              0x00007C00U
#define PCP_RB_CPUX_TRC_TRCIDR2_VMIDSIZE_RD(src)           (((src) & 0x00007C00U)>>10)
#define PCP_RB_CPUX_TRC_TRCIDR2_VMIDSIZE_SET(dst,src)      (((dst) & ~0x00007C00U) | (((u32)(src)<<10) & 0x00007C00U))
#define PCP_RB_CPUX_TRC_TRCIDR2_CIDSIZE_WIDTH              5
#define PCP_RB_CPUX_TRC_TRCIDR2_CIDSIZE_SHIFT              5
#define PCP_RB_CPUX_TRC_TRCIDR2_CIDSIZE_MASK               0x000003E0U
#define PCP_RB_CPUX_TRC_TRCIDR2_CIDSIZE_RD(src)            (((src) & 0x000003E0U)>>5)
#define PCP_RB_CPUX_TRC_TRCIDR2_CIDSIZE_SET(dst,src)       (((dst) & ~0x000003E0U) | (((u32)(src)<<5) & 0x000003E0U))
#define PCP_RB_CPUX_TRC_TRCIDR2_IASIZE_WIDTH               5
#define PCP_RB_CPUX_TRC_TRCIDR2_IASIZE_SHIFT               0
#define PCP_RB_CPUX_TRC_TRCIDR2_IASIZE_MASK                0x0000001FU
#define PCP_RB_CPUX_TRC_TRCIDR2_IASIZE_RD(src)             (((src) & 0x0000001FU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR2_IASIZE_SET(dst,src)        (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CPUX_TRC_TRCIDR3_TRCERR_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCIDR3_TRCERR_SHIFT               24
#define PCP_RB_CPUX_TRC_TRCIDR3_TRCERR_MASK                0x01000000U
#define PCP_RB_CPUX_TRC_TRCIDR3_TRCERR_RD(src)             (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCIDR3_TRCERR_SET(dst,src)        (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_NS_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_NS_SHIFT           20
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_NS_MASK            0x00F00000U
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_NS_RD(src)         (((src) & 0x00F00000U)>>20)
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_NS_SET(dst,src)    (((dst) & ~0x00F00000U) | (((u32)(src)<<20) & 0x00F00000U))
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_S_WIDTH            4
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_S_SHIFT            16
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_S_MASK             0x000F0000U
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_S_RD(src)          (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCIDR3_EXLEVEL_S_SET(dst,src)     (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_TRC_TRCIDR3_CCITMIN_WIDTH              12
#define PCP_RB_CPUX_TRC_TRCIDR3_CCITMIN_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCIDR3_CCITMIN_MASK               0x00000FFFU
#define PCP_RB_CPUX_TRC_TRCIDR3_CCITMIN_RD(src)            (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR3_CCITMIN_SET(dst,src)       (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMVMIDC_WIDTH             4
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMVMIDC_SHIFT             28
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMVMIDC_MASK              0xF0000000U
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMVMIDC_RD(src)           (((src) & 0xF0000000U)>>28)
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMVMIDC_SET(dst,src)      (((dst) & ~0xF0000000U) | (((u32)(src)<<28) & 0xF0000000U))
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMCIDC_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMCIDC_SHIFT              24
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMCIDC_MASK               0x0F000000U
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMCIDC_RD(src)            (((src) & 0x0F000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMCIDC_SET(dst,src)       (((dst) & ~0x0F000000U) | (((u32)(src)<<24) & 0x0F000000U))
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMSSCC_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMSSCC_SHIFT              20
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMSSCC_MASK               0x00F00000U
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMSSCC_RD(src)            (((src) & 0x00F00000U)>>20)
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMSSCC_SET(dst,src)       (((dst) & ~0x00F00000U) | (((u32)(src)<<20) & 0x00F00000U))
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMRSPAIR_WIDTH            4
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMRSPAIR_SHIFT            16
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMRSPAIR_MASK             0x000F0000U
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMRSPAIR_RD(src)          (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMRSPAIR_SET(dst,src)     (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMPC_WIDTH                4
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMPC_SHIFT                12
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMPC_MASK                 0x0000F000U
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMPC_RD(src)              (((src) & 0x0000F000U)>>12)
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMPC_SET(dst,src)         (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMACPAIRS_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMACPAIRS_SHIFT           0
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMACPAIRS_MASK            0x0000000FU
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMACPAIRS_RD(src)         (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR4_NUMACPAIRS_SET(dst,src)    (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMCNTR_WIDTH              3
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMCNTR_SHIFT              28
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMCNTR_MASK               0x70000000U
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMCNTR_RD(src)            (((src) & 0x70000000U)>>28)
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMCNTR_SET(dst,src)       (((dst) & ~0x70000000U) | (((u32)(src)<<28) & 0x70000000U))
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMSEQSTATE_WIDTH          3
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMSEQSTATE_SHIFT          25
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMSEQSTATE_MASK           0x0E000000U
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMSEQSTATE_RD(src)        (((src) & 0x0E000000U)>>25)
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMSEQSTATE_SET(dst,src)   (((dst) & ~0x0E000000U) | (((u32)(src)<<25) & 0x0E000000U))
#define PCP_RB_CPUX_TRC_TRCIDR5_LPOVERRIDE_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCIDR5_LPOVERRIDE_SHIFT           23
#define PCP_RB_CPUX_TRC_TRCIDR5_LPOVERRIDE_MASK            0x00800000U
#define PCP_RB_CPUX_TRC_TRCIDR5_LPOVERRIDE_RD(src)         (((src) & 0x00800000U)>>23)
#define PCP_RB_CPUX_TRC_TRCIDR5_LPOVERRIDE_SET(dst,src)    (((dst) & ~0x00800000U) | (((u32)(src)<<23) & 0x00800000U))
#define PCP_RB_CPUX_TRC_TRCIDR5_ATBTRIG_WIDTH              1
#define PCP_RB_CPUX_TRC_TRCIDR5_ATBTRIG_SHIFT              22
#define PCP_RB_CPUX_TRC_TRCIDR5_ATBTRIG_MASK               0x00400000U
#define PCP_RB_CPUX_TRC_TRCIDR5_ATBTRIG_RD(src)            (((src) & 0x00400000U)>>22)
#define PCP_RB_CPUX_TRC_TRCIDR5_ATBTRIG_SET(dst,src)       (((dst) & ~0x00400000U) | (((u32)(src)<<22) & 0x00400000U))
#define PCP_RB_CPUX_TRC_TRCIDR5_TRACEIDSIZE_WIDTH          6
#define PCP_RB_CPUX_TRC_TRCIDR5_TRACEIDSIZE_SHIFT          16
#define PCP_RB_CPUX_TRC_TRCIDR5_TRACEIDSIZE_MASK           0x003F0000U
#define PCP_RB_CPUX_TRC_TRCIDR5_TRACEIDSIZE_RD(src)        (((src) & 0x003F0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCIDR5_TRACEIDSIZE_SET(dst,src)   (((dst) & ~0x003F0000U) | (((u32)(src)<<16) & 0x003F0000U))
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTINSEL_WIDTH          3
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTINSEL_SHIFT          9
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTINSEL_MASK           0x00000E00U
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTINSEL_RD(src)        (((src) & 0x00000E00U)>>9)
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTINSEL_SET(dst,src)   (((dst) & ~0x00000E00U) | (((u32)(src)<<9) & 0x00000E00U))
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTIN_WIDTH             9
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTIN_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTIN_MASK              0x000001FFU
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTIN_RD(src)           (((src) & 0x000001FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR5_NUMEXTIN_SET(dst,src)      (((dst) & ~0x000001FFU) | (((u32)(src)<<0) & 0x000001FFU))
#define PCP_RB_CPUX_TRC_TRCIDR8_MAXSPEC_WIDTH              32
#define PCP_RB_CPUX_TRC_TRCIDR8_MAXSPEC_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCIDR8_MAXSPEC_MASK               0xFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCIDR8_MAXSPEC_RD(src)            (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR8_MAXSPEC_SET(dst,src)       (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CPUX_TRC_TRCIDR9_NUMP0KEY_WIDTH             32
#define PCP_RB_CPUX_TRC_TRCIDR9_NUMP0KEY_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCIDR9_NUMP0KEY_MASK              0xFFFFFFFFU
#define PCP_RB_CPUX_TRC_TRCIDR9_NUMP0KEY_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCIDR9_NUMP0KEY_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_TRCINT_TWEINT_WIDTH               1
#define PCP_RB_CSW_TRACE_TRCINT_TWEINT_SHIFT               2
#define PCP_RB_CSW_TRACE_TRCINT_TWEINT_MASK                0x00000004U
#define PCP_RB_CSW_TRACE_TRCINT_TWEINT_RD(src)             (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_TRACE_TRCINT_TWEINT_SET(dst,src)        (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_TRACE_TRCINT_TAEINT_WIDTH               1
#define PCP_RB_CSW_TRACE_TRCINT_TAEINT_SHIFT               1
#define PCP_RB_CSW_TRACE_TRCINT_TAEINT_MASK                0x00000002U
#define PCP_RB_CSW_TRACE_TRCINT_TAEINT_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_TRCINT_TAEINT_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_TRCINT_TIEINT_WIDTH               1
#define PCP_RB_CSW_TRACE_TRCINT_TIEINT_SHIFT               0
#define PCP_RB_CSW_TRACE_TRCINT_TIEINT_MASK                0x00000001U
#define PCP_RB_CSW_TRACE_TRCINT_TIEINT_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_TRCINT_TIEINT_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCITCTRL_IME_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCITCTRL_IME_SHIFT                0
#define PCP_RB_CPUX_TRC_TRCITCTRL_IME_MASK                 0x00000001U
#define PCP_RB_CPUX_TRC_TRCITCTRL_IME_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCITCTRL_IME_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCLSR_NTT_WIDTH                   1
#define PCP_RB_CPUX_TRC_TRCLSR_NTT_SHIFT                   2
#define PCP_RB_CPUX_TRC_TRCLSR_NTT_MASK                    0x00000004U
#define PCP_RB_CPUX_TRC_TRCLSR_NTT_RD(src)                 (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_TRC_TRCLSR_NTT_SET(dst,src)            (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_TRC_TRCLSR_SLK_WIDTH                   1
#define PCP_RB_CPUX_TRC_TRCLSR_SLK_SHIFT                   1
#define PCP_RB_CPUX_TRC_TRCLSR_SLK_MASK                    0x00000002U
#define PCP_RB_CPUX_TRC_TRCLSR_SLK_RD(src)                 (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_TRC_TRCLSR_SLK_SET(dst,src)            (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_TRC_TRCLSR_SLI_WIDTH                   1
#define PCP_RB_CPUX_TRC_TRCLSR_SLI_SHIFT                   0
#define PCP_RB_CPUX_TRC_TRCLSR_SLI_MASK                    0x00000001U
#define PCP_RB_CPUX_TRC_TRCLSR_SLI_RD(src)                 (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCLSR_SLI_SET(dst,src)            (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCOSLAR_OSLK_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCOSLAR_OSLK_SHIFT                0
#define PCP_RB_CPUX_TRC_TRCOSLAR_OSLK_MASK                 0x00000001U
#define PCP_RB_CPUX_TRC_TRCOSLAR_OSLK_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCOSLAR_OSLK_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM1_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM1_SHIFT               3
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM1_MASK                0x00000008U
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM1_RD(src)             (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM1_SET(dst,src)        (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_TRC_TRCOSLSR_NTT_WIDTH                 1
#define PCP_RB_CPUX_TRC_TRCOSLSR_NTT_SHIFT                 2
#define PCP_RB_CPUX_TRC_TRCOSLSR_NTT_MASK                  0x00000004U
#define PCP_RB_CPUX_TRC_TRCOSLSR_NTT_RD(src)               (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_TRC_TRCOSLSR_NTT_SET(dst,src)          (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLK_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLK_SHIFT                1
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLK_MASK                 0x00000002U
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLK_RD(src)              (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLK_SET(dst,src)         (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM0_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM0_SHIFT               0
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM0_MASK                0x00000001U
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM0_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCOSLSR_OSLM0_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCPDCR_PU_WIDTH                   1
#define PCP_RB_CPUX_TRC_TRCPDCR_PU_SHIFT                   3
#define PCP_RB_CPUX_TRC_TRCPDCR_PU_MASK                    0x00000008U
#define PCP_RB_CPUX_TRC_TRCPDCR_PU_RD(src)                 (((src) & 0x00000008U)>>3)
#define PCP_RB_CPUX_TRC_TRCPDCR_PU_SET(dst,src)            (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CPUX_TRC_TRCPDSR_OSLK_WIDTH                 1
#define PCP_RB_CPUX_TRC_TRCPDSR_OSLK_SHIFT                 5
#define PCP_RB_CPUX_TRC_TRCPDSR_OSLK_MASK                  0x00000020U
#define PCP_RB_CPUX_TRC_TRCPDSR_OSLK_RD(src)               (((src) & 0x00000020U)>>5)
#define PCP_RB_CPUX_TRC_TRCPDSR_OSLK_SET(dst,src)          (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CPUX_TRC_TRCPDSR_STICKYPD_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCPDSR_STICKYPD_SHIFT             1
#define PCP_RB_CPUX_TRC_TRCPDSR_STICKYPD_MASK              0x00000002U
#define PCP_RB_CPUX_TRC_TRCPDSR_STICKYPD_RD(src)           (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_TRC_TRCPDSR_STICKYPD_SET(dst,src)      (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_TRC_TRCPDSR_POWER_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCPDSR_POWER_SHIFT                0
#define PCP_RB_CPUX_TRC_TRCPDSR_POWER_MASK                 0x00000001U
#define PCP_RB_CPUX_TRC_TRCPDSR_POWER_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCPDSR_POWER_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCPIDR0_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCPIDR0_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCPIDR0_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCPIDR0_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCPIDR0_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCPIDR1_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCPIDR1_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCPIDR1_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCPIDR1_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCPIDR1_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCPIDR2_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCPIDR2_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCPIDR2_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCPIDR2_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCPIDR2_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCPIDR3_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCPIDR3_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCPIDR3_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCPIDR3_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCPIDR3_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCPIDR4_BIT7_0_WIDTH              8
#define PCP_RB_CPUX_TRC_TRCPIDR4_BIT7_0_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCPIDR4_BIT7_0_MASK               0x000000FFU
#define PCP_RB_CPUX_TRC_TRCPIDR4_BIT7_0_RD(src)            (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCPIDR4_BIT7_0_SET(dst,src)       (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCPRGCTLR_EN_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCPRGCTLR_EN_SHIFT                0
#define PCP_RB_CPUX_TRC_TRCPRGCTLR_EN_MASK                 0x00000001U
#define PCP_RB_CPUX_TRC_TRCPRGCTLR_EN_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCPRGCTLR_EN_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_PAIRINV_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_PAIRINV_SHIFT          21
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_PAIRINV_MASK           0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_PAIRINV_RD(src)        (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_PAIRINV_SET(dst,src)   (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_BITS_20_0_WIDTH        21
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_BITS_20_0_SHIFT        0
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_BITS_20_0_MASK         0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_BITS_20_0_RD(src)      (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR10_BITS_20_0_SET(dst,src) (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_PAIRINV_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_PAIRINV_SHIFT          21
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_PAIRINV_MASK           0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_PAIRINV_RD(src)        (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_PAIRINV_SET(dst,src)   (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_BITS_20_0_WIDTH        21
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_BITS_20_0_SHIFT        0
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_BITS_20_0_MASK         0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_BITS_20_0_RD(src)      (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR11_BITS_20_0_SET(dst,src) (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_PAIRINV_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_PAIRINV_SHIFT          21
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_PAIRINV_MASK           0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_PAIRINV_RD(src)        (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_PAIRINV_SET(dst,src)   (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_BITS_20_0_WIDTH        21
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_BITS_20_0_SHIFT        0
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_BITS_20_0_MASK         0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_BITS_20_0_RD(src)      (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR12_BITS_20_0_SET(dst,src) (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_PAIRINV_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_PAIRINV_SHIFT          21
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_PAIRINV_MASK           0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_PAIRINV_RD(src)        (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_PAIRINV_SET(dst,src)   (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_BITS_20_0_WIDTH        21
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_BITS_20_0_SHIFT        0
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_BITS_20_0_MASK         0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_BITS_20_0_RD(src)      (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR13_BITS_20_0_SET(dst,src) (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_PAIRINV_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_PAIRINV_SHIFT          21
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_PAIRINV_MASK           0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_PAIRINV_RD(src)        (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_PAIRINV_SET(dst,src)   (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_BITS_20_0_WIDTH        21
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_BITS_20_0_SHIFT        0
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_BITS_20_0_MASK         0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_BITS_20_0_RD(src)      (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR14_BITS_20_0_SET(dst,src) (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_PAIRINV_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_PAIRINV_SHIFT          21
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_PAIRINV_MASK           0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_PAIRINV_RD(src)        (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_PAIRINV_SET(dst,src)   (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_BITS_20_0_WIDTH        21
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_BITS_20_0_SHIFT        0
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_BITS_20_0_MASK         0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_BITS_20_0_RD(src)      (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR15_BITS_20_0_SET(dst,src) (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR2_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR3_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR4_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR5_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR6_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR7_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR8_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_PAIRINV_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_PAIRINV_SHIFT           21
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_PAIRINV_MASK            0x00200000U
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_PAIRINV_RD(src)         (((src) & 0x00200000U)>>21)
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_PAIRINV_SET(dst,src)    (((dst) & ~0x00200000U) | (((u32)(src)<<21) & 0x00200000U))
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_BITS_20_0_WIDTH         21
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_BITS_20_0_SHIFT         0
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_BITS_20_0_MASK          0x001FFFFFU
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_BITS_20_0_RD(src)       (((src) & 0x001FFFFFU)>>0)
#define PCP_RB_CPUX_TRC_TRCRSCTLR9_BITS_20_0_SET(dst,src)  (((dst) & ~0x001FFFFFU) | (((u32)(src)<<0) & 0x001FFFFFU))
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BTYPE_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BTYPE_SHIFT             15
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BTYPE_MASK              0x00008000U
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BTYPE_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BTYPE_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BSEL_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BSEL_SHIFT              8
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BSEL_MASK               0x00000F00U
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BSEL_RD(src)            (((src) & 0x00000F00U)>>8)
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_BSEL_SET(dst,src)       (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FTYPE_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FTYPE_SHIFT             7
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FTYPE_MASK              0x00000080U
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FTYPE_RD(src)           (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FTYPE_SET(dst,src)      (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FSEL_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FSEL_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FSEL_MASK               0x0000000FU
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FSEL_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCSEQEVR0_FSEL_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BTYPE_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BTYPE_SHIFT             15
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BTYPE_MASK              0x00008000U
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BTYPE_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BTYPE_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BSEL_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BSEL_SHIFT              8
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BSEL_MASK               0x00000F00U
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BSEL_RD(src)            (((src) & 0x00000F00U)>>8)
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_BSEL_SET(dst,src)       (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FTYPE_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FTYPE_SHIFT             7
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FTYPE_MASK              0x00000080U
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FTYPE_RD(src)           (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FTYPE_SET(dst,src)      (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FSEL_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FSEL_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FSEL_MASK               0x0000000FU
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FSEL_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCSEQEVR1_FSEL_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BTYPE_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BTYPE_SHIFT             15
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BTYPE_MASK              0x00008000U
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BTYPE_RD(src)           (((src) & 0x00008000U)>>15)
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BTYPE_SET(dst,src)      (((dst) & ~0x00008000U) | (((u32)(src)<<15) & 0x00008000U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BSEL_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BSEL_SHIFT              8
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BSEL_MASK               0x00000F00U
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BSEL_RD(src)            (((src) & 0x00000F00U)>>8)
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_BSEL_SET(dst,src)       (((dst) & ~0x00000F00U) | (((u32)(src)<<8) & 0x00000F00U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FTYPE_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FTYPE_SHIFT             7
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FTYPE_MASK              0x00000080U
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FTYPE_RD(src)           (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FTYPE_SET(dst,src)      (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FSEL_WIDTH              4
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FSEL_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FSEL_MASK               0x0000000FU
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FSEL_RD(src)            (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCSEQEVR2_FSEL_SET(dst,src)       (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTTYPE_WIDTH         1
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTTYPE_SHIFT         7
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTTYPE_MASK          0x00000080U
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTTYPE_RD(src)       (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTTYPE_SET(dst,src)  (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTSEL_WIDTH          4
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTSEL_SHIFT          0
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTSEL_MASK           0x0000000FU
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTSEL_RD(src)        (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCSEQRSTEVR_RSTSEL_SET(dst,src)   (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCSEQSTR_STATE_WIDTH              2
#define PCP_RB_CPUX_TRC_TRCSEQSTR_STATE_SHIFT              0
#define PCP_RB_CPUX_TRC_TRCSEQSTR_STATE_MASK               0x00000003U
#define PCP_RB_CPUX_TRC_TRCSEQSTR_STATE_RD(src)            (((src) & 0x00000003U)>>0)
#define PCP_RB_CPUX_TRC_TRCSEQSTR_STATE_SET(dst,src)       (((dst) & ~0x00000003U) | (((u32)(src)<<0) & 0x00000003U))
#define PCP_RB_CSW_TRACE_TRCSNPWP_OVERFLOW_WIDTH           1
#define PCP_RB_CSW_TRACE_TRCSNPWP_OVERFLOW_SHIFT           31
#define PCP_RB_CSW_TRACE_TRCSNPWP_OVERFLOW_MASK            0x80000000U
#define PCP_RB_CSW_TRACE_TRCSNPWP_OVERFLOW_RD(src)         (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_TRCSNPWP_OVERFLOW_SET(dst,src)    (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_TRCSNPWP_WRPTR_WIDTH              9
#define PCP_RB_CSW_TRACE_TRCSNPWP_WRPTR_SHIFT              0
#define PCP_RB_CSW_TRACE_TRCSNPWP_WRPTR_MASK               0x000001FFU
#define PCP_RB_CSW_TRACE_TRCSNPWP_WRPTR_RD(src)            (((src) & 0x000001FFU)>>0)
#define PCP_RB_CSW_TRACE_TRCSNPWP_WRPTR_SET(dst,src)       (((dst) & ~0x000001FFU) | (((u32)(src)<<0) & 0x000001FFU))
#define PCP_RB_CPUX_TRC_TRCSSCCR0_RST_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCSSCCR0_RST_SHIFT                24
#define PCP_RB_CPUX_TRC_TRCSSCCR0_RST_MASK                 0x01000000U
#define PCP_RB_CPUX_TRC_TRCSSCCR0_RST_RD(src)              (((src) & 0x01000000U)>>24)
#define PCP_RB_CPUX_TRC_TRCSSCCR0_RST_SET(dst,src)         (((dst) & ~0x01000000U) | (((u32)(src)<<24) & 0x01000000U))
#define PCP_RB_CPUX_TRC_TRCSSCCR0_ARC_WIDTH                4
#define PCP_RB_CPUX_TRC_TRCSSCCR0_ARC_SHIFT                16
#define PCP_RB_CPUX_TRC_TRCSSCCR0_ARC_MASK                 0x000F0000U
#define PCP_RB_CPUX_TRC_TRCSSCCR0_ARC_RD(src)              (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCSSCCR0_ARC_SET(dst,src)         (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_TRC_TRCSSCCR0_SAC_WIDTH                8
#define PCP_RB_CPUX_TRC_TRCSSCCR0_SAC_SHIFT                0
#define PCP_RB_CPUX_TRC_TRCSSCCR0_SAC_MASK                 0x000000FFU
#define PCP_RB_CPUX_TRC_TRCSSCCR0_SAC_RD(src)              (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCSSCCR0_SAC_SET(dst,src)         (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCSSCSR0_STATUS_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCSSCSR0_STATUS_SHIFT             31
#define PCP_RB_CPUX_TRC_TRCSSCSR0_STATUS_MASK              0x80000000U
#define PCP_RB_CPUX_TRC_TRCSSCSR0_STATUS_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CPUX_TRC_TRCSSCSR0_STATUS_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DV_WIDTH                 1
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DV_SHIFT                 2
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DV_MASK                  0x00000004U
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DV_RD(src)               (((src) & 0x00000004U)>>2)
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DV_SET(dst,src)          (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DA_WIDTH                 1
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DA_SHIFT                 1
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DA_MASK                  0x00000002U
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DA_RD(src)               (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_TRC_TRCSSCSR0_DA_SET(dst,src)          (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_TRC_TRCSSCSR0_INST_WIDTH               1
#define PCP_RB_CPUX_TRC_TRCSSCSR0_INST_SHIFT               0
#define PCP_RB_CPUX_TRC_TRCSSCSR0_INST_MASK                0x00000001U
#define PCP_RB_CPUX_TRC_TRCSSCSR0_INST_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCSSCSR0_INST_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_TRCST_TRACEACTIVE_WIDTH           1
#define PCP_RB_CSW_TRACE_TRCST_TRACEACTIVE_SHIFT           31
#define PCP_RB_CSW_TRACE_TRCST_TRACEACTIVE_MASK            0x80000000U
#define PCP_RB_CSW_TRACE_TRCST_TRACEACTIVE_RD(src)         (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_TRCST_TRACEACTIVE_SET(dst,src)    (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_TRCST_TRACECOUNT_WIDTH            12
#define PCP_RB_CSW_TRACE_TRCST_TRACECOUNT_SHIFT            0
#define PCP_RB_CSW_TRACE_TRCST_TRACECOUNT_MASK             0x00000FFFU
#define PCP_RB_CSW_TRACE_TRCST_TRACECOUNT_RD(src)          (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_TRCST_TRACECOUNT_SET(dst,src)     (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CPUX_TRC_TRCSTATR_PMSTABLE_WIDTH            1
#define PCP_RB_CPUX_TRC_TRCSTATR_PMSTABLE_SHIFT            1
#define PCP_RB_CPUX_TRC_TRCSTATR_PMSTABLE_MASK             0x00000002U
#define PCP_RB_CPUX_TRC_TRCSTATR_PMSTABLE_RD(src)          (((src) & 0x00000002U)>>1)
#define PCP_RB_CPUX_TRC_TRCSTATR_PMSTABLE_SET(dst,src)     (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CPUX_TRC_TRCSTATR_IDLE_WIDTH                1
#define PCP_RB_CPUX_TRC_TRCSTATR_IDLE_SHIFT                0
#define PCP_RB_CPUX_TRC_TRCSTATR_IDLE_MASK                 0x00000001U
#define PCP_RB_CPUX_TRC_TRCSTATR_IDLE_RD(src)              (((src) & 0x00000001U)>>0)
#define PCP_RB_CPUX_TRC_TRCSTATR_IDLE_SET(dst,src)         (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CPUX_TRC_TRCSYNCPR_PERIOD_WIDTH             5
#define PCP_RB_CPUX_TRC_TRCSYNCPR_PERIOD_SHIFT             0
#define PCP_RB_CPUX_TRC_TRCSYNCPR_PERIOD_MASK              0x0000001FU
#define PCP_RB_CPUX_TRC_TRCSYNCPR_PERIOD_RD(src)           (((src) & 0x0000001FU)>>0)
#define PCP_RB_CPUX_TRC_TRCSYNCPR_PERIOD_SET(dst,src)      (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CPUX_TRC_TRCTRACEIDR_TRACEID_WIDTH          7
#define PCP_RB_CPUX_TRC_TRCTRACEIDR_TRACEID_SHIFT          0
#define PCP_RB_CPUX_TRC_TRCTRACEIDR_TRACEID_MASK           0x0000007FU
#define PCP_RB_CPUX_TRC_TRCTRACEIDR_TRACEID_RD(src)        (((src) & 0x0000007FU)>>0)
#define PCP_RB_CPUX_TRC_TRCTRACEIDR_TRACEID_SET(dst,src)   (((dst) & ~0x0000007FU) | (((u32)(src)<<0) & 0x0000007FU))
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTTYPE_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTTYPE_SHIFT          7
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTTYPE_MASK           0x00000080U
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTTYPE_RD(src)        (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTTYPE_SET(dst,src)   (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTSEL_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTSEL_SHIFT           0
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTSEL_MASK            0x0000000FU
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTSEL_RD(src)         (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCTSCTLR_EVENTSEL_SET(dst,src)    (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCVICTLR_EXLEVEL_NS_WIDTH         3
#define PCP_RB_CPUX_TRC_TRCVICTLR_EXLEVEL_NS_SHIFT         20
#define PCP_RB_CPUX_TRC_TRCVICTLR_EXLEVEL_NS_MASK          0x00700000U
#define PCP_RB_CPUX_TRC_TRCVICTLR_EXLEVEL_NS_RD(src)       (((src) & 0x00700000U)>>20)
#define PCP_RB_CPUX_TRC_TRCVICTLR_EXLEVEL_NS_SET(dst,src)  (((dst) & ~0x00700000U) | (((u32)(src)<<20) & 0x00700000U))
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCERR_WIDTH             1
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCERR_SHIFT             11
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCERR_MASK              0x00000800U
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCERR_RD(src)           (((src) & 0x00000800U)>>11)
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCERR_SET(dst,src)      (((dst) & ~0x00000800U) | (((u32)(src)<<11) & 0x00000800U))
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCRESET_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCRESET_SHIFT           10
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCRESET_MASK            0x00000400U
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCRESET_RD(src)         (((src) & 0x00000400U)>>10)
#define PCP_RB_CPUX_TRC_TRCVICTLR_TRCRESET_SET(dst,src)    (((dst) & ~0x00000400U) | (((u32)(src)<<10) & 0x00000400U))
#define PCP_RB_CPUX_TRC_TRCVICTLR_SSSTATUS_WIDTH           1
#define PCP_RB_CPUX_TRC_TRCVICTLR_SSSTATUS_SHIFT           9
#define PCP_RB_CPUX_TRC_TRCVICTLR_SSSTATUS_MASK            0x00000200U
#define PCP_RB_CPUX_TRC_TRCVICTLR_SSSTATUS_RD(src)         (((src) & 0x00000200U)>>9)
#define PCP_RB_CPUX_TRC_TRCVICTLR_SSSTATUS_SET(dst,src)    (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTTYPE_WIDTH          1
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTTYPE_SHIFT          7
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTTYPE_MASK           0x00000080U
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTTYPE_RD(src)        (((src) & 0x00000080U)>>7)
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTTYPE_SET(dst,src)   (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTSEL_WIDTH           4
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTSEL_SHIFT           0
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTSEL_MASK            0x0000000FU
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTSEL_RD(src)         (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCVICTLR_EVENTSEL_SET(dst,src)    (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_EXCLUDE_WIDTH          4
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_EXCLUDE_SHIFT          16
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_EXCLUDE_MASK           0x000F0000U
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_EXCLUDE_RD(src)        (((src) & 0x000F0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_EXCLUDE_SET(dst,src)   (((dst) & ~0x000F0000U) | (((u32)(src)<<16) & 0x000F0000U))
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_INCLUDE_WIDTH          4
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_INCLUDE_SHIFT          0
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_INCLUDE_MASK           0x0000000FU
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_INCLUDE_RD(src)        (((src) & 0x0000000FU)>>0)
#define PCP_RB_CPUX_TRC_TRCVIIECTLR_INCLUDE_SET(dst,src)   (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_STOP_WIDTH             8
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_STOP_SHIFT             16
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_STOP_MASK              0x00FF0000U
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_STOP_RD(src)           (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_STOP_SET(dst,src)      (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_START_WIDTH            8
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_START_SHIFT            0
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_START_MASK             0x000000FFU
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_START_RD(src)          (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCVISSCTLR_START_SET(dst,src)     (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CPUX_TRC_TRCVMIDCVR0_VALUE_WIDTH            8
#define PCP_RB_CPUX_TRC_TRCVMIDCVR0_VALUE_SHIFT            0
#define PCP_RB_CPUX_TRC_TRCVMIDCVR0_VALUE_MASK             0x000000FFU
#define PCP_RB_CPUX_TRC_TRCVMIDCVR0_VALUE_RD(src)          (((src) & 0x000000FFU)>>0)
#define PCP_RB_CPUX_TRC_TRCVMIDCVR0_VALUE_SET(dst,src)     (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CSW_TRACE_TRCWM_TRACESTOP_WIDTH             1
#define PCP_RB_CSW_TRACE_TRCWM_TRACESTOP_SHIFT             31
#define PCP_RB_CSW_TRACE_TRCWM_TRACESTOP_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_TRCWM_TRACESTOP_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_TRCWM_TRACESTOP_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_TRCWM_WATERMARK_WIDTH             12
#define PCP_RB_CSW_TRACE_TRCWM_WATERMARK_SHIFT             0
#define PCP_RB_CSW_TRACE_TRCWM_WATERMARK_MASK              0x00000FFFU
#define PCP_RB_CSW_TRACE_TRCWM_WATERMARK_RD(src)           (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_TRCWM_WATERMARK_SET(dst,src)      (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_VREG0_ADDRLO_WIDTH                32
#define PCP_RB_CSW_TRACE_VREG0_ADDRLO_SHIFT                0
#define PCP_RB_CSW_TRACE_VREG0_ADDRLO_MASK                 0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_VREG0_ADDRLO_RD(src)              (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_VREG0_ADDRLO_SET(dst,src)         (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_VREG1_SNPTIMELO_WIDTH             12
#define PCP_RB_CSW_TRACE_VREG1_SNPTIMELO_SHIFT             20
#define PCP_RB_CSW_TRACE_VREG1_SNPTIMELO_MASK              0xFFF00000U
#define PCP_RB_CSW_TRACE_VREG1_SNPTIMELO_RD(src)           (((src) & 0xFFF00000U)>>20)
#define PCP_RB_CSW_TRACE_VREG1_SNPTIMELO_SET(dst,src)      (((dst) & ~0xFFF00000U) | (((u32)(src)<<20) & 0xFFF00000U))
#define PCP_RB_CSW_TRACE_VREG1_SNPRSRCID_WIDTH             6
#define PCP_RB_CSW_TRACE_VREG1_SNPRSRCID_SHIFT             14
#define PCP_RB_CSW_TRACE_VREG1_SNPRSRCID_MASK              0x000FC000U
#define PCP_RB_CSW_TRACE_VREG1_SNPRSRCID_RD(src)           (((src) & 0x000FC000U)>>14)
#define PCP_RB_CSW_TRACE_VREG1_SNPRSRCID_SET(dst,src)      (((dst) & ~0x000FC000U) | (((u32)(src)<<14) & 0x000FC000U))
#define PCP_RB_CSW_TRACE_VREG1_REQAGENTID_WIDTH            4
#define PCP_RB_CSW_TRACE_VREG1_REQAGENTID_SHIFT            9
#define PCP_RB_CSW_TRACE_VREG1_REQAGENTID_MASK             0x00001E00U
#define PCP_RB_CSW_TRACE_VREG1_REQAGENTID_RD(src)          (((src) & 0x00001E00U)>>9)
#define PCP_RB_CSW_TRACE_VREG1_REQAGENTID_SET(dst,src)     (((dst) & ~0x00001E00U) | (((u32)(src)<<9) & 0x00001E00U))
#define PCP_RB_CSW_TRACE_VREG1_REQTYPE_WIDTH               5
#define PCP_RB_CSW_TRACE_VREG1_REQTYPE_SHIFT               4
#define PCP_RB_CSW_TRACE_VREG1_REQTYPE_MASK                0x000001F0U
#define PCP_RB_CSW_TRACE_VREG1_REQTYPE_RD(src)             (((src) & 0x000001F0U)>>4)
#define PCP_RB_CSW_TRACE_VREG1_REQTYPE_SET(dst,src)        (((dst) & ~0x000001F0U) | (((u32)(src)<<4) & 0x000001F0U))
#define PCP_RB_CSW_TRACE_VREG1_ADDRHI_WIDTH                4
#define PCP_RB_CSW_TRACE_VREG1_ADDRHI_SHIFT                0
#define PCP_RB_CSW_TRACE_VREG1_ADDRHI_MASK                 0x0000000FU
#define PCP_RB_CSW_TRACE_VREG1_ADDRHI_RD(src)              (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG1_ADDRHI_SET(dst,src)         (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_VREG10_ACK1MISCLO_WIDTH           4
#define PCP_RB_CSW_TRACE_VREG10_ACK1MISCLO_SHIFT           28
#define PCP_RB_CSW_TRACE_VREG10_ACK1MISCLO_MASK            0xF0000000U
#define PCP_RB_CSW_TRACE_VREG10_ACK1MISCLO_RD(src)         (((src) & 0xF0000000U)>>28)
#define PCP_RB_CSW_TRACE_VREG10_ACK1MISCLO_SET(dst,src)    (((dst) & ~0xF0000000U) | (((u32)(src)<<28) & 0xF0000000U))
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQAGENTID_WIDTH       4
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQAGENTID_SHIFT       24
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQAGENTID_MASK        0x0F000000U
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQAGENTID_RD(src)     (((src) & 0x0F000000U)>>24)
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQAGENTID_SET(dst,src) (((dst) & ~0x0F000000U) | (((u32)(src)<<24) & 0x0F000000U))
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQRSRCID_WIDTH        8
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQRSRCID_SHIFT        16
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQRSRCID_MASK         0x00FF0000U
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQRSRCID_RD(src)      (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CSW_TRACE_VREG10_ACK1REQRSRCID_SET(dst,src) (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFAGENTID_WIDTH       4
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFAGENTID_SHIFT       12
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFAGENTID_MASK        0x0000F000U
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFAGENTID_RD(src)     (((src) & 0x0000F000U)>>12)
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFAGENTID_SET(dst,src) (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFRSRCID_WIDTH        8
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFRSRCID_SHIFT        4
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFRSRCID_MASK         0x00000FF0U
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFRSRCID_RD(src)      (((src) & 0x00000FF0U)>>4)
#define PCP_RB_CSW_TRACE_VREG10_ACK1DEFRSRCID_SET(dst,src) (((dst) & ~0x00000FF0U) | (((u32)(src)<<4) & 0x00000FF0U))
#define PCP_RB_CSW_TRACE_VREG10_ACK1ACKTYPE_WIDTH          4
#define PCP_RB_CSW_TRACE_VREG10_ACK1ACKTYPE_SHIFT          0
#define PCP_RB_CSW_TRACE_VREG10_ACK1ACKTYPE_MASK           0x0000000FU
#define PCP_RB_CSW_TRACE_VREG10_ACK1ACKTYPE_RD(src)        (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG10_ACK1ACKTYPE_SET(dst,src)   (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_VREG11_ACK1SNPRSRCID_WIDTH        6
#define PCP_RB_CSW_TRACE_VREG11_ACK1SNPRSRCID_SHIFT        10
#define PCP_RB_CSW_TRACE_VREG11_ACK1SNPRSRCID_MASK         0x0000FC00U
#define PCP_RB_CSW_TRACE_VREG11_ACK1SNPRSRCID_RD(src)      (((src) & 0x0000FC00U)>>10)
#define PCP_RB_CSW_TRACE_VREG11_ACK1SNPRSRCID_SET(dst,src) (((dst) & ~0x0000FC00U) | (((u32)(src)<<10) & 0x0000FC00U))
#define PCP_RB_CSW_TRACE_VREG11_ACK1ADDROFFSET_WIDTH       6
#define PCP_RB_CSW_TRACE_VREG11_ACK1ADDROFFSET_SHIFT       4
#define PCP_RB_CSW_TRACE_VREG11_ACK1ADDROFFSET_MASK        0x000003F0U
#define PCP_RB_CSW_TRACE_VREG11_ACK1ADDROFFSET_RD(src)     (((src) & 0x000003F0U)>>4)
#define PCP_RB_CSW_TRACE_VREG11_ACK1ADDROFFSET_SET(dst,src) (((dst) & ~0x000003F0U) | (((u32)(src)<<4) & 0x000003F0U))
#define PCP_RB_CSW_TRACE_VREG11_ACK1MISCHI_WIDTH           4
#define PCP_RB_CSW_TRACE_VREG11_ACK1MISCHI_SHIFT           0
#define PCP_RB_CSW_TRACE_VREG11_ACK1MISCHI_MASK            0x0000000FU
#define PCP_RB_CSW_TRACE_VREG11_ACK1MISCHI_RD(src)         (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG11_ACK1MISCHI_SET(dst,src)    (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_VREG12_ACK2MISCLO_WIDTH           4
#define PCP_RB_CSW_TRACE_VREG12_ACK2MISCLO_SHIFT           28
#define PCP_RB_CSW_TRACE_VREG12_ACK2MISCLO_MASK            0xF0000000U
#define PCP_RB_CSW_TRACE_VREG12_ACK2MISCLO_RD(src)         (((src) & 0xF0000000U)>>28)
#define PCP_RB_CSW_TRACE_VREG12_ACK2MISCLO_SET(dst,src)    (((dst) & ~0xF0000000U) | (((u32)(src)<<28) & 0xF0000000U))
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQAGENTID_WIDTH       4
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQAGENTID_SHIFT       24
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQAGENTID_MASK        0x0F000000U
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQAGENTID_RD(src)     (((src) & 0x0F000000U)>>24)
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQAGENTID_SET(dst,src) (((dst) & ~0x0F000000U) | (((u32)(src)<<24) & 0x0F000000U))
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQRSRCID_WIDTH        8
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQRSRCID_SHIFT        16
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQRSRCID_MASK         0x00FF0000U
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQRSRCID_RD(src)      (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CSW_TRACE_VREG12_ACK2REQRSRCID_SET(dst,src) (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFAGENTID_WIDTH       4
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFAGENTID_SHIFT       12
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFAGENTID_MASK        0x0000F000U
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFAGENTID_RD(src)     (((src) & 0x0000F000U)>>12)
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFAGENTID_SET(dst,src) (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFRSRCID_WIDTH        8
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFRSRCID_SHIFT        4
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFRSRCID_MASK         0x00000FF0U
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFRSRCID_RD(src)      (((src) & 0x00000FF0U)>>4)
#define PCP_RB_CSW_TRACE_VREG12_ACK2DEFRSRCID_SET(dst,src) (((dst) & ~0x00000FF0U) | (((u32)(src)<<4) & 0x00000FF0U))
#define PCP_RB_CSW_TRACE_VREG12_ACK2ACKTYPE_WIDTH          4
#define PCP_RB_CSW_TRACE_VREG12_ACK2ACKTYPE_SHIFT          0
#define PCP_RB_CSW_TRACE_VREG12_ACK2ACKTYPE_MASK           0x0000000FU
#define PCP_RB_CSW_TRACE_VREG12_ACK2ACKTYPE_RD(src)        (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG12_ACK2ACKTYPE_SET(dst,src)   (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_VREG13_ACK2SNPRSRCID_WIDTH        6
#define PCP_RB_CSW_TRACE_VREG13_ACK2SNPRSRCID_SHIFT        10
#define PCP_RB_CSW_TRACE_VREG13_ACK2SNPRSRCID_MASK         0x0000FC00U
#define PCP_RB_CSW_TRACE_VREG13_ACK2SNPRSRCID_RD(src)      (((src) & 0x0000FC00U)>>10)
#define PCP_RB_CSW_TRACE_VREG13_ACK2SNPRSRCID_SET(dst,src) (((dst) & ~0x0000FC00U) | (((u32)(src)<<10) & 0x0000FC00U))
#define PCP_RB_CSW_TRACE_VREG13_ACK2ADDROFFSET_WIDTH       6
#define PCP_RB_CSW_TRACE_VREG13_ACK2ADDROFFSET_SHIFT       4
#define PCP_RB_CSW_TRACE_VREG13_ACK2ADDROFFSET_MASK        0x000003F0U
#define PCP_RB_CSW_TRACE_VREG13_ACK2ADDROFFSET_RD(src)     (((src) & 0x000003F0U)>>4)
#define PCP_RB_CSW_TRACE_VREG13_ACK2ADDROFFSET_SET(dst,src) (((dst) & ~0x000003F0U) | (((u32)(src)<<4) & 0x000003F0U))
#define PCP_RB_CSW_TRACE_VREG13_ACK2MISCHI_WIDTH           4
#define PCP_RB_CSW_TRACE_VREG13_ACK2MISCHI_SHIFT           0
#define PCP_RB_CSW_TRACE_VREG13_ACK2MISCHI_MASK            0x0000000FU
#define PCP_RB_CSW_TRACE_VREG13_ACK2MISCHI_RD(src)         (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG13_ACK2MISCHI_SET(dst,src)    (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_VREG2_SNPTIMEHI_WIDTH             8
#define PCP_RB_CSW_TRACE_VREG2_SNPTIMEHI_SHIFT             0
#define PCP_RB_CSW_TRACE_VREG2_SNPTIMEHI_MASK              0x000000FFU
#define PCP_RB_CSW_TRACE_VREG2_SNPTIMEHI_RD(src)           (((src) & 0x000000FFU)>>0)
#define PCP_RB_CSW_TRACE_VREG2_SNPTIMEHI_SET(dst,src)      (((dst) & ~0x000000FFU) | (((u32)(src)<<0) & 0x000000FFU))
#define PCP_RB_CSW_TRACE_VREG4_ACKTIME_WIDTH               20
#define PCP_RB_CSW_TRACE_VREG4_ACKTIME_SHIFT               12
#define PCP_RB_CSW_TRACE_VREG4_ACKTIME_MASK                0xFFFFF000U
#define PCP_RB_CSW_TRACE_VREG4_ACKTIME_RD(src)             (((src) & 0xFFFFF000U)>>12)
#define PCP_RB_CSW_TRACE_VREG4_ACKTIME_SET(dst,src)        (((dst) & ~0xFFFFF000U) | (((u32)(src)<<12) & 0xFFFFF000U))
#define PCP_RB_CSW_TRACE_VREG4_ACK2VLD_WIDTH               1
#define PCP_RB_CSW_TRACE_VREG4_ACK2VLD_SHIFT               2
#define PCP_RB_CSW_TRACE_VREG4_ACK2VLD_MASK                0x00000004U
#define PCP_RB_CSW_TRACE_VREG4_ACK2VLD_RD(src)             (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_TRACE_VREG4_ACK2VLD_SET(dst,src)        (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_TRACE_VREG4_ACK1VLD_WIDTH               1
#define PCP_RB_CSW_TRACE_VREG4_ACK1VLD_SHIFT               1
#define PCP_RB_CSW_TRACE_VREG4_ACK1VLD_MASK                0x00000002U
#define PCP_RB_CSW_TRACE_VREG4_ACK1VLD_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_VREG4_ACK1VLD_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_VREG4_ACK0VLD_WIDTH               1
#define PCP_RB_CSW_TRACE_VREG4_ACK0VLD_SHIFT               0
#define PCP_RB_CSW_TRACE_VREG4_ACK0VLD_MASK                0x00000001U
#define PCP_RB_CSW_TRACE_VREG4_ACK0VLD_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_VREG4_ACK0VLD_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_VREG6_DATATIMELO_WIDTH            16
#define PCP_RB_CSW_TRACE_VREG6_DATATIMELO_SHIFT            16
#define PCP_RB_CSW_TRACE_VREG6_DATATIMELO_MASK             0xFFFF0000U
#define PCP_RB_CSW_TRACE_VREG6_DATATIMELO_RD(src)          (((src) & 0xFFFF0000U)>>16)
#define PCP_RB_CSW_TRACE_VREG6_DATATIMELO_SET(dst,src)     (((dst) & ~0xFFFF0000U) | (((u32)(src)<<16) & 0xFFFF0000U))
#define PCP_RB_CSW_TRACE_VREG6_MSGAGENTID_WIDTH            4
#define PCP_RB_CSW_TRACE_VREG6_MSGAGENTID_SHIFT            12
#define PCP_RB_CSW_TRACE_VREG6_MSGAGENTID_MASK             0x0000F000U
#define PCP_RB_CSW_TRACE_VREG6_MSGAGENTID_RD(src)          (((src) & 0x0000F000U)>>12)
#define PCP_RB_CSW_TRACE_VREG6_MSGAGENTID_SET(dst,src)     (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CSW_TRACE_VREG6_MSGRSRCID_WIDTH             8
#define PCP_RB_CSW_TRACE_VREG6_MSGRSRCID_SHIFT             4
#define PCP_RB_CSW_TRACE_VREG6_MSGRSRCID_MASK              0x00000FF0U
#define PCP_RB_CSW_TRACE_VREG6_MSGRSRCID_RD(src)           (((src) & 0x00000FF0U)>>4)
#define PCP_RB_CSW_TRACE_VREG6_MSGRSRCID_SET(dst,src)      (((dst) & ~0x00000FF0U) | (((u32)(src)<<4) & 0x00000FF0U))
#define PCP_RB_CSW_TRACE_VREG6_DATATYPE_WIDTH              3
#define PCP_RB_CSW_TRACE_VREG6_DATATYPE_SHIFT              1
#define PCP_RB_CSW_TRACE_VREG6_DATATYPE_MASK               0x0000000EU
#define PCP_RB_CSW_TRACE_VREG6_DATATYPE_RD(src)            (((src) & 0x0000000EU)>>1)
#define PCP_RB_CSW_TRACE_VREG6_DATATYPE_SET(dst,src)       (((dst) & ~0x0000000EU) | (((u32)(src)<<1) & 0x0000000EU))
#define PCP_RB_CSW_TRACE_VREG6_ERROR_WIDTH                 1
#define PCP_RB_CSW_TRACE_VREG6_ERROR_SHIFT                 0
#define PCP_RB_CSW_TRACE_VREG6_ERROR_MASK                  0x00000001U
#define PCP_RB_CSW_TRACE_VREG6_ERROR_RD(src)               (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_VREG6_ERROR_SET(dst,src)          (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_VREG7_DATATIMEHI_WIDTH            4
#define PCP_RB_CSW_TRACE_VREG7_DATATIMEHI_SHIFT            0
#define PCP_RB_CSW_TRACE_VREG7_DATATIMEHI_MASK             0x0000000FU
#define PCP_RB_CSW_TRACE_VREG7_DATATIMEHI_RD(src)          (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG7_DATATIMEHI_SET(dst,src)     (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_VREG7_ACK1VLD_WIDTH               1
#define PCP_RB_CSW_TRACE_VREG7_ACK1VLD_SHIFT               1
#define PCP_RB_CSW_TRACE_VREG7_ACK1VLD_MASK                0x00000002U
#define PCP_RB_CSW_TRACE_VREG7_ACK1VLD_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_VREG7_ACK1VLD_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_VREG7_ACK0VLD_WIDTH               1
#define PCP_RB_CSW_TRACE_VREG7_ACK0VLD_SHIFT               0
#define PCP_RB_CSW_TRACE_VREG7_ACK0VLD_MASK                0x00000001U
#define PCP_RB_CSW_TRACE_VREG7_ACK0VLD_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_VREG7_ACK0VLD_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_VREG8_ACK0MISCLO_WIDTH            4
#define PCP_RB_CSW_TRACE_VREG8_ACK0MISCLO_SHIFT            28
#define PCP_RB_CSW_TRACE_VREG8_ACK0MISCLO_MASK             0xF0000000U
#define PCP_RB_CSW_TRACE_VREG8_ACK0MISCLO_RD(src)          (((src) & 0xF0000000U)>>28)
#define PCP_RB_CSW_TRACE_VREG8_ACK0MISCLO_SET(dst,src)     (((dst) & ~0xF0000000U) | (((u32)(src)<<28) & 0xF0000000U))
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQAGENTID_WIDTH        4
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQAGENTID_SHIFT        24
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQAGENTID_MASK         0x0F000000U
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQAGENTID_RD(src)      (((src) & 0x0F000000U)>>24)
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQAGENTID_SET(dst,src) (((dst) & ~0x0F000000U) | (((u32)(src)<<24) & 0x0F000000U))
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQRSRCID_WIDTH         8
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQRSRCID_SHIFT         16
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQRSRCID_MASK          0x00FF0000U
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQRSRCID_RD(src)       (((src) & 0x00FF0000U)>>16)
#define PCP_RB_CSW_TRACE_VREG8_ACK0REQRSRCID_SET(dst,src)  (((dst) & ~0x00FF0000U) | (((u32)(src)<<16) & 0x00FF0000U))
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFAGENTID_WIDTH        4
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFAGENTID_SHIFT        12
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFAGENTID_MASK         0x0000F000U
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFAGENTID_RD(src)      (((src) & 0x0000F000U)>>12)
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFAGENTID_SET(dst,src) (((dst) & ~0x0000F000U) | (((u32)(src)<<12) & 0x0000F000U))
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFRSRCID_WIDTH         8
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFRSRCID_SHIFT         4
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFRSRCID_MASK          0x00000FF0U
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFRSRCID_RD(src)       (((src) & 0x00000FF0U)>>4)
#define PCP_RB_CSW_TRACE_VREG8_ACK0DEFRSRCID_SET(dst,src)  (((dst) & ~0x00000FF0U) | (((u32)(src)<<4) & 0x00000FF0U))
#define PCP_RB_CSW_TRACE_VREG8_ACK0ACKTYPE_WIDTH           4
#define PCP_RB_CSW_TRACE_VREG8_ACK0ACKTYPE_SHIFT           0
#define PCP_RB_CSW_TRACE_VREG8_ACK0ACKTYPE_MASK            0x0000000FU
#define PCP_RB_CSW_TRACE_VREG8_ACK0ACKTYPE_RD(src)         (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG8_ACK0ACKTYPE_SET(dst,src)    (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_VREG9_ACK0SNPRSRCID_WIDTH         6
#define PCP_RB_CSW_TRACE_VREG9_ACK0SNPRSRCID_SHIFT         10
#define PCP_RB_CSW_TRACE_VREG9_ACK0SNPRSRCID_MASK          0x0000FC00U
#define PCP_RB_CSW_TRACE_VREG9_ACK0SNPRSRCID_RD(src)       (((src) & 0x0000FC00U)>>10)
#define PCP_RB_CSW_TRACE_VREG9_ACK0SNPRSRCID_SET(dst,src)  (((dst) & ~0x0000FC00U) | (((u32)(src)<<10) & 0x0000FC00U))
#define PCP_RB_CSW_TRACE_VREG9_ACK0ADDROFFSET_WIDTH        6
#define PCP_RB_CSW_TRACE_VREG9_ACK0ADDROFFSET_SHIFT        4
#define PCP_RB_CSW_TRACE_VREG9_ACK0ADDROFFSET_MASK         0x000003F0U
#define PCP_RB_CSW_TRACE_VREG9_ACK0ADDROFFSET_RD(src)      (((src) & 0x000003F0U)>>4)
#define PCP_RB_CSW_TRACE_VREG9_ACK0ADDROFFSET_SET(dst,src) (((dst) & ~0x000003F0U) | (((u32)(src)<<4) & 0x000003F0U))
#define PCP_RB_CSW_TRACE_VREG9_ACK0MISCHI_WIDTH            4
#define PCP_RB_CSW_TRACE_VREG9_ACK0MISCHI_SHIFT            0
#define PCP_RB_CSW_TRACE_VREG9_ACK0MISCHI_MASK             0x0000000FU
#define PCP_RB_CSW_TRACE_VREG9_ACK0MISCHI_RD(src)          (((src) & 0x0000000FU)>>0)
#define PCP_RB_CSW_TRACE_VREG9_ACK0MISCHI_SET(dst,src)     (((dst) & ~0x0000000FU) | (((u32)(src)<<0) & 0x0000000FU))
#define PCP_RB_CSW_TRACE_WP0ADDRH_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP0ADDRH_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP0ADDRH_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP0ADDRH_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP0ADDRH_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP0ADDRH_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP0ADDRH_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP0ADDRH_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP0ADDRH_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP0ADDRH_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP0ADDRH_ADDRHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP0ADDRH_ADDRHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP0ADDRH_ADDRHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP0ADDRH_ADDRHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP0ADDRH_ADDRHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP0ADDRL_ADDRLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP0ADDRL_ADDRLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP0ADDRL_ADDRLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP0ADDRL_ADDRLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP0ADDRL_ADDRLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP0AGENT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP0AGENT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP0AGENT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP0AGENT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP0AGENT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP0AGENT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP0AGENT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP0AGENT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP0AGENT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP0AGENT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP0AGENT_L3C_WIDTH                1
#define PCP_RB_CSW_TRACE_WP0AGENT_L3C_SHIFT                5
#define PCP_RB_CSW_TRACE_WP0AGENT_L3C_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP0AGENT_L3C_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP0AGENT_L3C_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP0AGENT_IOBPA_WIDTH              1
#define PCP_RB_CSW_TRACE_WP0AGENT_IOBPA_SHIFT              4
#define PCP_RB_CSW_TRACE_WP0AGENT_IOBPA_MASK               0x00000010U
#define PCP_RB_CSW_TRACE_WP0AGENT_IOBPA_RD(src)            (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_TRACE_WP0AGENT_IOBPA_SET(dst,src)       (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD3_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD3_SHIFT               3
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD3_MASK                0x00000008U
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD3_RD(src)             (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD3_SET(dst,src)        (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD2_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD2_SHIFT               2
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD2_MASK                0x00000004U
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD2_RD(src)             (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD2_SET(dst,src)        (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD1_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD1_SHIFT               1
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD1_MASK                0x00000002U
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD1_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD1_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD0_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD0_SHIFT               0
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD0_MASK                0x00000001U
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD0_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_WP0AGENT_PMD0_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_WP0COUNT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP0COUNT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP0COUNT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP0COUNT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP0COUNT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP0COUNT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP0COUNT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP0COUNT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP0COUNT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP0COUNT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP0COUNT_COUNT_WIDTH              12
#define PCP_RB_CSW_TRACE_WP0COUNT_COUNT_SHIFT              0
#define PCP_RB_CSW_TRACE_WP0COUNT_COUNT_MASK               0x00000FFFU
#define PCP_RB_CSW_TRACE_WP0COUNT_COUNT_RD(src)            (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP0COUNT_COUNT_SET(dst,src)       (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP0EVENT_STARTEVENT_WIDTH         1
#define PCP_RB_CSW_TRACE_WP0EVENT_STARTEVENT_SHIFT         31
#define PCP_RB_CSW_TRACE_WP0EVENT_STARTEVENT_MASK          0x80000000U
#define PCP_RB_CSW_TRACE_WP0EVENT_STARTEVENT_RD(src)       (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP0EVENT_STARTEVENT_SET(dst,src)  (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPEVENT_WIDTH          1
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPEVENT_SHIFT          30
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPEVENT_MASK           0x40000000U
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPEVENT_RD(src)        (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPEVENT_SET(dst,src)   (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPCOUNT_WIDTH          12
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPCOUNT_SHIFT          0
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPCOUNT_MASK           0x00000FFFU
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPCOUNT_RD(src)        (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP0EVENT_STOPCOUNT_SET(dst,src)   (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP0MASKH_MASKHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP0MASKH_MASKHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP0MASKH_MASKHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP0MASKH_MASKHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP0MASKH_MASKHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP0MASKL_MASKLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP0MASKL_MASKLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP0MASKL_MASKLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP0MASKL_MASKLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP0MASKL_MASKLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP0TYPE_CONDEN_WIDTH              1
#define PCP_RB_CSW_TRACE_WP0TYPE_CONDEN_SHIFT              31
#define PCP_RB_CSW_TRACE_WP0TYPE_CONDEN_MASK               0x80000000U
#define PCP_RB_CSW_TRACE_WP0TYPE_CONDEN_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP0TYPE_CONDEN_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP0TYPE_NEGMATCH_WIDTH            1
#define PCP_RB_CSW_TRACE_WP0TYPE_NEGMATCH_SHIFT            30
#define PCP_RB_CSW_TRACE_WP0TYPE_NEGMATCH_MASK             0x40000000U
#define PCP_RB_CSW_TRACE_WP0TYPE_NEGMATCH_RD(src)          (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP0TYPE_NEGMATCH_SET(dst,src)     (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKRD_SHIFT               9
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKRD_MASK                0x00000200U
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKRD_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKRD_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKWR_SHIFT               8
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKWR_MASK                0x00000100U
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKWR_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_TRACE_WP0TYPE_BLKWR_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLRD_SHIFT               7
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLRD_MASK                0x00000080U
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLRD_RD(src)             (((src) & 0x00000080U)>>7)
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLRD_SET(dst,src)        (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLWR_SHIFT               6
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLWR_MASK                0x00000040U
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLWR_RD(src)             (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_TRACE_WP0TYPE_PTLWR_SET(dst,src)        (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_TRACE_WP0TYPE_MGMT_WIDTH                1
#define PCP_RB_CSW_TRACE_WP0TYPE_MGMT_SHIFT                5
#define PCP_RB_CSW_TRACE_WP0TYPE_MGMT_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP0TYPE_MGMT_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP0TYPE_MGMT_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP0TYPE_REQTYPE_WIDTH             5
#define PCP_RB_CSW_TRACE_WP0TYPE_REQTYPE_SHIFT             0
#define PCP_RB_CSW_TRACE_WP0TYPE_REQTYPE_MASK              0x0000001FU
#define PCP_RB_CSW_TRACE_WP0TYPE_REQTYPE_RD(src)           (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_TRACE_WP0TYPE_REQTYPE_SET(dst,src)      (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_TRACE_WP1ADDRH_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP1ADDRH_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP1ADDRH_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP1ADDRH_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP1ADDRH_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP1ADDRH_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP1ADDRH_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP1ADDRH_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP1ADDRH_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP1ADDRH_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP1ADDRH_ADDRHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP1ADDRH_ADDRHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP1ADDRH_ADDRHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP1ADDRH_ADDRHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP1ADDRH_ADDRHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP1ADDRL_ADDRLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP1ADDRL_ADDRLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP1ADDRL_ADDRLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP1ADDRL_ADDRLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP1ADDRL_ADDRLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP1AGENT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP1AGENT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP1AGENT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP1AGENT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP1AGENT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP1AGENT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP1AGENT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP1AGENT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP1AGENT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP1AGENT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP1AGENT_L3C_WIDTH                1
#define PCP_RB_CSW_TRACE_WP1AGENT_L3C_SHIFT                5
#define PCP_RB_CSW_TRACE_WP1AGENT_L3C_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP1AGENT_L3C_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP1AGENT_L3C_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP1AGENT_IOBPA_WIDTH              1
#define PCP_RB_CSW_TRACE_WP1AGENT_IOBPA_SHIFT              4
#define PCP_RB_CSW_TRACE_WP1AGENT_IOBPA_MASK               0x00000010U
#define PCP_RB_CSW_TRACE_WP1AGENT_IOBPA_RD(src)            (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_TRACE_WP1AGENT_IOBPA_SET(dst,src)       (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD3_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD3_SHIFT               3
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD3_MASK                0x00000008U
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD3_RD(src)             (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD3_SET(dst,src)        (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD2_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD2_SHIFT               2
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD2_MASK                0x00000004U
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD2_RD(src)             (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD2_SET(dst,src)        (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD1_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD1_SHIFT               1
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD1_MASK                0x00000002U
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD1_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD1_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD0_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD0_SHIFT               0
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD0_MASK                0x00000001U
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD0_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_WP1AGENT_PMD0_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_WP1COUNT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP1COUNT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP1COUNT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP1COUNT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP1COUNT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP1COUNT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP1COUNT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP1COUNT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP1COUNT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP1COUNT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP1COUNT_COUNT_WIDTH              12
#define PCP_RB_CSW_TRACE_WP1COUNT_COUNT_SHIFT              0
#define PCP_RB_CSW_TRACE_WP1COUNT_COUNT_MASK               0x00000FFFU
#define PCP_RB_CSW_TRACE_WP1COUNT_COUNT_RD(src)            (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP1COUNT_COUNT_SET(dst,src)       (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP1EVENT_STARTEVENT_WIDTH         1
#define PCP_RB_CSW_TRACE_WP1EVENT_STARTEVENT_SHIFT         31
#define PCP_RB_CSW_TRACE_WP1EVENT_STARTEVENT_MASK          0x80000000U
#define PCP_RB_CSW_TRACE_WP1EVENT_STARTEVENT_RD(src)       (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP1EVENT_STARTEVENT_SET(dst,src)  (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPEVENT_WIDTH          1
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPEVENT_SHIFT          30
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPEVENT_MASK           0x40000000U
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPEVENT_RD(src)        (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPEVENT_SET(dst,src)   (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPCOUNT_WIDTH          12
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPCOUNT_SHIFT          0
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPCOUNT_MASK           0x00000FFFU
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPCOUNT_RD(src)        (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP1EVENT_STOPCOUNT_SET(dst,src)   (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP1MASKH_MASKHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP1MASKH_MASKHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP1MASKH_MASKHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP1MASKH_MASKHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP1MASKH_MASKHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP1MASKL_MASKLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP1MASKL_MASKLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP1MASKL_MASKLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP1MASKL_MASKLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP1MASKL_MASKLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP1TYPE_CONDEN_WIDTH              1
#define PCP_RB_CSW_TRACE_WP1TYPE_CONDEN_SHIFT              31
#define PCP_RB_CSW_TRACE_WP1TYPE_CONDEN_MASK               0x80000000U
#define PCP_RB_CSW_TRACE_WP1TYPE_CONDEN_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP1TYPE_CONDEN_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP1TYPE_NEGMATCH_WIDTH            1
#define PCP_RB_CSW_TRACE_WP1TYPE_NEGMATCH_SHIFT            30
#define PCP_RB_CSW_TRACE_WP1TYPE_NEGMATCH_MASK             0x40000000U
#define PCP_RB_CSW_TRACE_WP1TYPE_NEGMATCH_RD(src)          (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP1TYPE_NEGMATCH_SET(dst,src)     (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKRD_SHIFT               9
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKRD_MASK                0x00000200U
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKRD_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKRD_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKWR_SHIFT               8
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKWR_MASK                0x00000100U
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKWR_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_TRACE_WP1TYPE_BLKWR_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLRD_SHIFT               7
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLRD_MASK                0x00000080U
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLRD_RD(src)             (((src) & 0x00000080U)>>7)
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLRD_SET(dst,src)        (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLWR_SHIFT               6
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLWR_MASK                0x00000040U
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLWR_RD(src)             (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_TRACE_WP1TYPE_PTLWR_SET(dst,src)        (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_TRACE_WP1TYPE_MGMT_WIDTH                1
#define PCP_RB_CSW_TRACE_WP1TYPE_MGMT_SHIFT                5
#define PCP_RB_CSW_TRACE_WP1TYPE_MGMT_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP1TYPE_MGMT_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP1TYPE_MGMT_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP1TYPE_REQTYPE_WIDTH             5
#define PCP_RB_CSW_TRACE_WP1TYPE_REQTYPE_SHIFT             0
#define PCP_RB_CSW_TRACE_WP1TYPE_REQTYPE_MASK              0x0000001FU
#define PCP_RB_CSW_TRACE_WP1TYPE_REQTYPE_RD(src)           (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_TRACE_WP1TYPE_REQTYPE_SET(dst,src)      (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_TRACE_WP2ADDRH_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP2ADDRH_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP2ADDRH_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP2ADDRH_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP2ADDRH_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP2ADDRH_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP2ADDRH_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP2ADDRH_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP2ADDRH_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP2ADDRH_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP2ADDRH_ADDRHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP2ADDRH_ADDRHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP2ADDRH_ADDRHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP2ADDRH_ADDRHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP2ADDRH_ADDRHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP2ADDRL_ADDRLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP2ADDRL_ADDRLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP2ADDRL_ADDRLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP2ADDRL_ADDRLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP2ADDRL_ADDRLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP2AGENT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP2AGENT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP2AGENT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP2AGENT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP2AGENT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP2AGENT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP2AGENT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP2AGENT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP2AGENT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP2AGENT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP2AGENT_L3C_WIDTH                1
#define PCP_RB_CSW_TRACE_WP2AGENT_L3C_SHIFT                5
#define PCP_RB_CSW_TRACE_WP2AGENT_L3C_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP2AGENT_L3C_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP2AGENT_L3C_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP2AGENT_IOBPA_WIDTH              1
#define PCP_RB_CSW_TRACE_WP2AGENT_IOBPA_SHIFT              4
#define PCP_RB_CSW_TRACE_WP2AGENT_IOBPA_MASK               0x00000010U
#define PCP_RB_CSW_TRACE_WP2AGENT_IOBPA_RD(src)            (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_TRACE_WP2AGENT_IOBPA_SET(dst,src)       (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD3_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD3_SHIFT               3
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD3_MASK                0x00000008U
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD3_RD(src)             (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD3_SET(dst,src)        (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD2_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD2_SHIFT               2
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD2_MASK                0x00000004U
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD2_RD(src)             (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD2_SET(dst,src)        (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD1_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD1_SHIFT               1
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD1_MASK                0x00000002U
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD1_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD1_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD0_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD0_SHIFT               0
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD0_MASK                0x00000001U
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD0_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_WP2AGENT_PMD0_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_WP2COUNT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP2COUNT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP2COUNT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP2COUNT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP2COUNT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP2COUNT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP2COUNT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP2COUNT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP2COUNT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP2COUNT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP2COUNT_COUNT_WIDTH              12
#define PCP_RB_CSW_TRACE_WP2COUNT_COUNT_SHIFT              0
#define PCP_RB_CSW_TRACE_WP2COUNT_COUNT_MASK               0x00000FFFU
#define PCP_RB_CSW_TRACE_WP2COUNT_COUNT_RD(src)            (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP2COUNT_COUNT_SET(dst,src)       (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP2EVENT_STARTEVENT_WIDTH         1
#define PCP_RB_CSW_TRACE_WP2EVENT_STARTEVENT_SHIFT         31
#define PCP_RB_CSW_TRACE_WP2EVENT_STARTEVENT_MASK          0x80000000U
#define PCP_RB_CSW_TRACE_WP2EVENT_STARTEVENT_RD(src)       (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP2EVENT_STARTEVENT_SET(dst,src)  (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPEVENT_WIDTH          1
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPEVENT_SHIFT          30
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPEVENT_MASK           0x40000000U
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPEVENT_RD(src)        (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPEVENT_SET(dst,src)   (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPCOUNT_WIDTH          12
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPCOUNT_SHIFT          0
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPCOUNT_MASK           0x00000FFFU
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPCOUNT_RD(src)        (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP2EVENT_STOPCOUNT_SET(dst,src)   (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP2MASKH_MASKHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP2MASKH_MASKHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP2MASKH_MASKHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP2MASKH_MASKHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP2MASKH_MASKHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP2MASKL_MASKLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP2MASKL_MASKLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP2MASKL_MASKLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP2MASKL_MASKLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP2MASKL_MASKLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP2TYPE_CONDEN_WIDTH              1
#define PCP_RB_CSW_TRACE_WP2TYPE_CONDEN_SHIFT              31
#define PCP_RB_CSW_TRACE_WP2TYPE_CONDEN_MASK               0x80000000U
#define PCP_RB_CSW_TRACE_WP2TYPE_CONDEN_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP2TYPE_CONDEN_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP2TYPE_NEGMATCH_WIDTH            1
#define PCP_RB_CSW_TRACE_WP2TYPE_NEGMATCH_SHIFT            30
#define PCP_RB_CSW_TRACE_WP2TYPE_NEGMATCH_MASK             0x40000000U
#define PCP_RB_CSW_TRACE_WP2TYPE_NEGMATCH_RD(src)          (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP2TYPE_NEGMATCH_SET(dst,src)     (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKRD_SHIFT               9
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKRD_MASK                0x00000200U
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKRD_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKRD_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKWR_SHIFT               8
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKWR_MASK                0x00000100U
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKWR_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_TRACE_WP2TYPE_BLKWR_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLRD_SHIFT               7
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLRD_MASK                0x00000080U
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLRD_RD(src)             (((src) & 0x00000080U)>>7)
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLRD_SET(dst,src)        (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLWR_SHIFT               6
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLWR_MASK                0x00000040U
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLWR_RD(src)             (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_TRACE_WP2TYPE_PTLWR_SET(dst,src)        (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_TRACE_WP2TYPE_MGMT_WIDTH                1
#define PCP_RB_CSW_TRACE_WP2TYPE_MGMT_SHIFT                5
#define PCP_RB_CSW_TRACE_WP2TYPE_MGMT_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP2TYPE_MGMT_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP2TYPE_MGMT_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP2TYPE_REQTYPE_WIDTH             5
#define PCP_RB_CSW_TRACE_WP2TYPE_REQTYPE_SHIFT             0
#define PCP_RB_CSW_TRACE_WP2TYPE_REQTYPE_MASK              0x0000001FU
#define PCP_RB_CSW_TRACE_WP2TYPE_REQTYPE_RD(src)           (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_TRACE_WP2TYPE_REQTYPE_SET(dst,src)      (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_CSW_TRACE_WP3ADDRH_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP3ADDRH_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP3ADDRH_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP3ADDRH_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP3ADDRH_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP3ADDRH_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP3ADDRH_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP3ADDRH_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP3ADDRH_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP3ADDRH_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP3ADDRH_ADDRHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP3ADDRH_ADDRHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP3ADDRH_ADDRHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP3ADDRH_ADDRHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP3ADDRH_ADDRHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP3ADDRL_ADDRLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP3ADDRL_ADDRLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP3ADDRL_ADDRLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP3ADDRL_ADDRLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP3ADDRL_ADDRLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP3AGENT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP3AGENT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP3AGENT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP3AGENT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP3AGENT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP3AGENT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP3AGENT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP3AGENT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP3AGENT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP3AGENT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP3AGENT_L3C_WIDTH                1
#define PCP_RB_CSW_TRACE_WP3AGENT_L3C_SHIFT                5
#define PCP_RB_CSW_TRACE_WP3AGENT_L3C_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP3AGENT_L3C_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP3AGENT_L3C_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP3AGENT_IOBPA_WIDTH              1
#define PCP_RB_CSW_TRACE_WP3AGENT_IOBPA_SHIFT              4
#define PCP_RB_CSW_TRACE_WP3AGENT_IOBPA_MASK               0x00000010U
#define PCP_RB_CSW_TRACE_WP3AGENT_IOBPA_RD(src)            (((src) & 0x00000010U)>>4)
#define PCP_RB_CSW_TRACE_WP3AGENT_IOBPA_SET(dst,src)       (((dst) & ~0x00000010U) | (((u32)(src)<<4) & 0x00000010U))
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD3_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD3_SHIFT               3
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD3_MASK                0x00000008U
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD3_RD(src)             (((src) & 0x00000008U)>>3)
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD3_SET(dst,src)        (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD2_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD2_SHIFT               2
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD2_MASK                0x00000004U
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD2_RD(src)             (((src) & 0x00000004U)>>2)
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD2_SET(dst,src)        (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD1_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD1_SHIFT               1
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD1_MASK                0x00000002U
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD1_RD(src)             (((src) & 0x00000002U)>>1)
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD1_SET(dst,src)        (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD0_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD0_SHIFT               0
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD0_MASK                0x00000001U
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD0_RD(src)             (((src) & 0x00000001U)>>0)
#define PCP_RB_CSW_TRACE_WP3AGENT_PMD0_SET(dst,src)        (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_CSW_TRACE_WP3COUNT_CONDEN_WIDTH             1
#define PCP_RB_CSW_TRACE_WP3COUNT_CONDEN_SHIFT             31
#define PCP_RB_CSW_TRACE_WP3COUNT_CONDEN_MASK              0x80000000U
#define PCP_RB_CSW_TRACE_WP3COUNT_CONDEN_RD(src)           (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP3COUNT_CONDEN_SET(dst,src)      (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP3COUNT_NEGMATCH_WIDTH           1
#define PCP_RB_CSW_TRACE_WP3COUNT_NEGMATCH_SHIFT           30
#define PCP_RB_CSW_TRACE_WP3COUNT_NEGMATCH_MASK            0x40000000U
#define PCP_RB_CSW_TRACE_WP3COUNT_NEGMATCH_RD(src)         (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP3COUNT_NEGMATCH_SET(dst,src)    (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP3COUNT_COUNT_WIDTH              12
#define PCP_RB_CSW_TRACE_WP3COUNT_COUNT_SHIFT              0
#define PCP_RB_CSW_TRACE_WP3COUNT_COUNT_MASK               0x00000FFFU
#define PCP_RB_CSW_TRACE_WP3COUNT_COUNT_RD(src)            (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP3COUNT_COUNT_SET(dst,src)       (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP3EVENT_STARTEVENT_WIDTH         1
#define PCP_RB_CSW_TRACE_WP3EVENT_STARTEVENT_SHIFT         31
#define PCP_RB_CSW_TRACE_WP3EVENT_STARTEVENT_MASK          0x80000000U
#define PCP_RB_CSW_TRACE_WP3EVENT_STARTEVENT_RD(src)       (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP3EVENT_STARTEVENT_SET(dst,src)  (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPEVENT_WIDTH          1
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPEVENT_SHIFT          30
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPEVENT_MASK           0x40000000U
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPEVENT_RD(src)        (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPEVENT_SET(dst,src)   (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPCOUNT_WIDTH          12
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPCOUNT_SHIFT          0
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPCOUNT_MASK           0x00000FFFU
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPCOUNT_RD(src)        (((src) & 0x00000FFFU)>>0)
#define PCP_RB_CSW_TRACE_WP3EVENT_STOPCOUNT_SET(dst,src)   (((dst) & ~0x00000FFFU) | (((u32)(src)<<0) & 0x00000FFFU))
#define PCP_RB_CSW_TRACE_WP3MASKH_MASKHI_WIDTH             10
#define PCP_RB_CSW_TRACE_WP3MASKH_MASKHI_SHIFT             0
#define PCP_RB_CSW_TRACE_WP3MASKH_MASKHI_MASK              0x000003FFU
#define PCP_RB_CSW_TRACE_WP3MASKH_MASKHI_RD(src)           (((src) & 0x000003FFU)>>0)
#define PCP_RB_CSW_TRACE_WP3MASKH_MASKHI_SET(dst,src)      (((dst) & ~0x000003FFU) | (((u32)(src)<<0) & 0x000003FFU))
#define PCP_RB_CSW_TRACE_WP3MASKL_MASKLO_WIDTH             32
#define PCP_RB_CSW_TRACE_WP3MASKL_MASKLO_SHIFT             0
#define PCP_RB_CSW_TRACE_WP3MASKL_MASKLO_MASK              0xFFFFFFFFU
#define PCP_RB_CSW_TRACE_WP3MASKL_MASKLO_RD(src)           (((src) & 0xFFFFFFFFU)>>0)
#define PCP_RB_CSW_TRACE_WP3MASKL_MASKLO_SET(dst,src)      (((dst) & ~0xFFFFFFFFU) | (((u32)(src)<<0) & 0xFFFFFFFFU))
#define PCP_RB_CSW_TRACE_WP3TYPE_CONDEN_WIDTH              1
#define PCP_RB_CSW_TRACE_WP3TYPE_CONDEN_SHIFT              31
#define PCP_RB_CSW_TRACE_WP3TYPE_CONDEN_MASK               0x80000000U
#define PCP_RB_CSW_TRACE_WP3TYPE_CONDEN_RD(src)            (((src) & 0x80000000U)>>31)
#define PCP_RB_CSW_TRACE_WP3TYPE_CONDEN_SET(dst,src)       (((dst) & ~0x80000000U) | (((u32)(src)<<31) & 0x80000000U))
#define PCP_RB_CSW_TRACE_WP3TYPE_NEGMATCH_WIDTH            1
#define PCP_RB_CSW_TRACE_WP3TYPE_NEGMATCH_SHIFT            30
#define PCP_RB_CSW_TRACE_WP3TYPE_NEGMATCH_MASK             0x40000000U
#define PCP_RB_CSW_TRACE_WP3TYPE_NEGMATCH_RD(src)          (((src) & 0x40000000U)>>30)
#define PCP_RB_CSW_TRACE_WP3TYPE_NEGMATCH_SET(dst,src)     (((dst) & ~0x40000000U) | (((u32)(src)<<30) & 0x40000000U))
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKRD_SHIFT               9
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKRD_MASK                0x00000200U
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKRD_RD(src)             (((src) & 0x00000200U)>>9)
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKRD_SET(dst,src)        (((dst) & ~0x00000200U) | (((u32)(src)<<9) & 0x00000200U))
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKWR_SHIFT               8
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKWR_MASK                0x00000100U
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKWR_RD(src)             (((src) & 0x00000100U)>>8)
#define PCP_RB_CSW_TRACE_WP3TYPE_BLKWR_SET(dst,src)        (((dst) & ~0x00000100U) | (((u32)(src)<<8) & 0x00000100U))
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLRD_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLRD_SHIFT               7
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLRD_MASK                0x00000080U
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLRD_RD(src)             (((src) & 0x00000080U)>>7)
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLRD_SET(dst,src)        (((dst) & ~0x00000080U) | (((u32)(src)<<7) & 0x00000080U))
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLWR_WIDTH               1
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLWR_SHIFT               6
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLWR_MASK                0x00000040U
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLWR_RD(src)             (((src) & 0x00000040U)>>6)
#define PCP_RB_CSW_TRACE_WP3TYPE_PTLWR_SET(dst,src)        (((dst) & ~0x00000040U) | (((u32)(src)<<6) & 0x00000040U))
#define PCP_RB_CSW_TRACE_WP3TYPE_MGMT_WIDTH                1
#define PCP_RB_CSW_TRACE_WP3TYPE_MGMT_SHIFT                5
#define PCP_RB_CSW_TRACE_WP3TYPE_MGMT_MASK                 0x00000020U
#define PCP_RB_CSW_TRACE_WP3TYPE_MGMT_RD(src)              (((src) & 0x00000020U)>>5)
#define PCP_RB_CSW_TRACE_WP3TYPE_MGMT_SET(dst,src)         (((dst) & ~0x00000020U) | (((u32)(src)<<5) & 0x00000020U))
#define PCP_RB_CSW_TRACE_WP3TYPE_REQTYPE_WIDTH             5
#define PCP_RB_CSW_TRACE_WP3TYPE_REQTYPE_SHIFT             0
#define PCP_RB_CSW_TRACE_WP3TYPE_REQTYPE_MASK              0x0000001FU
#define PCP_RB_CSW_TRACE_WP3TYPE_REQTYPE_RD(src)           (((src) & 0x0000001FU)>>0)
#define PCP_RB_CSW_TRACE_WP3TYPE_REQTYPE_SET(dst,src)      (((dst) & ~0x0000001FU) | (((u32)(src)<<0) & 0x0000001FU))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MWRACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MWRACCESSERR_SHIFT 3
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MWRACCESSERR_MASK 0x00000008U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MWRACCESSERR_RD(src) (((src) & 0x00000008U)>>3)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MWRACCESSERR_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_WRACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_WRACCESSERR_SHIFT 2
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_WRACCESSERR_MASK 0x00000004U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_WRACCESSERR_RD(src) (((src) & 0x00000004U)>>2)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_WRACCESSERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MRDACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MRDACCESSERR_SHIFT 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MRDACCESSERR_MASK 0x00000002U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MRDACCESSERR_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_MRDACCESSERR_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_RDACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_RDACCESSERR_SHIFT 0
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_RDACCESSERR_MASK 0x00000001U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_RDACCESSERR_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTMSK_RDACCESSERR_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MWRACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MWRACCESSERR_SHIFT 3
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MWRACCESSERR_MASK 0x00000008U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MWRACCESSERR_RD(src) (((src) & 0x00000008U)>>3)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MWRACCESSERR_SET(dst,src) (((dst) & ~0x00000008U) | (((u32)(src)<<3) & 0x00000008U))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_WRACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_WRACCESSERR_SHIFT 2
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_WRACCESSERR_MASK 0x00000004U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_WRACCESSERR_RD(src) (((src) & 0x00000004U)>>2)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_WRACCESSERR_SET(dst,src) (((dst) & ~0x00000004U) | (((u32)(src)<<2) & 0x00000004U))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MRDACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MRDACCESSERR_SHIFT 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MRDACCESSERR_MASK 0x00000002U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MRDACCESSERR_RD(src) (((src) & 0x00000002U)>>1)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_MRDACCESSERR_SET(dst,src) (((dst) & ~0x00000002U) | (((u32)(src)<<1) & 0x00000002U))
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_RDACCESSERR_WIDTH 1
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_RDACCESSERR_SHIFT 0
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_RDACCESSERR_MASK 0x00000001U
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_RDACCESSERR_RD(src) (((src) & 0x00000001U)>>0)
#define PCP_RB_IOB_ERR_XGICTRANSERRINTSTS_RDACCESSERR_SET(dst,src) (((dst) & ~0x00000001U) | (((u32)(src)<<0) & 0x00000001U))

#endif     /*  APM_PCP_CSR_H  */
